@@ -53,13 +53,25 @@ namespace MagickCore
 
 //
 // Provide appropriate DLL imports/exports for Visual C++,
-// Borland C++Builder and MinGW builds
+// Borland C++Builder and MinGW builds.
 //
 #if defined(WIN32) && !defined(__CYGWIN__) && !defined(__MINGW32__)
 # define MagickCplusPlusDLLSupported
 #endif
 #if defined(MagickCplusPlusDLLSupported)
-#  if defined(_MT) && defined(_DLL) && !defined(_LIB)
+#  if defined(_MT) && defined(_DLL) && !defined(_LIB) && !defined(STATIC_MAGICK)
+//
+// In a native Windows build, the following defines are used:
+//
+//   _MT         = Multithreaded
+//   _DLL        = Using code is part of a DLL
+//   _LIB        = Using code is being built as a library.
+//   _MAGICKMOD_ = Build uses loadable modules (Magick++ does not care about this)
+//
+// In the case where ImageMagick is built as a static library but the
+// using code is dynamic, STATIC_MAGICK may be defined in the project to
+// override triggering dynamic library behavior.
+//
 #    define MagickDLLBuild
 #    if defined(_VISUALC_)
 #      pragma warning( disable: 4273 )  /* Disable the stupid dll linkage warnings */
@@ -210,10 +222,12 @@ namespace Magick
   using MagickCore::FaxCompression;
   using MagickCore::Group4Compression;
   using MagickCore::JPEGCompression;
+  using MagickCore::LZMACompression;
   using MagickCore::LZWCompression;
   using MagickCore::RLECompression;
   using MagickCore::ZipCompression;
 
+  // Dispose methods
   using MagickCore::DisposeType;
   using MagickCore::UndefinedDispose;
   using MagickCore::NoneDispose;
@@ -451,6 +465,10 @@ namespace Magick
   using MagickCore::ShepardsColorInterpolate;
   using MagickCore::VoronoiColorInterpolate;
 
+  // Statistic type
+  using MagickCore::MedianStatistic;
+  using MagickCore::NonpeakStatistic;
+
   // StorageType type
   using MagickCore::StorageType;
   using MagickCore::CharPixel;
@@ -486,18 +504,41 @@ namespace Magick
   using MagickCore::OverlineDecoration;
   using MagickCore::LineThroughDecoration;
 
+  // Virtual pixel methods
+  using MagickCore::VirtualPixelMethod;
+  using MagickCore::UndefinedVirtualPixelMethod;
+  using MagickCore::BackgroundVirtualPixelMethod;
+  using MagickCore::DitherVirtualPixelMethod;
+  using MagickCore::EdgeVirtualPixelMethod;
+  using MagickCore::MirrorVirtualPixelMethod;
+  using MagickCore::RandomVirtualPixelMethod;
+  using MagickCore::TileVirtualPixelMethod;
+  using MagickCore::TransparentVirtualPixelMethod;
+  using MagickCore::MaskVirtualPixelMethod;
+  using MagickCore::BlackVirtualPixelMethod;
+  using MagickCore::GrayVirtualPixelMethod;
+  using MagickCore::WhiteVirtualPixelMethod;
+  using MagickCore::HorizontalTileVirtualPixelMethod;
+  using MagickCore::VerticalTileVirtualPixelMethod;
+  using MagickCore::HorizontalTileEdgeVirtualPixelMethod;
+  using MagickCore::VerticalTileEdgeVirtualPixelMethod;
+  using MagickCore::CheckerTileVirtualPixelMethod;
+
 #if defined(MAGICKCORE_IMPLEMENTATION)
   //
   // ImageMagick symbols used in implementation code
   //
   using MagickCore::AcquireCacheView;
-  using MagickCore::AcquireCacheViewPixels;
+  using MagickCore::AcquireExceptionInfo;
+  using MagickCore::GetCacheViewVirtualPixels;
   using MagickCore::AcquireImage;
-  using MagickCore::AcquireImagePixels;
+  using MagickCore::GetVirtualPixels;
+  using MagickCore::AcquireIndexes;
+  using MagickCore::AcquireKernelInfo;
   using MagickCore::AcquireMagickMemory;
   using MagickCore::AcquireQuantumInfo;
   using MagickCore::AcquireString;
-	using MagickCore::AcquireStringInfo;
+  using MagickCore::AcquireStringInfo;
   using MagickCore::AdaptiveBlurImage;
   using MagickCore::AdaptiveThresholdImage;
   using MagickCore::AddNoiseImage;
@@ -529,7 +570,9 @@ namespace Magick
   using MagickCore::CoderError;
   using MagickCore::CoderFatalError;
   using MagickCore::CoderWarning;
+  using MagickCore::ColorDecisionListImage;
   using MagickCore::ColorizeImage;
+  using MagickCore::ColorMatrixImage;
   using MagickCore::ColorPacket;
   using MagickCore::CompositeImage;
   using MagickCore::ConfigureError;
@@ -546,6 +589,7 @@ namespace Magick
   using MagickCore::CorruptImageWarning;
   using MagickCore::CropImage;
   using MagickCore::CycleColormapImage;
+  using MagickCore::DeconstructImages;
   using MagickCore::DelegateError;
   using MagickCore::DelegateFatalError;
   using MagickCore::DelegateWarning;
@@ -558,6 +602,7 @@ namespace Magick
   using MagickCore::DestroyExceptionInfo;
   using MagickCore::DestroyImageInfo;
   using MagickCore::DestroyImageList;
+  using MagickCore::DestroyKernelInfo;
   using MagickCore::DestroyMagickWand;
   using MagickCore::DestroyPixelWand;
   using MagickCore::DestroyQuantizeInfo;
@@ -660,6 +705,7 @@ namespace Magick
   using MagickCore::ExceptionType;
   using MagickCore::ExportImagePixels;
   using MagickCore::ExportQuantumPixels;
+  using MagickCore::ExtentImage;
   using MagickCore::FileOpenError;
   using MagickCore::FileOpenFatalError;
   using MagickCore::FileOpenWarning;
@@ -667,6 +713,7 @@ namespace Magick
   using MagickCore::FloodfillPaintImage;
   using MagickCore::FlopImage;
   using MagickCore::FormatMagickString;
+  using MagickCore::ForwardFourierTransformImage;
   using MagickCore::FrameImage;
   using MagickCore::FrameInfo;
   using MagickCore::FxImageChannel;
@@ -675,10 +722,11 @@ namespace Magick
   using MagickCore::GaussianBlurImage;
   using MagickCore::GaussianBlurImageChannel;
   using MagickCore::GetAffineMatrix;
+  using MagickCore::GetAuthenticIndexQueue;
   using MagickCore::GetBlobSize;
   using MagickCore::GetCacheViewException;
-  using MagickCore::GetCacheViewIndexes;
-  using MagickCore::GetCacheViewPixels;
+  using MagickCore::GetCacheViewAuthenticIndexQueue;
+  using MagickCore::GetCacheViewAuthenticPixels;
   using MagickCore::GetColorTuple;
   using MagickCore::GetDrawInfo;
   using MagickCore::GetExceptionInfo;
@@ -686,12 +734,14 @@ namespace Magick
   using MagickCore::GetImageBoundingBox;
   using MagickCore::GetImageChannelDepth;
   using MagickCore::GetImageChannelMean;
+  using MagickCore::GetImageChannelKurtosis;
   using MagickCore::GetImageChannelRange;
   using MagickCore::GetImageClipMask;
   using MagickCore::GetImageDepth;
   using MagickCore::GetImageInfo;
+  using MagickCore::GetImageInfoFile;
   using MagickCore::GetImageOption;
-  using MagickCore::GetImagePixels;
+  using MagickCore::GetAuthenticPixels;
   using MagickCore::GetImageProfile;
   using MagickCore::GetImageProperty;
   using MagickCore::GetImageQuantizeError;
@@ -706,8 +756,12 @@ namespace Magick
   using MagickCore::GetStringInfoDatum;
   using MagickCore::GetStringInfoLength;
   using MagickCore::GetTypeMetrics;
+  using MagickCore::GetVirtualIndexQueue;
+  using MagickCore::GetImageVirtualPixelMethod;
   using MagickCore::GlobExpression;
+  using MagickCore::GravityAdjustGeometry;
   using MagickCore::GreaterValue;
+  using MagickCore::HaldClutImage;
   using MagickCore::HeightValue;
   using MagickCore::ImageError;
   using MagickCore::ImageFatalError;
@@ -717,10 +771,12 @@ namespace Magick
   using MagickCore::ImageWarning;
   using MagickCore::ImplodeImage;
   using MagickCore::ImportQuantumPixels;
+  using MagickCore::InverseFourierTransformImage;
   using MagickCore::InvokeDynamicImageFilter;
   using MagickCore::IsEventLogging;
   using MagickCore::IsGeometry;
   using MagickCore::IsImagesEqual;
+  using MagickCore::KernelInfo;
   using MagickCore::LessValue;
   using MagickCore::LevelImage;
   using MagickCore::LevelImageChannel;
@@ -733,7 +789,6 @@ namespace Magick
   using MagickCore::MagickWand;
   using MagickCore::MagnifyImage;
   using MagickCore::MapImage;
-  using MagickCore::MedianFilterImage;
   using MagickCore::MergeImageLayers;
   using MagickCore::MinifyImage;
   using MagickCore::MissingDelegateError;
@@ -775,18 +830,20 @@ namespace Magick
   using MagickCore::QuantumInfo;
   using MagickCore::QueryColorDatabase;
   using MagickCore::QueryMagickColor;
+  using MagickCore::QueueAuthenticPixels;
+  using MagickCore::QueueCacheViewAuthenticPixels;
   using MagickCore::RaiseImage;
   using MagickCore::RandomThresholdImageChannel;
   using MagickCore::ReadImage;
+  using MagickCore::RecolorImage;
   using MagickCore::RectangleInfo;
-  using MagickCore::RectangleInfo;
-  using MagickCore::ReduceNoiseImage;
   using MagickCore::RegisterMagickInfo;
   using MagickCore::RegistryError;
   using MagickCore::RegistryFatalError;
   using MagickCore::RegistryType;
   using MagickCore::RegistryWarning;
   using MagickCore::RelinquishMagickMemory;
+  using MagickCore::RemapImage;
   using MagickCore::ResizeImage;
   using MagickCore::ResizeMagickMemory;
   using MagickCore::ResourceLimitError;
@@ -799,17 +856,17 @@ namespace Magick
   using MagickCore::ScaleImage;
   using MagickCore::SegmentImage;
   using MagickCore::SeparateImageChannel;
-  using MagickCore::SetCacheViewPixels;
   using MagickCore::SetClientName;
+  using MagickCore::SetGeometry;
   using MagickCore::SetImageBackgroundColor;
   using MagickCore::SetImageChannelDepth;
   using MagickCore::SetImageClipMask;
   using MagickCore::SetImageDepth;
   using MagickCore::SetImageExtent;
   using MagickCore::SetImageInfo;
+  using MagickCore::SetImageInfoFile;
   using MagickCore::SetImageOpacity;
   using MagickCore::SetImageOption;
-  using MagickCore::SetImagePixels;
   using MagickCore::SetImageProfile;
   using MagickCore::SetImageProperty;
   using MagickCore::SetImageRegistry;
@@ -818,6 +875,7 @@ namespace Magick
   using MagickCore::SetMagickInfo;
   using MagickCore::SetMagickResourceLimit;
   using MagickCore::SetStringInfoDatum;
+  using MagickCore::SetImageVirtualPixelMethod;
   using MagickCore::ShadeImage;
   using MagickCore::SharpenImage;
   using MagickCore::SharpenImageChannel;
@@ -827,7 +885,9 @@ namespace Magick
   using MagickCore::SignatureImage;
   using MagickCore::SolarizeImage;
   using MagickCore::SparseColorImage;
+  using MagickCore::SpliceImage;
   using MagickCore::SpreadImage;
+  using MagickCore::StatisticImage;
   using MagickCore::SteganoImage;
   using MagickCore::StereoImage;
   using MagickCore::StreamError;
@@ -835,14 +895,15 @@ namespace Magick
   using MagickCore::StreamWarning;
   using MagickCore::StringInfo;
   using MagickCore::SwirlImage;
-  using MagickCore::SyncCacheViewPixels;
+  using MagickCore::SyncCacheViewAuthenticPixels;
   using MagickCore::SyncImage;
-  using MagickCore::SyncImagePixels;
+  using MagickCore::SyncAuthenticPixels;
   using MagickCore::TextureImage;
   using MagickCore::ThrowException;
   using MagickCore::TransformImage;
   using MagickCore::TransformRGBImage;
   using MagickCore::TransparentPaintImage;
+  using MagickCore::TransparentPaintImageChroma;
   using MagickCore::TrimImage;
   using MagickCore::TypeError;
   using MagickCore::TypeFatalError;
@@ -852,7 +913,7 @@ namespace Magick
   using MagickCore::UnregisterMagickInfo;
   using MagickCore::UnsharpMaskImage;
   using MagickCore::UnsharpMaskImageChannel;
-  using MagickCore::ViewInfo;
+  using MagickCore::CacheView;
   using MagickCore::WaveImage;
   using MagickCore::WidthValue;
   using MagickCore::WriteImage;
