@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,7 +42,9 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -96,9 +98,6 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  long
-    y;
-
   QuantumInfo
     *quantum_info;
 
@@ -108,15 +107,13 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register PixelPacket
-    *q;
-
-  ssize_t
-    count;
-
   size_t
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
     *pixels;
 
@@ -140,9 +137,9 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->depth=1;
   image->endian=MSBEndian;
   (void) ReadBlobLSBShort(image);
-  image->columns=(unsigned long) ReadBlobLSBShort(image);
+  image->columns=(size_t) ReadBlobLSBShort(image);
   (void) ReadBlobLSBShort(image);
-  image->rows=(unsigned long) ReadBlobLSBShort(image);
+  image->rows=(size_t) ReadBlobLSBShort(image);
   /*
     Initialize image colormap.
   */
@@ -153,29 +150,30 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   /*
     Convert bi-level image to pixel packets.
   */
   quantum_type=IndexQuantum;
   quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   pixels=GetQuantumPixels(quantum_info);
   length=GetQuantumExtent(image,quantum_info,quantum_type);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    register PixelPacket
+      *restrict q;
+
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
     count=ReadBlob(image,length,pixels);
     if (count != (ssize_t) length)
       ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-    (void) ImportQuantumPixels(image,quantum_info,quantum_type,pixels);
+    (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+      quantum_type,pixels,exception);
     count=ReadBlob(image,(size_t) (-(ssize_t) length) & 0x01,pixels);
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
     if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
       break;
@@ -209,10 +207,10 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterARTImage method is:
 %
-%      unsigned long RegisterARTImage(void)
+%      size_t RegisterARTImage(void)
 %
 */
-ModuleExport unsigned long RegisterARTImage(void)
+ModuleExport size_t RegisterARTImage(void)
 {
   MagickInfo
     *entry;
@@ -278,14 +276,11 @@ ModuleExport void UnregisterARTImage(void)
 */
 static MagickBooleanType WriteARTImage(const ImageInfo *image_info,Image *image)
 {
-  long
-    y;
-
   MagickBooleanType
     status;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
 
   register const PixelPacket
     *p;
@@ -294,7 +289,8 @@ static MagickBooleanType WriteARTImage(const ImageInfo *image_info,Image *image)
     count;
 
   size_t
-    length;
+    length,
+    y;
 
   unsigned char
     *pixels;
@@ -320,7 +316,7 @@ static MagickBooleanType WriteARTImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobLSBShort(image,0);
   (void) WriteBlobLSBShort(image,(unsigned short) image->rows);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   length=(image->columns+7)/8;
   pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
@@ -329,20 +325,24 @@ static MagickBooleanType WriteARTImage(const ImageInfo *image_info,Image *image)
     Convert image to a bi-level image.
   */
   (void) SetImageType(image,BilevelType);
-  GetQuantumInfo(image_info,&quantum_info);
-  for (y=0; y < (long) image->rows; y++)
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ExportQuantumPixels(image,&quantum_info,GrayQuantum,pixels);
+    (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+      GrayQuantum,pixels,&image->exception);
     count=WriteBlob(image,length,pixels);
     if (count != (ssize_t) length)
       ThrowWriterException(CorruptImageError,"UnableToWriteImageData");
     count=WriteBlob(image,(size_t) (-(ssize_t) length) & 0x01,pixels);
-    if (SetImageProgress(image,SaveImageTag,y,image->rows) == MagickFalse)
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
       break;
   }
+  quantum_info=DestroyQuantumInfo(quantum_info);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   (void) CloseBlob(image);
   return(MagickTrue);
