@@ -20,7 +20,7 @@
 %                               October 2003                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,6 +49,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -66,6 +67,7 @@
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/module.h"
 
 /*
@@ -79,7 +81,7 @@ typedef struct _CINDataFormatInfo
     sign,
     sense;
 
-  unsigned long
+  size_t
     line_pad,
     channel_pad;
 
@@ -89,7 +91,7 @@ typedef struct _CINDataFormatInfo
 
 typedef struct _CINFileInfo
 {
-  unsigned long
+  size_t
     magic,
     image_offset,
     generic_length,
@@ -113,14 +115,14 @@ typedef struct _CINFilmInfo
     offset,
     reserve1;
 
-  unsigned long
+  size_t
     prefix,
     count;
 
   char
     format[32];
 
-  unsigned long
+  size_t
     frame_position;
 
   float
@@ -139,7 +141,7 @@ typedef struct _CINImageChannel
     bits_per_pixel,
     reserve;
 
-  unsigned long
+  size_t
     pixels_per_line,
     lines_per_image;
 
@@ -173,7 +175,7 @@ typedef struct _CINImageInfo
 
 typedef struct _CINOriginationInfo
 {
-  long
+  ssize_t
     x_offset,
     y_offset;
 
@@ -247,8 +249,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -290,8 +291,8 @@ static MagickBooleanType IsCIN(const unsigned char *magick,const size_t length)
 %
 */
 
-static size_t GetBytesPerRow(unsigned long columns,
-  unsigned long samples_per_pixel,unsigned long bits_per_pixel,
+static size_t GetBytesPerRow(size_t columns,
+  size_t samples_per_pixel,size_t bits_per_pixel,
   MagickBooleanType pad)
 {
   size_t
@@ -360,21 +361,21 @@ static inline MagickBooleanType IsFloatDefined(const float value)
 {
   union
   {
-    unsigned long
+    unsigned int
       unsigned_value;
 
     double
       float_value;
   } quantum;
 
+  quantum.unsigned_value=0U;
   quantum.float_value=value;
   if (quantum.unsigned_value == 0U)
     return(MagickFalse);
   return(MagickTrue);
 }
 
-static Image *ReadCINImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
+static Image *ReadCINImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define MonoColorType  1
 #define RGBColorType  3
@@ -388,9 +389,6 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   Image
     *image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -403,24 +401,23 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register long
+  register ssize_t
     i;
 
   register PixelPacket
     *q;
 
   size_t
-    length;
+    length,
+    lsb_first;
 
   ssize_t
-    count;
+    count,
+    y;
 
   unsigned char
     *pixels;
 
-  unsigned long
-    lsb_first;
-
   /*
     Open image file.
   */
@@ -463,16 +460,16 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   offset+=4;
   offset+=ReadBlob(image,sizeof(cin.file.version),(unsigned char *)
     cin.file.version);
-  (void) SetImageProperty(image,"cin:file.version",cin.file.version);
+  (void) SetImageProperty(image,"dpx:file.version",cin.file.version);
   offset+=ReadBlob(image,sizeof(cin.file.filename),(unsigned char *)
     cin.file.filename);
-  (void) SetImageProperty(image,"cin:file.filename",cin.file.filename);
+  (void) SetImageProperty(image,"dpx:file.filename",cin.file.filename);
   offset+=ReadBlob(image,sizeof(cin.file.create_date),(unsigned char *)
     cin.file.create_date);
-  (void) SetImageProperty(image,"cin:file.create_date",cin.file.create_date);
+  (void) SetImageProperty(image,"dpx:file.create_date",cin.file.create_date);
   offset+=ReadBlob(image,sizeof(cin.file.create_time),(unsigned char *)
     cin.file.create_time);
-  (void) SetImageProperty(image,"cin:file.create_time",cin.file.create_time);
+  (void) SetImageProperty(image,"dpx:file.create_time",cin.file.create_time);
   offset+=ReadBlob(image,sizeof(cin.file.reserve),(unsigned char *)
     cin.file.reserve);
   /*
@@ -481,19 +478,19 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   cin.image.orientation=(unsigned char) ReadBlobByte(image);
   offset++;
   if (cin.image.orientation != (unsigned char) (~0U))
-    (void) FormatImageProperty(image,"cin:image.orientation","%d",
+    (void) FormatImageProperty(image,"dpx:image.orientation","%d",
       cin.image.orientation);
   switch (cin.image.orientation)
   {
     default:
-    case 0:  image->orientation=TopLeftOrientation; break;
-    case 1:  image->orientation=TopRightOrientation; break;
-    case 2:  image->orientation=BottomLeftOrientation; break;
-    case 3:  image->orientation=BottomRightOrientation; break;
-    case 4:  image->orientation=LeftTopOrientation; break;
-    case 5:  image->orientation=RightTopOrientation; break;
-    case 6:  image->orientation=LeftBottomOrientation; break;
-    case 7:  image->orientation=RightBottomOrientation; break;
+    case 0: image->orientation=TopLeftOrientation; break;
+    case 1: image->orientation=TopRightOrientation; break;
+    case 2: image->orientation=BottomLeftOrientation; break;
+    case 3: image->orientation=BottomRightOrientation; break;
+    case 4: image->orientation=LeftTopOrientation; break;
+    case 5: image->orientation=RightTopOrientation; break;
+    case 6: image->orientation=LeftBottomOrientation; break;
+    case 7: image->orientation=RightBottomOrientation; break;
   }
   cin.image.number_channels=(unsigned char) ReadBlobByte(image);
   offset++;
@@ -556,7 +553,7 @@ static Image *ReadCINImage(const ImageInfo *image_info,
     image->chromaticity.blue_primary.y=cin.image.blue_primary_chromaticity[1];
   offset+=ReadBlob(image,sizeof(cin.image.label),(unsigned char *)
     cin.image.label);
-  (void) SetImageProperty(image,"cin:image.label",cin.image.label);
+  (void) SetImageProperty(image,"dpx:image.label",cin.image.label);
   offset+=ReadBlob(image,sizeof(cin.image.reserve),(unsigned char *)
     cin.image.reserve);
   /*
@@ -579,38 +576,38 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   /*
     Image origination information.
   */
-  cin.origination.x_offset=(long) ReadBlobLong(image);
+  cin.origination.x_offset=(int) ReadBlobLong(image);
   offset+=4;
-  if ((unsigned long) cin.origination.x_offset != ~0UL)
-    (void) FormatImageProperty(image,"cin:origination.x_offset","%ld",
-      cin.origination.x_offset);
-  cin.origination.y_offset=(long) ReadBlobLong(image);
+  if ((size_t) cin.origination.x_offset != ~0UL)
+    (void) FormatImageProperty(image,"dpx:origination.x_offset","%.20g",
+      (double) cin.origination.x_offset);
+  cin.origination.y_offset=(ssize_t) ReadBlobLong(image);
   offset+=4;
-  if ((unsigned long) cin.origination.y_offset != ~0UL)
-    (void) FormatImageProperty(image,"cin:origination.y_offset","%ld",
-      cin.origination.y_offset);
+  if ((size_t) cin.origination.y_offset != ~0UL)
+    (void) FormatImageProperty(image,"dpx:origination.y_offset","%.20g",
+      (double) cin.origination.y_offset);
   offset+=ReadBlob(image,sizeof(cin.origination.filename),(unsigned char *)
     cin.origination.filename);
-  (void) SetImageProperty(image,"cin:origination.filename",
+  (void) SetImageProperty(image,"dpx:origination.filename",
     cin.origination.filename);
   offset+=ReadBlob(image,sizeof(cin.origination.create_date),(unsigned char *)
     cin.origination.create_date);
-  (void) SetImageProperty(image,"cin:origination.create_date",
+  (void) SetImageProperty(image,"dpx:origination.create_date",
     cin.origination.create_date);
   offset+=ReadBlob(image,sizeof(cin.origination.create_time),(unsigned char *)
     cin.origination.create_time);
-  (void) SetImageProperty(image,"cin:origination.create_time",
+  (void) SetImageProperty(image,"dpx:origination.create_time",
     cin.origination.create_time);
   offset+=ReadBlob(image,sizeof(cin.origination.device),(unsigned char *)
     cin.origination.device);
-  (void) SetImageProperty(image,"cin:origination.device",
+  (void) SetImageProperty(image,"dpx:origination.device",
     cin.origination.device);
   offset+=ReadBlob(image,sizeof(cin.origination.model),(unsigned char *)
     cin.origination.model);
-  (void) SetImageProperty(image,"cin:origination.model",cin.origination.model);
+  (void) SetImageProperty(image,"dpx:origination.model",cin.origination.model);
   offset+=ReadBlob(image,sizeof(cin.origination.serial),(unsigned char *)
     cin.origination.serial);
-  (void) SetImageProperty(image,"cin:origination.serial",
+  (void) SetImageProperty(image,"dpx:origination.serial",
     cin.origination.serial);
   cin.origination.x_pitch=ReadBlobFloat(image);
   offset+=4;
@@ -629,45 +626,45 @@ static Image *ReadCINImage(const ImageInfo *image_info,
       */
       cin.film.id=ReadBlobByte(image);
       offset++;
-      if ((unsigned long) cin.film.id != ~0UL)
-        (void) FormatImageProperty(image,"cin:film.id","%d",cin.film.id);
+      if (((size_t) cin.film.id) != ~0UL)
+        (void) FormatImageProperty(image,"dpx:film.id","%d",cin.film.id);
       cin.film.type=ReadBlobByte(image);
       offset++;
-      if ((unsigned long) cin.film.type != ~0UL)
-        (void) FormatImageProperty(image,"cin:film.type","%d",cin.film.type);
+      if (((size_t) cin.film.type) != ~0UL)
+        (void) FormatImageProperty(image,"dpx:film.type","%d",cin.film.type);
       cin.film.offset=ReadBlobByte(image);
       offset++;
-      if ((unsigned long) cin.film.offset != ~0UL)
-        (void) FormatImageProperty(image,"cin:film.offset","%d",
+      if (((size_t) cin.film.offset) != ~0UL)
+        (void) FormatImageProperty(image,"dpx:film.offset","%d",
           cin.film.offset);
       cin.film.reserve1=ReadBlobByte(image);
       offset++;
       cin.film.prefix=ReadBlobLong(image);
       offset+=4;
       if (cin.film.prefix != ~0UL)
-        (void) FormatImageProperty(image,"cin:film.prefix","%lu",
+        (void) FormatImageProperty(image,"dpx:film.prefix","%.20g",(double)
           cin.film.prefix);
       cin.film.count=ReadBlobLong(image);
       offset+=4;
       offset+=ReadBlob(image,sizeof(cin.film.format),(unsigned char *)
         cin.film.format);
-      (void) SetImageProperty(image,"cin:film.format",cin.film.format);
+      (void) SetImageProperty(image,"dpx:film.format",cin.film.format);
       cin.film.frame_position=ReadBlobLong(image);
       offset+=4;
       if (cin.film.frame_position != ~0UL)
-        (void) FormatImageProperty(image,"cin:film.frame_position","%lu",
-          cin.film.frame_position);
+        (void) FormatImageProperty(image,"dpx:film.frame_position","%.20g",
+          (double) cin.film.frame_position);
       cin.film.frame_rate=ReadBlobFloat(image);
       offset+=4;
       if (IsFloatDefined(cin.film.frame_rate) != MagickFalse)
-        (void) FormatImageProperty(image,"cin:film.frame_rate","%g",
+        (void) FormatImageProperty(image,"dpx:film.frame_rate","%g",
           cin.film.frame_rate);
       offset+=ReadBlob(image,sizeof(cin.film.frame_id),(unsigned char *)
         cin.film.frame_id);
-      (void) SetImageProperty(image,"cin:film.frame_id",cin.film.frame_id);
+      (void) SetImageProperty(image,"dpx:film.frame_id",cin.film.frame_id);
       offset+=ReadBlob(image,sizeof(cin.film.slate_info),(unsigned char *)
         cin.film.slate_info);
-      (void) SetImageProperty(image,"cin:film.slate_info",cin.film.slate_info);
+      (void) SetImageProperty(image,"dpx:film.slate_info",cin.film.slate_info);
       offset+=ReadBlob(image,sizeof(cin.film.reserve),(unsigned char *)
         cin.film.reserve);
     }
@@ -682,10 +679,10 @@ static Image *ReadCINImage(const ImageInfo *image_info,
       profile=AcquireStringInfo(cin.file.user_length);
       offset+=ReadBlob(image,GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
-      (void) SetImageProfile(image,"cin:user.data",profile);
+      (void) SetImageProfile(image,"dpx:user.data",profile);
       profile=DestroyStringInfo(profile);
     }
-  for ( ; offset < (long) cin.file.image_offset; offset++)
+  for ( ; offset < (MagickOffsetType) cin.file.image_offset; offset++)
     (void) ReadBlobByte(image);
   image->depth=cin.image.channel[0].bits_per_pixel;
   image->columns=cin.image.channel[0].pixels_per_line;
@@ -695,15 +692,12 @@ static Image *ReadCINImage(const ImageInfo *image_info,
       (void) CloseBlob(image);
       return(image);
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   /*
     Convert CIN raster image to pixel packets.
   */
   quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   quantum_info->quantum=32;
   quantum_info->pack=MagickFalse;
   quantum_type=RGBQuantum;
@@ -715,20 +709,22 @@ static Image *ReadCINImage(const ImageInfo *image_info,
       quantum_type=GrayQuantum;
       length=GetBytesPerRow(image->columns,1,image->depth,MagickTrue);
     }
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
     count=ReadBlob(image,length,pixels);
     if ((size_t) count != length)
       break;
-    (void) ImportQuantumPixels(image,quantum_info,quantum_type,pixels);
-    if (SyncImagePixels(image) == MagickFalse)
+    (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+      quantum_type,pixels,exception);
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
     if (image->previous == (Image *) NULL)
       {
-        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+          image->rows);
         if (status == MagickFalse)
           break;
       }
@@ -762,10 +758,10 @@ static Image *ReadCINImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterCINImage method is:
 %
-%      unsigned long RegisterCINImage(void)
+%      size_t RegisterCINImage(void)
 %
 */
-ModuleExport unsigned long RegisterCINImage(void)
+ModuleExport size_t RegisterCINImage(void)
 {
   MagickInfo
     *entry;
@@ -774,6 +770,7 @@ ModuleExport unsigned long RegisterCINImage(void)
   entry->decoder=(DecodeImageHandler *) ReadCINImage;
   entry->encoder=(EncodeImageHandler *) WriteCINImage;
   entry->magick=(IsImageFormatHandler *) IsCIN;
+  entry->adjoin=MagickFalse;
   entry->description=ConstantString("Cineon Image File");
   entry->module=ConstantString("CIN");
   (void) RegisterMagickInfo(entry);
@@ -852,9 +849,6 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   const StringInfo
     *profile;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -870,14 +864,18 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     i;
 
   size_t
     length;
 
   ssize_t
-    count;
+    count,
+    y;
+
+  struct tm
+    local_time;
 
   time_t
     seconds;
@@ -898,39 +896,34 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != LogColorspace)
-    (void) SetImageColorspace(image,LogColorspace);
+    (void) TransformImageColorspace(image,LogColorspace);
   /*
     Write image information.
   */
   (void) ResetMagickMemory(&cin,0,sizeof(cin));
   offset=0;
   cin.file.magic=0x802A5FD7UL;
-  offset+=WriteBlobLong(image,cin.file.magic);
+  offset+=WriteBlobLong(image,(unsigned int) cin.file.magic);
   cin.file.image_offset=0x800;
-  profile=GetImageProfile(image,"cin:user.data");
-  if (profile != (StringInfo *) NULL)
-    {
-      cin.file.image_offset+=(unsigned long) GetStringInfoLength(profile);
-      cin.file.image_offset=(((cin.file.image_offset+0x2000-1)/0x2000)*0x2000);
-    }
-  offset+=WriteBlobLong(image,cin.file.image_offset);
+  offset+=WriteBlobLong(image,(unsigned int) cin.file.image_offset);
   cin.file.generic_length=0x400;
-  offset+=WriteBlobLong(image,cin.file.generic_length);
+  offset+=WriteBlobLong(image,(unsigned int) cin.file.generic_length);
   cin.file.industry_length=0x400;
-  offset+=WriteBlobLong(image,cin.file.industry_length);
+  offset+=WriteBlobLong(image,(unsigned int) cin.file.industry_length);
   cin.file.user_length=0x00;
+  profile=GetImageProfile(image,"dpx:user.data");
   if (profile != (StringInfo *) NULL)
     {
-      cin.file.user_length+=(unsigned long) GetStringInfoLength(profile);
+      cin.file.user_length+=(size_t) GetStringInfoLength(profile);
       cin.file.user_length=(((cin.file.user_length+0x2000-1)/0x2000)*0x2000);
     }
-  offset+=WriteBlobLong(image,cin.file.user_length);
+  offset+=WriteBlobLong(image,(unsigned int) cin.file.user_length);
   cin.file.file_size=4*image->columns*image->rows+0x2000;
-  offset+=WriteBlobLong(image,cin.file.file_size);
+  offset+=WriteBlobLong(image,(unsigned int) cin.file.file_size);
   (void) CopyMagickString(cin.file.version,"V4.5",sizeof(cin.file.version));
   offset+=WriteBlob(image,sizeof(cin.file.version),(unsigned char *)
     cin.file.version);
-  value=GetCINProperty(image_info,image,"cin:file.filename");
+  value=GetCINProperty(image_info,image,"dpx:file.filename");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.file.filename,value,sizeof(cin.file.filename));
   else
@@ -939,12 +932,17 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   offset+=WriteBlob(image,sizeof(cin.file.filename),(unsigned char *)
     cin.file.filename);
   seconds=time((time_t *) NULL);
+#if defined(MAGICKCORE_HAVE_LOCALTIME_R)
+  (void) localtime_r(&seconds,&local_time);
+#else
+  (void) memcpy(&local_time,localtime(&seconds),sizeof(local_time));
+#endif
   (void) strftime(cin.file.create_date,sizeof(cin.file.create_date),"%Y:%m:%d",
-    localtime(&seconds));
+    &local_time);
   offset+=WriteBlob(image,sizeof(cin.file.create_date),(unsigned char *)
     cin.file.create_date);
   (void) strftime(cin.file.create_time,sizeof(cin.file.create_time),
-    "%H:%M:%S%Z",localtime(&seconds));
+    "%H:%M:%S%Z",&local_time);
   offset+=WriteBlob(image,sizeof(cin.file.create_time),(unsigned char *)
     cin.file.create_time);
   offset+=WriteBlob(image,sizeof(cin.file.reserve),(unsigned char *)
@@ -965,9 +963,11 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     offset+=WriteBlobByte(image,cin.image.channel[0].bits_per_pixel);
     offset+=WriteBlobByte(image,cin.image.channel[0].reserve);
     cin.image.channel[i].pixels_per_line=image->columns;
-    offset+=WriteBlobLong(image,cin.image.channel[0].pixels_per_line);
+    offset+=WriteBlobLong(image,(unsigned int)
+      cin.image.channel[0].pixels_per_line);
     cin.image.channel[i].lines_per_image=image->rows;
-    offset+=WriteBlobLong(image,cin.image.channel[0].lines_per_image);
+    offset+=WriteBlobLong(image,(unsigned int)
+      cin.image.channel[0].lines_per_image);
     cin.image.channel[i].min_data=0;
     offset+=WriteBlobFloat(image,cin.image.channel[0].min_data);
     cin.image.channel[i].min_quantity=0.0;
@@ -986,7 +986,7 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   offset+=WriteBlobFloat(image,image->chromaticity.green_primary.y);
   offset+=WriteBlobFloat(image,image->chromaticity.blue_primary.x);
   offset+=WriteBlobFloat(image,image->chromaticity.blue_primary.y);
-  value=GetCINProperty(image_info,image,"cin:image.label");
+  value=GetCINProperty(image_info,image,"dpx:image.label");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.image.label,value,sizeof(cin.image.label));
   offset+=WriteBlob(image,sizeof(cin.image.label),(unsigned char *)
@@ -998,32 +998,32 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   */
   cin.data_format.interleave=0; /* pixel interleave (rgbrgbr...) */
   offset+=WriteBlobByte(image,cin.data_format.interleave);
-  cin.data_format.packing=5; /* packing longword (32bit) boundaries */
+  cin.data_format.packing=5; /* packing ssize_tword (32bit) boundaries */
   offset+=WriteBlobByte(image,cin.data_format.packing);
   cin.data_format.sign=0; /* unsigned data */
   offset+=WriteBlobByte(image,cin.data_format.sign);
   cin.data_format.sense=0; /* image sense: positive image */
   offset+=WriteBlobByte(image,cin.data_format.sense);
   cin.data_format.line_pad=0;
-  offset+=WriteBlobLong(image,cin.data_format.line_pad);
+  offset+=WriteBlobLong(image,(unsigned int) cin.data_format.line_pad);
   cin.data_format.channel_pad=0;
-  offset+=WriteBlobLong(image,cin.data_format.channel_pad);
+  offset+=WriteBlobLong(image,(unsigned int) cin.data_format.channel_pad);
   offset+=WriteBlob(image,sizeof(cin.data_format.reserve),(unsigned char *)
     cin.data_format.reserve);
   /*
     Write origination information.
   */
   cin.origination.x_offset=0UL;
-  value=GetCINProperty(image_info,image,"cin:origination.x_offset");
+  value=GetCINProperty(image_info,image,"dpx:origination.x_offset");
   if (value != (const char *) NULL)
-    cin.origination.x_offset=atol(value);
-  offset+=WriteBlobLong(image,(unsigned long) cin.origination.x_offset);
+    cin.origination.x_offset=(ssize_t) StringToLong(value);
+  offset+=WriteBlobLong(image,(unsigned int) cin.origination.x_offset);
   cin.origination.y_offset=0UL;
-  value=GetCINProperty(image_info,image,"cin:origination.y_offset");
+  value=GetCINProperty(image_info,image,"dpx:origination.y_offset");
   if (value != (const char *) NULL)
-    cin.origination.y_offset=atol(value);
-  offset+=WriteBlobLong(image,(unsigned long) cin.origination.y_offset);
-  value=GetCINProperty(image_info,image,"cin:origination.filename");
+    cin.origination.y_offset=(ssize_t) StringToLong(value);
+  offset+=WriteBlobLong(image,(unsigned int) cin.origination.y_offset);
+  value=GetCINProperty(image_info,image,"dpx:origination.filename");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.origination.filename,value,
       sizeof(cin.origination.filename));
@@ -1034,40 +1034,40 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     cin.origination.filename);
   seconds=time((time_t *) NULL);
   (void) strftime(cin.origination.create_date,
-    sizeof(cin.origination.create_date),"%Y:%m:%d",localtime(&seconds));
+    sizeof(cin.origination.create_date),"%Y:%m:%d",&local_time);
   offset+=WriteBlob(image,sizeof(cin.origination.create_date),(unsigned char *)
     cin.origination.create_date);
   (void) strftime(cin.origination.create_time,
-    sizeof(cin.origination.create_time),"%H:%M:%S%Z",localtime(&seconds));
+    sizeof(cin.origination.create_time),"%H:%M:%S%Z",&local_time);
   offset+=WriteBlob(image,sizeof(cin.origination.create_time),(unsigned char *)
     cin.origination.create_time);
-  value=GetCINProperty(image_info,image,"cin:origination.device");
+  value=GetCINProperty(image_info,image,"dpx:origination.device");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.origination.device,value,
       sizeof(cin.origination.device));
   offset+=WriteBlob(image,sizeof(cin.origination.device),(unsigned char *)
     cin.origination.device);
-  value=GetCINProperty(image_info,image,"cin:origination.model");
+  value=GetCINProperty(image_info,image,"dpx:origination.model");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.origination.model,value,
       sizeof(cin.origination.model));
   offset+=WriteBlob(image,sizeof(cin.origination.model),(unsigned char *)
     cin.origination.model);
-  value=GetCINProperty(image_info,image,"cin:origination.serial");
+  value=GetCINProperty(image_info,image,"dpx:origination.serial");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.origination.serial,value,
       sizeof(cin.origination.serial));
   offset+=WriteBlob(image,sizeof(cin.origination.serial),(unsigned char *)
     cin.origination.serial);
   cin.origination.x_pitch=0.0f;
-  value=GetCINProperty(image_info,image,"cin:origination.x_pitch");
+  value=GetCINProperty(image_info,image,"dpx:origination.x_pitch");
   if (value != (const char *) NULL)
-    cin.origination.x_pitch=atof(value);
+    cin.origination.x_pitch=StringToDouble(value);
   offset+=WriteBlobFloat(image,cin.origination.x_pitch);
   cin.origination.y_pitch=0.0f;
-  value=GetCINProperty(image_info,image,"cin:origination.y_pitch");
+  value=GetCINProperty(image_info,image,"dpx:origination.y_pitch");
   if (value != (const char *) NULL)
-    cin.origination.y_pitch=atof(value);
+    cin.origination.y_pitch=StringToDouble(value);
   offset+=WriteBlobFloat(image,cin.origination.y_pitch);
   cin.origination.gamma=image->gamma;
   offset+=WriteBlobFloat(image,cin.origination.gamma);
@@ -1077,52 +1077,52 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     Image film information.
   */
   cin.film.id=0;
-  value=GetCINProperty(image_info,image,"cin:film.id");
+  value=GetCINProperty(image_info,image,"dpx:film.id");
   if (value != (const char *) NULL)
-    cin.film.id=(char) atoi(value);
+    cin.film.id=(char) StringToLong(value);
   offset+=WriteBlobByte(image,(unsigned char) cin.film.id);
   cin.film.type=0;
-  value=GetCINProperty(image_info,image,"cin:film.type");
+  value=GetCINProperty(image_info,image,"dpx:film.type");
   if (value != (const char *) NULL)
-    cin.film.type=(char) atoi(value);
+    cin.film.type=(char) StringToLong(value);
   offset+=WriteBlobByte(image,(unsigned char) cin.film.type);
   cin.film.offset=0;
-  value=GetCINProperty(image_info,image,"cin:film.offset");
+  value=GetCINProperty(image_info,image,"dpx:film.offset");
   if (value != (const char *) NULL)
-    cin.film.offset=(char) atoi(value);
+    cin.film.offset=(char) StringToLong(value);
   offset+=WriteBlobByte(image,(unsigned char) cin.film.offset);
   offset+=WriteBlobByte(image,(unsigned char) cin.film.reserve1);
   cin.film.prefix=0UL;
-  value=GetCINProperty(image_info,image,"cin:film.prefix");
+  value=GetCINProperty(image_info,image,"dpx:film.prefix");
   if (value != (const char *) NULL)
-    cin.film.prefix=(unsigned long) atol(value);
-  offset+=WriteBlobLong(image,cin.film.prefix);
+    cin.film.prefix=StringToUnsignedLong(value);
+  offset+=WriteBlobLong(image,(unsigned int) cin.film.prefix);
   cin.film.count=0UL;
-  value=GetCINProperty(image_info,image,"cin:film.count");
+  value=GetCINProperty(image_info,image,"dpx:film.count");
   if (value != (const char *) NULL)
-    cin.film.count=(unsigned long) atol(value);
-  offset+=WriteBlobLong(image,cin.film.count);
-  value=GetCINProperty(image_info,image,"cin:film.format");
+    cin.film.count=StringToUnsignedLong(value);
+  offset+=WriteBlobLong(image,(unsigned int) cin.film.count);
+  value=GetCINProperty(image_info,image,"dpx:film.format");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.film.format,value,sizeof(cin.film.format));
   offset+=WriteBlob(image,sizeof(cin.film.format),(unsigned char *)
     cin.film.format);
   cin.film.frame_position=0UL;
-  value=GetCINProperty(image_info,image,"cin:film.frame_position");
+  value=GetCINProperty(image_info,image,"dpx:film.frame_position");
   if (value != (const char *) NULL)
-    cin.film.frame_position=(unsigned long) atol(value);
-  offset+=WriteBlobLong(image,cin.film.frame_position);
+    cin.film.frame_position=StringToUnsignedLong(value);
+  offset+=WriteBlobLong(image,(unsigned int) cin.film.frame_position);
   cin.film.frame_rate=0.0f;
-  value=GetCINProperty(image_info,image,"cin:film.frame_rate");
+  value=GetCINProperty(image_info,image,"dpx:film.frame_rate");
   if (value != (const char *) NULL)
-    cin.film.frame_rate=atof(value);
+    cin.film.frame_rate=StringToDouble(value);
   offset+=WriteBlobFloat(image,cin.film.frame_rate);
-  value=GetCINProperty(image_info,image,"cin:film.frame_id");
+  value=GetCINProperty(image_info,image,"dpx:film.frame_id");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.film.frame_id,value,sizeof(cin.film.frame_id));
   offset+=WriteBlob(image,sizeof(cin.film.frame_id),(unsigned char *)
     cin.film.frame_id);
-  value=GetCINProperty(image_info,image,"cin:film.slate_info");
+  value=GetCINProperty(image_info,image,"dpx:film.slate_info");
   if (value != (const char *) NULL)
     (void) CopyMagickString(cin.film.slate_info,value,
       sizeof(cin.film.slate_info));
@@ -1139,6 +1139,8 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     Convert pixel packets to CIN raster image.
   */
   quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   quantum_info->quantum=32;
   quantum_info->pack=MagickFalse;
   quantum_type=RGBQuantum;
@@ -1149,16 +1151,18 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
       quantum_type=GrayQuantum;
       length=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
     }
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
+    (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+      quantum_type,pixels,&image->exception);
     count=WriteBlob(image,length,pixels);
     if (count != (ssize_t) length)
       break;
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
     if (status == MagickFalse)
       break;
   }
