@@ -9,7 +9,7 @@
 %                              CCC   UUU     T                                %
 %                                                                             %
 %                                                                             %
-%                         Read DR Halo Image Format.                          %
+%                         Read DR Halo Image Format                           %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
@@ -50,8 +50,11 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
+#include "magick/colormap-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -89,149 +92,126 @@ typedef struct
 } CUTPalHeader;
 
 
-static void InsertRow(long depth,unsigned char *p,long y,Image *image)
+static void InsertRow(ssize_t depth,unsigned char *p,ssize_t y,Image *image)
 {
-  unsigned long bit; long x;
+  ExceptionInfo
+    *exception;
+
+  size_t bit; ssize_t x;
   register PixelPacket *q;
   IndexPacket index;
   register IndexPacket *indexes;
 
-
   index=(IndexPacket) 0;
+  exception=(&image->exception);
   switch (depth)
-    {
+  {
     case 1:  /* Convert bitmap scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-7); x+=8)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-7); x+=8)
+        {
+          for (bit=0; bit < 8; bit++)
           {
-            for (bit=0; bit < 8; bit++)
-              {
-                index=(IndexPacket) ((((*p) & (0x80 >> bit)) != 0) ? 0x01 : 0x00);
-                indexes[x+bit]=index;
-                *q++=image->colormap[(long) index];
-              }
-            p++;
+            index=(IndexPacket) ((((*p) & (0x80 >> bit)) != 0) ? 0x01 : 0x00);
+            SetIndexPixelComponent(indexes+x+bit,index);
           }
+          p++;
+        }
         if ((image->columns % 8) != 0)
           {
             for (bit=0; bit < (image->columns % 8); bit++)
               {
                 index=(IndexPacket) ((((*p) & (0x80 >> bit)) != 0) ? 0x01 : 0x00);
-                indexes[x+bit]=index;
-                *q++=image->colormap[(long) index];
+                SetIndexPixelComponent(indexes+x+bit,index);
               }
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-        /*            if (image->previous == (Image *) NULL)
-                      if (QuantumTick(y,image->rows) != MagickFalse)
-                      ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
     case 2:  /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-1); x+=2)
           {
             index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
             index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
             index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
             index=ConstrainColormapIndex(image,(*p) & 0x3);
-            indexes[x+1]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x+1,index);
             p++;
           }
         if ((image->columns % 4) != 0)
           {
             index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
             if ((image->columns % 4) >= 1)
 
               {
                 index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
-                indexes[x]=index;
-                *q++=image->colormap[(long) index];
+                SetIndexPixelComponent(indexes+x,index);
                 if ((image->columns % 4) >= 2)
 
                   {
                     index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
-                    indexes[x]=index;
-                    *q++=image->colormap[(long) index];
+                    SetIndexPixelComponent(indexes+x,index);
                   }
               }
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-        /*         if (image->previous == (Image *) NULL)
-                   if (QuantumTick(y,image->rows) != MagickFalse)
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
 
     case 4:  /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-1); x+=2)
           {
             index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
             index=ConstrainColormapIndex(image,(*p) & 0xf);
-            indexes[x+1]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x+1,index);
             p++;
           }
         if ((image->columns % 2) != 0)
           {
             index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-        /*         if (image->previous == (Image *) NULL)
-                   if (QuantumTick(y,image->rows) != MagickFalse)
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
     case 8: /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL) break;
-        indexes=GetIndexes(image);
-
-        for (x=0; x < (long) image->columns; x++)
-          {
-            index=ConstrainColormapIndex(image,*p);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-            p++;
-          }
-        if (SyncImagePixels(image) == MagickFalse)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < (ssize_t) image->columns; x++)
+        {
+          index=ConstrainColormapIndex(image,*p);
+          SetIndexPixelComponent(indexes+x,index);
+          p++;
+        }
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-        /*           if (image->previous == (Image *) NULL)
-                     if (QuantumTick(y,image->rows) != MagickFalse)
-                     ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
       }
       break;
 
@@ -243,26 +223,30 @@ static void InsertRow(long depth,unsigned char *p,long y,Image *image)
 */
 static int GetCutColors(Image *image)
 {
-  long
-    x,
-    y;
-
-  PixelPacket
-    *q;
+  ExceptionInfo
+    *exception;
 
   Quantum
     intensity,
     scale_intensity;
 
+  register PixelPacket
+    *q;
+
+  ssize_t
+    x,
+    y;
+
+  exception=(&image->exception);
   intensity=0;
   scale_intensity=ScaleCharToQuantum(16);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
-    for (x=0; x < (long) image->columns; x++)
+    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if (intensity < q->red)
-        intensity=q->red;
+      if (intensity < GetRedPixelComponent(q))
+        intensity=GetRedPixelComponent(q);
       if (intensity >= scale_intensity)
         return(255);
       q++;
@@ -296,7 +280,7 @@ static int GetCutColors(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -310,19 +294,16 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickOffsetType
     offset;
 
-  unsigned long EncodedByte;
+  size_t EncodedByte;
   unsigned char RunCount,RunValue,RunCountMasked;
   CUTHeader  Header;
   CUTPalHeader PalHeader;
-  long depth;
-  long i,j;
-  long ldblk;
+  ssize_t depth;
+  ssize_t i,j;
+  ssize_t ldblk;
   unsigned char *BImgBuff=NULL,*ptrB;
   PixelPacket *q;
 
-  ssize_t
-    count;
-
   /*
     Open image file.
   */
@@ -333,7 +314,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -360,13 +341,13 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   while((int) RunCountMasked!=0)  /*end of line?*/
     {
       i=1;
-      if((int) RunCount<0x80) i=(long) RunCountMasked;
+      if((int) RunCount<0x80) i=(ssize_t) RunCountMasked;
       offset=SeekBlob(image,TellBlob(image)+i,SEEK_SET);
       if (offset < 0)
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       if(EOFBlob(image) != MagickFalse) goto CUT_KO;  /*wrong data*/
       EncodedByte-=i+1;
-      ldblk+=(long) RunCountMasked;
+      ldblk+=(ssize_t) RunCountMasked;
 
       RunCount=(unsigned char) ReadBlobByte(image);
       if(EOFBlob(image) != MagickFalse)  goto CUT_KO;  /*wrong data: unexpected eof in line*/
@@ -383,20 +364,15 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=Header.Width;
   image->rows=Header.Height;
   image->depth=8;
-  image->colors=1UL << (unsigned long) i;
+  image->colors=(size_t) (GetQuantumRange(1UL*i)+1);
 
   if (image_info->ping) goto Finish;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
 
   /* ----- Do something with palette ----- */
   if ((clone_info=CloneImageInfo(image_info)) == NULL) goto NoPalette;
 
 
-  i=(long) strlen(clone_info->filename);
+  i=(ssize_t) strlen(clone_info->filename);
   j=i;
   while(--i>0)
     {
@@ -414,14 +390,14 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   (void) CopyMagickString(clone_info->filename+i,".PAL",(size_t)
     (MaxTextExtent-i));
-  if((clone_info->file=MagickOpenStream(clone_info->filename,"rb"))==NULL)
+  if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
     {
       (void) CopyMagickString(clone_info->filename+i,".pal",(size_t)
         (MaxTextExtent-i));
-      if((clone_info->file=MagickOpenStream(clone_info->filename,"rb"))==NULL)
+      if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
         {
           clone_info->filename[i]='\0';
-          if((clone_info->file=MagickOpenStream(clone_info->filename,"rb"))==NULL)
+          if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
             {
               clone_info=DestroyImageInfo(clone_info);
               clone_info=NULL;
@@ -430,7 +406,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
 
-  if( (palette=AllocateImage(clone_info))==NULL ) goto NoPalette;
+  if( (palette=AcquireImage(clone_info))==NULL ) goto NoPalette;
   status=OpenBlob(clone_info,palette,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -443,7 +419,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   if(palette!=NULL)
     {
-      count=ReadBlob(palette,2,(unsigned char *) PalHeader.FileId);
+      (void) ReadBlob(palette,2,(unsigned char *) PalHeader.FileId);
       if(strncmp(PalHeader.FileId,"AH",2) != 0) goto ErasePalette;
       PalHeader.Version=ReadBlobLSBShort(palette);
       PalHeader.Size=ReadBlobLSBShort(palette);
@@ -455,11 +431,11 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       PalHeader.MaxRed=ReadBlobLSBShort(palette);
       PalHeader.MaxGreen=ReadBlobLSBShort(palette);
       PalHeader.MaxBlue=ReadBlobLSBShort(palette);
-      count=ReadBlob(palette,20,(unsigned char *) PalHeader.PaletteId);
+      (void) ReadBlob(palette,20,(unsigned char *) PalHeader.PaletteId);
 
       if(PalHeader.MaxIndex<1) goto ErasePalette;
       image->colors=PalHeader.MaxIndex+1;
-      if (AllocateImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
+      if (AcquireImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
 
       if(PalHeader.MaxRed==0) PalHeader.MaxRed=(unsigned int) QuantumRange;  /*avoid division by 0*/
       if(PalHeader.MaxGreen==0) PalHeader.MaxGreen=(unsigned int) QuantumRange;
@@ -467,7 +443,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       for(i=0;i<=(int) PalHeader.MaxIndex;i++)
         {      /*this may be wrong- I don't know why is palette such strange*/
-          j=(long) TellBlob(palette);
+          j=(ssize_t) TellBlob(palette);
           if((j % 512)>512-6)
             {
               j=((j / 512)+1)*512;
@@ -476,22 +452,22 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 ThrowReaderException(CorruptImageError,"ImproperImageHeader");
             }
           image->colormap[i].red=(Quantum) ReadBlobLSBShort(palette);
-          if(QuantumRange!=(Quantum) PalHeader.MaxRed)
+          if (QuantumRange != (Quantum) PalHeader.MaxRed)
             {
-              image->colormap[i].red=RoundToQuantum(((double)
+              image->colormap[i].red=ClampToQuantum(((double)
                 image->colormap[i].red*QuantumRange+(PalHeader.MaxRed>>1))/
                 PalHeader.MaxRed);
             }
           image->colormap[i].green=(Quantum) ReadBlobLSBShort(palette);
-          if(QuantumRange!=(Quantum) PalHeader.MaxGreen)
+          if (QuantumRange != (Quantum) PalHeader.MaxGreen)
             {
-              image->colormap[i].green=RoundToQuantum
+              image->colormap[i].green=ClampToQuantum
                 (((double) image->colormap[i].green*QuantumRange+(PalHeader.MaxGreen>>1))/PalHeader.MaxGreen);
             }
           image->colormap[i].blue=(Quantum) ReadBlobLSBShort(palette);
-          if(QuantumRange!=(Quantum) PalHeader.MaxBlue)
+          if (QuantumRange != (Quantum) PalHeader.MaxBlue)
             {
-              image->colormap[i].blue=RoundToQuantum
+              image->colormap[i].blue=ClampToQuantum
                 (((double)image->colormap[i].blue*QuantumRange+(PalHeader.MaxBlue>>1))/PalHeader.MaxBlue);
             }
 
@@ -505,13 +481,13 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
 
       image->colors=256;
-      if (AllocateImageColormap(image,image->colors) == MagickFalse)
+      if (AcquireImageColormap(image,image->colors) == MagickFalse)
         {
         NoMemory:
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
             }
 
-      for (i=0; i < (long)image->colors; i++)
+      for (i=0; i < (ssize_t)image->colors; i++)
         {
           image->colormap[i].red=ScaleCharToQuantum((unsigned char) i);
           image->colormap[i].green=ScaleCharToQuantum((unsigned char) i);
@@ -538,9 +514,9 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       RunCount=(unsigned char) ReadBlobByte(image);
       RunCountMasked=RunCount & 0x7F;
 
-      while((int) RunCountMasked!=0)
-        {
-          if((long) RunCountMasked>j)
+      while ((int) RunCountMasked != 0)
+      {
+          if((ssize_t) RunCountMasked>j)
             {    /*Wrong Data*/
               RunCountMasked=(unsigned char) j;
               if(j==0)
@@ -555,7 +531,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
               (void) ResetMagickMemory(ptrB,(int) RunValue,(size_t) RunCountMasked);
             }
           else {
-            count=ReadBlob(image,(size_t) RunCountMasked,ptrB);
+            (void) ReadBlob(image,(size_t) RunCountMasked,ptrB);
           }
 
           ptrB+=(int) RunCountMasked;
@@ -568,6 +544,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       InsertRow(depth,BImgBuff,i,image);
     }
+  (void) SyncImage(image);
 
 
   /*detect monochrome image*/
@@ -579,7 +556,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           if(GetCutColors(image)==2)
             {
-              for (i=0; i < (long)image->colors; i++)
+              for (i=0; i < (ssize_t)image->colors; i++)
                 {
                   register Quantum
                     sample;
@@ -589,19 +566,22 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   if(image->colormap[i].blue!=sample) goto Finish;
                 }
 
-              image->colormap[1].red=image->colormap[1].green=image->colormap[1].blue=(Quantum) QuantumRange;
-              for (i=0; i < (long)image->rows; i++)
+              image->colormap[1].red=image->colormap[1].green=
+                image->colormap[1].blue=(Quantum) QuantumRange;
+              for (i=0; i < (ssize_t)image->rows; i++)
                 {
-                  q=SetImagePixels(image,0,i,image->columns,1);
-                  for (j=0; j < (long)image->columns; j++)
+                  q=QueueAuthenticPixels(image,0,i,image->columns,1,exception);
+                  for (j=0; j < (ssize_t)image->columns; j++)
                     {
-                      if(q->red==ScaleCharToQuantum(1))
+                      if (GetRedPixelComponent(q) == ScaleCharToQuantum(1))
                         {
-                          q->red=q->green=q->blue=(Quantum) QuantumRange;
+                          SetRedPixelComponent(q,QuantumRange);
+                          SetGreenPixelComponent(q,QuantumRange);
+                          SetBluePixelComponent(q,QuantumRange);
                         }
                       q++;
                     }
-                  if (SyncImagePixels(image) == MagickFalse) goto Finish;
+                  if (SyncAuthenticPixels(image,exception) == MagickFalse) goto Finish;
                 }
             }
         }
@@ -617,7 +597,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -641,10 +621,10 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterCUTImage method is:
 %
-%      unsigned long RegisterCUTImage(void)
+%      size_t RegisterCUTImage(void)
 %
 */
-ModuleExport unsigned long RegisterCUTImage(void)
+ModuleExport size_t RegisterCUTImage(void)
 {
   MagickInfo
     *entry;
