@@ -17,7 +17,7 @@
 %                                March 2008                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -115,7 +116,7 @@
 */
 typedef struct _DDSPixelFormat
 {
-  unsigned long
+  size_t
     flags,
     fourcc,
     rgb_bitcount,
@@ -127,13 +128,13 @@ typedef struct _DDSPixelFormat
 
 typedef struct _DDSInfo
 {
-	unsigned long
-		flags,
-		height,
-		width,
-		pitchOrLinearSize,
-		depth,
-		mipmapcount,
+  size_t
+    flags,
+    height,
+    width,
+    pitchOrLinearSize,
+    depth,
+    mipmapcount,
     ddscaps1,
     ddscaps2;
   
@@ -224,7 +225,7 @@ static void
 %
 */
 
-static inline unsigned long Min(unsigned long one, unsigned long two)
+static inline size_t Min(size_t one, size_t two)
 {
   if (one < two)
     return one;
@@ -246,12 +247,12 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     compression;
 
   DDSInfo
-  	dds_info;
+    dds_info;
   
   DDSDecoder
     *decoder;
   
-  unsigned long
+  size_t
     n, num_images;
   
   /*
@@ -276,7 +277,7 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Initialize image structure.
   */
   if (ReadDDSInfo(image, &dds_info) != MagickTrue) {
-	  ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   }
   
   if (dds_info.ddscaps2 & DDSCAPS2_CUBEMAP)
@@ -407,33 +408,33 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
 static MagickBooleanType ReadDDSInfo(Image *image, DDSInfo *dds_info)
 {
-	unsigned long
+  size_t
     hdr_size,
     required;
-	
-	/* Seek to start of header */
-	(void) SeekBlob(image, 4, SEEK_SET);
-	
-	/* Check header field */
-	hdr_size = ReadBlobLSBLong(image);
-	if (hdr_size != 124)
-		return MagickFalse;
-	
-	/* Fill in DDS info struct */
-	dds_info->flags = ReadBlobLSBLong(image);
   
-	/* Check required flags */
-	required=(unsigned long) (DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT);
+  /* Seek to start of header */
+  (void) SeekBlob(image, 4, SEEK_SET);
+  
+  /* Check header field */
+  hdr_size = ReadBlobLSBLong(image);
+  if (hdr_size != 124)
+    return MagickFalse;
+  
+  /* Fill in DDS info struct */
+  dds_info->flags = ReadBlobLSBLong(image);
+  
+  /* Check required flags */
+  required=(size_t) (DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT);
   if ((dds_info->flags & required) != required)
-		return MagickFalse;
+    return MagickFalse;
   
-	dds_info->height = ReadBlobLSBLong(image);
-	dds_info->width = ReadBlobLSBLong(image);
-	dds_info->pitchOrLinearSize = ReadBlobLSBLong(image);
-	dds_info->depth = ReadBlobLSBLong(image);
-	dds_info->mipmapcount = ReadBlobLSBLong(image);
+  dds_info->height = ReadBlobLSBLong(image);
+  dds_info->width = ReadBlobLSBLong(image);
+  dds_info->pitchOrLinearSize = ReadBlobLSBLong(image);
+  dds_info->depth = ReadBlobLSBLong(image);
+  dds_info->mipmapcount = ReadBlobLSBLong(image);
   
-	(void) SeekBlob(image, 44, SEEK_CUR); 	/* reserved region of 11 DWORDs */
+  (void) SeekBlob(image, 44, SEEK_CUR);   /* reserved region of 11 DWORDs */
   
   /* Read pixel format structure */
   hdr_size = ReadBlobLSBLong(image);
@@ -448,11 +449,11 @@ static MagickBooleanType ReadDDSInfo(Image *image, DDSInfo *dds_info)
   dds_info->pixelformat.b_bitmask = ReadBlobLSBLong(image);
   dds_info->pixelformat.alpha_bitmask = ReadBlobLSBLong(image);
   
-	dds_info->ddscaps1 = ReadBlobLSBLong(image);
-	dds_info->ddscaps2 = ReadBlobLSBLong(image);
-	(void) SeekBlob(image, 12, SEEK_CUR); /* 3 reserved DWORDs */
+  dds_info->ddscaps1 = ReadBlobLSBLong(image);
+  dds_info->ddscaps2 = ReadBlobLSBLong(image);
+  (void) SeekBlob(image, 12, SEEK_CUR); /* 3 reserved DWORDs */
   
-	return MagickTrue;
+  return MagickTrue;
 }
 
 static void CalculateColors(unsigned short c0, unsigned short c1,
@@ -494,34 +495,38 @@ static MagickBooleanType ReadDXT1(Image *image, DDSInfo *dds_info)
   DDSColors
     colors;
 
-  long
-    j,
-    y;
-	
+  ExceptionInfo
+    *exception;
+
   PixelPacket
     *q;
   
-  register long
+  register ssize_t
     i,
     x;
   
+  size_t
+    bits;
+
+  ssize_t
+    j,
+    y;
+  
   unsigned char
     code;
   
-	unsigned short
+  unsigned short
     c0,
     c1;
-	
-  unsigned long
-    bits;
-
-  for (y = 0; y < (long) dds_info->height; y += 4)
+  
+  exception=(&image->exception);
+  for (y = 0; y < (ssize_t) dds_info->height; y += 4)
   {
-    for (x = 0; x < (long) dds_info->width; x += 4)
+    for (x = 0; x < (ssize_t) dds_info->width; x += 4)
     {
       /* Get 4x4 patch of pixels to write on */
-      q = SetImagePixels(image, x, y, Min(4, dds_info->width - x),
-	                     Min(4, dds_info->height - y));
+      q = QueueAuthenticPixels(image, x, y, Min(4, dds_info->width - x),
+        Min(4, dds_info->height - y),exception);
       
       if (q == (PixelPacket *) NULL)
         return MagickFalse;
@@ -538,13 +543,13 @@ static MagickBooleanType ReadDXT1(Image *image, DDSInfo *dds_info)
       {
         for (i = 0; i < 4; i++)
         {
-          if ((x + i) < (long) dds_info->width && (y + j) < (long) dds_info->height)
+          if ((x + i) < (ssize_t) dds_info->width && (y + j) < (ssize_t) dds_info->height)
             {
               code = (unsigned char) ((bits >> ((j*4+i)*2)) & 0x3);
-              q->red     = ScaleCharToQuantum( colors.r[code] );
-              q->green   = ScaleCharToQuantum( colors.g[code] );
-              q->blue    = ScaleCharToQuantum( colors.b[code] );
-              q->opacity = ScaleCharToQuantum( colors.a[code] );
+              SetRedPixelComponent(q,ScaleCharToQuantum(colors.r[code]));
+              SetGreenPixelComponent(q,ScaleCharToQuantum(colors.g[code]));
+              SetBluePixelComponent(q,ScaleCharToQuantum(colors.b[code]));
+              SetOpacityPixelComponent(q,ScaleCharToQuantum(colors.a[code]));
               if (colors.a[code] && image->matte == MagickFalse)
                 /* Correct matte */
                 image->matte = MagickTrue;
@@ -553,7 +558,7 @@ static MagickBooleanType ReadDXT1(Image *image, DDSInfo *dds_info)
         }
       }
       
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         return MagickFalse;
     }
   }
@@ -568,21 +573,24 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info)
   DDSColors
     colors;
   
-  long
+  ExceptionInfo
+    *exception;
+
+  ssize_t
     j,
     y;
 
   PixelPacket
     *q;
   
-  register long
+  register ssize_t
     i,
     x;
   
   unsigned char
     alpha;
   
-  unsigned long
+  size_t
     a0,
     a1,
     bits,
@@ -592,13 +600,14 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info)
     c0,
     c1;
   
-  for (y = 0; y < (long) dds_info->height; y += 4)
+  exception=(&image->exception);
+  for (y = 0; y < (ssize_t) dds_info->height; y += 4)
   {
-    for (x = 0; x < (long) dds_info->width; x += 4)
+    for (x = 0; x < (ssize_t) dds_info->width; x += 4)
     {
       /* Get 4x4 patch of pixels to write on */
-      q = SetImagePixels(image, x, y, Min(4, dds_info->width - x),
-                         Min(4, dds_info->height - y));
+      q = QueueAuthenticPixels(image, x, y, Min(4, dds_info->width - x),
+                         Min(4, dds_info->height - y),exception);
       
       if (q == (PixelPacket *) NULL)
         return MagickFalse;
@@ -619,13 +628,12 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info)
       {
         for (i = 0; i < 4; i++)
         {
-          if ((x + i) < (long) dds_info->width && (y + j) < (long) dds_info->height)
+          if ((x + i) < (ssize_t) dds_info->width && (y + j) < (ssize_t) dds_info->height)
             {
               code = (bits >> ((4*j+i)*2)) & 0x3;
-              q->red   = ScaleCharToQuantum( colors.r[code] );
-              q->green = ScaleCharToQuantum( colors.g[code] );
-              q->blue  = ScaleCharToQuantum( colors.b[code] );
-              
+              SetRedPixelComponent(q,ScaleCharToQuantum(colors.r[code]));
+              SetGreenPixelComponent(q,ScaleCharToQuantum(colors.g[code]));
+              SetBluePixelComponent(q,ScaleCharToQuantum(colors.b[code]));
               /*
                 Extract alpha value: multiply 0..15 by 17 to get range 0..255
               */
@@ -633,15 +641,14 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info)
                 alpha = 17U * (unsigned char) ((a0 >> (4*(4*j+i))) & 0xf);
               else
                 alpha = 17U * (unsigned char) ((a1 >> (4*(4*(j-2)+i))) & 0xf);
-              
-              q->opacity = ScaleCharToQuantum( (unsigned char) (255 - alpha) );
-              
+              SetOpacityPixelComponent(q,ScaleCharToQuantum((unsigned char)
+                (255-alpha)));
               q++;
             }
         }
       }
       
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         return MagickFalse;
     }
   }
@@ -656,7 +663,10 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
   DDSColors
     colors;
   
-  long
+  ExceptionInfo
+    *exception;
+
+  ssize_t
     j,
     y;
 
@@ -666,7 +676,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
   PixelPacket
     *q;
   
-  register long
+  register ssize_t
     i,
     x;
 
@@ -674,7 +684,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
     a0,
     a1;
   
-  unsigned long
+  size_t
     alpha,
     bits,
     code,
@@ -684,13 +694,14 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
     c0,
     c1;
   
-  for (y = 0; y < (long) dds_info->height; y += 4)
+  exception=(&image->exception);
+  for (y = 0; y < (ssize_t) dds_info->height; y += 4)
   {
-    for (x = 0; x < (long) dds_info->width; x += 4)
+    for (x = 0; x < (ssize_t) dds_info->width; x += 4)
     {
       /* Get 4x4 patch of pixels to write on */
-      q = SetImagePixels(image, x, y, Min(4, dds_info->width - x),
-                         Min(4, dds_info->height - y));
+      q = QueueAuthenticPixels(image, x, y, Min(4, dds_info->width - x),
+                         Min(4, dds_info->height - y),exception);
       
       if (q == (PixelPacket *) NULL)
         return MagickFalse;
@@ -714,15 +725,14 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
       {
         for (i = 0; i < 4; i++)
         {
-          if ((x + i) < (long) dds_info->width && (y + j) < (long) dds_info->height)
+          if ((x + i) < (ssize_t) dds_info->width && (y + j) < (ssize_t) dds_info->height)
             {
               code = (bits >> ((4*j+i)*2)) & 0x3;
-              q->red   = ScaleCharToQuantum( colors.r[code] );
-              q->green = ScaleCharToQuantum( colors.g[code] );
-              q->blue  = ScaleCharToQuantum( colors.b[code] );
-              
+              SetRedPixelComponent(q,ScaleCharToQuantum(colors.r[code]));
+              SetGreenPixelComponent(q,ScaleCharToQuantum(colors.g[code]));
+              SetBluePixelComponent(q,ScaleCharToQuantum(colors.b[code]));
               /* Extract alpha value */
-              alpha_code = (unsigned long) (alpha_bits >> (3*(4*j+i))) & 0x7;
+              alpha_code = (size_t) (alpha_bits >> (3*(4*j+i))) & 0x7;
               if (alpha_code == 0)
                 alpha = a0;
               else if (alpha_code == 1)
@@ -735,15 +745,14 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
                 alpha = 255;
               else
                 alpha = (((6-alpha_code) * a0 + (alpha_code-1) * a1) / 5);
-              
-              q->opacity = ScaleCharToQuantum( (unsigned char) (255 - alpha) );
-              
+              SetOpacityPixelComponent(q,ScaleCharToQuantum((unsigned char)
+                (255-alpha)));
               q++;
             }
         }
       }
       
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         return MagickFalse;
     }
   }
@@ -755,28 +764,37 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
 
 static MagickBooleanType ReadUncompressedRGB(Image *image, DDSInfo *dds_info)
 {
-  long
+  ExceptionInfo
+    *exception;
+
+  ssize_t
     x, y;
   
   PixelPacket
     *q;
   
-  for (y = 0; y < (long) dds_info->height; y++)
+  exception=(&image->exception);
+  for (y = 0; y < (ssize_t) dds_info->height; y++)
   {
-    q = SetImagePixels(image, 0, y, dds_info->width, 1);
+    q = QueueAuthenticPixels(image, 0, y, dds_info->width, 1,exception);
     
     if (q == (PixelPacket *) NULL)
       return MagickFalse;
     
-    for (x = 0; x < (long) dds_info->width; x++)
+    for (x = 0; x < (ssize_t) dds_info->width; x++)
     {
-      q->blue  = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
-      q->green = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
-      q->red   = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
+      SetBluePixelComponent(q,ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
+      SetGreenPixelComponent(q,ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
+      SetRedPixelComponent(q,ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
+      if (dds_info->pixelformat.rgb_bitcount == 32)
+        (void) ReadBlobByte(image);
       q++;
     }
     
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       return MagickFalse;
   }
   
@@ -787,29 +805,37 @@ static MagickBooleanType ReadUncompressedRGB(Image *image, DDSInfo *dds_info)
 
 static MagickBooleanType ReadUncompressedRGBA(Image *image, DDSInfo *dds_info)
 {
-  long
+  ExceptionInfo
+    *exception;
+
+  ssize_t
     x, y;
   
   PixelPacket
     *q;
   
-  for (y = 0; y < (long) dds_info->height; y++)
+  exception=(&image->exception);
+  for (y = 0; y < (ssize_t) dds_info->height; y++)
   {
-    q = SetImagePixels(image, 0, y, dds_info->width, 1);
+    q = QueueAuthenticPixels(image, 0, y, dds_info->width, 1,exception);
     
     if (q == (PixelPacket *) NULL)
       return MagickFalse;
     
-    for (x = 0; x < (long) dds_info->width; x++)
+    for (x = 0; x < (ssize_t) dds_info->width; x++)
     {
-      q->blue    = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
-      q->green   = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
-      q->red     = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
-      q->opacity = ScaleCharToQuantum( (unsigned char) (255 - ReadBlobByte(image)) );
+      SetBluePixelComponent(q,ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
+      SetGreenPixelComponent(q,ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
+      SetRedPixelComponent(q,ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
+      SetOpacityPixelComponent(q,ScaleCharToQuantum((unsigned char)
+        (255-ReadBlobByte(image))));
       q++;
     }
     
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       return MagickFalse;
   }
   
@@ -823,13 +849,13 @@ static MagickBooleanType ReadUncompressedRGBA(Image *image, DDSInfo *dds_info)
 */
 static void SkipDXTMipmaps(Image *image, DDSInfo *dds_info, int texel_size)
 {
-  register long
+  register ssize_t
     i;
 
   MagickOffsetType
     offset;
 
-  unsigned long
+  size_t
     h,
     w;
   
@@ -846,7 +872,7 @@ static void SkipDXTMipmaps(Image *image, DDSInfo *dds_info, int texel_size)
       /*
         Mipmapcount includes the main image, so start from one
       */
-      for (i = 1; (i < (long) dds_info->mipmapcount) && w && h; i++)
+      for (i = 1; (i < (ssize_t) dds_info->mipmapcount) && w && h; i++)
       {
         offset = (MagickOffsetType) ((w + 3) / 4) * ((h + 3) / 4) * texel_size;
         (void) SeekBlob(image, offset, SEEK_CUR);
@@ -865,10 +891,10 @@ static void SkipRGBMipmaps(Image *image, DDSInfo *dds_info, int pixel_size)
   MagickOffsetType
     offset;
   
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     h,
     w;
 
@@ -885,7 +911,7 @@ static void SkipRGBMipmaps(Image *image, DDSInfo *dds_info, int pixel_size)
       /*
         Mipmapcount includes the main image, so start from one
       */
-      for (i=1; (i < (long) dds_info->mipmapcount) && w && h; i++)
+      for (i=1; (i < (ssize_t) dds_info->mipmapcount) && w && h; i++)
       {
         offset = (MagickOffsetType) w * h * pixel_size;
         (void) SeekBlob(image, offset, SEEK_CUR);
@@ -916,8 +942,7 @@ static void SkipRGBMipmaps(Image *image, DDSInfo *dds_info, int pixel_size)
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -954,7 +979,7 @@ static MagickBooleanType IsDDS(const unsigned char *magick,const size_t length)
 %      RegisterDDSImage(void)
 %
 */
-ModuleExport unsigned long RegisterDDSImage(void)
+ModuleExport size_t RegisterDDSImage(void)
 {
   MagickInfo
     *entry;
@@ -962,6 +987,7 @@ ModuleExport unsigned long RegisterDDSImage(void)
   entry = SetMagickInfo("DDS");
   entry->decoder = (DecodeImageHandler *) ReadDDSImage;
   entry->magick = (IsImageFormatHandler *) IsDDS;
+  entry->seekable_stream=MagickTrue;
   entry->description = ConstantString("Microsoft DirectDraw Surface");
   entry->module = ConstantString("DDS");
   (void) RegisterMagickInfo(entry);
