@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,8 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -78,8 +80,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -162,14 +163,14 @@ pump_data(Image *image, LoadContext* lc)
 static ddjvu_message_t*
 pump_data_until_message(LoadContext *lc,Image *image) /* ddjvu_context_t *context, type ddjvu_document_type_t */
 {
-        unsigned long blocksize = BLOCKSIZE;
+        size_t blocksize = BLOCKSIZE;
         unsigned char data[BLOCKSIZE];
-        unsigned long size;
+        size_t size;
         ddjvu_message_t *message;
 
         /* i might check for a condition! */
         while (!(message = ddjvu_message_peek(lc->context))
-               && (size = (unsigned long) ReadBlob(image,(size_t) blocksize,data)) == blocksize) {
+               && (size = (size_t) ReadBlob(image,(size_t) blocksize,data)) == blocksize) {
                 ddjvu_stream_write(lc->document, lc->streamid, (char *) data, size);
         }
         if (message)
@@ -210,7 +211,7 @@ message_tag_name(ddjvu_message_tag_t tag)
 /* write out nice info on the message,
  * and store in *user* data the info on progress.
  * */
-void
+int
 process_message(ddjvu_message_t *message)
 {
 
@@ -219,7 +220,7 @@ process_message(ddjvu_message_t *message)
 #endif
 
    if (! message)
-      return;
+      return(-1);
 #if DEBUG
    printf("*** %s: %s.\n",__FUNCTION__, message_tag_name(message->m_any.tag));
 #endif
@@ -231,7 +232,7 @@ process_message(ddjvu_message_t *message)
       ddjvu_document_t* document= message->m_any.document;
       /* ddjvu_document_decoding_status  is set by libdjvu! */
       /* we have some info on the document  */
-      LoadContext *lc = ddjvu_document_get_user_data(document);
+      LoadContext *lc = (LoadContext *) ddjvu_document_get_user_data(document);
       lc->pages = ddjvu_document_get_pagenum(document);
 #if DEBUG
       printf("the doc has %d pages\n", ddjvu_document_get_pagenum(document));
@@ -303,6 +304,7 @@ process_message(ddjvu_message_t *message)
    default:
       printf("unexpected\n");
    };
+  return(message->m_any.tag);
 }
 
 
@@ -320,8 +322,21 @@ process_message(ddjvu_message_t *message)
 static void
 get_page_image(LoadContext *lc, ddjvu_page_t *page, int x, int y, int w, int h, QuantumInfo* quantum_info)
 {
+  ddjvu_format_t
+    *format;
+
+  ddjvu_page_type_t
+    type;
+
+  Image
+    *image;
+
   int
-    ret;
+    ret,
+    stride;
+
+  unsigned char
+    *q;
 
         ddjvu_rect_t rect;
         rect.x = x;
@@ -329,18 +344,18 @@ get_page_image(LoadContext *lc, ddjvu_page_t *page, int x, int y, int w, int h,
         rect.w = (unsigned int) w;             /* /10 */
         rect.h = (unsigned int) h;             /* /10 */
 
-        Image* image = lc->image;
-        ddjvu_page_type_t type = ddjvu_page_get_type(lc->page);
+        image = lc->image;
+        type = ddjvu_page_get_type(lc->page);
 
         /* stride of this temporary buffer: */
-        int stride = (type == DDJVU_PAGETYPE_BITONAL)?
+        stride = (type == DDJVU_PAGETYPE_BITONAL)?
                 (lc->image->columns + 7)/8:
                 lc->image->columns *3;
 
-        unsigned char* q = malloc(lc->image->rows *  stride);
+        q = (unsigned char *) AcquireQuantumMemory(lc->image->rows,stride);
 
 
-        ddjvu_format_t *format = ddjvu_format_create(
+        format = ddjvu_format_create(
                 (type == DDJVU_PAGETYPE_BITONAL)?DDJVU_FORMAT_LSBTOMSB : DDJVU_FORMAT_RGB24,
                 /* DDJVU_FORMAT_RGB24
                  * DDJVU_FORMAT_RGBMASK32*/
@@ -366,6 +381,7 @@ get_page_image(LoadContext *lc, ddjvu_page_t *page, int x, int y, int w, int h,
                                     format,
                                     stride, /* ?? */
                                     (char*)q);
+        (void) ret;
         ddjvu_format_release(format);
 
 
@@ -375,38 +391,39 @@ get_page_image(LoadContext *lc, ddjvu_page_t *page, int x, int y, int w, int h,
                 printf("%s: expanding BITONAL page/image\n", __FUNCTION__);
 #endif
                 register IndexPacket *indexes;
-                unsigned long bit, byte;
+                size_t bit, byte;
 
-                for (y=0; y < (long) image->rows; y++)
+                for (y=0; y < (ssize_t) image->rows; y++)
                         {
-                                PixelPacket * o = SetImagePixels(image,0,y,image->columns,1);
+                                PixelPacket * o = QueueAuthenticPixels(image,0,y,image->columns,1,&image->exception);
                                 if (o == (PixelPacket *) NULL)
                                         break;
-                                indexes=GetIndexes(image);
+                                indexes=GetAuthenticIndexQueue(image);
                                 bit=0;
                                 byte=0;
 
                                 /* fixme:  the non-aligned, last =<7 bits ! that's ok!!!*/
-                                for (x= 0; x < (long) image->columns; x++)
+                                for (x= 0; x < (ssize_t) image->columns; x++)
                                         {
-                                                if (bit == 0) byte= (unsigned long) q[(y * stride) + (x / 8)];
+                                                if (bit == 0) byte= (size_t) q[(y * stride) + (x / 8)];
 
-                                                indexes[x]=(IndexPacket) (((byte & 0x01) != 0) ? 0x00 : 0x01);
+                                                SetIndexPixelComponent(indexes+x,(IndexPacket) (((byte & 0x01) != 0) ? 0x00 : 0x01));
                                                 bit++;
                                                 if (bit == 8)
                                                         bit=0;
                                                 byte>>=1;
                                         }
-                                if (SyncImagePixels(image) == MagickFalse)
+                                if (SyncAuthenticPixels(image,&image->exception) == MagickFalse)
                                         break;
                         }
-                SyncImage(image);
+                if (!image->ping)
+                  SyncImage(image);
         } else {
 #if DEBUG
                 printf("%s: expanding PHOTO page/image\n", __FUNCTION__);
 #endif
                 /* now transfer line-wise: */
-                long i;
+                ssize_t i;
 #if 0
                 /* old: */
                 char* r;
@@ -414,67 +431,44 @@ get_page_image(LoadContext *lc, ddjvu_page_t *page, int x, int y, int w, int h,
                 register PixelPacket *r;
 #endif
 
-                for (i = 0;i< (long) lc->image->rows; i++)
+                for (i = 0;i< (ssize_t) lc->image->rows; i++)
                         {
 #if DEBUG
                                if (i % 1000 == 0) printf("%d\n",i);
 #endif
-                               r = SetImagePixels(lc->image,0,i,lc->image->columns,1);
+                               r = QueueAuthenticPixels(lc->image,0,i,lc->image->columns,1,&image->exception);
+                               if (r == (PixelPacket *) NULL)
+                                 break;
 
-                                ImportQuantumPixels(lc->image,
+                               ImportQuantumPixels(lc->image,
+                                                    (CacheView *) NULL,
                                                     quantum_info,
                                                     RGBQuantum, /*GrayQuantum*/
-                                                    q+i*stride);
-                                SyncImagePixels(lc->image);
+                                                    q+i*stride,&image->exception);
+                              SyncAuthenticPixels(lc->image,&image->exception);
                         }
         }
+        q=(unsigned char *) RelinquishMagickMemory(q);
 }
 
 
+#if defined(MAGICKCORE_DJVU_DELEGATE)
+
+#if 0
 static int
-get_page_line(PixelPacket *q, LoadContext *lc, int row)
+get_page_line(LoadContext *lc, int row, QuantumInfo* quantum_info)
 {
+  ddjvu_format_t
+    *format;
 
-        ddjvu_rect_t rect, pagerect;
-        rect.x = 0;
-        rect.y = row;
-        rect.h = 1;             /* /10 */
+  int
+    ret;
 
-        pagerect.x = 0;
-        pagerect.y = 0;
-        pagerect.w = lc->image->columns;
-        pagerect.h = lc->image->rows;
+  size_t
+    stride;
 
-
-        ddjvu_format_t *format = ddjvu_format_create(
-#if RGB
-                DDJVU_FORMAT_RGB24
-#else
-                DDJVU_FORMAT_GREY8
-#endif
-                ,
-                0, NULL);
-        ddjvu_format_set_row_order(format, 0);
-        ddjvu_format_set_y_direction(format, 1);
-
-        int ret = ddjvu_page_render(lc->page,
-                                    DDJVU_RENDER_COLOR, /* ddjvu_render_mode_t */
-                                    &pagerect,
-                                    &rect,     /* mmc: ?? */
-                                    format,
-                                    pagerect.w
-#if RGB
-                                    * 3
-#endif
-                                    , /* ?? */
-                                    (char*)q);
-        ddjvu_format_release(format);
-        return ret;
-}
-
-static int
-get_page_line_new(LoadContext *lc, int row, QuantumInfo* quantum_info)
-{
+  unsigned char
+    *q;
 
         ddjvu_rect_t rect, pagerect;
         rect.x = 0;
@@ -488,7 +482,7 @@ get_page_line_new(LoadContext *lc, int row, QuantumInfo* quantum_info)
         pagerect.h = lc->image->rows;
 
 
-        ddjvu_format_t *format = ddjvu_format_create(
+        format = ddjvu_format_create(
 #if RGB
                 DDJVU_FORMAT_RGB24
 #else
@@ -499,13 +493,13 @@ get_page_line_new(LoadContext *lc, int row, QuantumInfo* quantum_info)
         ddjvu_format_set_row_order(format, 1);
         ddjvu_format_set_y_direction(format, 1);
 
-        unsigned char* q = malloc(lc->image->columns
+        stride=1;
 #if RGB
-                         * 3
+        stride=3;
 #endif
-                );
+        q = (unsigned char *) AcquireQuantumMemory(lc->image->columns,stride);
 
-        int ret = ddjvu_page_render(lc->page,
+        ret = ddjvu_page_render(lc->page,
                                     DDJVU_RENDER_COLOR, /* ddjvu_render_mode_t */
                                     &pagerect,
                                     &rect,     /* mmc: ?? */
@@ -514,17 +508,20 @@ get_page_line_new(LoadContext *lc, int row, QuantumInfo* quantum_info)
                                     (char*)q);
 
         ImportQuantumPixels(lc->image,
+                            (CacheView *) NULL,
                             quantum_info,
 #if RGB
                             RGBQuantum
 #else
                             GrayQuantum
 #endif
-                            ,q);
-        free(q);
+                            ,q,&lc->image->exception);
+        q=(unsigned char *) RelinquishMagickMemory(q);
         ddjvu_format_release(format);
         return ret;
 }
+#endif
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -557,20 +554,35 @@ get_page_line_new(LoadContext *lc, int row, QuantumInfo* quantum_info)
 %
 */
 
+static inline double MagickMax(const double x,const double y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
 static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
   const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  ddjvu_page_type_t
+     type;
+
+  ddjvu_pageinfo_t info;
+  QuantumInfo *quantum_info;
+  ddjvu_message_t *message;
+  Image *image;
+  int logging;
+  int tag;
+
         /* so, we know that the page is there! Get its dimension, and  */
 
         /* Read one DJVU image */
-        Image *image = lc->image;
-        int logging;
+        image = lc->image;
+
         /* register PixelPacket *q; */
 
-        QuantumInfo *quantum_info;
-
-        ddjvu_message_t *message;
         logging=LogMagickEvent(CoderEvent,GetMagickModule(), "  enter ReadOneDJVUImage()");
+        (void) logging;
 
 #if DEBUG
         printf("====  Loading the page %d\n", pagenum);
@@ -578,10 +590,12 @@ static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
         lc->page = ddjvu_page_create_by_pageno(lc->document, pagenum); /*  0? */
 
         /* pump data untill the page is ready for rendering. */
+        tag=(-1);
         do {
                 while ((message = ddjvu_message_peek(lc->context)))
                         {
-                                process_message(message);
+                                tag=process_message(message);
+                                if (tag == 0) break;
                                 ddjvu_message_pop(lc->context);
                         }
                 /* fixme: maybe exit? */
@@ -590,15 +604,37 @@ static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
                 message = pump_data_until_message(lc,image);
                 if (message)
                         do {
-                                process_message(message);
+                                tag=process_message(message);
+                                if (tag == 0) break;
                                 ddjvu_message_pop(lc->context);
                         } while ((message = ddjvu_message_peek(lc->context)));
         } while (!ddjvu_page_decoding_done(lc->page));
 
-        ddjvu_pageinfo_t info;
         ddjvu_document_get_pageinfo(lc->document, pagenum, &info);
 
-        ddjvu_page_type_t type = ddjvu_page_get_type(lc->page);
+        image->x_resolution = (float) info.dpi;
+        image->y_resolution =(float) info.dpi;
+        if (image_info->density != (char *) NULL)
+          {
+            int
+              flags;
+
+            GeometryInfo
+              geometry_info;
+
+            /*
+              Set rendering resolution.
+            */
+            flags=ParseGeometry(image_info->density,&geometry_info);
+            image->x_resolution=geometry_info.rho;
+            image->y_resolution=geometry_info.sigma;
+            if ((flags & SigmaValue) == 0)
+              image->y_resolution=image->x_resolution;
+            info.width*=image->x_resolution/info.dpi;
+            info.height*=image->y_resolution/info.dpi;
+            info.dpi=(ssize_t) MagickMax(image->x_resolution,image->y_resolution);
+          }
+        type = ddjvu_page_get_type(lc->page);
 
         /* double -> float! */
         /* image->gamma = (float)ddjvu_page_get_gamma(lc->page); */
@@ -606,10 +642,8 @@ static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
         /* mmc:  set  image->depth  */
         /* mmc:  This from the type */
 
-        SetImageExtent(image,info.width,info.height);
-
-        image->x_resolution = (float) info.dpi;
-        image->y_resolution =(float) info.dpi;
+        image->columns=(size_t) info.width;
+        image->rows=(size_t) info.height;
 
         /* mmc: bitonal should be palettized, and compressed! */
         if (type == DDJVU_PAGETYPE_BITONAL){
@@ -630,41 +664,40 @@ static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
                 /* is this useful? */
         }
 #if DEBUG
-        printf("now filling %ld x %ld\n", image->columns,image->rows);
+        printf("now filling %.20g x %.20g\n",(double) image->columns,(double)
+          image->rows);
 #endif
 
 
 #if 1                           /* per_line */
         quantum_info=AcquireQuantumInfo(image_info,image);
+        if (quantum_info == (QuantumInfo *) NULL)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
 
-        /* q = SetImagePixels(image,0,0,image->columns,image->rows); */
+        /* q = QueueAuthenticPixels(image,0,0,image->columns,image->rows); */
         get_page_image(lc, lc->page, 0, 0, info.width, info.height, quantum_info);
 #else
         int i;
         for (i = 0;i< image->rows; i++)
                 {
                         printf("%d\n",i);
-#if 0
-                        q = SetImagePixels(image,0,i,image->columns,1);
-                        get_page_line(q,lc, i);
-                        SyncImagePixels(image);
-#else
-                        q = SetImagePixels(image,0,i,image->columns,1);
-                        get_page_line_new(lc, i, quantum_info);
-                        SyncImagePixels(image);
-#endif
+                        q = QueueAuthenticPixels(image,0,i,image->columns,1);
+                        get_page_line(lc, i, quantum_info);
+                        SyncAuthenticPixels(image);
                 }
 
 #endif /* per_line */
 
 
 #if DEBUG
-        printf("END: finished filling %ld x %ld\n", image->columns,image->rows);
+        printf("END: finished filling %.20g x %.20g\n",(double) image->columns,
+          (double) image->rows);
 #endif
 
-        SyncImage(image);
+        if (!image->ping)
+          SyncImage(image);
         quantum_info=DestroyQuantumInfo(quantum_info);
-        /* indexes=GetIndexes(image); */
+        /* indexes=GetAuthenticIndexQueue(image); */
         /* mmc: ??? Convert PNM pixels to runlength-encoded MIFF packets. */
         /* image->colors =  */
 
@@ -676,6 +709,8 @@ static Image *ReadOneDJVUImage(LoadContext* lc,const int pagenum,
         }
 
         /* image->page.y=mng_info->y_off[mng_info->object_id]; */
+        if (tag == 0)
+          image=DestroyImage(image);
         return image;
         /* end of reading one DJVU page/image */
 }
@@ -709,12 +744,19 @@ static void djvu_close_lc(LoadContext* lc)
 static Image *ReadDJVUImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+  const char
+    *url;
+
+  ddjvu_message_t
+    *message;
+
   Image
     *image,
     *images;
 
   int
-    logging;
+    logging,
+    use_cache;
 
   LoadContext
     *lc;
@@ -722,7 +764,7 @@ static Image *ReadDJVUImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -740,6 +782,7 @@ static Image *ReadDJVUImage(const ImageInfo *image_info,
 
 
   logging = LogMagickEvent(CoderEvent,GetMagickModule(),"enter ReadDJVUImage()");
+  (void) logging;
 
   image = AcquireImage(image_info); /* mmc: ?? */
 
@@ -777,11 +820,10 @@ static Image *ReadDJVUImage(const ImageInfo *image_info,
   lc->pages = 0;
   lc->context = ddjvu_context_create("ImageMagick djvu loader"); /* g_program_name */
 
-  ddjvu_message_t *message;
   ddjvu_cache_set_size(lc->context, 1); /* right? */
-  int use_cache = 0;
-  char* url = "http://maruska.dyndns.org/fake.djvu";
+  use_cache = 0;
   /* document: here we don't have a filename, but, for the sake of generality, a FILE* ! */
+  url="http://www.imagemagick.org/fake.djvu";
   lc->document = ddjvu_document_create(lc->context, url, use_cache); /* don't cache */
   ddjvu_document_set_user_data(lc->document, lc);
 
@@ -822,15 +864,24 @@ static Image *ReadDJVUImage(const ImageInfo *image_info,
   }
 
   images=NewImageList();
-  for (i=0; i < (long) lc->pages; i++)
+  i=0;
+  if (image_info->number_scenes != 0)
+    i=image_info->scene;
+  for ( ; i < (ssize_t) lc->pages; i++)
   {
     image=ReadOneDJVUImage(lc,i,image_info,exception);
+    if (image == (Image *) NULL)
+      break;
+    image->scene=i;
     AppendImageToList(&images,CloneImageList(image,exception));
+    if (image_info->number_scenes != 0)
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+        break;
   }
-
   djvu_close_lc(lc);
-  (void) CloseBlob(image);
-  image=DestroyImageList(image);
+  (void) CloseBlob(images);
+  if (image != (Image *) NULL)
+    image=DestroyImageList(image);
 
 #if 0
   if ((image->page.width == 0) && (image->page.height == 0))
@@ -875,10 +926,10 @@ static Image *ReadDJVUImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterDJVUImage method is:
 %
-%      unsigned long RegisterDJVUImage(void)
+%      size_t RegisterDJVUImage(void)
 %
 */
-ModuleExport unsigned long RegisterDJVUImage(void)
+ModuleExport size_t RegisterDJVUImage(void)
 {
   char
     version[MaxTextExtent];
