@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -130,9 +130,10 @@ static Image *ReadDOTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (status == MagickFalse)
     return((Image *) NULL);
   read_info=CloneImageInfo(image_info);
-  (void) CopyMagickString(read_info->magick,"PS",MaxTextExtent);
+  SetImageInfoBlob(read_info,(void *) NULL,0);
+  (void) CopyMagickString(read_info->magick,"SVG",MaxTextExtent);
   (void) AcquireUniqueFilename(read_info->filename);
-  (void) FormatMagickString(command,MaxTextExtent,"-Tps2 -o%s %s",
+  (void) FormatMagickString(command,MaxTextExtent,"-Tsvg -o%s %s",
     read_info->filename,image_info->filename);
   graph=agread(GetBlobFileHandle(image));
   if (graph == (graph_t *) NULL)
@@ -142,10 +143,10 @@ static Image *ReadDOTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     gvLayout(graphic_context,graph,(char *) "dot");
   else
     gvLayout(graphic_context,graph,(char *) option);
-  gvRenderFilename(graphic_context,graph,(char *) "ps2",read_info->filename);
+  gvRenderFilename(graphic_context,graph,(char *) "svg",read_info->filename);
   gvFreeLayout(graphic_context,graph);
   /*
-    Read Postscript graph.
+    Read SVG graph.
   */
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
@@ -176,10 +177,10 @@ static Image *ReadDOTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterDOTImage method is:
 %
-%      unsigned long RegisterDOTImage(void)
+%      size_t RegisterDOTImage(void)
 %
 */
-ModuleExport unsigned long RegisterDOTImage(void)
+ModuleExport size_t RegisterDOTImage(void)
 {
   MagickInfo
     *entry;
