@@ -10,39 +10,26 @@
 %                            DDDD   P      SSSSS                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%            Read Postscript Using the Display Postscript System.             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,11 +39,32 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
-#if defined(HasDPS)
-#include "xwindow.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/client.h"
+#include "magick/colormap.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/image.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/module.h"
+#include "magick/utility.h"
+#include "magick/xwindow-private.h"
+#if defined(MAGICKCORE_DPS_DELEGATE)
+#include <DPS/dpsXclient.h>
+#include <DPS/dpsXpreview.h>
+#endif
 
+#if defined(MAGICKCORE_DPS_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -68,7 +76,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadDPSImage reads a Adobe Postscript image file and returns it.  It
+%  ReadDPSImage() reads a Adobe Postscript image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -78,19 +86,22 @@
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadDPSImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
+static inline double MagickMin(const double x,const double y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  char
+  const char
     *client_name;
 
   Display
@@ -106,28 +117,28 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     sans,
     status;
 
-  long
-    x,
-    y;
-
   Pixmap
     pixmap;
 
   register IndexPacket
     *indexes;
 
-  register int
+  register ssize_t
     i;
 
   register PixelPacket
     *q;
 
-  register unsigned long
+  register size_t
     pixel;
 
   Screen
     *screen;
 
+  ssize_t
+    x,
+    y;
+
   XColor
     *colors;
 
@@ -155,36 +166,38 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   display=XOpenDisplay(image_info->server_name);
   if (display == (Display *) NULL)
     return((Image *) NULL);
   /*
-    Set our forgiving error handler.
+    Set our forgiving exception handler.
   */
   (void) XSetErrorHandler(XError);
   /*
     Open image file.
   */
-  image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
+  image=AcquireImage(image_info);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
     return((Image *) NULL);
   /*
     Get user defaults from X resource database.
   */
-  client_name=SetClientName((char *) NULL);
+  client_name=GetClientName();
   resource_database=XGetResourceDatabase(display,client_name);
-  XGetResourceInfo(resource_database,client_name,&resource_info);
+  XGetResourceInfo(image_info,resource_database,client_name,&resource_info);
   /*
     Allocate standard colormap.
   */
   map_info=XAllocStandardColormap();
   visual_info=(XVisualInfo *) NULL;
   if (map_info == (XStandardColormap *) NULL)
-    ThrowReaderException(ResourceLimitWarning,
-      "Unable to create standard colormap",image)
+    ThrowReaderException(ResourceLimitError,"UnableToCreateStandardColormap")
   else
     {
       /*
@@ -197,7 +210,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((map_info == (XStandardColormap *) NULL) ||
       (visual_info == (XVisualInfo *) NULL))
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -208,12 +221,14 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   screen=ScreenOfDisplay(display,visual_info->screen);
   pixels_per_point=XDPSPixelsPerPoint(screen);
   if ((image->x_resolution != 0.0) && (image->y_resolution != 0.0))
-    pixels_per_point=Min(image->x_resolution,image->y_resolution)/72.0;
-  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,image->file,
-    visual_info->depth,pixels_per_point,&pixmap,&bits_per_pixel,&page);
+    pixels_per_point=MagickMin(image->x_resolution,image->y_resolution)/
+      DefaultResolution;
+  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,
+    GetBlobFileHandle(image),visual_info->depth,pixels_per_point,&pixmap,
+    &bits_per_pixel,&page);
   if ((status == dps_status_failure) || (status == dps_status_no_extension))
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -221,12 +236,13 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Rasterize the file into the pixmap.
   */
-  status=XDPSImageFileIntoDrawable((DPSContext) NULL,screen,pixmap,image->file,
-    bits_per_pixel.height,visual_info->depth,&page,-page.x,
-    -page.y,pixels_per_point,True,False,True,&sans);
+  status=XDPSImageFileIntoDrawable((DPSContext) NULL,screen,pixmap,
+    GetBlobFileHandle(image),(int) bits_per_pixel.height,visual_info->depth,
+    &page,-page.x,-page.y,pixels_per_point,MagickTrue,MagickFalse,MagickTrue,
+    &sans);
   if (status != dps_status_success)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -239,7 +255,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) XFreePixmap(display,pixmap);
   if (dps_image == (XImage *) NULL)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -247,25 +263,25 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Get the colormap colors.
   */
-  colors=(XColor *) AcquireMemory(visual_info->colormap_size*sizeof(XColor));
+  colors=(XColor *) AcquireQuantumMemory(visual_info->colormap_size,
+    sizeof(*colors));
   if (colors == (XColor *) NULL)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XDestroyImage(dps_image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
     }
-  if ((visual_info->storage_class != DirectColor) &&
-      (visual_info->storage_class != TrueColor))
+  if ((visual_info->klass != DirectColor) && (visual_info->klass != TrueColor))
     for (i=0; i < visual_info->colormap_size; i++)
     {
-      colors[i].pixel=i;
+      colors[i].pixel=(size_t) i;
       colors[i].pad=0;
     }
   else
     {
-      unsigned long
+      size_t
         blue,
         blue_bit,
         green,
@@ -302,26 +318,25 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Convert X image to MIFF format.
   */
-  if ((visual_info->storage_class != TrueColor) &&
-      (visual_info->storage_class != DirectColor))
+  if ((visual_info->klass != TrueColor) && (visual_info->klass != DirectColor))
     image->storage_class=PseudoClass;
-  image->columns=dps_image->width;
-  image->rows=dps_image->height;
-  if (image_info->ping)
+  image->columns=(size_t) dps_image->width;
+  image->rows=(size_t) dps_image->height;
+  if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
-      return(image);
+      (void) CloseBlob(image);
+      return(GetFirstImageInList(image));
     }
   switch (image->storage_class)
   {
     case DirectClass:
     default:
     {
-      register unsigned long
+      register size_t
         color,
         index;
 
-      unsigned long
+      size_t
         blue_mask,
         blue_shift,
         green_mask,
@@ -334,21 +349,21 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       red_mask=visual_info->red_mask;
       red_shift=0;
-      while ((red_mask & 0x01) == 0)
+      while ((red_mask != 0) && ((red_mask & 0x01) == 0))
       {
         red_mask>>=1;
         red_shift++;
       }
       green_mask=visual_info->green_mask;
       green_shift=0;
-      while ((green_mask & 0x01) == 0)
+      while ((green_mask != 0) && ((green_mask & 0x01) == 0))
       {
         green_mask>>=1;
         green_shift++;
       }
       blue_mask=visual_info->blue_mask;
       blue_shift=0;
-      while ((blue_mask & 0x01) == 0)
+      while ((blue_mask != 0) && ((blue_mask & 0x01) == 0))
       {
         blue_mask>>=1;
         blue_shift++;
@@ -357,49 +372,54 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         Convert X image to DirectClass packets.
       */
       if ((visual_info->colormap_size > 0) &&
-          (visual_info->storage_class == DirectColor))
-        for (y=0; y < (long) image->rows; y++)
+          (visual_info->klass == DirectColor))
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             pixel=XGetPixel(dps_image,x,y);
             index=(pixel >> red_shift) & red_mask;
-            q->red=XDownscale(colors[index].red);
+            SetRedPixelComponent(q,ScaleShortToQuantum(colors[index].red));
             index=(pixel >> green_shift) & green_mask;
-            q->green=XDownscale(colors[index].green);
+            SetGreenPixelComponent(q,ScaleShortToQuantum(colors[index].green));
             index=(pixel >> blue_shift) & blue_mask;
-            q->blue=XDownscale(colors[index].blue);
+            SetBluePixelComponent(q,ScaleShortToQuantum(colors[index].blue));
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
+            break;
+          if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,y,image->rows);
         }
       else
-        for (y=0; y < (long) image->rows; y++)
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             pixel=XGetPixel(dps_image,x,y);
             color=(pixel >> red_shift) & red_mask;
-            q->red=XDownscale((color*65535L)/red_mask);
+            color=(color*65535L)/red_mask;
+            SetRedPixelComponent(q,ScaleShortToQuantum((unsigned short) color));
             color=(pixel >> green_shift) & green_mask;
-            q->green=XDownscale((color*65535L)/green_mask);
+            color=(color*65535L)/green_mask;
+            SetGreenPixelComponent(q,ScaleShortToQuantum((unsigned short)
+              color));
             color=(pixel >> blue_shift) & blue_mask;
-            q->blue=XDownscale((color*65535L)/blue_mask);
+            color=(color*65535L)/blue_mask;
+            SetBluePixelComponent(q,ScaleShortToQuantum((unsigned short)
+              color));
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
+            break;
+          if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,y,image->rows);
         }
       break;
     }
@@ -408,54 +428,57 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Create colormap.
       */
-      if (!AllocateImageColormap(image,visual_info->colormap_size))
+      if (AcquireImageColormap(image,(size_t) visual_info->colormap_size) == MagickFalse)
         {
-          DestroyImage(image);
-          LiberateMemory((void **) &colors);
+          image=DestroyImage(image);
+          colors=(XColor *) RelinquishMagickMemory(colors);
           XDestroyImage(dps_image);
           XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
             (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
           return((Image *) NULL);
         }
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (ssize_t) image->colors; i++)
       {
-        image->colormap[colors[i].pixel].red=XDownscale(colors[i].red);
-        image->colormap[colors[i].pixel].green=XDownscale(colors[i].green);
-        image->colormap[colors[i].pixel].blue=XDownscale(colors[i].blue);
+        image->colormap[colors[i].pixel].red=ScaleShortToQuantum(colors[i].red);
+        image->colormap[colors[i].pixel].green=
+          ScaleShortToQuantum(colors[i].green);
+        image->colormap[colors[i].pixel].blue=
+          ScaleShortToQuantum(colors[i].blue);
       }
       /*
         Convert X image to PseudoClass packets.
       */
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < (long) image->columns; x++)
-          indexes[x]=(unsigned short) XGetPixel(dps_image,x,y);
-        if (!SyncImagePixels(image))
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < (ssize_t) image->columns; x++)
+          SetIndexPixelComponent(indexes+x,(unsigned short)
+            XGetPixel(dps_image,x,y));
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
+          break;
+        if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
       }
       break;
     }
   }
-  LiberateMemory((void **) &colors);
+  colors=(XColor *) RelinquishMagickMemory(colors);
   XDestroyImage(dps_image);
   if (image->storage_class == PseudoClass)
-    SyncImage(image);
+    (void) SyncImage(image);
   /*
     Rasterize matte image.
   */
-  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,image->file,1,
-    pixels_per_point,&pixmap,&bits_per_pixel,&page);
+  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,
+    GetBlobFileHandle(image),1,pixels_per_point,&pixmap,&bits_per_pixel,&page);
   if ((status != dps_status_failure) && (status != dps_status_no_extension))
     {
       status=XDPSImageFileIntoDrawable((DPSContext) NULL,screen,pixmap,
-        image->file,bits_per_pixel.height,1,&page,-page.x,
-        -page.y,pixels_per_point,True,True,True,&sans);
+        GetBlobFileHandle(image),(int) bits_per_pixel.height,1,&page,-page.x,
+        -page.y,pixels_per_point,MagickTrue,MagickTrue,MagickTrue,&sans);
       if (status == dps_status_success)
         {
           XImage
@@ -470,20 +493,20 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (matte_image != (XImage *) NULL)
             {
               image->storage_class=DirectClass;
-              image->matte=True;
-              for (y=0; y < (long) image->rows; y++)
+              image->matte=MagickTrue;
+              for (y=0; y < (ssize_t) image->rows; y++)
               {
-                q=SetImagePixels(image,0,y,image->columns,1);
+                q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
                 if (q == (PixelPacket *) NULL)
                   break;
-                for (x=0; x < (long) image->columns; x++)
+                for (x=0; x < (ssize_t) image->columns; x++)
                 {
-                  q->opacity=OpaqueOpacity;
-                  if (!XGetPixel(matte_image,x,y))
-                    q->opacity=TransparentOpacity;
+                  SetOpacityPixelComponent(q,OpaqueOpacity);
+                  if (XGetPixel(matte_image,x,y) == 0)
+                    SetOpacityPixelComponent(q,TransparentOpacity);
                   q++;
                 }
-                if (!SyncImagePixels(image))
+                if (SyncAuthenticPixels(image,exception) == MagickFalse)
                   break;
               }
               XDestroyImage(matte_image);
@@ -491,19 +514,12 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
   /*
-    Free resources.
+    Relinquish resources.
   */
   XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
     (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
-  CloseBlob(image);
-  return(image);
-}
-#else
-static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  ThrowException(exception,MissingDelegateWarning,"Cannot read DPS images",
-    image_info->filename);
-  return((Image *) NULL);
+  (void) CloseBlob(image);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -518,7 +534,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterDPSImage adds attributes for the Display Postscript image
+%  RegisterDPSImage() adds attributes for the Display Postscript image
 %  format to the list of supported formats.  The attributes include the image
 %  format tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -527,20 +543,23 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterDPSImage method is:
 %
-%      RegisterDPSImage(void)
+%      size_t RegisterDPSImage(void)
 %
 */
-ModuleExport void RegisterDPSImage(void)
+ModuleExport size_t RegisterDPSImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("DPS");
-  entry->decoder=ReadDPSImage;
-  entry->blob_support=False;
-  entry->description=AcquireString("Display Postscript");
-  entry->module=AcquireString("DPS");
+#if defined(MAGICKCORE_DPS_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadDPSImage;
+#endif
+  entry->blob_support=MagickFalse;
+  entry->description=ConstantString("Display Postscript Interpreter");
+  entry->module=ConstantString("DPS");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -554,7 +573,7 @@ ModuleExport void RegisterDPSImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterDPSImage removes format registrations made by the
+%  UnregisterDPSImage() removes format registrations made by the
 %  DPS module from the list of supported formats.
 %
 %  The format of the UnregisterDPSImage method is:
