@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,12 +49,14 @@
 #include "magick/exception-private.h"
 #include "magick/delegate.h"
 #include "magick/geometry.h"
+#include "magick/histogram.h"
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantize.h"
 #include "magick/resource_.h"
 #include "magick/quantum-private.h"
@@ -69,7 +71,7 @@
 */
 typedef struct _EPTInfo
 {
-  unsigned long
+  size_t
     magick;
 
   MagickOffsetType
@@ -111,8 +113,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -148,7 +149,7 @@ static MagickBooleanType IsEPT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -164,15 +165,15 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ImageInfo
     *read_info;
 
-  ssize_t
-    count;
-
   MagickBooleanType
     status;
 
   MagickOffsetType
     offset;
 
+  ssize_t
+    count;
+
   /*
     Open image file.
   */
@@ -183,7 +184,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -201,25 +202,32 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ept_info.tiff_length=ReadBlobLSBLong(image);
   (void) ReadBlobLSBShort(image);
   ept_info.postscript=(unsigned char *) AcquireQuantumMemory(
-    ept_info.postscript_length,sizeof(*ept_info.postscript));
-  ept_info.tiff=(unsigned char *) AcquireQuantumMemory(ept_info.tiff_length,
-    sizeof(*ept_info.tiff));
-  if ((ept_info.postscript == (unsigned char *) NULL) ||
-      (ept_info.tiff == (unsigned char *) NULL))
+    ept_info.postscript_length+1,sizeof(*ept_info.postscript));
+  if (ept_info.postscript == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) ResetMagickMemory(ept_info.postscript,0,(ept_info.postscript_length+1)*
+    sizeof(*ept_info.postscript));
+  ept_info.tiff=(unsigned char *) AcquireQuantumMemory(ept_info.tiff_length+1,
+    sizeof(*ept_info.tiff));
+  if (ept_info.tiff == (unsigned char *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) ResetMagickMemory(ept_info.tiff,0,(ept_info.tiff_length+1)*
+    sizeof(*ept_info.tiff));
   offset=SeekBlob(image,ept_info.tiff_offset,SEEK_SET);
   if (offset < 0)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   count=ReadBlob(image,ept_info.tiff_length,ept_info.tiff);
   if (count != (ssize_t) (ept_info.tiff_length))
-    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageWarning,
+      "InsufficientImageDataInFile","`%s'",image->filename);
   offset=SeekBlob(image,ept_info.postscript_offset,SEEK_SET);
   if (offset < 0)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   count=ReadBlob(image,ept_info.postscript_length,ept_info.postscript);
   if (count != (ssize_t) (ept_info.postscript_length))
-    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-  CloseBlob(image);
+    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageWarning,
+      "InsufficientImageDataInFile","`%s'",image->filename);
+  (void) CloseBlob(image);
   image=DestroyImage(image);
   read_info=CloneImageInfo(image_info);
   (void) CopyMagickString(read_info->magick,"EPS",MaxTextExtent);
@@ -259,10 +267,10 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterEPTImage method is:
 %
-%      unsigned long RegisterEPTImage(void)
+%      size_t RegisterEPTImage(void)
 %
 */
-ModuleExport unsigned long RegisterEPTImage(void)
+ModuleExport size_t RegisterEPTImage(void)
 {
   MagickInfo
     *entry;
@@ -271,6 +279,7 @@ ModuleExport unsigned long RegisterEPTImage(void)
   entry->decoder=(DecodeImageHandler *) ReadEPTImage;
   entry->encoder=(EncodeImageHandler *) WriteEPTImage;
   entry->magick=(IsImageFormatHandler *) IsEPT;
+  entry->seekable_stream=MagickTrue;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->description=ConstantString(
@@ -282,6 +291,7 @@ ModuleExport unsigned long RegisterEPTImage(void)
   entry->encoder=(EncodeImageHandler *) WriteEPTImage;
   entry->magick=(IsImageFormatHandler *) IsEPT;
   entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->blob_support=MagickFalse;
   entry->description=ConstantString(
     "Encapsulated PostScript Level II with TIFF preview");
@@ -291,6 +301,7 @@ ModuleExport unsigned long RegisterEPTImage(void)
   entry->decoder=(DecodeImageHandler *) ReadEPTImage;
   entry->encoder=(EncodeImageHandler *) WriteEPTImage;
   entry->magick=(IsImageFormatHandler *) IsEPT;
+  entry->seekable_stream=MagickTrue;
   entry->blob_support=MagickFalse;
   entry->description=ConstantString(
     "Encapsulated PostScript Level III with TIFF preview");
@@ -343,13 +354,16 @@ ModuleExport void UnregisterEPTImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
 */
 static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
 {
+  char
+     filename[MaxTextExtent];
+
   EPTInfo
     ept_info;
 
@@ -377,8 +391,6 @@ static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
   write_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (write_image == (Image *) NULL)
     return(MagickFalse);
-  DestroyBlob(write_image);
-  write_image->blob=CloneBlobInfo((BlobInfo *) NULL);
   write_info=CloneImageInfo(image_info);
   (void) CopyMagickString(write_info->magick,"EPS",MaxTextExtent);
   if (LocaleCompare(image_info->magick,"EPT2") == 0)
@@ -396,12 +408,11 @@ static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
   write_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (write_image == (Image *) NULL)
     return(MagickFalse);
-  DestroyBlob(write_image);
-  write_image->blob=CloneBlobInfo((BlobInfo *) NULL);
   write_info=CloneImageInfo(image_info);
   (void) CopyMagickString(write_info->magick,"TIFF",MaxTextExtent);
-  (void) FormatMagickString(write_info->filename,MaxTextExtent,"tiff:%.1024s",
+  (void) FormatMagickString(filename,MaxTextExtent,"tiff:%s",
     write_info->filename); 
+  (void) CopyMagickString(write_info->filename,filename,MaxTextExtent);
   (void) TransformImage(&write_image,(char *) NULL,"512x512>");
   if ((write_image->storage_class == DirectClass) ||
       (write_image->colors > 256))
@@ -417,7 +428,7 @@ static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
         MagickFalse ? MagickTrue : MagickFalse;
       (void) QuantizeImage(&quantize_info,write_image);
     }
-  write_image->compression=NoCompression;
+  write_info->compression=NoCompression;
   ept_info.tiff=(unsigned char *) ImageToBlob(write_info,write_image,
     &ept_info.tiff_length,&image->exception);
   write_image=DestroyImage(write_image);
@@ -431,13 +442,13 @@ static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
   /*
     Write EPT image.
   */
-  (void) WriteBlobLSBLong(image,ept_info.magick);
+  (void) WriteBlobLSBLong(image,(unsigned int) ept_info.magick);
   (void) WriteBlobLSBLong(image,30);
-  (void) WriteBlobLSBLong(image,(unsigned long) ept_info.postscript_length);
+  (void) WriteBlobLSBLong(image,(unsigned int) ept_info.postscript_length);
   (void) WriteBlobLSBLong(image,0);
   (void) WriteBlobLSBLong(image,0);
-  (void) WriteBlobLSBLong(image,(unsigned long) ept_info.postscript_length+30);
-  (void) WriteBlobLSBLong(image,(unsigned long) ept_info.tiff_length);
+  (void) WriteBlobLSBLong(image,(unsigned int) ept_info.postscript_length+30);
+  (void) WriteBlobLSBLong(image,(unsigned int) ept_info.tiff_length);
   (void) WriteBlobLSBShort(image,0xffff);
   (void) WriteBlob(image,ept_info.postscript_length,ept_info.postscript);
   (void) WriteBlob(image,ept_info.tiff_length,ept_info.tiff);
@@ -447,6 +458,6 @@ static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
   ept_info.postscript=(unsigned char *) RelinquishMagickMemory(
     ept_info.postscript);
   ept_info.tiff=(unsigned char *) RelinquishMagickMemory(ept_info.tiff);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
