@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,9 +40,10 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/property.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
@@ -53,15 +54,22 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/module.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
-#include "magick/quantum-private.h"
-#include "magick/quantum-private.h"
+#include "magick/pixel-private.h"
+#include "magick/pixel-private.h"
+#include "magick/property.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/module.h"
-#include "magick/module.h"
+
+/*
+  Forward declarations.
+*/
+#define FITSBlocksize  2880UL
 
 /*
   Forward declarations.
@@ -89,8 +97,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -99,9 +106,9 @@ static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 {
   if (length < 6)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"IT0",3) == 0)
+  if (LocaleNCompare((const char *) magick,"IT0",3) == 0)
     return(MagickTrue);
-  if (LocaleNCompare((char *) magick,"SIMPLE",6) == 0)
+  if (LocaleNCompare((const char *) magick,"SIMPLE",6) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -134,24 +141,24 @@ static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 %
 */
 
-static inline double GetFITSQuantum(Image *image,int bits_per_pixel)
+static inline double GetFITSPixel(Image *image,int bits_per_pixel)
 {
   switch (image->depth >> 3)
   {
     case 1:
       return((double) ReadBlobByte(image));
     case 2:
-      return((double) ReadBlobShort(image));
+      return((double) ((short) ReadBlobShort(image)));
     case 4:
     {
       if (bits_per_pixel > 0)
-        return((double) ReadBlobLong(image));
+        return((double) ((int) ReadBlobLong(image)));
       return((double) ReadBlobFloat(image));
     }
     case 8:
     {
       if (bits_per_pixel > 0)
-        return((double) ReadBlobLongLong(image));
+        return((double) ((MagickOffsetType) ReadBlobLongLong(image)));
     }
     default:
       break;
@@ -159,11 +166,11 @@ static inline double GetFITSQuantum(Image *image,int bits_per_pixel)
   return(ReadBlobDouble(image));
 }
 
-static void GetFITSQuantumExtrema(Image *image,const int bits_per_pixel,
+static void GetFITSPixelExtrema(Image *image,const int bits_per_pixel,
   double *minima,double *maxima)
 {
   double
-    quantum;
+    pixel;
 
   MagickOffsetType
     offset;
@@ -176,31 +183,31 @@ static void GetFITSQuantumExtrema(Image *image,const int bits_per_pixel,
 
   offset=TellBlob(image);
   number_pixels=(MagickSizeType) image->columns*image->rows;
-  *minima=GetFITSQuantum(image,bits_per_pixel);
+  *minima=GetFITSPixel(image,bits_per_pixel);
   *maxima=(*minima);
   for (i=1; i < (MagickOffsetType) number_pixels; i++)
   {
-    quantum=GetFITSQuantum(image,bits_per_pixel);
-    if (quantum < *minima)
-      *minima=quantum;
-    if (quantum > *maxima)
-      *maxima=quantum;
+    pixel=GetFITSPixel(image,bits_per_pixel);
+    if (pixel < *minima)
+      *minima=pixel;
+    if (pixel > *maxima)
+      *maxima=pixel;
   }
   (void) SeekBlob(image,offset,SEEK_SET);
 }
 
-static inline double GetFITSQuantumRange(const unsigned long depth)
+static inline double GetFITSPixelRange(const size_t depth)
 {
   return((double) ((MagickOffsetType) GetQuantumRange(depth)));
 }
 
-static void SetFITSUnsignedQuantum(const size_t length,
-  const unsigned long bits_per_pixel,unsigned char *pixels)
+static void SetFITSUnsignedPixels(const size_t length,
+  const size_t bits_per_pixel,unsigned char *pixels)
 {
-  register long
+  register ssize_t
     i;
 
-  for (i=0; i < (long) length; i++)
+  for (i=0; i < (ssize_t) length; i++)
   {
     *pixels^=0x80;
     pixels+=bits_per_pixel >> 3;
@@ -212,11 +219,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 {
   typedef struct _FITSInfo
   {
-    EndianType
-      endian;
+    MagickBooleanType
+      extend,
+      simple;
 
     int
-      simple,
       bits_per_pixel,
       columns,
       rows,
@@ -228,11 +235,20 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
       max_data,
       zero,
       scale;
+
+    EndianType
+      endian;
   } FITSInfo;
 
   char
-    keyword[MaxTextExtent],
-    value[MaxTextExtent];
+    *comment,
+    keyword[9],
+    property[MaxTextExtent],
+    value[73];
+
+  double
+    pixel,
+    scale;
 
   FITSInfo
     fits_info;
@@ -243,31 +259,23 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   int
     c;
 
-  long
-    scene,
-    y;
-
   MagickBooleanType
-    status,
-    value_expected;
+    status;
 
   MagickSizeType
     number_pixels;
 
-  QuantumInfo
-    *quantum_info;
+  register ssize_t
+    i,
+    x;
 
   register PixelPacket
     *q;
 
   ssize_t
-    count;
-
-  size_t
-    length;
-
-  unsigned char
-    *pixels;
+    count,
+    scene,
+    y;
 
   /*
     Open image file.
@@ -290,6 +298,8 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     Initialize image header.
   */
   (void) ResetMagickMemory(&fits_info,0,sizeof(fits_info));
+  fits_info.extend=MagickFalse;
+  fits_info.simple=MagickFalse;
   fits_info.bits_per_pixel=8;
   fits_info.columns=1;
   fits_info.rows=1;
@@ -303,161 +313,152 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   /*
     Decode image header.
   */
-  c=ReadBlobByte(image);
-  if (c == EOF)
-    {
-      image=DestroyImage(image);
-      return((Image *) NULL);
-    }
-  for ( ; c != EOF; )
+  for (comment=(char *) NULL; EOFBlob(image) == MagickFalse; )
   {
-    if (isalnum((int) ((unsigned char) c)) == 0)
-      c=ReadBlobByte(image);
-    else
-      {
-        register char
-          *p;
+    for ( ; EOFBlob(image) == MagickFalse; )
+    {
+      register char
+        *p;
 
-        /*
-          Determine a keyword and its value.
-        */
-        p=keyword;
-        do
-        {
-          if ((size_t) (p-keyword) < MaxTextExtent)
-            *p++=(char) c;
-          c=ReadBlobByte(image);
-        } while ((isalnum((int) ((unsigned char) c)) != MagickFalse) ||
-                 ((int) c == '_'));
-        *p='\0';
-        if (LocaleCompare(keyword,"END") == 0)
+      count=ReadBlob(image,8,(unsigned char *) keyword);
+      if (count != 8)
+        break;
+      for (i=0; i < 8; i++)
+      {
+        if (isspace((int) ((unsigned char) keyword[i])) != 0)
           break;
-        value_expected=MagickFalse;
-        while ((isspace((int) ((unsigned char) c)) != 0) || ((int) c == '='))
-        {
-          if ((int) c == '=')
-            value_expected=MagickTrue;
-          c=ReadBlobByte(image);
-        }
-        if (value_expected == MagickFalse)
-          continue;
-        p=value;
-        while (isalnum(c) || (c == '-') || (c == '+') || (c == '.'))
-        {
-          if ((size_t) (p-value) < MaxTextExtent)
-            *p++=c;
-          c=ReadBlobByte(image);
-        }
-        *p='\0';
-        /*
-          Assign a value to the specified keyword.
-        */
-        if (LocaleCompare(keyword,"SIMPLE") == 0)
-          fits_info.simple=(int) ((*value == 'T') || (*value == 't'));
-        if (LocaleCompare(keyword,"BITPIX") == 0)
-          fits_info.bits_per_pixel=atoi(value);
-        if (LocaleCompare(keyword,"NAXIS") == 0)
-          fits_info.number_axes=atoi(value);
-        if (LocaleCompare(keyword,"NAXIS1") == 0)
-          fits_info.columns=atoi(value);
-        if (LocaleCompare(keyword,"NAXIS2") == 0)
-          fits_info.rows=atoi(value);
-        if (LocaleCompare(keyword,"NAXIS3") == 0)
-          fits_info.number_planes=atoi(value);
-        if (LocaleCompare(keyword,"DATAMAX") == 0)
-          fits_info.max_data=atof(value);
-        if (LocaleCompare(keyword,"DATAMIN") == 0)
-          fits_info.min_data=atof(value);
-        if (LocaleCompare(keyword,"BZERO") == 0)
-          fits_info.zero=atof(value);
-        if (LocaleCompare(keyword,"BSCALE") == 0)
-          fits_info.scale=atof(value);
-        if (LocaleCompare(keyword,"XENDIAN") == 0)
-          {
-            if (LocaleCompare(keyword,"BIG") == 0)
-              fits_info.endian=MSBEndian;
-            else
-              fits_info.endian=LSBEndian;
-          }
-        (void) SetImageProperty(image,keyword,value);
+        keyword[i]=tolower((int) ((unsigned char) keyword[i]));
       }
-    while (((TellBlob(image) % 80) != 0) && (c != EOF))
+      keyword[i]='\0';
+      count=ReadBlob(image,72,(unsigned char *) value);
+      if (count != 72)
+        break;
+      value[72]='\0';
+      p=value;
+      if (*p == '=')
+        {
+          p+=2;
+          while (isspace((int) ((unsigned char) *p)) != 0)
+            p++;
+        }
+      if (LocaleCompare(keyword,"end") == 0)
+        break;
+      if (LocaleCompare(keyword,"extend") == 0)
+        fits_info.extend=(*p == 'T') || (*p == 't') ? MagickTrue : MagickFalse;
+      if (LocaleCompare(keyword,"simple") == 0)
+        fits_info.simple=(*p == 'T') || (*p == 't') ? MagickTrue : MagickFalse;
+      if (LocaleCompare(keyword,"bitpix") == 0)
+        fits_info.bits_per_pixel=StringToLong(p);
+      if (LocaleCompare(keyword,"naxis") == 0)
+        fits_info.number_axes=StringToLong(p);
+      if (LocaleCompare(keyword,"naxis1") == 0)
+        fits_info.columns=StringToLong(p);
+      if (LocaleCompare(keyword,"naxis2") == 0)
+        fits_info.rows=StringToLong(p);
+      if (LocaleCompare(keyword,"naxis3") == 0)
+        fits_info.number_planes=StringToLong(p);
+      if (LocaleCompare(keyword,"datamax") == 0)
+        fits_info.max_data=StringToDouble(p);
+      if (LocaleCompare(keyword,"datamin") == 0)
+        fits_info.min_data=StringToDouble(p);
+      if (LocaleCompare(keyword,"bzero") == 0)
+        fits_info.zero=StringToDouble(p);
+      if (LocaleCompare(keyword,"bscale") == 0)
+        fits_info.scale=StringToDouble(p);
+      if (LocaleCompare(keyword,"comment") == 0)
+        {
+          if (comment == (char *) NULL)
+            comment=ConstantString(p);
+          else
+            (void) ConcatenateString(&comment,p);
+        }
+      if (LocaleCompare(keyword,"xendian") == 0)
+        {
+          if (LocaleNCompare(p,"big",3) == 0)
+            fits_info.endian=MSBEndian;
+          else
+            fits_info.endian=LSBEndian;
+        }
+      (void) FormatMagickString(property,MaxTextExtent,"fits:%s",keyword);
+      (void) SetImageProperty(image,property,p);
+    }
+    c=0;
+    while (((TellBlob(image) % FITSBlocksize) != 0) && (c != EOF))
       c=ReadBlobByte(image);
-    c=ReadBlobByte(image);
+    if (fits_info.extend == MagickFalse)
+      break;
+    number_pixels=(MagickSizeType) fits_info.columns*fits_info.rows;
+    if ((fits_info.simple != MagickFalse) && (fits_info.number_axes >= 1) &&
+        (fits_info.number_axes <= 4) && (number_pixels != 0))
+      break;
   }
-  while (((TellBlob(image) % 2880) != 0) && (c != EOF))
-    c=ReadBlobByte(image);
   /*
     Verify that required image information is defined.
   */
+  if (comment != (char *) NULL)
+    {
+      (void) SetImageProperty(image,"comment",comment);
+      comment=DestroyString(comment);
+    }
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   number_pixels=(MagickSizeType) fits_info.columns*fits_info.rows;
-  if ((fits_info.simple == 0) || (fits_info.number_axes < 1) ||
+  if ((fits_info.simple == MagickFalse) || (fits_info.number_axes < 1) ||
       (fits_info.number_axes > 4) || (number_pixels == 0))
     ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
-  for (scene=0; scene < (long) fits_info.number_planes; scene++)
+  for (scene=0; scene < (ssize_t) fits_info.number_planes; scene++)
   {
-    image->columns=(unsigned long) fits_info.columns;
-    image->rows=(unsigned long) fits_info.rows;
-    image->depth=(unsigned long) (fits_info.bits_per_pixel < 0 ? -1 : 1)*
+    image->columns=(size_t) fits_info.columns;
+    image->rows=(size_t) fits_info.rows;
+    image->depth=(size_t) (fits_info.bits_per_pixel < 0 ? -1 : 1)*
       fits_info.bits_per_pixel;
     image->endian=fits_info.endian;
-    image->scene=(unsigned long) scene;
+    image->scene=(size_t) scene;
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
       Initialize image structure.
     */
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
     if ((fits_info.min_data != 0.0) || (fits_info.max_data != 0.0))
       {
         if ((fits_info.bits_per_pixel != 0) && (fits_info.max_data == 0.0))
-          fits_info.max_data=GetFITSQuantumRange((unsigned long)
+          fits_info.max_data=GetFITSPixelRange((size_t)
             fits_info.bits_per_pixel);
       }
     else
-      GetFITSQuantumExtrema(image,fits_info.bits_per_pixel,&fits_info.min_data,
+      GetFITSPixelExtrema(image,fits_info.bits_per_pixel,&fits_info.min_data,
         &fits_info.max_data);
     /*
       Convert FITS pixels to pixel packets.
     */
-    quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
-    if (fits_info.bits_per_pixel < 0)
-      quantum_info->format=FloatingPointQuantumFormat;
-    quantum_info->scale=QuantumRange*fits_info.scale;
-    quantum_info->minimum=fits_info.min_data;
-    quantum_info->maximum=fits_info.max_data;
-    pixels=GetQuantumPixels(quantum_info);
-    length=GetQuantumExtent(image,quantum_info,GrayQuantum);
-    for (y=(long) image->rows-1; y >= 0; y--)
+    scale=(double) QuantumRange/(fits_info.scale*(fits_info.max_data-
+      fits_info.min_data)+fits_info.zero);
+    for (y=(ssize_t) image->rows-1; y >= 0; y--)
     {
-      q=SetImagePixels(image,0,y,image->columns,1);
+      q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      count=ReadBlob(image,length,pixels);
-      if ((size_t) count != length)
-        break;
-      if (fits_info.bits_per_pixel == 16)
-        SetFITSUnsignedQuantum(image->columns,image->depth,pixels);
-      if (fits_info.bits_per_pixel == 32)
-        SetFITSUnsignedQuantum(image->columns,image->depth,pixels);
-      length=ImportQuantumPixels(image,quantum_info,GrayQuantum,pixels);
-      if (count != (ssize_t) length)
-        ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-      if (SyncImagePixels(image) == MagickFalse)
+      for (x=0; x < (ssize_t) image->columns; x++)
+      {
+        pixel=GetFITSPixel(image,fits_info.bits_per_pixel);
+        SetRedPixelComponent(q,ClampToQuantum(scale*(fits_info.scale*(pixel-
+          fits_info.min_data)+fits_info.zero)));
+        SetGreenPixelComponent(q,GetRedPixelComponent(q));
+        SetBluePixelComponent(q,GetRedPixelComponent(q));
+        q++;
+      }
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
           if (status == MagickFalse)
             break;
         }
     }
-    quantum_info=DestroyQuantumInfo(quantum_info);
     if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
@@ -470,7 +471,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (scene < (long) (fits_info.number_planes-1))
+    if (scene < (ssize_t) (fits_info.number_planes-1))
       {
         /*
           Allocate next image structure.
@@ -512,10 +513,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterFITSImage method is:
 %
-%      unsigned long RegisterFITSImage(void)
+%      size_t RegisterFITSImage(void)
 %
 */
-ModuleExport unsigned long RegisterFITSImage(void)
+ModuleExport size_t RegisterFITSImage(void)
 {
   MagickInfo
     *entry;
@@ -596,12 +597,9 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   Image *image)
 {
   char
-    header[MaxTextExtent],
+    header[FITSBlocksize],
     *fits_info;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -616,7 +614,8 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
 
   ssize_t
     count,
-    offset;
+    offset,
+    y;
 
   unsigned char
     *pixels;
@@ -634,14 +633,14 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Allocate image memory.
   */
-  fits_info=(char *) AcquireQuantumMemory(2880UL,sizeof(*fits_info));
+  fits_info=(char *) AcquireQuantumMemory(FITSBlocksize,sizeof(*fits_info));
   if (fits_info == (char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(fits_info,' ',2880UL*sizeof(*fits_info));
+  (void) ResetMagickMemory(fits_info,' ',FITSBlocksize*sizeof(*fits_info));
   /*
     Initialize image header.
   */
@@ -650,79 +649,158 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   if (quantum_info == (QuantumInfo *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   offset=0;
-  (void) FormatMagickString(header,MaxTextExtent,
+  (void) FormatMagickString(header,FITSBlocksize,
     "SIMPLE  =                    T");
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"BITPIX  =           %10ld",
-    (quantum_info->format == FloatingPointQuantumFormat ? -1 : 1)*image->depth);
+  (void) FormatMagickString(header,FITSBlocksize,"BITPIX  =           %10ld",
+    (long) (quantum_info->format == FloatingPointQuantumFormat ? -1 : 1)*
+    image->depth);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"NAXIS   =           %10lu",
-    2UL);
+  (void) FormatMagickString(header,FITSBlocksize,"NAXIS   =           %10lu",
+    IsGrayImage(image,&image->exception) != MagickFalse ? 2UL : 3UL);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"NAXIS1  =           %10lu",
-    image->columns);
+  (void) FormatMagickString(header,FITSBlocksize,"NAXIS1  =           %10lu",
+    (unsigned long) image->columns);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"NAXIS2  =           %10lu",
-    image->rows);
+  (void) FormatMagickString(header,FITSBlocksize,"NAXIS2  =           %10lu",
+    (unsigned long) image->rows);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"BSCALE  =         %E",1.0);
+  if (IsGrayImage(image,&image->exception) == MagickFalse)
+    {
+      (void) FormatMagickString(header,FITSBlocksize,
+        "NAXIS3  =           %10lu",3UL);
+      (void) strncpy(fits_info+offset,header,strlen(header));
+      offset+=80;
+    }
+  (void) FormatMagickString(header,FITSBlocksize,"BSCALE  =         %E",1.0);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"BZERO   =         %E",
-    image->depth > 8 ? GetFITSQuantumRange(image->depth) : 0.0);
+  (void) FormatMagickString(header,FITSBlocksize,"BZERO   =         %E",
+    image->depth > 8 ? GetFITSPixelRange(image->depth) : 0.0);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"DATAMAX =         %E",
+  (void) FormatMagickString(header,FITSBlocksize,"DATAMAX =         %E",
     1.0*((MagickOffsetType) GetQuantumRange(image->depth)));
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) FormatMagickString(header,MaxTextExtent,"DATAMIN =         %E",0.0);
+  (void) FormatMagickString(header,FITSBlocksize,"DATAMIN =         %E",0.0);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
   if (image->endian == LSBEndian)
     {
-      (void) FormatMagickString(header,MaxTextExtent,"XENDIAN = 'SMALL'");
+      (void) FormatMagickString(header,FITSBlocksize,"XENDIAN = 'SMALL'");
       (void) strncpy(fits_info+offset,header,strlen(header));
       offset+=80;
     }
-  (void) FormatMagickString(header,MaxTextExtent,"HISTORY %.72s",
-    GetMagickVersion((unsigned long *) NULL));
+  (void) FormatMagickString(header,FITSBlocksize,"HISTORY %.72s",
+    GetMagickVersion((size_t *) NULL));
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) strncpy(header,"END",MaxTextExtent);
+  (void) strncpy(header,"END",FITSBlocksize);
   (void) strncpy(fits_info+offset,header,strlen(header));
   offset+=80;
-  (void) WriteBlob(image,2880,(unsigned char *) fits_info);
+  (void) WriteBlob(image,FITSBlocksize,(unsigned char *) fits_info);
   /*
     Convert image to fits scale PseudoColor class.
   */
   pixels=GetQuantumPixels(quantum_info);
-  length=GetQuantumExtent(image,quantum_info,GrayQuantum);
-  for (y=(long) image->rows-1; y >= 0; y--)
-  {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
-      break;
-    length=ExportQuantumPixels(image,quantum_info,GrayQuantum,pixels);
-    if (image->depth == 16)
-      SetFITSUnsignedQuantum(image->columns,image->depth,pixels);
-    if ((image->depth == 32) &&
-        (quantum_info->format != FloatingPointQuantumFormat))
-      SetFITSUnsignedQuantum(image->columns,image->depth,pixels);
-    count=WriteBlob(image,length,pixels);
-    if (count != (ssize_t) length)
-      break;
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
-    if (status == MagickFalse)
-      break;
-  }
+  if (IsGrayImage(image,&image->exception) != MagickFalse)
+    {
+      length=GetQuantumExtent(image,quantum_info,GrayQuantum);
+      for (y=(ssize_t) image->rows-1; y >= 0; y--)
+      {
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+        if (p == (const PixelPacket *) NULL)
+          break;
+        length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+          GrayQuantum,pixels,&image->exception);
+        if (image->depth == 16)
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        if (((image->depth == 32) || (image->depth == 64)) &&
+            (quantum_info->format != FloatingPointQuantumFormat))
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        count=WriteBlob(image,length,pixels);
+        if (count != (ssize_t) length)
+          break;
+        status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+          image->rows);
+        if (status == MagickFalse)
+          break;
+      }
+    }
+  else
+    {
+      length=GetQuantumExtent(image,quantum_info,RedQuantum);
+      for (y=(ssize_t) image->rows-1; y >= 0; y--)
+      {
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+        if (p == (const PixelPacket *) NULL)
+          break;
+        length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+          RedQuantum,pixels,&image->exception);
+        if (image->depth == 16)
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        if (((image->depth == 32) || (image->depth == 64)) &&
+            (quantum_info->format != FloatingPointQuantumFormat))
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        count=WriteBlob(image,length,pixels);
+        if (count != (ssize_t) length)
+          break;
+        status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+          image->rows);
+        if (status == MagickFalse)
+          break;
+      }
+      length=GetQuantumExtent(image,quantum_info,GreenQuantum);
+      for (y=(ssize_t) image->rows-1; y >= 0; y--)
+      {
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+        if (p == (const PixelPacket *) NULL)
+          break;
+        length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+          GreenQuantum,pixels,&image->exception);
+        if (image->depth == 16)
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        if (((image->depth == 32) || (image->depth == 64)) &&
+            (quantum_info->format != FloatingPointQuantumFormat))
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        count=WriteBlob(image,length,pixels);
+        if (count != (ssize_t) length)
+          break;
+        status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+          image->rows);
+        if (status == MagickFalse)
+          break;
+      }
+      length=GetQuantumExtent(image,quantum_info,BlueQuantum);
+      for (y=(ssize_t) image->rows-1; y >= 0; y--)
+      {
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+        if (p == (const PixelPacket *) NULL)
+          break;
+        length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+          BlueQuantum,pixels,&image->exception);
+        if (image->depth == 16)
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        if (((image->depth == 32) || (image->depth == 64)) &&
+            (quantum_info->format != FloatingPointQuantumFormat))
+          SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+        count=WriteBlob(image,length,pixels);
+        if (count != (ssize_t) length)
+          break;
+        status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+          image->rows);
+        if (status == MagickFalse)
+          break;
+      }
+    }
   quantum_info=DestroyQuantumInfo(quantum_info);
-  length=(size_t) (2880-TellBlob(image) % 2880);
+  length=(size_t) (FITSBlocksize-TellBlob(image) % FITSBlocksize);
   if (length != 0)
     {
       (void) ResetMagickMemory(fits_info,0,length*sizeof(*fits_info));
