@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,6 +43,7 @@
 #include "magick/property.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -63,7 +64,7 @@
 #include "magick/string_.h"
 #include "magick/module.h"
 #if defined(MAGICKCORE_FPX_DELEGATE)
-#if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
+#if !defined(vms) && !defined(macintosh) && !defined(MAGICKCORE_WINDOWS_SUPPORT)
 #include <fpxlib.h>
 #else
 #include "Fpxlib.h"
@@ -98,8 +99,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -169,16 +169,13 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     i,
     x;
 
@@ -194,6 +191,9 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     memory_limit;
 
+  ssize_t
+    y;
+
   unsigned char
     *pixels;
 
@@ -203,7 +203,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     tile_height,
     width;
 
-  unsigned long
+  size_t
     scene;
 
   /*
@@ -282,7 +282,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       fpx_status=FPX_GetImageResultAspectRatio(flashpix,&aspect_ratio);
       if (fpx_status != FPX_OK)
         ThrowReaderException(DelegateError,"UnableToReadAspectRatio");
-      if (width != (unsigned long) ((aspect_ratio*height)+0.5))
+      if (width != (size_t) floor((aspect_ratio*height)+0.5))
         Swap(width,height);
     }
   fpx_status=FPX_GetSummaryInformation(flashpix,&summary_info);
@@ -382,13 +382,6 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       FPX_ClearSystem();
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      (void) FPX_CloseImage(flashpix);
-      FPX_ClearSystem();
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   /*
     Allocate memory for the image and pixel buffer.
   */
@@ -428,12 +421,12 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image pixels.
   */
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetAuthenticIndexQueue(image);
     if ((y % tile_height) == 0)
       {
         /*
@@ -445,8 +438,8 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         else
           fpx_status=FPX_ReadImageTransformRectangle(flashpix,0.0F,
             (float) y/image->rows,(float) image->columns/image->rows,
-            (float) (y+tile_height-1)/image->rows,(long) image->columns,
-            (long) tile_height,&fpx_info);
+            (float) (y+tile_height-1)/image->rows,(ssize_t) image->columns,
+            (ssize_t) tile_height,&fpx_info);
         if (fpx_status == FPX_LOW_MEMORY_ERROR)
           {
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
@@ -462,33 +455,35 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     g=green_component->theData+(y % tile_height)*green_component->lineStride;
     b=blue_component->theData+(y % tile_height)*blue_component->lineStride;
     a=alpha_component->theData+(y % tile_height)*alpha_component->lineStride;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       if (fpx_info.numberOfComponents > 2)
         {
-          q->red=ScaleCharToQuantum(*r);
-          q->green=ScaleCharToQuantum(*g);
-          q->blue=ScaleCharToQuantum(*b);
+          SetRedPixelComponent(q,ScaleCharToQuantum(*r));
+          SetGreenPixelComponent(q,ScaleCharToQuantum(*g));
+          SetBluePixelComponent(q,ScaleCharToQuantum(*b));
         }
       else
         {
           index=ScaleCharToQuantum(*r);
-          indexes[x]=index;
-          q->red=index;
-          q->green=index;
-          q->blue=index;
+          SetIndexPixelComponent(indexes+x,index);
+          SetRedPixelComponent(q,index);
+          SetGreenPixelComponent(q,index);
+          SetBluePixelComponent(q,index);
         }
+      SetOpacityPixelComponent(q,OpaqueOpacity);
       if (image->matte != MagickFalse)
-        q->opacity=ScaleCharToQuantum(255-*a);
+        SetOpacityPixelComponent(q,ScaleCharToQuantum(255-*a));
       q++;
       r+=red_component->columnStride;
       g+=green_component->columnStride;
       b+=blue_component->columnStride;
       a+=alpha_component->columnStride;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
-    status=SetImageProgress(image,LoadImageTag,y,image->rows);
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
     if (status == MagickFalse)
       break;
   }
@@ -519,10 +514,10 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterFPXImage method is:
 %
-%      unsigned long RegisterFPXImage(void)
+%      size_t RegisterFPXImage(void)
 %
 */
-ModuleExport unsigned long RegisterFPXImage(void)
+ModuleExport size_t RegisterFPXImage(void)
 {
   MagickInfo
     *entry;
@@ -810,9 +805,6 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   FPXSummaryInformation
     summary_info;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -825,13 +817,16 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     i;
 
   size_t
     length,
     memory_limit;
 
+  ssize_t
+    y;
+
   unsigned char
     *pixels;
 
@@ -857,7 +852,7 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   */
   image->depth=8;
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
@@ -878,6 +873,8 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   background_color.color3_value=0;
   background_color.color4_value=0;
   compression=NONE;
+  if (image->compression == JPEGCompression)
+    compression=JPEG_UNSPECIFIED;
   if (image_info->compression == JPEGCompression)
     compression=JPEG_UNSPECIFIED;
   {
@@ -963,9 +960,11 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
     Initialize FlashPix image description.
   */
   quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   pixels=GetQuantumPixels(quantum_info);
   fpx_info.numberOfComponents=colorspace.numberOfComponents;
-  for (i=0; i < (long) fpx_info.numberOfComponents; i++)
+  for (i=0; i < (ssize_t) fpx_info.numberOfComponents; i++)
   {
     fpx_info.components[i].myColorType.myDataType=DATA_TYPE_UNSIGNED_BYTE;
     fpx_info.components[i].horzSubSampFactor=1;
@@ -988,16 +987,18 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
     quantum_type=RGBAQuantum;
   if (fpx_info.numberOfComponents == 1)
     quantum_type=GrayQuantum;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
+    length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+      quantum_type,pixels,&image->exception);
     fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
     if (fpx_status != FPX_OK)
       break;
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
     if (status == MagickFalse)
       break;
   }
