@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
@@ -80,7 +81,7 @@ static MagickBooleanType
 %
 %  ReadGRAYImage() reads an image of raw grayscale samples and returns
 %  it.  It allocates the memory necessary for the new Image structure and
-%   returns a pointer to the new image.
+%  returns a pointer to the new image.
 %
 %  The format of the ReadGRAYImage method is:
 %
@@ -101,34 +102,25 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     *canvas_image,
     *image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    scene;
+
   QuantumInfo
     *quantum_info;
 
   QuantumType
     quantum_type;
 
-  register const PixelPacket
-    *p;
-
-  register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
-
-  ssize_t
-    count;
-
   size_t
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
     *pixels;
 
@@ -151,21 +143,19 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  for (i=0; i < image->offset; i++)
-    if (ReadBlobByte(image) == EOF)
-      {
-        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
-          image->filename);
-        break;
-      }
+  if (DiscardBlobBytes(image,(size_t) image->offset) == MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   /*
     Create virtual canvas to support cropping (i.e. image.gray[100x100+10+20]).
   */
-  canvas_image=CloneImage(image,image->extract_info.width,1,MagickTrue,
+  canvas_image=CloneImage(image,image->extract_info.width,1,MagickFalse,
     exception);
   (void) SetImageVirtualPixelMethod(canvas_image,BlackVirtualPixelMethod);
   quantum_type=GrayQuantum;
   quantum_info=AcquireQuantumInfo(image_info,canvas_image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   pixels=GetQuantumPixels(quantum_info);
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
@@ -175,13 +165,16 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       */
       image->scene++;
       length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
         count=ReadBlob(image,length,pixels);
         if (count != (ssize_t) length)
           break;
       }
     }
+  scene=0;
+  count=0;
+  length=0;
   do
   {
     /*
@@ -190,47 +183,63 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if ((SetImageExtent(image,0,0) == MagickFalse) ||
-        (SetImageExtent(canvas_image,0,0) == MagickFalse))
-      break;
-    length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
-    count=ReadBlob(image,length,pixels);
-    if (count != (ssize_t) length)
-      break;
-    for (y=0; y < (long) image->extract_info.height; y++)
+    if (scene == 0)
+      {
+        length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
+        count=ReadBlob(image,length,pixels);
+      }
+    for (y=0; y < (ssize_t) image->extract_info.height; y++)
     {
-      q=GetImagePixels(canvas_image,0,0,canvas_image->columns,1);
+      register const PixelPacket
+        *restrict p;
+
+      register ssize_t
+        x;
+
+      register PixelPacket
+        *restrict q;
+
+      if (count != (ssize_t) length)
+        {
+          ThrowFileException(exception,CorruptImageError,
+            "UnexpectedEndOfFile",image->filename);
+          break;
+        }
+      q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      length=ImportQuantumPixels(canvas_image,quantum_info,quantum_type,pixels);
-      if (SyncImagePixels(canvas_image) == MagickFalse)
+      length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,quantum_info,
+        quantum_type,pixels,exception);
+      if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
         break;
-      count=ReadBlob(image,length,pixels);
       if (((y-image->extract_info.y) >= 0) && 
-          ((y-image->extract_info.y) < (long) image->rows))
+          ((y-image->extract_info.y) < (ssize_t) image->rows))
         {
-          p=AcquireImagePixels(canvas_image,canvas_image->extract_info.x,0,
+          p=GetVirtualPixels(canvas_image,canvas_image->extract_info.x,0,
             image->columns,1,exception);
-          q=SetImagePixels(image,0,y-image->extract_info.y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y-image->extract_info.y,image->columns,
+            1,exception);
           if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=p->red;
-            q->green=p->green;
-            q->blue=p->blue;
+            SetRedPixelComponent(q,GetRedPixelComponent(p));
+            SetGreenPixelComponent(q,GetGreenPixelComponent(p));
+            SetBluePixelComponent(q,GetBluePixelComponent(p));
             p++;
             q++;
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
         }
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
+      count=ReadBlob(image,length,pixels);
     }
     SetQuantumImageType(image,quantum_type);
     /*
@@ -256,9 +265,10 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
         if (status == MagickFalse)
           break;
       }
+    scene++;
   } while (count == (ssize_t) length);
   quantum_info=DestroyQuantumInfo(quantum_info);
-  InheritException(exception,&canvas_image->exception);
+  InheritException(&image->exception,&canvas_image->exception);
   canvas_image=DestroyImage(canvas_image);
   (void) CloseBlob(image);
   return(GetFirstImageInList(image));
@@ -284,10 +294,10 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterGRAYImage method is:
 %
-%      unsigned long RegisterGRAYImage(void)
+%      size_t RegisterGRAYImage(void)
 %
 */
-ModuleExport unsigned long RegisterGRAYImage(void)
+ModuleExport size_t RegisterGRAYImage(void)
 {
   MagickInfo
     *entry;
@@ -297,7 +307,6 @@ ModuleExport unsigned long RegisterGRAYImage(void)
   entry->encoder=(EncodeImageHandler *) WriteGRAYImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw gray samples");
   entry->module=ConstantString("GRAY");
   (void) RegisterMagickInfo(entry);
@@ -357,9 +366,6 @@ ModuleExport void UnregisterGRAYImage(void)
 static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
   Image *image)
 {
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -372,15 +378,13 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register const PixelPacket
-    *p;
-
-  ssize_t
-    count;
-
   size_t
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
     *pixels;
 
@@ -403,22 +407,29 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
       Write grayscale pixels.
     */
     if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
+      (void) TransformImageColorspace(image,RGBColorspace);
     quantum_type=GrayQuantum;
     quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=GetQuantumPixels(quantum_info);
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+      register const PixelPacket
+        *restrict p;
+
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
+      length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+        quantum_type,pixels,&image->exception);
       count=WriteBlob(image,length,pixels);
       if (count != (ssize_t) length)
         break;
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
