@@ -10,39 +10,27 @@
 %                        H   H    T    M   M  LLLLL                           %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                  Write A Client-Side Image Map Using                        %
+%                 Image Montage & Directory Information.                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,13 +40,30 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/color-private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/geometry.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/paint.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/module.h"
+#include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteHTMLImage(const ImageInfo *,Image *);
 
 /*
@@ -72,31 +77,27 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsHTML returns True if the image format type, identified by the
+%  IsHTML() returns MagickTrue if the image format type, identified by the
 %  magick string, is HTML.
 %
 %  The format of the IsHTML method is:
 %
-%      unsigned int IsHTML(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsHTML(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsHTML returns True if the image format type is HTML.
-%
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
-static unsigned int IsHTML(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsHTML(const unsigned char *magick,const size_t length)
 {
   if (length < 5)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"<html",5) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -110,8 +111,8 @@ static unsigned int IsHTML(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterHTMLImage adds attributes for the HTML image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterHTMLImage() adds properties for the HTML image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -119,37 +120,39 @@ static unsigned int IsHTML(const unsigned char *magick,const size_t length)
 %
 %  The format of the RegisterHTMLImage method is:
 %
-%      RegisterHTMLImage(void)
+%      size_t RegisterHTMLImage(void)
 %
 */
-ModuleExport void RegisterHTMLImage(void)
+ModuleExport size_t RegisterHTMLImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("HTM");
-  entry->encoder=WriteHTMLImage;
-  entry->magick=IsHTML;
-  entry->adjoin=False;
-  entry->description=
-    AcquireString("Hypertext Markup Language and a client-side image map");
-  entry->module=AcquireString("HTML");
+  entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
+  entry->magick=(IsImageFormatHandler *) IsHTML;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString(
+    "Hypertext Markup Language and a client-side image map");
+  entry->module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("HTML");
-  entry->encoder=WriteHTMLImage;
-  entry->magick=IsHTML;
-  entry->adjoin=False;
-  entry->description=
-    AcquireString("Hypertext Markup Language and a client-side image map");
+  entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
+  entry->magick=(IsImageFormatHandler *) IsHTML;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString(
+    "Hypertext Markup Language and a client-side image map");
+  entry->module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SHTML");
-  entry->encoder=WriteHTMLImage;
-  entry->magick=IsHTML;
-  entry->adjoin=False;
-  entry->description=
-    AcquireString("Hypertext Markup Language and a client-side image map");
-  entry->module=AcquireString("HTML");
+  entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
+  entry->magick=(IsImageFormatHandler *) IsHTML;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString(
+    "Hypertext Markup Language and a client-side image map");
+  entry->module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -163,7 +166,7 @@ ModuleExport void RegisterHTMLImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterHTMLImage removes format registrations made by the
+%  UnregisterHTMLImage() removes format registrations made by the
 %  HTML module from the list of supported formats.
 %
 %  The format of the UnregisterHTMLImage method is:
@@ -189,25 +192,22 @@ ModuleExport void UnregisterHTMLImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteHTMLImage writes an image in the HTML encoded image format.
+%  WriteHTMLImage() writes an image in the HTML encoded image format.
 %
 %  The format of the WriteHTMLImage method is:
 %
-%      unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteHTMLImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
+%    o image_info: the image info.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
-%
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
-static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     basename[MaxTextExtent],
@@ -220,7 +220,10 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
     *next;
 
   ImageInfo
-    *clone_info;
+    *write_info;
+
+  MagickBooleanType
+    status;
 
   RectangleInfo
     geometry;
@@ -228,9 +231,6 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
   register char
     *p;
 
-  unsigned int
-    status;
-
   /*
     Open image.
   */
@@ -238,11 +238,15 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  CloseBlob(image);
-  (void) TransformRGBImage(image,RGBColorspace);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  (void) CloseBlob(image);
+  if (image->colorspace != RGBColorspace)
+    (void) TransformImageColorspace(image,RGBColorspace);
   *url='\0';
   if ((LocaleCompare(image_info->magick,"FTP") == 0) ||
       (LocaleCompare(image_info->magick,"HTTP") == 0))
@@ -254,195 +258,194 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
       if (p)
         {
           p++;
-          (void) strncpy(url,image_info->magick,MaxTextExtent-1);
-          (void) strcat(url,":");
+          (void) CopyMagickString(url,image_info->magick,MaxTextExtent);
+          (void) ConcatenateMagickString(url,":",MaxTextExtent);
           url[strlen(url)+p-image->filename]='\0';
-          (void) strncat(url,image->filename,p-image->filename);
-          (void) strncpy(image->filename,p,MaxTextExtent-1);
+          (void) ConcatenateMagickString(url,image->filename,
+            p-image->filename+2);
+          (void) CopyMagickString(image->filename,p,MaxTextExtent);
         }
     }
   /*
     Refer to image map file.
   */
-  (void) strncpy(filename,image->filename,MaxTextExtent-1);
+  (void) CopyMagickString(filename,image->filename,MaxTextExtent);
   AppendImageFormat("map",filename);
   GetPathComponent(filename,BasePath,basename);
-  (void) strncpy(mapname,basename,MaxTextExtent-1);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-  (void) strncpy(filename,image->filename,MaxTextExtent-1);
-  clone_info=CloneImageInfo(image_info);
-  clone_info->adjoin=True;
-  status=True;
+  (void) CopyMagickString(mapname,basename,MaxTextExtent);
+  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
+  (void) CopyMagickString(filename,image->filename,MaxTextExtent);
+  write_info=CloneImageInfo(image_info);
+  write_info->adjoin=MagickTrue;
+  status=MagickTrue;
   if (LocaleCompare(image_info->magick,"SHTML") != 0)
     {
-      const ImageAttribute
-        *attribute;
+      const char
+        *value;
 
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-      if (status == False)
-        ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+      if (status == MagickFalse)
+        return(status);
       /*
         Write the HTML image file.
       */
-      (void) WriteBlobString(image,"<html version=\"2.0\">\n");
+      (void) WriteBlobString(image,"<?xml version=\"1.0\" "
+        "encoding=\"US-ASCII\"?>\n");
+      (void) WriteBlobString(image,"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML "
+        "1.0 Strict//EN\" "
+        "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
+      (void) WriteBlobString(image,"<html>\n");
       (void) WriteBlobString(image,"<head>\n");
-      attribute=GetImageAttribute(image,"label");
-      if (attribute != (const ImageAttribute *) NULL)
-        FormatString(buffer,"<title>%.1024s</title>\n",attribute->value);
+      value=GetImageProperty(image,"label");
+      if (value != (const char *) NULL)
+        (void) FormatMagickString(buffer,MaxTextExtent,"<title>%s</title>\n",
+          value);
       else
         {
           GetPathComponent(filename,BasePath,basename);
-          FormatString(buffer,"<title>%.1024s</title>\n",basename);
+          (void) FormatMagickString(buffer,MaxTextExtent,
+            "<title>%s</title>\n",basename);
         }
       (void) WriteBlobString(image,buffer);
       (void) WriteBlobString(image,"</head>\n");
-      (void) WriteBlobString(image,"<body>\n");
-      (void) WriteBlobString(image,"<center>\n");
-      FormatString(buffer,"<h1>%.1024s</h1>\n",image->filename);
+      (void) WriteBlobString(image,"<body style=\"text-align: center;\">\n");
+      (void) FormatMagickString(buffer,MaxTextExtent,"<h1>%s</h1>\n",
+        image->filename);
       (void) WriteBlobString(image,buffer);
-      (void) WriteBlobString(image,"<br><br>\n");
-      (void) strncpy(filename,image->filename,MaxTextExtent-1);
-      AppendImageFormat("gif",filename);
-      FormatString(buffer,
-        "<img ismap usemap=\"#%.1024s\" src=\"%.1024s\" border=0>\n",
-        mapname,filename);
+      (void) WriteBlobString(image,"<div>\n");
+      (void) CopyMagickString(filename,image->filename,MaxTextExtent);
+      AppendImageFormat("png",filename);
+      (void) FormatMagickString(buffer,MaxTextExtent,"<img usemap=\"#%s\" "
+        "src=\"%s\" style=\"border: 0;\" alt=\"Image map\" />\n",mapname,
+        filename);
       (void) WriteBlobString(image,buffer);
       /*
         Determine the size and location of each image tile.
       */
       SetGeometry(image,&geometry);
       if (image->montage != (char *) NULL)
-        {
-          (void) GetGeometry(image->montage,&geometry.x,&geometry.y,
-            &geometry.width,&geometry.height);
-          (void) GetMagickGeometry(image->montage,&geometry.x,&geometry.y,
-            &geometry.width,&geometry.height);
-        }
+        (void) ParseAbsoluteGeometry(image->montage,&geometry);
       /*
         Write an image map.
       */
-      FormatString(buffer,"<map name=\"%.1024s\">\n",mapname);
+      (void) FormatMagickString(buffer,MaxTextExtent,
+        "<map id=\"%s\" name=\"%s\">\n",mapname,mapname);
       (void) WriteBlobString(image,buffer);
-      FormatString(buffer,"  <area href=\"%.1024s",url);
+      (void) FormatMagickString(buffer,MaxTextExtent,"  <area href=\"%s",
+        url);
       (void) WriteBlobString(image,buffer);
       if (image->directory == (char *) NULL)
         {
-          FormatString(buffer,"%.1024s\" shape=rect coords=\"0,0,%lu,%lu\">\n",
-            image->filename,geometry.width-1,geometry.height-1);
+          (void) FormatMagickString(buffer,MaxTextExtent,
+            "%s\" shape=\"rect\" coords=\"0,0,%.20g,%.20g\" alt=\"\" />\n",
+            image->filename,(double) geometry.width-1,(double) geometry.height-
+            1);
           (void) WriteBlobString(image,buffer);
         }
       else
         for (p=image->directory; *p != '\0'; p++)
           if (*p != '\n')
-            (void) WriteBlobByte(image,*p);
+            (void) WriteBlobByte(image,(unsigned char) *p);
           else
             {
-              FormatString(buffer,"\" shape=rect coords=\"%ld,%ld,%ld,%ld\">\n",
-                geometry.x,geometry.y,geometry.x+(long) geometry.width-1,
-                geometry.y+(long) geometry.height-1);
+              (void) FormatMagickString(buffer,MaxTextExtent,"\" shape="
+                "\"rect\" coords=\"%.20g,%.20g,%.20g,%.20g\" alt=\"\" />\n",
+                (double) geometry.x,(double) geometry.y,(double) (geometry.x+
+                geometry.width-1),(double) (geometry.y+geometry.height-1));
               (void) WriteBlobString(image,buffer);
               if (*(p+1) != '\0')
                 {
-                  FormatString(buffer,"  <area href=%.1024s\"",url);
+                  (void) FormatMagickString(buffer,MaxTextExtent,
+                    "  <area href=%s\"",url);
                   (void) WriteBlobString(image,buffer);
                 }
-              geometry.x+=geometry.width;
-              if (geometry.x >= (long) image->columns)
+              geometry.x+=(ssize_t) geometry.width;
+              if ((geometry.x+4) >= (ssize_t) image->columns)
                 {
                   geometry.x=0;
-                  geometry.y+=geometry.height;
+                  geometry.y+=(ssize_t) geometry.height;
                 }
             }
       (void) WriteBlobString(image,"</map>\n");
-      if (image->montage != (char *) NULL)
-        (void) TransparentImage(image,GetOnePixel(image,0,0),
-          TransparentOpacity);
-      (void) strncpy(filename,image->filename,MaxTextExtent-1);
-      (void) WriteBlobString(image,"</center>\n");
+      (void) CopyMagickString(filename,image->filename,MaxTextExtent);
+      (void) WriteBlobString(image,"</div>\n");
       (void) WriteBlobString(image,"</body>\n");
       (void) WriteBlobString(image,"</html>\n");
-      CloseBlob(image);
+      (void) CloseBlob(image);
       /*
-        Write the image as transparent GIF.
+        Write the image as PNG.
       */
-      (void) strncpy(image->filename,filename,MaxTextExtent-1);
-      AppendImageFormat("gif",image->filename);
-      next=image->next;
-      image->next=(Image *) NULL;
-      (void) strcpy(image->magick,"GIF");
-      (void) WriteImage(clone_info,image);
+      (void) CopyMagickString(image->filename,filename,MaxTextExtent);
+      AppendImageFormat("png",image->filename);
+      next=GetNextImageInList(image);
+      image->next=NewImageList();
+      (void) CopyMagickString(image->magick,"PNG",MaxTextExtent);
+      (void) WriteImage(write_info,image);
       image->next=next;
       /*
         Determine image map filename.
       */
-      (void) strncpy(image->filename,filename,MaxTextExtent-1);
-      for (p=filename+strlen(filename)-1; p > (filename+1); p--)
-        if (*p == '.')
-          {
-            (void) strncpy(image->filename,filename,p-filename);
-            image->filename[p-filename]='\0';
-            break;
-          }
-      (void) strcat(image->filename,"_map.shtml");
+      GetPathComponent(image->filename,BasePath,filename);
+      (void) ConcatenateMagickString(filename,"_map.shtml",MaxTextExtent);
+      (void) CopyMagickString(image->filename,filename,MaxTextExtent);
     }
   /*
     Open image map.
   */
-  status=OpenBlob(clone_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  DestroyImageInfo(clone_info);
+  status=OpenBlob(write_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  write_info=DestroyImageInfo(write_info);
   /*
     Determine the size and location of each image tile.
   */
   SetGeometry(image,&geometry);
   if (image->montage != (char *) NULL)
-    {
-      (void) GetGeometry(image->montage,&geometry.x,&geometry.y,
-        &geometry.width,&geometry.height);
-      (void) GetMagickGeometry(image->montage,&geometry.x,&geometry.y,
-        &geometry.width,&geometry.height);
-    }
+    (void) ParseAbsoluteGeometry(image->montage,&geometry);
   /*
     Write an image map.
   */
-  FormatString(buffer,"<map name=\"%.1024s\">\n",mapname);
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "<map id=\"%s\" name=\"%s\">\n",mapname,mapname);
   (void) WriteBlobString(image,buffer);
-  FormatString(buffer,"  <area href=\"%.1024s",url);
+  (void) FormatMagickString(buffer,MaxTextExtent,"  <area href=\"%s",url);
   (void) WriteBlobString(image,buffer);
   if (image->directory == (char *) NULL)
     {
-      FormatString(buffer,"%.1024s\" shape=rect coords=\"0,0,%lu,%lu\">\n",
-        image->filename,geometry.width-1,geometry.height-1);
+      (void) FormatMagickString(buffer,MaxTextExtent,
+        "%s\" shape=\"rect\" coords=\"0,0,%.20g,%.20g\" alt=\"\" />\n",
+        image->filename,(double) geometry.width-1,(double) geometry.height-1);
       (void) WriteBlobString(image,buffer);
     }
   else
     for (p=image->directory; *p != '\0'; p++)
       if (*p != '\n')
-        (void) WriteBlobByte(image,*p);
+        (void) WriteBlobByte(image,(unsigned char) *p);
       else
         {
-          FormatString(buffer,"\" shape=rect coords=\"%ld,%ld,%ld,%ld\">\n",
-            geometry.x,geometry.y,geometry.x+(long) geometry.width-1,
-            geometry.y+(long) geometry.height-1);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\" shape=\"rect\""
+            " coords=\"%.20g,%.20g,%.20g,%.20g\" alt=\"\" />\n",
+            (double) geometry.x,(double) geometry.y,geometry.x+(double)
+            geometry.width-1,geometry.y+(double) geometry.height-1);
           (void) WriteBlobString(image,buffer);
           if (*(p+1) != '\0')
             {
-              FormatString(buffer,"  <area href=%.1024s\"",url);
+              (void) FormatMagickString(buffer,MaxTextExtent,
+                "  <area href=%s\"",url);
               (void) WriteBlobString(image,buffer);
             }
-          geometry.x+=geometry.width;
-          if (geometry.x >= (long) image->columns)
+          geometry.x+=(ssize_t) geometry.width;
+          if ((geometry.x+4) >= (ssize_t) image->columns)
             {
               geometry.x=0;
-              geometry.y+=geometry.height;
+              geometry.y+=(ssize_t) geometry.height;
             }
         }
   (void) WriteBlobString(image,"</map>\n");
-  CloseBlob(image);
-  (void) strncpy(image->filename,filename,MaxTextExtent-1);
+  (void) CloseBlob(image);
+  (void) CopyMagickString(image->filename,filename,MaxTextExtent);
   return(status);
 }
