@@ -10,39 +10,26 @@
 %                        JJJ    BBBB   IIIII   GGG                            %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                       Read/Write JBIG Image Format                          %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,17 +39,42 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/color-private.h"
+#include "magick/colormap.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/nt-feature.h"
+#include "magick/quantum-private.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/module.h"
+#if defined(MAGICKCORE_JBIG_DELEGATE)
+#include "jbig.h"
+#endif
 
 /*
   Forward declarations.
 */
-static unsigned int
+#if defined(MAGICKCORE_JBIG_DELEGATE)
+static MagickBooleanType
   WriteJBIGImage(const ImageInfo *,Image *);
+#endif
 
-#if defined(HasJBIG)
-#include "jbig.h"
+#if defined(MAGICKCORE_JBIG_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -74,7 +86,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadJBIGImage reads a JBIG image file and returns it.  It
+%  ReadJBIGImage() reads a JBIG image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -85,38 +97,27 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadJBIGImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadJBIGImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define MaxBufferSize  8192
-
   Image
     *image;
 
   IndexPacket
     index;
 
-  int
+  MagickBooleanType
     status;
 
-  long
-    length,
-    y;
-
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
@@ -125,17 +126,17 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   register unsigned char
     *p;
 
-  size_t
-    count;
+  ssize_t
+    count,
+    length,
+    y;
 
   struct jbg_dec_state
     jbig_info;
 
   unsigned char
     bit,
-    *buffer;
-
-  unsigned int
+    *buffer,
     byte;
 
   /*
@@ -143,42 +144,52 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+  image=AcquireImage(image_info);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize JBIG toolkit.
   */
   jbg_dec_init(&jbig_info);
-  jbg_dec_maxsize(&jbig_info,(unsigned long) image->columns,
-    (unsigned long) image->rows);
-  image->columns= jbg_dec_getwidth(&jbig_info);
-  image->rows= jbg_dec_getheight(&jbig_info);
+  jbg_dec_maxsize(&jbig_info,(unsigned long) image->columns,(unsigned long)
+    image->rows);
+  image->columns=jbg_dec_getwidth(&jbig_info);
+  image->rows=jbg_dec_getheight(&jbig_info);
   image->depth=8;
   image->storage_class=PseudoClass;
   image->colors=2;
   /*
     Read JBIG file.
   */
-  buffer=(unsigned char *) AcquireMemory(MaxBufferSize);
+  buffer=(unsigned char *) AcquireQuantumMemory(MagickMaxBufferExtent,
+    sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   status=JBG_EAGAIN;
   do
   {
-    length=(long) ReadBlob(image,MaxBufferSize,(char *) buffer);
+    length=(ssize_t) ReadBlob(image,MagickMaxBufferExtent,buffer);
     if (length == 0)
       break;
     p=buffer;
     count=0;
     while ((length > 0) && ((status == JBG_EAGAIN) || (status == JBG_EOK)))
     {
+      size_t
+        count;
+
       status=jbg_dec_in(&jbig_info,p,length,&count);
       p+=count;
-      length-=count;
+      length-=(ssize_t) count;
     }
   } while ((status == JBG_EAGAIN) || (status == JBG_EOK));
   /*
@@ -186,38 +197,38 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   */
   image->columns=jbg_dec_getwidth(&jbig_info);
   image->rows=jbg_dec_getheight(&jbig_info);
-  if (!AllocateImageColormap(image,2))
+  image->compression=JBIG2Compression;
+  if (AcquireImageColormap(image,2) == MagickFalse)
     {
-      LiberateMemory((void **) &buffer);
-      ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-        image)
+      buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   image->colormap[0].red=0;
   image->colormap[0].green=0;
   image->colormap[0].blue=0;
-  image->colormap[1].red=MaxRGB;
-  image->colormap[1].green=MaxRGB;
-  image->colormap[1].blue=MaxRGB;
+  image->colormap[1].red=QuantumRange;
+  image->colormap[1].green=QuantumRange;
+  image->colormap[1].blue=QuantumRange;
   image->x_resolution=300;
   image->y_resolution=300;
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
-      return(image);
+      (void) CloseBlob(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Convert X bitmap image to pixel packets.
   */
   p=jbg_dec_getimage(&jbig_info,0);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetAuthenticIndexQueue(image);
     bit=0;
     byte=0;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       if (bit == 0)
         byte=(*p++);
@@ -226,29 +237,26 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
       byte<<=1;
       if (bit == 8)
         bit=0;
-      indexes[x]=index;
-      *q++=image->colormap[index];
+      SetIndexPixelComponent(indexes+x,index);
+      SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+      SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+      SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+      q++;
     }
-    if (!SyncImagePixels(image))
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
   }
   /*
     Free scale resource.
   */
   jbg_dec_free(&jbig_info);
-  LiberateMemory((void **) &buffer);
-  CloseBlob(image);
-  return(image);
-}
-#else
-static Image *ReadJBIGImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
-{
-  ThrowException(exception,MissingDelegateWarning,
-    "JBIG library is not available",image_info->filename);
-  return((Image *) NULL);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  (void) CloseBlob(image);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -263,7 +271,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterJBIGImage adds attributes for the JBIG image format to
+%  RegisterJBIGImage() adds attributes for the JBIG image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -272,48 +280,55 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterJBIGImage method is:
 %
-%      RegisterJBIGImage(void)
+%      size_t RegisterJBIGImage(void)
 %
 */
-ModuleExport void RegisterJBIGImage(void)
+ModuleExport size_t RegisterJBIGImage(void)
 {
 #define JBIGDescription  "Joint Bi-level Image experts Group interchange format"
 
   char
-	  version[MaxTextExtent];
+    version[MaxTextExtent];
 
   MagickInfo
     *entry;
 
   *version='\0';
 #if defined(JBG_VERSION)
-  (void) strncpy(version,JBG_VERSION,MaxTextExtent-1);
+  (void) CopyMagickString(version,JBG_VERSION,MaxTextExtent);
 #endif
   entry=SetMagickInfo("BIE");
-  entry->decoder=ReadJBIGImage;
-  entry->encoder=WriteJBIGImage;
-  entry->adjoin=False;
-  entry->description=AcquireString(JBIGDescription);
+#if defined(MAGICKCORE_JBIG_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadJBIGImage;
+  entry->encoder=(EncodeImageHandler *) WriteJBIGImage;
+#endif
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBG");
-  entry->decoder=ReadJBIGImage;
-  entry->encoder=WriteJBIGImage;
-  entry->description=AcquireString(JBIGDescription);
+#if defined(MAGICKCORE_JBIG_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadJBIGImage;
+  entry->encoder=(EncodeImageHandler *) WriteJBIGImage;
+#endif
+  entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBIG");
-  entry->decoder=ReadJBIGImage;
-  entry->encoder=WriteJBIGImage;
-  entry->description=AcquireString(JBIGDescription);
+#if defined(MAGICKCORE_JBIG_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadJBIGImage;
+  entry->encoder=(EncodeImageHandler *) WriteJBIGImage;
+#endif
+  entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -327,7 +342,7 @@ ModuleExport void RegisterJBIGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterJBIGImage removes format registrations made by the
+%  UnregisterJBIGImage() removes format registrations made by the
 %  JBIG module from the list of supported formats.
 %
 %  The format of the UnregisterJBIGImage method is:
@@ -342,7 +357,7 @@ ModuleExport void UnregisterJBIGImage(void)
   (void) UnregisterMagickInfo("JBIG");
 }
 
-#if defined(HasJBIG)
+#if defined(MAGICKCORE_JBIG_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -354,21 +369,17 @@ ModuleExport void UnregisterJBIGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteJBIGImage writes an image in the JBIG encoded image format.
+%  WriteJBIGImage() writes an image in the JBIG encoded image format.
 %
 %  The format of the WriteJBIGImage method is:
 %
-%      unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteJBIGImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
+%    o image_info: the image info.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
-%
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
@@ -382,38 +393,43 @@ static void JBIGEncode(unsigned char *pixels,size_t length,void *data)
   (void) WriteBlob(image,length,pixels);
 }
 
-static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
+  Image *image)
 {
-  long
-    y;
+  double
+    version;
+
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
+  register const IndexPacket
+    *indexes;
 
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
-  register long
+  register ssize_t
     x;
 
   register unsigned char
     *q;
 
+  size_t
+    number_packets;
+
+  ssize_t
+    y;
+
   struct jbg_enc_state
     jbig_info;
 
   unsigned char
     bit,
-    *pixels,
-    polarity;
-
-  unsigned int
     byte,
-    scene,
-    status;
-
-  unsigned long
-    number_packets;
+    *pixels;
 
   /*
     Open image file.
@@ -422,43 +438,42 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  version=strtod(JBG_VERSION,(char **) NULL);
   scene=0;
   do
   {
     /*
       Allocate pixel data.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
-    number_packets=((image->columns+7) >> 3)*image->rows;
-    pixels=(unsigned char *) AcquireMemory(number_packets);
+    if (image->colorspace != RGBColorspace)
+      (void) TransformImageColorspace(image,RGBColorspace);
+    number_packets=(image->columns+7)/8;
+    pixels=(unsigned char *) AcquireQuantumMemory(number_packets,
+      image->rows*sizeof(*pixels));
     if (pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Convert pixels to a bitmap.
     */
-    if ((image->storage_class == DirectClass) ||
-        !IsMonochromeImage(image,&image->exception))
-      SetImageType(image,BilevelType);
-    polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
-    if (image->colors == 2)
-      polarity=Intensity(&image->colormap[0]) > Intensity(&image->colormap[1]);
+    (void) SetImageType(image,BilevelType);
     q=pixels;
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
+      indexes=GetVirtualIndexQueue(image);
       bit=0;
       byte=0;
-      for (x=0; x < (long) image->columns; x++)
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
         byte<<=1;
-        if (indexes[x] == polarity)
+        if (PixelIntensity(p) < (QuantumRange/2.0))
           byte|=0x01;
         bit++;
         if (bit == 8)
@@ -467,60 +482,78 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
             bit=0;
             byte=0;
           }
-       }
+        p++;
+      }
       if (bit != 0)
         *q++=byte << (8-bit);
-      if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+      if (image->previous == (Image *) NULL)
+        {
+          status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+            image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
     /*
       Initialize JBIG info structure.
     */
-    jbg_enc_init(&jbig_info,image->columns,image->rows,1,&pixels,
-      (void (*)(unsigned char *,size_t,void *)) JBIGEncode,image);
-    if (image_info->subimage != 0)
-      jbg_enc_layers(&jbig_info,(int) image_info->subimage);
+    jbg_enc_init(&jbig_info,(unsigned long) image->columns,(unsigned long)
+      image->rows,1,&pixels,(void (*)(unsigned char *,size_t,void *))
+      JBIGEncode,image);
+    if (image_info->scene != 0)
+      jbg_enc_layers(&jbig_info,(int) image_info->scene);
     else
       {
-        long
-          sans_offset;
-
-        unsigned long
+        size_t
           x_resolution,
           y_resolution;
 
+        ssize_t
+          sans_offset;
+
         x_resolution=640;
         y_resolution=480;
         sans_offset=0;
         if (image_info->density != (char *) NULL)
-          (void) GetGeometry(image_info->density,&sans_offset,&sans_offset,
-            &x_resolution,&y_resolution);
-        (void) jbg_enc_lrlmax(&jbig_info,x_resolution,y_resolution);
+          {
+            GeometryInfo
+              geometry_info;
+
+            MagickStatusType
+              flags;
+
+            flags=ParseGeometry(image_info->density,&geometry_info);
+            x_resolution=geometry_info.rho;
+            y_resolution=geometry_info.sigma;
+            if ((flags & SigmaValue) == 0)
+              y_resolution=x_resolution;
+          }
+        if (image->units == PixelsPerCentimeterResolution)
+          {
+            x_resolution=(size_t) (100.0*2.54*x_resolution+0.5)/100.0;
+            y_resolution=(size_t) (100.0*2.54*y_resolution+0.5)/100.0;
+          }
+        (void) jbg_enc_lrlmax(&jbig_info,(unsigned long) x_resolution,
+          (unsigned long) y_resolution);
       }
     (void) jbg_enc_lrange(&jbig_info,-1,-1);
     jbg_enc_options(&jbig_info,JBG_ILEAVE | JBG_SMID,JBG_TPDON | JBG_TPBON |
-      JBG_DPON,-1,-1,-1);
+      JBG_DPON,version < 1.6 ? -1 : 0,-1,-1);
     /*
       Write JBIG image.
     */
     jbg_enc_out(&jbig_info);
     jbg_enc_free(&jbig_info);
-    LiberateMemory((void **) &pixels);
-    if (image->next == (Image *) NULL)
+    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
-  CloseBlob(image);
-  return(True);
-}
-#else
-static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateWarning,"JBIG library is not available",
-    image->filename);
+    image=SyncNextImageInList(image);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
+  } while (image_info->adjoin != MagickFalse);
+  (void) CloseBlob(image);
+  return(MagickTrue);
 }
 #endif
