@@ -50,7 +50,9 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -62,6 +64,7 @@
 #include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/option.h"
+#include "magick/pixel.h"
 #include "magick/resource_.h"
 #include "magick/shear.h"
 #include "magick/static.h"
@@ -124,43 +127,43 @@ static const char *OsDesc=
 
 typedef enum
   {
-    miINT8 = 1,			/* 8 bit signed */
-    miUINT8,			/* 8 bit unsigned */
-    miINT16,			/* 16 bit signed */
-    miUINT16,			/* 16 bit unsigned */
-    miINT32,			/* 32 bit signed */
-    miUINT32,			/* 32 bit unsigned */
-    miSINGLE,			/* IEEE 754 single precision float */
+    miINT8 = 1,      /* 8 bit signed */
+    miUINT8,      /* 8 bit unsigned */
+    miINT16,      /* 16 bit signed */
+    miUINT16,      /* 16 bit unsigned */
+    miINT32,      /* 32 bit signed */
+    miUINT32,      /* 32 bit unsigned */
+    miSINGLE,      /* IEEE 754 single precision float */
     miRESERVE1,
-    miDOUBLE,			/* IEEE 754 double precision float */
+    miDOUBLE,      /* IEEE 754 double precision float */
     miRESERVE2,
     miRESERVE3,
-    miINT64,			/* 64 bit signed */
-    miUINT64,			/* 64 bit unsigned */
-    miMATRIX,		        /* MATLAB array */
-    miCOMPRESSED,	        /* Compressed Data */
-    miUTF8,		        /* Unicode UTF-8 Encoded Character Data */
-    miUTF16,		        /* Unicode UTF-16 Encoded Character Data */
-    miUTF32			/* Unicode UTF-32 Encoded Character Data */
+    miINT64,      /* 64 bit signed */
+    miUINT64,      /* 64 bit unsigned */
+    miMATRIX,            /* MATLAB array */
+    miCOMPRESSED,          /* Compressed Data */
+    miUTF8,            /* Unicode UTF-8 Encoded Character Data */
+    miUTF16,            /* Unicode UTF-16 Encoded Character Data */
+    miUTF32      /* Unicode UTF-32 Encoded Character Data */
   } mat5_data_type;
 
 typedef enum
   {
-    mxCELL_CLASS=1,		/* cell array */
-    mxSTRUCT_CLASS,		/* structure */
-    mxOBJECT_CLASS,		/* object */
-    mxCHAR_CLASS,		/* character array */
-    mxSPARSE_CLASS,		/* sparse array */
-    mxDOUBLE_CLASS,		/* double precision array */
-    mxSINGLE_CLASS,		/* single precision floating point */
-    mxINT8_CLASS,		/* 8 bit signed integer */
-    mxUINT8_CLASS,		/* 8 bit unsigned integer */
-    mxINT16_CLASS,		/* 16 bit signed integer */
-    mxUINT16_CLASS,		/* 16 bit unsigned integer */
-    mxINT32_CLASS,		/* 32 bit signed integer */
-    mxUINT32_CLASS,		/* 32 bit unsigned integer */
-    mxINT64_CLASS,		/* 64 bit signed integer */
-    mxUINT64_CLASS,		/* 64 bit unsigned integer */
+    mxCELL_CLASS=1,    /* cell array */
+    mxSTRUCT_CLASS,    /* structure */
+    mxOBJECT_CLASS,    /* object */
+    mxCHAR_CLASS,    /* character array */
+    mxSPARSE_CLASS,    /* sparse array */
+    mxDOUBLE_CLASS,    /* double precision array */
+    mxSINGLE_CLASS,    /* single precision floating point */
+    mxINT8_CLASS,    /* 8 bit signed integer */
+    mxUINT8_CLASS,    /* 8 bit unsigned integer */
+    mxINT16_CLASS,    /* 16 bit signed integer */
+    mxUINT16_CLASS,    /* 16 bit unsigned integer */
+    mxINT32_CLASS,    /* 32 bit signed integer */
+    mxUINT32_CLASS,    /* 32 bit unsigned integer */
+    mxINT64_CLASS,    /* 64 bit signed integer */
+    mxUINT64_CLASS,    /* 64 bit unsigned integer */
     mxFUNCTION_CLASS            /* Function handle */
   } arrayclasstype;
 
@@ -168,12 +171,15 @@ typedef enum
 #define FLAG_GLOBAL  0x4
 #define FLAG_LOGICAL 0x2
 
-static const QuantumType z2qtype[4] = {IndexQuantum, BlueQuantum, GreenQuantum, RedQuantum};
+static const QuantumType z2qtype[4] = {GrayQuantum, BlueQuantum, GreenQuantum, RedQuantum};
 
 
 static void InsertComplexDoubleRow(double *p, int y, Image * image, double MinVal,
                                   double MaxVal)
 {
+  ExceptionInfo
+    *exception;
+
   double f;
   int x;
   register PixelPacket *q;
@@ -183,43 +189,53 @@ static void InsertComplexDoubleRow(double *p, int y, Image * image, double MinVa
   if (MaxVal == 0)
     MaxVal = 1;
 
-  q = SetImagePixels(image, 0, y, image->columns, 1);
+  exception=(&image->exception);
+  q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
   if (q == (PixelPacket *) NULL)
     return;
-  for (x = 0; x < (long) image->columns; x++)
+  for (x = 0; x < (ssize_t) image->columns; x++)
   {
     if (*p > 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->red);
-      if (f + q->red > QuantumRange)
-        q->red = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetRedPixelComponent(q));
+      if (f + GetRedPixelComponent(q) > QuantumRange)
+        SetRedPixelComponent(q,QuantumRange);
       else
-        q->red += (int) f;
-      if ((int) f / 2.0 > q->green)
-        q->green = q->blue = 0;
+        SetRedPixelComponent(q,GetRedPixelComponent(q)+(int) f);
+      if ((int) f / 2.0 > GetGreenPixelComponent(q))
+        {
+          SetGreenPixelComponent(q,0);
+          SetBluePixelComponent(q,0);
+        }
       else
-        q->green = q->blue -= (int) (f / 2.0);
+        {
+          SetBluePixelComponent(q,GetBluePixelComponent(q)-(int) (f/2.0));
+          SetGreenPixelComponent(q,GetBluePixelComponent(q));
+        }
     }
     if (*p < 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->blue);
-      if (f + q->blue > QuantumRange)
-        q->blue = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetBluePixelComponent(q));
+      if (f + GetBluePixelComponent(q) > QuantumRange)
+        SetBluePixelComponent(q,QuantumRange);
       else
-        q->blue += (int) f;
+        SetBluePixelComponent(q,GetBluePixelComponent(q)+(int) f);
       if ((int) f / 2.0 > q->green)
-        q->green = q->red = 0;
+        {
+          SetRedPixelComponent(q,0);
+          SetGreenPixelComponent(q,0);
+        }
       else
-        q->green = q->red -= (int) (f / 2.0);
+        {
+          SetRedPixelComponent(q,GetRedPixelComponent(q)-(int) (f/2.0));
+          SetGreenPixelComponent(q,GetRedPixelComponent(q));
+        }
     }
     p++;
     q++;
   }
-  if (!SyncImagePixels(image))
+  if (!SyncAuthenticPixels(image,exception))
     return;
-  /*          if (image->previous == (Image *) NULL)
-     if (QuantumTick(y,image->rows))
-     MagickMonitor(LoadImageText,image->rows-y-1,image->rows); */
   return;
 }
 
@@ -227,6 +243,9 @@ static void InsertComplexDoubleRow(double *p, int y, Image * image, double MinVa
 static void InsertComplexFloatRow(float *p, int y, Image * image, double MinVal,
                                   double MaxVal)
 {
+  ExceptionInfo
+    *exception;
+
   double f;
   int x;
   register PixelPacket *q;
@@ -236,43 +255,53 @@ static void InsertComplexFloatRow(float *p, int y, Image * image, double MinVal,
   if (MaxVal == 0)
     MaxVal = 1;
 
-  q = SetImagePixels(image, 0, y, image->columns, 1);
+  exception=(&image->exception);
+  q = QueueAuthenticPixels(image, 0, y, image->columns, 1,exception);
   if (q == (PixelPacket *) NULL)
     return;
-  for (x = 0; x < (long) image->columns; x++)
+  for (x = 0; x < (ssize_t) image->columns; x++)
   {
     if (*p > 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->red);
-      if (f + q->red > QuantumRange)
-        q->red = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetRedPixelComponent(q));
+      if (f + GetRedPixelComponent(q) > QuantumRange)
+        SetRedPixelComponent(q,QuantumRange);
       else
-        q->red += (int) f;
-      if ((int) f / 2.0 > q->green)
-        q->green = q->blue = 0;
+        SetRedPixelComponent(q,GetRedPixelComponent(q)+(int) f);
+      if ((int) f / 2.0 > GetGreenPixelComponent(q))
+        {
+          SetGreenPixelComponent(q,0);
+          SetBluePixelComponent(q,0);
+        }
       else
-        q->green = q->blue -= (int) (f / 2.0);
+        {
+          SetBluePixelComponent(q,GetBluePixelComponent(q)-(int) (f/2.0));
+          SetGreenPixelComponent(q,GetBluePixelComponent(q));
+        }
     }
     if (*p < 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->blue);
-      if (f + q->blue > QuantumRange)
-        q->blue = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetBluePixelComponent(q));
+      if (f + GetBluePixelComponent(q) > QuantumRange)
+        SetBluePixelComponent(q,QuantumRange);
       else
-        q->blue += (int) f;
+        SetBluePixelComponent(q,GetBluePixelComponent(q)+(int) f);
       if ((int) f / 2.0 > q->green)
-        q->green = q->red = 0;
+        {
+          SetGreenPixelComponent(q,0);
+          SetRedPixelComponent(q,0);
+        }
       else
-        q->green = q->red -= (int) (f / 2.0);
+        {
+          SetRedPixelComponent(q,GetRedPixelComponent(q)-(int) (f/2.0));
+          SetGreenPixelComponent(q,GetRedPixelComponent(q));
+        }
     }
     p++;
     q++;
   }
-  if (!SyncImagePixels(image))
+  if (!SyncAuthenticPixels(image,exception))
     return;
-  /*          if (image->previous == (Image *) NULL)
-     if (QuantumTick(y,image->rows))
-     MagickMonitor(LoadImageText,image->rows-y-1,image->rows); */
   return;
 }
 
@@ -326,7 +355,7 @@ static void ReadBlobDoublesMSB(Image * image, size_t len, double *data)
 }
 
 /* Calculate minimum and maximum from a given block of data */
-static void CalcMinMax(Image *image, int endian_indicator, int SizeX, int SizeY, unsigned long CellType, unsigned ldblk, void *BImgBuff, double *Min, double *Max)
+static void CalcMinMax(Image *image, int endian_indicator, int SizeX, int SizeY, size_t CellType, unsigned ldblk, void *BImgBuff, double *Min, double *Max)
 {
 MagickOffsetType filepos;
 int i, x;
@@ -340,13 +369,13 @@ float *fltrow;
     ReadBlobDoublesXXX = ReadBlobDoublesLSB;
     ReadBlobFloatsXXX = ReadBlobFloatsLSB;   
   } 
-  else		/* MI */
+  else    /* MI */
   {    
     ReadBlobDoublesXXX = ReadBlobDoublesMSB;
     ReadBlobFloatsXXX = ReadBlobFloatsMSB;   
   }
 
-  filepos = TellBlob(image);	   /* Please note that file seeking occurs only in the case of doubles */
+  filepos = TellBlob(image);     /* Please note that file seeking occurs only in the case of doubles */
   for (i = 0; i < SizeY; i++)
   {
     if (CellType==miDOUBLE)
@@ -374,7 +403,7 @@ float *fltrow;
       {
         *Min = *Max = *fltrow;
       }
-    for (x = 0; x < (long) SizeX; x++)
+    for (x = 0; x < (ssize_t) SizeX; x++)
       {
         if (*Min > *fltrow)
           *Min = *fltrow;
@@ -394,10 +423,10 @@ static void FixSignedValues(PixelPacket *q, int y)
   {
      /* Please note that negative values will overflow
         Q=8; QuantumRange=255: <0;127> + 127+1 = <128; 255> 
-		       <-1;-128> + 127+1 = <0; 127> */
-    q->red += QuantumRange/2 + 1;
-    q->green += QuantumRange/ + 1;
-    q->blue += QuantumRange/ + 1;
+           <-1;-128> + 127+1 = <0; 127> */
+    SetRedPixelComponent(q,GetRedPixelComponent(q)+QuantumRange/2+1);
+    SetGreenPixelComponent(q,GetGreenPixelComponent(q)+QuantumRange/2+1);
+    SetBluePixelComponent(q,GetBluePixelComponent(q)+QuantumRange/2+1);
     q++;
   }
 }
@@ -427,22 +456,37 @@ unsigned char val = 0;
   *BuffL = val;
 }
 
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
+static voidpf AcquireZIPMemory(voidpf context,unsigned int items,
+  unsigned int size)
+{
+  (void) context;
+  return((voidpf) AcquireQuantumMemory(items,size));
+}
 
+static void RelinquishZIPMemory(voidpf context,voidpf memory)
+{
+  (void) context;
+  memory=RelinquishMagickMemory(memory);
+}
+#endif
+
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
 /** This procedure decompreses an image block for a new MATLAB format. */
 static Image *DecompressBlock(Image *orig, MagickOffsetType Size, ImageInfo *clone_info, ExceptionInfo *exception)
 {
-#if defined(MAGICKCORE_ZLIB_DELEGATE)
 
 Image *image2;
 void *CacheBlock, *DecompressBlock;
 z_stream zip_info;
 FILE *mat_file;
 size_t magick_size;
+size_t extent;
 
 int status;
 
   if(clone_info==NULL) return NULL;
-  if(clone_info->file)		/* Close file opened from previous transaction. */
+  if(clone_info->file)    /* Close file opened from previous transaction. */
   {
     fclose(clone_info->file);
     clone_info->file = NULL;
@@ -467,8 +511,8 @@ int status;
     return NULL;
   }
 
-  zip_info.zalloc = (alloc_func) NULL;
-  zip_info.zfree = (free_func) NULL;
+  zip_info.zalloc=AcquireZIPMemory;
+  zip_info.zfree=RelinquishZIPMemory;
   zip_info.opaque = (voidpf) NULL;
   inflateInit(&zip_info);
   /* zip_info.next_out = 8*4;*/
@@ -486,7 +530,8 @@ int status;
       zip_info.avail_out = 4096;    
       zip_info.next_out = (Bytef *) DecompressBlock;
       status = inflate(&zip_info,Z_NO_FLUSH);      
-      fwrite(DecompressBlock, 4096-zip_info.avail_out, 1, mat_file);
+      extent=fwrite(DecompressBlock, 4096-zip_info.avail_out, 1, mat_file);
+      (void) extent;
 
       if(status == Z_STREAM_END) goto DblBreak;
     }
@@ -514,10 +559,8 @@ UnlinkFile:
   }
 
   return image2;
-#else
-  return NULL;
-#endif
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -565,22 +608,23 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   unsigned int status;
   MATHeader MATLAB_HDR;
-  unsigned long size;  
-  unsigned long CellType;
+  size_t size;  
+  size_t CellType;
   QuantumInfo *quantum_info;
   ImageInfo *clone_info;
   int i;
-  long ldblk;
+  ssize_t ldblk;
   unsigned char *BImgBuff = NULL;
   double MinVal, MaxVal;
-  unsigned long Unknown6;
+  size_t Unknown6;
   unsigned z;
   int logging;
   int sample_size;
   MagickOffsetType filepos=0x80;
   BlobInfo *blob;
+  size_t one;
   
-  unsigned long (*ReadBlobXXXLong)(Image *image);
+  unsigned int (*ReadBlobXXXLong)(Image *image);
   unsigned short (*ReadBlobXXXShort)(Image *image);
   void (*ReadBlobDoublesXXX)(Image * image, size_t len, double *data);
   void (*ReadBlobFloatsXXX)(Image * image, size_t len, float *data);
@@ -670,7 +714,7 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
 
     MATLAB_HDR.unknown3 = ReadBlobXXXLong(image2);
     if(image!=image2)
-      MATLAB_HDR.unknown4 = ReadBlobXXXLong(image2);	/* ??? don't understand why ?? */
+      MATLAB_HDR.unknown4 = ReadBlobXXXLong(image2);  /* ??? don't understand why ?? */
     MATLAB_HDR.unknown4 = ReadBlobXXXLong(image2);
     MATLAB_HDR.DimFlag = ReadBlobXXXLong(image2);
     MATLAB_HDR.SizeX = ReadBlobXXXLong(image2);
@@ -679,11 +723,12 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
 
     switch(MATLAB_HDR.DimFlag)
     {     
-      case  8: z=1; break;			/* 2D matrix*/
-      case 12: z = ReadBlobXXXLong(image2);	/* 3D matrix RGB*/
-  	       Unknown6 = ReadBlobXXXLong(image2);
-	       if(z!=3) ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
-	       break;
+      case  8: z=1; break;      /* 2D matrix*/
+      case 12: z = ReadBlobXXXLong(image2);  /* 3D matrix RGB*/
+           Unknown6 = ReadBlobXXXLong(image2);
+           (void) Unknown6;
+         if(z!=3) ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
+         break;
       default: ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
     }  
 
@@ -693,23 +738,23 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
           "MATLAB_HDR.StructureClass %d",MATLAB_HDR.StructureClass);
     if (MATLAB_HDR.StructureClass != mxCHAR_CLASS && 
-        MATLAB_HDR.StructureClass != mxSINGLE_CLASS &&		/* float + complex float */
-        MATLAB_HDR.StructureClass != mxDOUBLE_CLASS &&		/* double + complex double */
+        MATLAB_HDR.StructureClass != mxSINGLE_CLASS &&    /* float + complex float */
+        MATLAB_HDR.StructureClass != mxDOUBLE_CLASS &&    /* double + complex double */
         MATLAB_HDR.StructureClass != mxINT8_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT8_CLASS &&		/* uint8 + uint8 3D */
+        MATLAB_HDR.StructureClass != mxUINT8_CLASS &&    /* uint8 + uint8 3D */
         MATLAB_HDR.StructureClass != mxINT16_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT16_CLASS &&		/* uint16 + uint16 3D */
+        MATLAB_HDR.StructureClass != mxUINT16_CLASS &&    /* uint16 + uint16 3D */
         MATLAB_HDR.StructureClass != mxINT32_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT32_CLASS &&		/* uint32 + uint32 3D */
+        MATLAB_HDR.StructureClass != mxUINT32_CLASS &&    /* uint32 + uint32 3D */
         MATLAB_HDR.StructureClass != mxINT64_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT64_CLASS)		/* uint64 + uint64 3D */
+        MATLAB_HDR.StructureClass != mxUINT64_CLASS)    /* uint64 + uint64 3D */
       ThrowReaderException(CoderError,"UnsupportedCellTypeInTheMatrix");
 
     switch (MATLAB_HDR.NameFlag)
     {
       case 0:
-        size = ReadBlobXXXLong(image2);	/* Object name string size */
-        size = 4 * (long) ((size + 3 + 1) / 4);
+        size = ReadBlobXXXLong(image2);  /* Object name string size */
+        size = 4 * (ssize_t) ((size + 3 + 1) / 4);
         (void) SeekBlob(image2, size, SEEK_CUR);
         break;
       case 1:
@@ -723,8 +768,9 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
 
     CellType = ReadBlobXXXLong(image2);    /* Additional object type */
-    if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
-          "MATLAB_HDR.CellType: %ld",CellType);
+    if (logging)
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "MATLAB_HDR.CellType: %.20g",(double) CellType);
   
     (void) ReadBlob(image2, 4, (unsigned char *) &size);     /* data size */
 
@@ -742,34 +788,34 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
           image->depth = 1;
         else
           image->depth = 8;         /* Byte type cell */
-        ldblk = (long) MATLAB_HDR.SizeX;      
+        ldblk = (ssize_t) MATLAB_HDR.SizeX;      
         break;
       case miINT16:
       case miUINT16:
         sample_size = 16;
         image->depth = 16;        /* Word type cell */
-        ldblk = (long) (2 * MATLAB_HDR.SizeX);
+        ldblk = (ssize_t) (2 * MATLAB_HDR.SizeX);
         break;
       case miINT32:
       case miUINT32:
         sample_size = 32;
         image->depth = 32;        /* Dword type cell */
-        ldblk = (long) (4 * MATLAB_HDR.SizeX);      
+        ldblk = (ssize_t) (4 * MATLAB_HDR.SizeX);      
         break;
       case miINT64:
       case miUINT64:
         sample_size = 64;
         image->depth = 64;        /* Qword type cell */
-        ldblk = (long) (8 * MATLAB_HDR.SizeX);      
+        ldblk = (ssize_t) (8 * MATLAB_HDR.SizeX);      
         break;   
       case miSINGLE:
         sample_size = 32;
         image->depth = 32;        /* double type cell */
         (void) SetImageOption(clone_info,"quantum:format","floating-point");
         if (MATLAB_HDR.StructureFlag & FLAG_COMPLEX)
-	{					    /* complex float type cell */
-	}
-        ldblk = (long) (4 * MATLAB_HDR.SizeX);
+  {              /* complex float type cell */
+  }
+        ldblk = (ssize_t) (4 * MATLAB_HDR.SizeX);
         break;
       case miDOUBLE:
         sample_size = 64; 
@@ -778,17 +824,21 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
         if (sizeof(double) != 8)
           ThrowReaderException(CoderError, "IncompatibleSizeOfDouble");
         if (MATLAB_HDR.StructureFlag & FLAG_COMPLEX)
-	{                         /* complex double type cell */        
-	}
-        ldblk = (long) (8 * MATLAB_HDR.SizeX);
+  {                         /* complex double type cell */        
+  }
+        ldblk = (ssize_t) (8 * MATLAB_HDR.SizeX);
         break;
       default:
         ThrowReaderException(CoderError, "UnsupportedCellTypeInTheMatrix");
     }
+    (void) sample_size;
     image->columns = MATLAB_HDR.SizeX;
     image->rows = MATLAB_HDR.SizeY;    
     quantum_info=AcquireQuantumInfo(clone_info,image);
-    image->colors = 1l << image->depth;
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    one=1;
+    image->colors = one << image->depth;
     if (image->columns == 0 || image->rows == 0)
       goto MATLAB_KO;
 
@@ -809,7 +859,7 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     */
     if (image_info->ping)
     {
-      unsigned long temp = image->columns;
+      size_t temp = image->columns;
       image->columns = image->rows;
       image->rows = temp;
       goto done_reading; /* !!!!!! BAD  !!!! */
@@ -829,52 +879,52 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
 
     /* Main loop for reading all scanlines */
     if(z==1) z=0; /* read grey scanlines */
-		/* else read color scanlines */
+    /* else read color scanlines */
     do
     {
-      for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
+      for (i = 0; i < (ssize_t) MATLAB_HDR.SizeY; i++)
       {
-        q=SetImagePixels(image,0,MATLAB_HDR.SizeY-i-1,image->columns,1);
+        q=QueueAuthenticPixels(image,0,MATLAB_HDR.SizeY-i-1,image->columns,1,exception);
         if (q == (PixelPacket *)NULL)
-	{
-	  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+  {
+    if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
               "  MAT set image pixels returns unexpected NULL on a row %u.", (unsigned)(MATLAB_HDR.SizeY-i-1));
-	  goto done_reading;		/* Skip image rotation, when cannot set image pixels	  */
-	}
+    goto done_reading;    /* Skip image rotation, when cannot set image pixels    */
+  }
         if(ReadBlob(image2,ldblk,(unsigned char *)BImgBuff) != (ssize_t) ldblk)
-	{
-	  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+  {
+    if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
              "  MAT cannot read scanrow %u from a file.", (unsigned)(MATLAB_HDR.SizeY-i-1));
-	  goto ExitLoop;
-	}
+    goto ExitLoop;
+  }
         if((CellType==miINT8 || CellType==miUINT8) && (MATLAB_HDR.StructureFlag & FLAG_LOGICAL))
         {
           FixLogical((unsigned char *)BImgBuff,ldblk);
-          if(ImportQuantumPixels(image,quantum_info,z2qtype[z],BImgBuff) <= 0)
-	  {
+          if(ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,z2qtype[z],BImgBuff,exception) <= 0)
+    {
 ImportQuantumPixelsFailed:
-	    if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+      if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
               "  MAT failed to ImportQuantumPixels for a row %u", (unsigned)(MATLAB_HDR.SizeY-i-1));
-	    break;
-	  }
+      break;
+    }
         }
         else
         {
-          if(ImportQuantumPixels(image,quantum_info,z2qtype[z],BImgBuff) <= 0)
-	    goto ImportQuantumPixelsFailed;
+          if(ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,z2qtype[z],BImgBuff,exception) <= 0)
+      goto ImportQuantumPixelsFailed;
 
 
-          if (z<=1 &&			 /* fix only during a last pass z==0 || z==1 */
-	        (CellType==miINT8 || CellType==miINT16 || CellType==miINT32 || CellType==miINT64))
-	    FixSignedValues(q,MATLAB_HDR.SizeX);
+          if (z<=1 &&       /* fix only during a last pass z==0 || z==1 */
+          (CellType==miINT8 || CellType==miINT16 || CellType==miINT32 || CellType==miINT64))
+      FixSignedValues(q,MATLAB_HDR.SizeX);
         }
 
-        if (!SyncImagePixels(image))
-	{
-	  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+        if (!SyncAuthenticPixels(image,exception))
+  {
+    if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
             "  MAT failed to sync image pixels for a row %u", (unsigned)(MATLAB_HDR.SizeY-i-1));
-	  goto ExitLoop;
-	}
+    goto ExitLoop;
+  }
       }
     } while(z-- >= 2);
 ExitLoop:
@@ -892,18 +942,18 @@ ExitLoop:
       }
 
       if (CellType==miDOUBLE)
-        for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
-	{
+        for (i = 0; i < (ssize_t) MATLAB_HDR.SizeY; i++)
+  {
           ReadBlobDoublesXXX(image2, ldblk, (double *)BImgBuff);
           InsertComplexDoubleRow((double *)BImgBuff, i, image, MinVal, MaxVal);
-	}
+  }
 
       if (CellType==miSINGLE)
-        for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
-	{
+        for (i = 0; i < (ssize_t) MATLAB_HDR.SizeY; i++)
+  {
           ReadBlobFloatsXXX(image2, ldblk, (float *)BImgBuff);
           InsertComplexFloatRow((float *)BImgBuff, i, image, MinVal, MaxVal);
-	}    
+  }    
     }
 
       /* Image is gray when no complex flag is set and 2D Matrix AGAIN!!! */
@@ -938,14 +988,14 @@ done_reading:
       if(image2!=image)
       {
         DeleteImageFromList(&image2); 
-	if(clone_info)
-	{
+  if(clone_info)
+  {
           if(clone_info->file)
-	  {
+    {
             fclose(clone_info->file);
             clone_info->file = NULL;
             (void) unlink(clone_info->filename);
-	  }
+    }
         }    
       }
 
@@ -968,7 +1018,7 @@ done_reading:
 
   {
     Image *p;    
-    long scene=0;
+    ssize_t scene=0;
     
     /*
       Rewind list, removing any empty images while rewinding.
@@ -994,7 +1044,7 @@ done_reading:
       p->scene=scene++;
   }
 
-  if(clone_info != NULL)	/* cleanup garbage file from compression */
+  if(clone_info != NULL)  /* cleanup garbage file from compression */
   {
     if(clone_info->file)
     {
@@ -1031,10 +1081,10 @@ done_reading:
 %
 %  The format of the RegisterMATImage method is:
 %
-%      unsigned long RegisterMATImage(void)
+%      size_t RegisterMATImage(void)
 %
 */
-ModuleExport unsigned long RegisterMATImage(void)
+ModuleExport size_t RegisterMATImage(void)
 {
   MagickInfo
     *entry;
@@ -1044,7 +1094,7 @@ ModuleExport unsigned long RegisterMATImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMATImage;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("MATLAB image format");
+  entry->description=AcquireString("MATLAB level 5 image format");
   entry->module=AcquireString("MAT");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
@@ -1104,17 +1154,20 @@ ModuleExport void UnregisterMATImage(void)
 */
 static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
 {
-  long y;
+  ExceptionInfo
+    *exception;
+
+  ssize_t y;
   unsigned z;
   const PixelPacket *p;
 
   unsigned int status;
   int logging;
-  unsigned long DataSize;
+  size_t DataSize;
   char padding;
   char MATLAB_HDR[0x80];
   time_t current_time;
-  const struct tm *t;
+  struct tm local_time;
   unsigned char *pixels;
   int is_gray;
 
@@ -1132,21 +1185,23 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   logging=LogMagickEvent(CoderEvent,GetMagickModule(),"enter MAT");
+  (void) logging;
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(MagickFalse);
   image->depth=8;
 
   current_time=time((time_t *) NULL);
-  t=localtime(&current_time);
+#if defined(MAGICKCORE_HAVE_LOCALTIME_R)
+  (void) localtime_r(&current_time,&local_time);
+#else
+  (void) memcpy(&local_time,localtime(&current_time),sizeof(local_time));
+#endif
   (void) memset(MATLAB_HDR,' ',MagickMin(sizeof(MATLAB_HDR),124));
   FormatMagickString(MATLAB_HDR,MaxTextExtent,"MATLAB 5.0 MAT-file, Platform: %s, Created on: %s %s %2d %2d:%2d:%2d %d",
-    OsDesc,
-    DayOfWTab[t->tm_wday],
-    MonthsTab[t->tm_mon],
-    t->tm_mday,
-    t->tm_hour,t->tm_min,t->tm_sec,
-    t->tm_year+1900);
+    OsDesc,DayOfWTab[local_time.tm_wday],MonthsTab[local_time.tm_mon],
+    local_time.tm_mday,local_time.tm_hour,local_time.tm_min,
+    local_time.tm_sec,local_time.tm_year+1900);
   MATLAB_HDR[0x7C]=0;
   MATLAB_HDR[0x7D]=1;
   MATLAB_HDR[0x7E]='I';
@@ -1156,7 +1211,7 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
   do
   {
     if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
+      (void) TransformImageColorspace(image,RGBColorspace);
 
     is_gray = IsGrayImage(image,&image->exception);
     z = is_gray ? 0 : 3;
@@ -1169,15 +1224,15 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
     padding=((unsigned char)(DataSize-1) & 0x7) ^ 0x7;
 
     (void) WriteBlobLSBLong(image, miMATRIX);
-    (void) WriteBlobLSBLong(image, DataSize+padding+(is_gray ? 48l : 56l));
+    (void) WriteBlobLSBLong(image, (unsigned int) DataSize+padding+(is_gray ? 48 : 56));
     (void) WriteBlobLSBLong(image, 0x6); /* 0x88 */
     (void) WriteBlobLSBLong(image, 0x8); /* 0x8C */
     (void) WriteBlobLSBLong(image, 0x6); /* 0x90 */  
     (void) WriteBlobLSBLong(image, 0);   
     (void) WriteBlobLSBLong(image, 0x5); /* 0x98 */
     (void) WriteBlobLSBLong(image, is_gray ? 0x8 : 0xC); /* 0x9C - DimFlag */
-    (void) WriteBlobLSBLong(image, image->rows);    /* x: 0xA0 */  
-    (void) WriteBlobLSBLong(image, image->columns); /* y: 0xA4 */  
+    (void) WriteBlobLSBLong(image, (unsigned int) image->rows);    /* x: 0xA0 */  
+    (void) WriteBlobLSBLong(image, (unsigned int) image->columns); /* y: 0xA4 */  
     if(!is_gray)
     {
       (void) WriteBlobLSBLong(image, 3); /* z: 0xA8 */  
@@ -1187,24 +1242,28 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobLSBShort(image, 1);  /* 0xB2 */
     (void) WriteBlobLSBLong(image, 'M'); /* 0xB4 */
     (void) WriteBlobLSBLong(image, 0x2); /* 0xB8 */  
-    (void) WriteBlobLSBLong(image, DataSize); /* 0xBC */
+    (void) WriteBlobLSBLong(image, (unsigned int) DataSize); /* 0xBC */
 
     /*
       Store image data.
     */
+    exception=(&image->exception);
     quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=GetQuantumPixels(quantum_info);
     do
     {
-      for (y=0; y < (long)image->columns; y++)
+      for (y=0; y < (ssize_t)image->columns; y++)
       {
-        p=AcquireImagePixels(image,y,0,1,image->rows,&image->exception);
+        p=GetVirtualPixels(image,y,0,1,image->rows,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
-        (void) ExportQuantumPixels(image,quantum_info,z2qtype[z],pixels);
+        (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+          z2qtype[z],pixels,exception);
         (void) WriteBlob(image,image->rows,pixels);
       }    
-      if (!SyncImagePixels(image))
+      if (!SyncAuthenticPixels(image,exception))
         break;
     } while(z-- >= 2);
     while(padding-->0) (void) WriteBlobByte(image,0);
