@@ -17,7 +17,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -72,6 +72,40 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s A V I                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsAVI() returns MagickTrue if the image format type, identified by the
+%  magick string, is Audio/Video Interleaved file format.
+%
+%  The format of the IsAVI method is:
+%
+%      size_t IsAVI(const unsigned char *magick,const size_t length)
+%
+%  A description of each parameter follows:
+%
+%    o magick: compare image format pattern against these bytes.
+%
+%    o length: Specifies the length of the magick string.
+%
+*/
+static MagickBooleanType IsAVI(const unsigned char *magick,const size_t length)
+{
+  if (length < 4)
+    return(MagickFalse);
+  if (memcmp(magick,"RIFF",4) == 0)
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I s M P E G                                                               %
 %                                                                             %
 %                                                                             %
@@ -87,8 +121,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -144,9 +177,6 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register long
-    i;
-
   /*
     Open image file.
   */
@@ -167,43 +197,16 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
   (void) CloseBlob(image);
   (void) DestroyImageList(image);
   /*
-    Convert MPEG to PPM with delegate.
+    Convert MPEG to PAM with delegate.
   */
-  image=AcquireImage(image_info);
   read_info=CloneImageInfo(image_info);
+  image=AcquireImage(image_info);
   (void) InvokeDelegate(read_info,image,"mpeg:decode",(char *) NULL,exception);
   image=DestroyImage(image);
-  /*
-    Read PPM files.
-  */
-  images=NewImageList();
-  for (i=(long) read_info->scene; ; i++)
-  {
-    (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s%ld.%s",
-      read_info->unique,i,ReadMPEGIntermediateFormat);
-    if (IsPathAccessible(read_info->filename) == MagickFalse)
-      break;
-    image=ReadImage(read_info,exception);
-    if (image == (Image *) NULL)
-      break;
-    (void) CopyMagickString(image->magick,image_info->magick,MaxTextExtent);
-    image->scene=(unsigned long) i;
-    AppendImageToList(&images,image);
-    if (read_info->number_scenes != 0)
-      if (i >= (long) (read_info->scene+read_info->number_scenes-1))
-        break;
-  }
-  /*
-    Relinquish resources.
-  */
-  for (i=0; ; i++)
-  {
-    (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s%ld.%s",
-      read_info->unique,i,ReadMPEGIntermediateFormat);
-    if (IsPathAccessible(read_info->filename) == MagickFalse)
-      break;
-    (void) RelinquishUniqueFileResource(read_info->filename);
-  }
+  (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s.%s",
+    read_info->unique,ReadMPEGIntermediateFormat);
+  images=ReadImage(read_info,exception);
+  (void) RelinquishUniqueFileResource(read_info->filename);
   read_info=DestroyImageInfo(read_info);
   return(images);
 }
@@ -228,14 +231,21 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterMPEGImage method is:
 %
-%      unsigned long RegisterMPEGImage(void)
+%      size_t RegisterMPEGImage(void)
 %
 */
-ModuleExport unsigned long RegisterMPEGImage(void)
+ModuleExport size_t RegisterMPEGImage(void)
 {
   MagickInfo
     *entry;
 
+  entry=SetMagickInfo("AVI");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->magick=(IsImageFormatHandler *) IsAVI;
+  entry->blob_support=MagickFalse;
+  entry->description=ConstantString("Microsoft Audio/Visual Interleaved");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MOV");
   entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
@@ -323,6 +333,7 @@ ModuleExport void UnregisterMPEGImage(void)
   (void) UnregisterMagickInfo("MPG");
   (void) UnregisterMagickInfo("MPEG");
   (void) UnregisterMagickInfo("MOV");
+  (void) UnregisterMagickInfo("AVI");
 }
 
 /*
@@ -415,7 +426,8 @@ static MagickBooleanType CopyDelegateFile(const char *source,
     }
   quantum=(size_t) MagickMaxBufferExtent;
   if ((fstat(source_file,&attributes) == 0) && (attributes.st_size != 0))
-    quantum=MagickMin((size_t) attributes.st_size,MagickMaxBufferExtent);
+    quantum=(size_t) MagickMin((double) attributes.st_size,
+      MagickMaxBufferExtent);
   buffer=(unsigned char *) AcquireQuantumMemory(quantum,sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     {
@@ -467,19 +479,17 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
   register Image
     *p;
 
-  register long
+  register ssize_t
     i;
 
   size_t
-    length;
+    count,
+    length,
+    scene;
 
   unsigned char
     *blob;
 
-  unsigned long
-    count,
-    scene;
-
   /*
     Open output image file.
   */
@@ -494,7 +504,7 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
     return(status);
   (void) CloseBlob(image);
   /*
-    Write JPEG files.
+    Write intermediate files.
   */
   coalesce_image=CoalesceImages(image,&image->exception);
   if (coalesce_image == (Image *) NULL)
@@ -515,7 +525,7 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
     length=0;
     scene=p->scene;
     delay=100.0*p->delay/MagickMax(1.0*p->ticks_per_second,1.0);
-    for (i=0; i < (long) MagickMax((1.0*delay+1.0)/3.0,1.0); i++)
+    for (i=0; i < (ssize_t) MagickMax((1.0*delay+1.0)/3.0,1.0); i++)
     {
       p->scene=count;
       count++;
@@ -527,12 +537,13 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
           Image
             *frame;
 
-          (void) FormatMagickString(p->filename,MaxTextExtent,"%s%lu.%s",
-            basename,p->scene,WriteMPEGIntermediateFormat);
-          (void) FormatMagickString(filename,MaxTextExtent,"%s%lu.%s",
-            basename,p->scene,WriteMPEGIntermediateFormat);
+          (void) FormatMagickString(p->filename,MaxTextExtent,"%s%.20g.%s",
+            basename,(double) p->scene,WriteMPEGIntermediateFormat);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s%.20g.%s",
+            basename,(double) p->scene,WriteMPEGIntermediateFormat);
           (void) FormatMagickString(previous_image,MaxTextExtent,
-            "%s%lu.%s",basename,p->scene,WriteMPEGIntermediateFormat);
+            "%s%.20g.%s",basename,(double) p->scene,
+            WriteMPEGIntermediateFormat);
           frame=CloneImage(p,0,0,MagickTrue,&p->exception);
           if (frame == (Image *) NULL)
             break;
@@ -547,8 +558,8 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
         }
         default:
         {
-          (void) FormatMagickString(filename,MaxTextExtent,"%s%lu.%s",
-            basename,p->scene,WriteMPEGIntermediateFormat);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s%.20g.%s",
+            basename,(double) p->scene,WriteMPEGIntermediateFormat);
           if (length > 0)
             status=BlobToFile(filename,blob,length,&image->exception);
           break;
@@ -558,12 +569,13 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
         {
           if (status != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "%lu. Wrote JPEG file for scene %lu:",i,p->scene);
+              "%.20g. Wrote %s file for scene %.20g:",(double) i,
+              WriteMPEGIntermediateFormat,(double) p->scene);
           else
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "%lu. Failed to write JPEG file for scene %lu:",i,p->scene);
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",
-            filename);
+              "%.20g. Failed to write %s file for scene %.20g:",(double) i,
+              WriteMPEGIntermediateFormat,(double) p->scene);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",filename);
         }
     }
     p->scene=scene;
@@ -595,10 +607,10 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
   for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
   {
     delay=100.0*p->delay/MagickMax(1.0*p->ticks_per_second,1.0);
-    for (i=0; i < (long) MagickMax((1.0*delay+1.0)/3.0,1.0); i++)
+    for (i=0; i < (ssize_t) MagickMax((1.0*delay+1.0)/3.0,1.0); i++)
     {
-      (void) FormatMagickString(p->filename,MaxTextExtent,"%s%lu.%s",
-        basename,count++,WriteMPEGIntermediateFormat);
+      (void) FormatMagickString(p->filename,MaxTextExtent,"%s%.20g.%s",
+        basename,(double) count++,WriteMPEGIntermediateFormat);
       (void) RelinquishUniqueFileResource(p->filename);
     }
     (void) CopyMagickString(p->filename,image_info->filename,MaxTextExtent);
