@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,6 +43,7 @@
 #include "magick/property.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/client.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
@@ -143,13 +144,10 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       key;
   } PCDTable;
 
-  long
-    quantum;
-
   PCDTable
     *pcd_table[3];
 
-  register long
+  register ssize_t
     i,
     j;
 
@@ -161,21 +159,20 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     *q;
 
   size_t
-    length;
-
-  ssize_t
-    count;
-
-  unsigned char
-    *buffer;
-
-  unsigned long
     bits,
+    length,
     plane,
     pcd_length[3],
     row,
     sum;
 
+  ssize_t
+    count,
+    quantum;
+
+  unsigned char
+    *buffer;
+
   /*
     Initialize Huffman tables.
   */
@@ -206,7 +203,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           image->filename);
       }
     r=pcd_table[i];
-    for (j=0; j < (long) length; j++)
+    for (j=0; j < (ssize_t) length; j++)
     {
       PCDGetBits(8);
       r->length=(unsigned int) (sum & 0xff)+1;
@@ -222,7 +219,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       r->mask=(~((1U << (32-r->length))-1));
       r++;
     }
-    pcd_length[i]=(unsigned long) length;
+    pcd_length[i]=(size_t) length;
   }
   /*
     Search for Sync byte.
@@ -292,10 +289,9 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       Decode luminance or chrominance deltas.
     */
     r=pcd_table[plane];
-    for (i=0; ((i < (long) length) && ((sum & r->mask) != r->sequence)); i++)
+    for (i=0; ((i < (ssize_t) length) && ((sum & r->mask) != r->sequence)); i++)
       r++;
-    if ((row > image->rows) || (r == (PCDTable *) NULL) ||
-        ((size_t) (q-luma) > (image->columns*image->rows)))
+    if ((row > image->rows) || (r == (PCDTable *) NULL))
       {
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
           CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
@@ -306,9 +302,9 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
         continue;
       }
     if (r->key < 128)
-      quantum=(long) (*q)+r->key;
+      quantum=(ssize_t) (*q)+r->key;
     else
-      quantum=(long) (*q)+r->key-256;
+      quantum=(ssize_t) (*q)+r->key-256;
     *q=(unsigned char) ((quantum < 0) ? 0 : (quantum > 255) ? 255 : quantum);
     q++;
     PCDGetBits(r->length);
@@ -343,18 +339,16 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 {
   if (length < 2052)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick+2048,"PCD_",4) == 0)
+  if (LocaleNCompare((const char *) magick+2048,"PCD_",4) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -425,10 +419,10 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   return(montage_image);
 }
 
-static void Upsample(const unsigned long width,const unsigned long height,
-  const unsigned long scaled_width,unsigned char *pixels)
+static void Upsample(const size_t width,const size_t height,
+  const size_t scaled_width,unsigned char *pixels)
 {
-  register long
+  register ssize_t
     x,
     y;
 
@@ -441,39 +435,36 @@ static void Upsample(const unsigned long width,const unsigned long height,
     Create a new image that is a integral size greater than an existing one.
   */
   assert(pixels != (unsigned char *) NULL);
-  for (y=0; y < (long) height; y++)
+  for (y=0; y < (ssize_t) height; y++)
   {
     p=pixels+(height-1-y)*scaled_width+(width-1);
     q=pixels+((height-1-y) << 1)*scaled_width+((width-1) << 1);
     *q=(*p);
     *(q+1)=(*(p));
-    for (x=1; x < (long) width; x++)
+    for (x=1; x < (ssize_t) width; x++)
     {
       p--;
       q-=2;
       *q=(*p);
-      *(q+1)=(unsigned char) ((((unsigned long) *p)+
-        ((unsigned long) *(p+1))+1) >> 1);
+      *(q+1)=(unsigned char) ((((size_t) *p)+((size_t) *(p+1))+1) >> 1);
     }
   }
-  for (y=0; y < (long) (height-1); y++)
+  for (y=0; y < (ssize_t) (height-1); y++)
   {
-    p=pixels+((unsigned long) y << 1)*scaled_width;
+    p=pixels+((size_t) y << 1)*scaled_width;
     q=p+scaled_width;
     r=q+scaled_width;
-    for (x=0; x < (long) (width-1); x++)
+    for (x=0; x < (ssize_t) (width-1); x++)
     {
-      *q=(unsigned char) ((((unsigned long) *p)+((unsigned long) *r)+1) >> 1);
-      *(q+1)=(unsigned char) ((((unsigned long) *p)+((unsigned long) *(p+2))+
-        ((unsigned long) *r)+((unsigned long) *(r+2))+2) >> 2);
+      *q=(unsigned char) ((((size_t) *p)+((size_t) *r)+1) >> 1);
+      *(q+1)=(unsigned char) ((((size_t) *p)+((size_t) *(p+2))+
+        ((size_t) *r)+((size_t) *(r+2))+2) >> 2);
       q+=2;
       p+=2;
       r+=2;
     }
-    *q++=(unsigned char) ((((unsigned long) *p++)+
-      ((unsigned long) *r++)+1) >> 1);
-    *q++=(unsigned char) ((((unsigned long) *p++)+
-      ((unsigned long) *r++)+1) >> 1);
+    *q++=(unsigned char) ((((size_t) *p++)+((size_t) *r++)+1) >> 1);
+    *q++=(unsigned char) ((((size_t) *p++)+((size_t) *r++)+1) >> 1);
   }
   p=pixels+(2*height-2)*scaled_width;
   q=pixels+(2*height-1)*scaled_width;
@@ -485,9 +476,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  long
-    x;
-
   MagickBooleanType
     status;
 
@@ -497,7 +485,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickSizeType
     number_pixels;
 
-  register long
+  register ssize_t
     i,
     y;
 
@@ -509,8 +497,16 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *c2,
     *yy;
 
+  size_t
+    height,
+    number_images,
+    rotate,
+    scene,
+    width;
+
   ssize_t
-    count;
+    count,
+    x;
 
   unsigned char
     *chroma1,
@@ -521,13 +517,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned int
     overview;
 
-  unsigned long
-    height,
-    number_images,
-    rotate,
-    scene,
-    width;
-
   /*
     Open image file.
   */
@@ -577,7 +566,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     }
   if (image_info->number_scenes != 0)
-    scene=(unsigned long) MagickMin(image_info->scene,6);
+    scene=(size_t) MagickMin(image_info->scene,6);
   if (overview)
     scene=1;
   /*
@@ -585,7 +574,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   width=192;
   height=128;
-  for (i=1; i < (long) MagickMin(scene,3); i++)
+  for (i=1; i < (ssize_t) MagickMin(scene,3); i++)
   {
     width<<=1;
     height<<=1;
@@ -593,19 +582,14 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=width;
   image->rows=height;
   image->depth=8;
-  for ( ; i < (long) scene; i++)
+  for ( ; i < (ssize_t) scene; i++)
   {
     image->columns<<=1;
     image->rows<<=1;
   }
   /*
-    Allocate lumand chroma memory.
+    Allocate luma and chroma memory.
   */
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels != (size_t) number_pixels)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -630,7 +614,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     else
       if (scene <= 1)
         offset=1;
-  for (i=0; i < (long) (offset*0x800); i++)
+  for (i=0; i < (ssize_t) (offset*0x800); i++)
     (void) ReadBlobByte(image);
   if (overview)
     {
@@ -640,28 +624,28 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       MagickProgressMonitor
         progress_monitor;
 
-      register long
+      register ssize_t
         j;
 
       /*
         Read thumbnails from overview image.
       */
-      for (j=1; j <= (long) number_images; j++)
+      for (j=1; j <= (ssize_t) number_images; j++)
       {
         progress_monitor=SetImageProgressMonitor(image,
           (MagickProgressMonitor) NULL,image->client_data);
         (void) FormatMagickString(image->filename,MaxTextExtent,
-          "images/img%04ld.pcd",j);
+          "images/img%04ld.pcd",(long) j);
         (void) FormatMagickString(image->magick_filename,MaxTextExtent,
-          "images/img%04ld.pcd",j);
-        image->scene=(unsigned long) j;
+          "images/img%04ld.pcd",(long) j);
+        image->scene=(size_t) j;
         image->columns=width;
         image->rows=height;
         image->depth=8;
         yy=luma;
         c1=chroma1;
         c2=chroma2;
-        for (y=0; y < (long) height; y+=2)
+        for (y=0; y < (ssize_t) height; y+=2)
         {
           count=ReadBlob(image,width,yy);
           yy+=image->columns;
@@ -680,25 +664,25 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         yy=luma;
         c1=chroma1;
         c2=chroma2;
-        for (y=0; y < (long) image->rows; y++)
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(*yy++);
-            q->green=ScaleCharToQuantum(*c1++);
-            q->blue=ScaleCharToQuantum(*c2++);
+            SetRedPixelComponent(q,ScaleCharToQuantum(*yy++));
+            SetGreenPixelComponent(q,ScaleCharToQuantum(*c1++));
+            SetBluePixelComponent(q,ScaleCharToQuantum(*c2++));
             q++;
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
         }
         image->colorspace=YCCColorspace;
         if (LocaleCompare(image_info->magick,"PCDS") == 0)
           image->colorspace=sRGBColorspace;
-        if (j < (long) number_images)
+        if (j < (ssize_t) number_images)
           {
             /*
               Allocate next image structure.
@@ -733,7 +717,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   yy=luma;
   c1=chroma1;
   c2=chroma2;
-  for (y=0; y < (long) height; y+=2)
+  for (y=0; y < (ssize_t) height; y+=2)
   {
     count=ReadBlob(image,width,yy);
     yy+=image->columns;
@@ -788,23 +772,24 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   yy=luma;
   c1=chroma1;
   c2=chroma2;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      q->red=ScaleCharToQuantum(*yy++);
-      q->green=ScaleCharToQuantum(*c1++);
-      q->blue=ScaleCharToQuantum(*c2++);
+      SetRedPixelComponent(q,ScaleCharToQuantum(*yy++));
+      SetGreenPixelComponent(q,ScaleCharToQuantum(*c1++));
+      SetBluePixelComponent(q,ScaleCharToQuantum(*c2++));
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
     if (image->previous == (Image *) NULL)
       {
-        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
         if (status == MagickFalse)
           break;
       }
@@ -874,10 +859,10 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPCDImage method is:
 %
-%      unsigned long RegisterPCDImage(void)
+%      size_t RegisterPCDImage(void)
 %
 */
-ModuleExport unsigned long RegisterPCDImage(void)
+ModuleExport size_t RegisterPCDImage(void)
 {
   MagickInfo
     *entry;
@@ -887,6 +872,7 @@ ModuleExport unsigned long RegisterPCDImage(void)
   entry->encoder=(EncodeImageHandler *) WritePCDImage;
   entry->magick=(IsImageFormatHandler *) IsPCD;
   entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Photo CD");
   entry->module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
@@ -894,6 +880,7 @@ ModuleExport unsigned long RegisterPCDImage(void)
   entry->decoder=(DecodeImageHandler *) ReadPCDImage;
   entry->encoder=(EncodeImageHandler *) WritePCDImage;
   entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Photo CD");
   entry->module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
@@ -936,8 +923,7 @@ ModuleExport void UnregisterPCDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  WritePCDImage() writes an image in the Photo CD encoded image
-%  format.
+%  WritePCDImage() writes an image in the Photo CD encoded image format.
 %
 %  The format of the WritePCDImage method is:
 %
@@ -949,7 +935,6 @@ ModuleExport void UnregisterPCDImage(void)
 %
 %    o image:  The image.
 %
-%
 */
 
 static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
@@ -962,9 +947,6 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
     *downsample_image,
     *tile_image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -978,10 +960,13 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
     *p,
     *q;
 
-  register long
+  register ssize_t
     i,
     x;
 
+  ssize_t
+    y;
+
   /*
     Scale image to tile size.
   */
@@ -997,8 +982,8 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
   if (tile_image == (Image *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(page_geometry,&geometry_info);
-  geometry.width=(unsigned long) geometry_info.rho;
-  geometry.height=(unsigned long) geometry_info.sigma;
+  geometry.width=(size_t) geometry_info.rho;
+  geometry.height=(size_t) geometry_info.sigma;
   if ((flags & SigmaValue) == 0)
     geometry.height=geometry.width;
   if ((tile_image->columns != geometry.width) ||
@@ -1023,7 +1008,7 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
     }
   (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(tile_image,YCCColorspace);
+    (void) TransformImageColorspace(tile_image,YCCColorspace);
   downsample_image=ResizeImage(tile_image,tile_image->columns/2,
     tile_image->rows/2,TriangleFilter,1.0,&image->exception);
   if (downsample_image == (Image *) NULL)
@@ -1031,33 +1016,33 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
   /*
     Write tile to PCD file.
   */
-  for (y=0; y < (long) tile_image->rows; y+=2)
+  for (y=0; y < (ssize_t) tile_image->rows; y+=2)
   {
-    p=AcquireImagePixels(tile_image,0,y,tile_image->columns,2,
+    p=GetVirtualPixels(tile_image,0,y,tile_image->columns,2,
       &tile_image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) (tile_image->columns << 1); x++)
+    for (x=0; x < (ssize_t) (tile_image->columns << 1); x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(GetRedPixelComponent(p)));
       p++;
     }
-    q=AcquireImagePixels(downsample_image,0,y >> 1,downsample_image->columns,
+    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,
       1,&downsample_image->exception);
     if (q == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) downsample_image->columns; x++)
+    for (x=0; x < (ssize_t) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(q->green));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(GetGreenPixelComponent(q)));
       q++;
     }
-    q=AcquireImagePixels(downsample_image,0,y >> 1,downsample_image->columns,
+    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,
       1,&downsample_image->exception);
     if (q == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) downsample_image->columns; x++)
+    for (x=0; x < (ssize_t) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(q->blue));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(GetBluePixelComponent(q)));
       q++;
     }
     status=SetImageProgress(image,SaveImageTag,y,tile_image->rows);
@@ -1079,7 +1064,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     i;
 
   assert(image_info != (const ImageInfo *) NULL);
@@ -1109,7 +1094,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(pcd_image,RGBColorspace);
+    (void) TransformImageColorspace(pcd_image,RGBColorspace);
   /*
     Write PCD image header.
   */
@@ -1133,7 +1118,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(pcd_image,0x01);
   for (i=0; i < 1944; i++)
     (void) WriteBlobByte(pcd_image,'\0');
-  (void) WriteBlob(pcd_image,7,(unsigned char *) "PCD_IPI");
+  (void) WriteBlob(pcd_image,7,(const unsigned char *) "PCD_IPI");
   (void) WriteBlobByte(pcd_image,0x06);
   for (i=0; i < 1530; i++)
     (void) WriteBlobByte(pcd_image,'\0');
