@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,6 +43,7 @@
 #include "magick/property.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -59,6 +60,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
+#include "magick/option.h"
 #include "magick/profile.h"
 #include "magick/resource_.h"
 #include "magick/quantum-private.h"
@@ -95,21 +97,19 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsPCL(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
     return(MagickFalse);
-  if (memcmp(magick,"\033E\033",3) == 0)
-    return(MagickTrue);
   if (memcmp(magick,"\033E\033&",4) == 0)
     return(MagickFalse);
+  if (memcmp(magick,"\033E\033",3) == 0)
+    return(MagickTrue);
   return(MagickFalse);
 }
 
@@ -178,19 +178,19 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register char
     *p;
 
-  register long
+  register ssize_t
     c;
 
   SegmentInfo
     bounds;
 
-  ssize_t
-    count;
-
-  unsigned long
+  size_t
     height,
     width;
 
+  ssize_t
+    count;
+
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
@@ -254,7 +254,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     *p++=(char) c;
     if ((c != (int) '/') && (c != '\n') &&
-        ((size_t) (p-command) < MaxTextExtent))
+        ((size_t) (p-command) < (MaxTextExtent-1)))
       continue;
     *p='\0';
     p=command;
@@ -290,8 +290,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Set PCL render geometry.
     */
-    width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-    height=(unsigned long) (bounds.y2-bounds.y1+0.5);
+    width=(size_t) floor(bounds.x2-bounds.x1+0.5);
+    height=(size_t) floor(bounds.y2-bounds.y1+0.5);
     if (width > page.width)
       page.width=width;
     if (height > page.height)
@@ -305,8 +305,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
-  (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
-    page.width,page.height);
+  (void) FormatMagickString(geometry,MaxTextExtent,"%.20gx%.20g",(double)
+    page.width,(double) page.height);
   if (image_info->monochrome != MagickFalse)
     delegate_info=GetDelegateInfo("pcl:mono",(char *) NULL,exception);
   else
@@ -323,22 +323,23 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
   (void) FormatMagickString(density,MaxTextExtent,"%gx%g",
     image->x_resolution,image->y_resolution);
-  page.width=(unsigned long) (page.width*image->x_resolution/delta.x+0.5);
-  page.height=(unsigned long) (page.height*image->y_resolution/delta.y+0.5);
-  (void) FormatMagickString(options,MaxTextExtent,"-g%lux%lu ",
-    page.width,page.height);
+  page.width=(size_t) floor(page.width*image->x_resolution/delta.x+0.5);
+  page.height=(size_t) floor(page.height*image->y_resolution/delta.y+
+    0.5);
+  (void) FormatMagickString(options,MaxTextExtent,"-g%.20gx%.20g ",(double)
+     page.width,(double) page.height);
   image=DestroyImage(image);
   read_info=CloneImageInfo(image_info);
   *read_info->magick='\0';
   if (read_info->number_scenes != 0)
     {
       if (read_info->number_scenes != 1)
-        (void) FormatMagickString(options,MaxTextExtent,"-dLastPage=%lu",
-          read_info->scene+read_info->number_scenes);
+        (void) FormatMagickString(options,MaxTextExtent,"-dLastPage=%.20g",
+          (double) (read_info->scene+read_info->number_scenes));
       else
         (void) FormatMagickString(options,MaxTextExtent,
-          "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,read_info->scene+
-          read_info->number_scenes);
+          "-dFirstPage=%.20g -dLastPage=%.20g",(double) read_info->scene+1,
+          (double) (read_info->scene+read_info->number_scenes));
       read_info->number_scenes=0;
       if (read_info->scenes != (char *) NULL)
         *read_info->scenes='\0';
@@ -353,8 +354,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,density,options,
     read_info->filename,input_filename);
-  status=SystemCommand(read_info->verbose,command) != 0 ? MagickTrue :
-    MagickFalse;
+  status=SystemCommand(MagickFalse,read_info->verbose,command,exception) != 0 ?
+    MagickTrue : MagickFalse;
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
   (void) RelinquishUniqueFileResource(input_filename);
@@ -398,16 +399,16 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %  RegisterPCLImage() adds attributes for the PCL image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
-%  supports the saving of more than one frame to the same file or blob,
+%  supports the saving of more than one frame to the i file or blob,
 %  whether the format supports native in-memory I/O, and a brief
 %  description of the format.
 %
 %  The format of the RegisterPCLImage method is:
 %
-%      unsigned long RegisterPCLImage(void)
+%      size_t RegisterPCLImage(void)
 %
 */
-ModuleExport unsigned long RegisterPCLImage(void)
+ModuleExport size_t RegisterPCLImage(void)
 {
   MagickInfo
     *entry;
@@ -416,7 +417,6 @@ ModuleExport unsigned long RegisterPCLImage(void)
   entry->decoder=(DecodeImageHandler *) ReadPCLImage;
   entry->encoder=(EncodeImageHandler *) WritePCLImage;
   entry->magick=(IsImageFormatHandler *) IsPCL;
-  entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->thread_support=EncoderThreadSupport;
@@ -437,8 +437,8 @@ ModuleExport unsigned long RegisterPCLImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  UnregisterPCLImage() removes format registrations made by the
-%  PCL module from the list of supported formats.
+%  UnregisterPCLImage() removes format registrations made by the PCL module
+%  from the list of supported formats.
 %
 %  The format of the UnregisterPCLImage method is:
 %
@@ -475,28 +475,208 @@ ModuleExport void UnregisterPCLImage(void)
 %    o image:  The image.
 %
 */
-static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
+
+static size_t PCLDeltaCompressImage(const size_t length,
+  const unsigned char *previous_pixels,const unsigned char *pixels,
+  unsigned char *compress_pixels)
 {
-  char
-    buffer[MaxTextExtent];
+  int
+    delta,
+    j,
+    replacement;
 
-  long
-    y;
-
-  MagickBooleanType
-    status;
-
-  register const PixelPacket
-    *p;
-
-  register long
+  register ssize_t
+    i,
     x;
 
   register unsigned char
     *q;
 
-  unsigned long
-    density;
+  q=compress_pixels;
+  for (x=0; x < (ssize_t) length; )
+  {
+    j=0;
+    for (i=0; x < (ssize_t) length; x++)
+    {
+      if (*pixels++ != *previous_pixels++)
+        {
+          i=1;
+          break;
+        }
+      j++;
+    }
+    while (x < (ssize_t) length)
+    {
+      x++;
+      if (*pixels == *previous_pixels)
+        break;
+      i++;
+      previous_pixels++;
+      pixels++;
+    }
+    if (i == 0)
+      break;
+    replacement=j >= 31 ? 31 : j;
+    j-=replacement;
+    delta=i >= 8 ? 8 : i;
+    *q++=(unsigned char) (((delta-1) << 5) | replacement);
+    if (replacement == 31)
+      {
+        for (replacement=255; j != 0; )
+        {
+          if (replacement > j)
+            replacement=j;
+          *q++=(unsigned char) replacement;
+          j-=replacement;
+        }
+        if (replacement == 255)
+          *q++='\0';
+      }
+    for (pixels-=i; i != 0; )
+    {
+      for (i-=delta; delta != 0; delta--)
+        *q++=(*pixels++);
+      if (i == 0)
+        break;
+      delta=i;
+      if (i >= 8)
+        delta=8;
+      *q++=(unsigned char) ((delta-1) << 5);
+    }
+  }
+  return((size_t) (q-compress_pixels));
+}
+
+static size_t PCLPackbitsCompressImage(const size_t length,
+  const unsigned char *pixels,unsigned char *compress_pixels)
+{
+  int
+    count;
+
+  register ssize_t
+    x;
+
+  register unsigned char
+    *q;
+
+  ssize_t
+    j;
+
+  unsigned char
+    packbits[128];
+
+  /*
+    Compress pixels with Packbits encoding.
+  */
+  q=compress_pixels;
+  for (x=(ssize_t) length; x != 0; )
+  {
+    switch (x)
+    {
+      case 1:
+      {
+        x--;
+        *q++=0;
+        *q++=(*pixels);
+        break;
+      }
+      case 2:
+      {
+        x-=2;
+        *q++=1;
+        *q++=(*pixels);
+        *q++=pixels[1];
+        break;
+      }
+      case 3:
+      {
+        x-=3;
+        if ((*pixels == *(pixels+1)) && (*(pixels+1) == *(pixels+2)))
+          {
+            *q++=(unsigned char) ((256-3)+1);
+            *q++=(*pixels);
+            break;
+          }
+        *q++=2;
+        *q++=(*pixels);
+        *q++=pixels[1];
+        *q++=pixels[2];
+        break;
+      }
+      default:
+      {
+        if ((*pixels == *(pixels+1)) && (*(pixels+1) == *(pixels+2)))
+          {
+            /*
+              Packed run.
+            */
+            count=3;
+            while (((ssize_t) count < x) && (*pixels == *(pixels+count)))
+            {
+              count++;
+              if (count >= 127)
+                break;
+            }
+            x-=count;
+            *q++=(unsigned char) ((256-count)+1);
+            *q++=(*pixels);
+            pixels+=count;
+            break;
+          }
+        /*
+          Literal run.
+        */
+        count=0;
+        while ((*(pixels+count) != *(pixels+count+1)) ||
+               (*(pixels+count+1) != *(pixels+count+2)))
+        {
+          packbits[count+1]=pixels[count];
+          count++;
+          if (((ssize_t) count >= (x-3)) || (count >= 127))
+            break;
+        }
+        x-=count;
+        *packbits=(unsigned char) (count-1);
+        for (j=0; j <= (ssize_t) count; j++)
+          *q++=packbits[j];
+        pixels+=count;
+        break;
+      }
+    }
+  }
+  *q++=128; /* EOD marker */
+  return((size_t) (q-compress_pixels));
+}
+
+static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
+{
+  char
+    buffer[MaxTextExtent];
+
+  const char
+    *option;
+
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
+  register const IndexPacket
+    *indexes;
+
+  register const PixelPacket *p; register ssize_t i, x; register unsigned char *q; size_t density, length,
+    one,
+    packets;
+
+  ssize_t
+    y;
+
+  unsigned char
+    bits_per_pixel,
+    *compress_pixels,
+    *pixels,
+    *previous_pixels;
 
   /*
     Open output image file.
@@ -510,13 +690,6 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
-  /*
-    Initialize the printer.
-  */
-  (void) WriteBlobString(image,"\033E");  /* printer reset */
-  (void) WriteBlobString(image,"\033&l0E");  /* top margin 0 */
   density=75;
   if (image_info->density != (char *) NULL)
     {
@@ -524,108 +697,269 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
         geometry;
 
       (void) ParseGeometry(image_info->density,&geometry);
-      density=(unsigned long) geometry.rho;
+      density=(size_t) geometry.rho;
     }
-  if (IsMonochromeImage(image,&image->exception) != MagickFalse)
-    {
-      register unsigned char
-        bit,
-        byte;
-
-      /*
-        Write PCL monochrome image.
-      */
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%ldR",density);
-      (void) WriteBlobString(image,buffer);
-      (void) WriteBlobString(image,"\033*r1A");  /* start graphics */
-      (void) WriteBlobString(image,"\033*b0M");  /* no compression */
-      for (y=0; y < (long) image->rows; y++)
+  scene=0;
+  one=1;
+  do
+  {
+    if (image->colorspace != RGBColorspace)
+      (void) TransformImageColorspace(image,RGBColorspace);
+    /*
+      Initialize the printer.
+    */
+    (void) WriteBlobString(image,"\033E");  /* printer reset */
+    (void) WriteBlobString(image,"\033*r3F");  /* set presentation mode */
+    (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%.20gs%.20gT",
+      (double) image->columns,(double) image->rows);
+    (void) WriteBlobString(image,buffer);
+    (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%.20gR",(double)
+      density);
+    (void) WriteBlobString(image,buffer);
+    (void) WriteBlobString(image,"\033&l0E");  /* top margin 0 */
+    if (IsMonochromeImage(image,&image->exception) != MagickFalse)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-        if (p == (const PixelPacket *) NULL)
-          break;
-        bit=0;
-        byte=0;
-        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
-          (image->columns+7)/8);
-        (void) WriteBlobString(image,buffer);
-        for (x=0; x < (long) image->columns; x++)
-        {
-          byte<<=1;
-          if (PixelIntensity(p) < ((MagickRealType) QuantumRange/2.0))
-            byte|=0x01;
-          bit++;
-          if (bit == 8)
-            {
-              (void) WriteBlobByte(image,byte);
-              bit=0;
-              byte=0;
-            }
-          p++;
-        }
-        if (bit != 0)
-          (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
-        status=SetImageProgress(image,SaveImageTag,y,image->rows);
-        if (status == MagickFalse)
-          break;
+        /*
+          Monochrome image: use default printer monochrome setup.
+        */
+        bits_per_pixel=1;
       }
-      (void) WriteBlobString(image,"\033*rB");  /* end graphics */
-    }
-  else
-    {
-      static char
-        color_mode[6] = { 0, 3, 0, 8, 8, 8 };
-
-      unsigned char
-        *pixels;
-
-      /*
-        Allocate pixel buffers.
-      */
-      pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
-        3UL*sizeof(*pixels));
-      if (pixels == (unsigned char *) NULL)
-        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-      /*
-        Write PCL color image.
-      */
-      (void) WriteBlobString(image,"\033*r3F");  /* set presentation mode */
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%luR",density);  /* set resolution */
-      (void) WriteBlobString(image,buffer);
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%luT",image->rows);
-      (void) WriteBlobString(image,buffer);
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%luS",
-        image->columns);
-      (void) WriteBlobString(image,buffer);
-      (void) WriteBlobString(image,"\033*v6W");  /* set color mode */
-      (void) WriteBlob(image,6,(unsigned char *) color_mode);
+    else
+      if (image->storage_class == DirectClass)
+        {
+          /*
+            DirectClass image.
+          */
+          bits_per_pixel=24;
+          (void) WriteBlobString(image,"\033*v6W"); /* set color mode */
+          (void) WriteBlobByte(image,0); /* RGB */
+          (void) WriteBlobByte(image,3); /* direct by pixel */
+          (void) WriteBlobByte(image,0); /* bits per index (ignored) */
+          (void) WriteBlobByte(image,8); /* bits per red component */
+          (void) WriteBlobByte(image,8); /* bits per green component */
+          (void) WriteBlobByte(image,8); /* bits per blue component */
+        }
+      else
+        {
+          /*
+            Colormapped image.
+          */
+          bits_per_pixel=8;
+          (void) WriteBlobString(image,"\033*v6W"); /* set color mode... */
+          (void) WriteBlobByte(image,0); /* RGB */
+          (void) WriteBlobByte(image,1); /* indexed by pixel */
+          (void) WriteBlobByte(image,bits_per_pixel); /* bits per index */
+          (void) WriteBlobByte(image,8); /* bits per red component */
+          (void) WriteBlobByte(image,8); /* bits per green component */
+          (void) WriteBlobByte(image,8); /* bits per blue component */
+          for (i=0; i < (ssize_t) image->colors; i++)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,
+              "\033*v%da%db%dc%.20gI",
+              ScaleQuantumToChar(image->colormap[i].red),
+              ScaleQuantumToChar(image->colormap[i].green),
+              ScaleQuantumToChar(image->colormap[i].blue),(double) i);
+            (void) WriteBlobString(image,buffer);
+          }
+          for (one=1; i < (ssize_t) (one << bits_per_pixel); i++)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,"\033*v%.20gI",
+              (double) i);
+            (void) WriteBlobString(image,buffer);
+          }
+        }
+    option=GetImageOption(image_info,"pcl:fit-to-page");
+    if ((option != (const char *) NULL) &&
+        (IsMagickTrue(option) != MagickFalse))
+      (void) WriteBlobString(image,"\033*r3A");
+    else
       (void) WriteBlobString(image,"\033*r1A");  /* start raster graphics */
-      (void) WriteBlobString(image,"\033*b0Y");  /* set y offset */
-      (void) WriteBlobString(image,"\033*b0M");  /* no compression */
-      for (y=0; y < (long) image->rows; y++)
+    (void) WriteBlobString(image,"\033*b0Y");  /* set y offset */
+    length=(image->columns*bits_per_pixel+7)/8;
+    pixels=(unsigned char *) AcquireQuantumMemory(length+1,sizeof(*pixels));
+    if (pixels == (unsigned char *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    (void) ResetMagickMemory(pixels,0,(length+1)*sizeof(*pixels));
+    compress_pixels=(unsigned char *) NULL;
+    previous_pixels=(unsigned char *) NULL;
+    switch (image->compression)
+    {
+      case NoCompression:
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-        if (p == (const PixelPacket *) NULL)
-          break;
-        q=pixels;
-        for (x=0; x < (long) image->columns; x++)
-        {
-          *q++=ScaleQuantumToChar(p->red);
-          *q++=ScaleQuantumToChar(p->green);
-          *q++=ScaleQuantumToChar(p->blue);
-          p++;
-        }
-        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
-          3*image->columns);  /* send row */
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b0M");
         (void) WriteBlobString(image,buffer);
-        (void) WriteBlob(image,3*image->columns,pixels);
-        status=SetImageProgress(image,SaveImageTag,y,image->rows);
-        if (status == MagickFalse)
-          break;
+        break;
+      }
+      case RLECompression:
+      {
+        compress_pixels=(unsigned char *) AcquireQuantumMemory(length+256,
+          sizeof(*compress_pixels));
+        if (compress_pixels == (unsigned char *) NULL)
+          {
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+        (void) ResetMagickMemory(compress_pixels,0,(length+256)*
+          sizeof(*compress_pixels));
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b2M");
+        (void) WriteBlobString(image,buffer);
+        break;
+      }
+      default:
+      {
+        compress_pixels=(unsigned char *) AcquireQuantumMemory(3*length+256,
+          sizeof(*compress_pixels));
+        if (compress_pixels == (unsigned char *) NULL)
+          {
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+        (void) ResetMagickMemory(compress_pixels,0,(3*length+256)*
+          sizeof(*compress_pixels));
+        previous_pixels=(unsigned char *) AcquireQuantumMemory(length+1,
+          sizeof(*previous_pixels));
+        if (previous_pixels == (unsigned char *) NULL)
+          {
+            compress_pixels=(unsigned char *) RelinquishMagickMemory(
+              compress_pixels);
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+        (void) ResetMagickMemory(previous_pixels,0,(length+1)*
+          sizeof(*previous_pixels));
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b3M");
+        (void) WriteBlobString(image,buffer);
+        break;
       }
-      (void) WriteBlobString(image,"\033*r0C");  /* end graphics */
-      pixels=(unsigned char *) RelinquishMagickMemory(pixels);
     }
+    for (y=0; y < (ssize_t) image->rows; y++)
+    {
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+      if (p == (const PixelPacket *) NULL)
+        break;
+      indexes=GetAuthenticIndexQueue(image);
+      q=pixels;
+      switch (bits_per_pixel)
+      {
+        case 1:
+        {
+          register unsigned char
+            bit,
+            byte;
+
+          /*
+            Monochrome image.
+          */
+          bit=0;
+          byte=0;
+          for (x=0; x < (ssize_t) image->columns; x++)
+          {
+            byte<<=1;
+            if (PixelIntensity(p) < ((MagickRealType) QuantumRange/2.0))
+              byte|=0x01;
+            bit++;
+            if (bit == 8)
+              {
+                *q++=byte;
+                bit=0;
+                byte=0;
+              }
+            p++;
+          }
+          if (bit != 0)
+            *q++=byte << (8-bit);
+          break;
+        }
+        case 8:
+        {
+          /*
+            Colormapped image.
+          */
+          for (x=0; x < (ssize_t) image->columns; x++)
+            *q++=(unsigned char) GetIndexPixelComponent(indexes+x);
+          break;
+        }
+        case 24:
+        case 32:
+        {
+          /*
+            Truecolor image.
+          */
+          for (x=0; x < (ssize_t) image->columns; x++)
+          {
+            *q++=ScaleQuantumToChar(GetRedPixelComponent(p));
+            *q++=ScaleQuantumToChar(GetGreenPixelComponent(p));
+            *q++=ScaleQuantumToChar(GetBluePixelComponent(p));
+            p++;
+          }
+          break;
+        }
+      }
+      switch (image->compression)
+      {
+        case NoCompression:
+        {
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%.20gW",
+            (double) length);
+          (void) WriteBlobString(image,buffer);
+          (void) WriteBlob(image,length,pixels);
+          break;
+        }
+        case RLECompression:
+        {
+          packets=PCLPackbitsCompressImage(length,pixels,compress_pixels);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%.20gW",
+            (double) packets);
+          (void) WriteBlobString(image,buffer);
+          (void) WriteBlob(image,packets,compress_pixels);
+          break;
+        }
+        default:
+        {
+          if (y == 0)
+            for (i=0; i < (ssize_t) length; i++)
+              previous_pixels[i]=(~pixels[i]);
+          packets=PCLDeltaCompressImage(length,previous_pixels,pixels,
+            compress_pixels);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%.20gW",
+            (double) packets);
+          (void) WriteBlobString(image,buffer);
+          (void) WriteBlob(image,packets,compress_pixels);
+          (void) CopyMagickMemory(previous_pixels,pixels,length*
+            sizeof(*pixels));
+          break;
+        }
+      }
+    }
+    (void) WriteBlobString(image,"\033*rB");  /* end graphics */
+    switch (image->compression)
+    {
+      case NoCompression:
+        break;
+      case RLECompression:
+      {
+        compress_pixels=(unsigned char *) RelinquishMagickMemory(
+          compress_pixels);
+        break;
+      }
+      default:
+      {
+        previous_pixels=(unsigned char *) RelinquishMagickMemory(
+          previous_pixels);
+        compress_pixels=(unsigned char *) RelinquishMagickMemory(
+          compress_pixels);
+        break;
+      }
+    }
+    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+    if (GetNextImageInList(image) == (Image *) NULL)
+      break;
+    image=SyncNextImageInList(image);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
+  } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"\033E");
   (void) CloseBlob(image);
   return(MagickTrue);
