@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -37,9 +37,9 @@
     20071202 TS * rewrote RLE decoder - old version could cause buffer overflows
                 * failure of RLE decoding now thows error RLEDecoderError
                 * fixed bug in RLE decoding - now all rows are decoded, not just
-		  the first one
-		* fixed bug in reader - record offsets now handled correctly
-		* fixed bug in reader - only bits 0..2 indicate compression type
+      the first one
+    * fixed bug in reader - record offsets now handled correctly
+    * fixed bug in reader - only bits 0..2 indicate compression type
                 * in writer: now using image color count instead of depth
 */
 
@@ -47,9 +47,13 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
@@ -80,7 +84,7 @@ typedef struct _PDBInfo
     attributes,
     version;
 
-  unsigned long
+  size_t
     create_time,
     modify_time,
     archive_time,
@@ -92,7 +96,7 @@ typedef struct _PDBInfo
     type[4],  /* database type identifier "vIMG" */
     id[4];    /* database creator identifier "View" */
 
-  unsigned long
+  size_t
     seed,
     next_record;
 
@@ -107,7 +111,7 @@ typedef struct _PDBImage
     version,
     type;
 
-  unsigned long
+  size_t
     reserved_1,
     note;
 
@@ -115,7 +119,7 @@ typedef struct _PDBImage
     x_last,
     y_last;
 
-  unsigned long
+  size_t
     reserved_2;
 
   short int
@@ -166,33 +170,33 @@ static MagickBooleanType DecodeImage(Image *image, unsigned char *pixels,
 #define RLE_MODE_COPY  0
 #define RLE_MODE_RUN   1
 
-	int           data = 0, count = 0;
-	unsigned char *p;
-	int           mode = RLE_MODE_NONE;
+  int           data = 0, count = 0;
+  unsigned char *p;
+  int           mode = RLE_MODE_NONE;
 
-	for (p = pixels; p < pixels + length; p++) {
-		if (0 == count) {
-			data = ReadBlobByte( image );
-			if (-1 == data) return MagickFalse;
-			if (data > 128) {
-				mode  = RLE_MODE_RUN;
-				count = data - 128 + 1;
-				data  = ReadBlobByte( image );
-				if (-1 == data) return MagickFalse;
-			} else {
-				mode  = RLE_MODE_COPY;
-				count = data + 1;
-			}
-		}
+  for (p = pixels; p < pixels + length; p++) {
+    if (0 == count) {
+      data = ReadBlobByte( image );
+      if (-1 == data) return MagickFalse;
+      if (data > 128) {
+        mode  = RLE_MODE_RUN;
+        count = data - 128 + 1;
+        data  = ReadBlobByte( image );
+        if (-1 == data) return MagickFalse;
+      } else {
+        mode  = RLE_MODE_COPY;
+        count = data + 1;
+      }
+    }
 
-		if (RLE_MODE_COPY == mode) {
-			data = ReadBlobByte( image );
-			if (-1 == data) return MagickFalse;
-		}
-		*p = (unsigned char)data;
-		--count;
-	}
-	return MagickTrue;
+    if (RLE_MODE_COPY == mode) {
+      data = ReadBlobByte( image );
+      if (-1 == data) return MagickFalse;
+    }
+    *p = (unsigned char)data;
+    --count;
+  }
+  return MagickTrue;
 }
 
 /*
@@ -215,8 +219,7 @@ static MagickBooleanType DecodeImage(Image *image, unsigned char *pixels,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -268,11 +271,6 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  long
-    img_offset, /* TS */
-    comment_offset = 0,
-    y;
-
   MagickBooleanType
     status;
 
@@ -285,7 +283,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
@@ -294,17 +292,21 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
+  size_t
+    bits_per_pixel,
+    num_pad_bytes, /* TS */
+    one,
+    packets;
+
   ssize_t
-    count;
+    count,
+    img_offset, /* TS */
+    comment_offset = 0,
+    y;
 
   unsigned char
     *pixels;
 
-  unsigned long
-    num_pad_bytes, /* TS */
-    bits_per_pixel,
-    packets;
-
   /*
     Open image file.
   */
@@ -347,22 +349,21 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read record header.
   */
-  img_offset=(long) ReadBlobMSBLong(image); /* TS */
+  img_offset=(int) ReadBlobMSBLong(image); /* TS */
   attributes=(unsigned char) ReadBlobByte(image);
+  (void) attributes;
   count=ReadBlob(image,3,(unsigned char *) tag);
   if (count != 3  ||  memcmp(tag,"\x6f\x80\x00",3) != 0)
     ThrowReaderException(CorruptImageError,"CorruptImage");
-
   if (pdb_info.number_records > 1)
     {
-      comment_offset=(long) ReadBlobMSBLong(image);
+      comment_offset=(int) ReadBlobMSBLong(image);
       attributes=(unsigned char) ReadBlobByte(image);
       count=ReadBlob(image,3,(unsigned char *) tag);
       if (count != 3  ||  memcmp(tag,"\x6f\x80\x01",3) != 0)
         ThrowReaderException(CorruptImageError,"CorruptImage");
     }
-
-  num_pad_bytes = (unsigned long) (img_offset - TellBlob( image ));
+  num_pad_bytes = (size_t) (img_offset - TellBlob( image ));
   while (num_pad_bytes--) ReadBlobByte( image );
   /*
     Read image header.
@@ -382,23 +383,19 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image structure.
   */
-  image->columns=(unsigned long) pdb_image.width;
-  image->rows=(unsigned long) pdb_image.height;
+  image->columns=(size_t) pdb_image.width;
+  image->rows=(size_t) pdb_image.height;
   image->depth=8;
   image->storage_class=PseudoClass;
   bits_per_pixel=pdb_image.type == 0 ? 2UL : pdb_image.type == 2 ? 4UL : 1UL;
-  if (AcquireImageColormap(image,1 << bits_per_pixel) == MagickFalse)
+  one=1;
+  if (AcquireImageColormap(image,one << bits_per_pixel) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->ping != MagickFalse)
     {
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   packets=bits_per_pixel*image->columns/8;
   pixels=(unsigned char *) AcquireQuantumMemory(packets+256UL,image->rows*
     sizeof(*pixels));
@@ -417,12 +414,12 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       image->compression=RLECompression;
       if (!DecodeImage(image, pixels, packets * image -> rows))
-      	ThrowReaderException( CorruptImageError, "RLEDecoderError" ); /* TS */
+        ThrowReaderException( CorruptImageError, "RLEDecoderError" ); /* TS */
       break;
     }
     default:
       ThrowReaderException(CorruptImageError,
-			   "UnrecognizedImageCompressionType" );
+         "UnrecognizedImageCompressionType" );
   }
   p=pixels;
   switch (bits_per_pixel)
@@ -435,28 +432,29 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Read 1-bit PDB image.
       */
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-7); x+=8)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-7); x+=8)
         {
           for (bit=0; bit < 8; bit++)
           {
             index=(IndexPacket) (*p & (0x80 >> bit) ? 0x00 : 0x01);
-            indexes[x+bit]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x+bit,index);
           }
           p++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+          image->rows);
         if (status == MagickFalse)
           break;
       }
+      (void) SyncImage(image);
       break;
     }
     case 2:
@@ -464,34 +462,32 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Read 2-bit PDB image.
       */
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < (long) image->columns; x+=4)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < (ssize_t) image->columns; x+=4)
         {
           index=ConstrainColormapIndex(image,3UL-((*p >> 6) & 0x03));
-          indexes[x]=index;
-          *q++=image->colormap[(long) index];
+          SetIndexPixelComponent(indexes+x,index);
           index=ConstrainColormapIndex(image,3UL-((*p >> 4) & 0x03));
-          indexes[x+1]=index;
-          *q++=image->colormap[(long) index];
+          SetIndexPixelComponent(indexes+x+1,index);
           index=ConstrainColormapIndex(image,3UL-((*p >> 2) & 0x03));
-          indexes[x+2]=index;
-          *q++=image->colormap[(long) index];
+          SetIndexPixelComponent(indexes+x+2,index);
           index=ConstrainColormapIndex(image,3UL-((*p) & 0x03));
-          indexes[x+3]=index;
-          *q++=image->colormap[(long) index];
+          SetIndexPixelComponent(indexes+x+3,index);
           p++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+          image->rows);
         if (status == MagickFalse)
           break;
       }
+      (void) SyncImage(image);
       break;
     }
     case 4:
@@ -499,28 +495,28 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Read 4-bit PDB image.
       */
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < (long) image->columns; x+=2)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < (ssize_t) image->columns; x+=2)
         {
           index=ConstrainColormapIndex(image,15UL-((*p >> 4) & 0x0f));
-          indexes[x]=index;
-          *q++=image->colormap[(long) index];
+          SetIndexPixelComponent(indexes+x,index);
           index=ConstrainColormapIndex(image,15UL-((*p) & 0x0f));
-          indexes[x+1]=index;
-          *q++=image->colormap[(long) index];
+          SetIndexPixelComponent(indexes+x+1,index);
           p++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+          image->rows);
         if (status == MagickFalse)
           break;
       }
+      (void) SyncImage(image);
       break;
     }
     default:
@@ -542,10 +538,10 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       register char
         *p;
 
-      unsigned long
+      size_t
         length;
 
-      num_pad_bytes = (unsigned long) (comment_offset - TellBlob( image ));
+      num_pad_bytes = (size_t) (comment_offset - TellBlob( image ));
       while (num_pad_bytes--) ReadBlobByte( image );
 
       /*
@@ -600,10 +596,10 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPDBImage method is:
 %
-%      unsigned long RegisterPDBImage(void)
+%      size_t RegisterPDBImage(void)
 %
 */
-ModuleExport unsigned long RegisterPDBImage(void)
+ModuleExport size_t RegisterPDBImage(void)
 {
   MagickInfo
     *entry;
@@ -669,7 +665,7 @@ ModuleExport void UnregisterPDBImage(void)
 */
 
 static unsigned char *EncodeRLE(unsigned char *destination,
-  unsigned char *source,unsigned long literal,unsigned long repeat)
+  unsigned char *source,size_t literal,size_t repeat)
 {
   if (literal > 0)
     *destination++=(unsigned char) (literal-1);
@@ -691,9 +687,6 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   int
     bits;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -704,31 +697,32 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     pdb_info;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
 
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     x;
 
   register unsigned char
     *q;
 
   size_t
-    packet_size;
+    bits_per_pixel,
+    literal,
+    packets,
+    packet_size,
+    repeat;
+
+  ssize_t
+    y;
 
   unsigned char
     *buffer,
     *runlength,
     *scanline;
 
-  unsigned long
-    bits_per_pixel,
-    literal,
-    packets,
-    repeat;
-
   /*
     Open output image file.
   */
@@ -742,7 +736,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
 
   if (image -> colors <= 2  ||  GetImageType( image, &image -> exception ) == BilevelType) { /* TS */
     bits_per_pixel = 1;
@@ -773,16 +767,16 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlob(image,32,(unsigned char *) pdb_info.name);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_info.attributes);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_info.version);
-  (void) WriteBlobMSBLong(image,pdb_info.create_time);
-  (void) WriteBlobMSBLong(image,pdb_info.modify_time);
-  (void) WriteBlobMSBLong(image,pdb_info.archive_time);
-  (void) WriteBlobMSBLong(image,pdb_info.modify_number);
-  (void) WriteBlobMSBLong(image,pdb_info.application_info);
-  (void) WriteBlobMSBLong(image,pdb_info.sort_info);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.create_time);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.modify_time);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.archive_time);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.modify_number);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.application_info);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.sort_info);
   (void) WriteBlob(image,4,(unsigned char *) pdb_info.type);
   (void) WriteBlob(image,4,(unsigned char *) pdb_info.id);
-  (void) WriteBlobMSBLong(image,(unsigned long) pdb_info.seed);
-  (void) WriteBlobMSBLong(image,(unsigned long) pdb_info.next_record);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.seed);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_info.next_record);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_info.number_records);
   (void) CopyMagickString(pdb_image.name,pdb_info.name,32);
   pdb_image.version=1;  /* RLE Compressed */
@@ -816,26 +810,29 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Convert to GRAY raster scanline.
   */
-  GetQuantumInfo(image_info,&quantum_info);
-  bits=8/(long) bits_per_pixel-1;  /* start at most significant bits */
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  bits=8/(int) bits_per_pixel-1;  /* start at most significant bits */
   literal=0;
   repeat=0;
   q=runlength;
   buffer[0]=0x00;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ExportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
+    (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+      GrayQuantum,scanline,&image->exception);
     for (x=0; x < pdb_image.width; x++)
     {
-      if (x < (long) image->columns)
+      if (x < (ssize_t) image->columns)
         buffer[literal+repeat]|=(0xff-scanline[x*packet_size]) >>
           (8-bits_per_pixel) << bits*bits_per_pixel;
       bits--;
@@ -876,21 +873,23 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
                   literal-=0x80;
                 }
             }
-        bits=8/(long) bits_per_pixel-1;
+        bits=8/(int) bits_per_pixel-1;
         buffer[literal+repeat]=0x00;
       }
     }
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
     if (status == MagickFalse)
       break;
   }
   q=EncodeRLE(q,buffer,literal,repeat);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  quantum_info=DestroyQuantumInfo(quantum_info);
   /*
     Write the Image record header.
   */
-  (void) WriteBlobMSBLong(image,(unsigned long)
+  (void) WriteBlobMSBLong(image,(unsigned int)
     (TellBlob(image)+8*pdb_info.number_records));
   (void) WriteBlobByte(image,0x40);
   (void) WriteBlobByte(image,0x6f);
@@ -901,7 +900,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
       /*
         Write the comment record header.
       */
-      (void) WriteBlobMSBLong(image,(unsigned long) (TellBlob(image)+8+58+q-
+      (void) WriteBlobMSBLong(image,(unsigned int) (TellBlob(image)+8+58+q-
         runlength));
       (void) WriteBlobByte(image,0x40);
       (void) WriteBlobByte(image,0x6f);
@@ -914,11 +913,11 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlob(image,32,(unsigned char *) pdb_image.name);
   (void) WriteBlobByte(image,(unsigned char) pdb_image.version);
   (void) WriteBlobByte(image,(unsigned char) pdb_image.type);
-  (void) WriteBlobMSBLong(image,pdb_image.reserved_1);
-  (void) WriteBlobMSBLong(image,pdb_image.note);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_image.reserved_1);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_image.note);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.x_last);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.y_last);
-  (void) WriteBlobMSBLong(image,pdb_image.reserved_2);
+  (void) WriteBlobMSBLong(image,(unsigned int) pdb_image.reserved_2);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.x_anchor);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.y_anchor);
   (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.width);
