@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,7 +42,10 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
+#include "magick/colormap-private.h"
 #include "magick/colorspace.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
@@ -106,7 +109,7 @@ typedef struct _PICTCode
   const char
     *name;
 
-  long
+  ssize_t
     length;
 
   const char
@@ -119,7 +122,7 @@ typedef struct _PICTPixmap
     version,
     pack_type;
 
-  unsigned long
+  size_t
     pack_size,
     horizontal_resolution,
     vertical_resolution;
@@ -130,7 +133,7 @@ typedef struct _PICTPixmap
     component_count,
     component_size;
 
-  unsigned long
+  size_t
     plane_bytes,
     table,
     reserved;
@@ -162,8 +165,8 @@ static const PICTCode
     /* 0x0b */ { "OvSize", 4, "oval size (point)" },
     /* 0x0c */ { "Origin", 4, "dh, dv (word)" },
     /* 0x0d */ { "TxSize", 2, "text size (word)" },
-    /* 0x0e */ { "FgColor", 4, "foreground color (longword)" },
-    /* 0x0f */ { "BkColor", 4, "background color (longword)" },
+    /* 0x0e */ { "FgColor", 4, "foreground color (ssize_tword)" },
+    /* 0x0f */ { "BkColor", 4, "background color (ssize_tword)" },
     /* 0x10 */ { "TxRatio", 8, "numerator (point), denominator (point)" },
     /* 0x11 */ { "Version", 1, "version (byte)" },
     /* 0x12 */ { "BkPixPat", 0, "color background pattern" },
@@ -335,7 +338,7 @@ static MagickBooleanType
 %  The format of the DecodeImage method is:
 %
 %      unsigned char *DecodeImage(Image *blob,Image *image,
-%        unsigned long bytes_per_line,const int bits_per_pixel,
+%        size_t bytes_per_line,const int bits_per_pixel,
 %        unsigned size_t extent)
 %
 %  A description of each parameter follows:
@@ -356,7 +359,7 @@ static MagickBooleanType
 static unsigned char *ExpandBuffer(unsigned char *pixels,
   MagickSizeType *bytes_per_line,const unsigned int bits_per_pixel)
 {
-  register long
+  register ssize_t
     i;
 
   register unsigned char
@@ -376,7 +379,7 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
       return(pixels);
     case 4:
     {
-      for (i=0; i < (long) *bytes_per_line; i++)
+      for (i=0; i < (ssize_t) *bytes_per_line; i++)
       {
         *q++=(*p >> 4) & 0xff;
         *q++=(*p & 15);
@@ -387,7 +390,7 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
     }
     case 2:
     {
-      for (i=0; i < (long) *bytes_per_line; i++)
+      for (i=0; i < (ssize_t) *bytes_per_line; i++)
       {
         *q++=(*p >> 6) & 0x03;
         *q++=(*p >> 4) & 0x03;
@@ -400,7 +403,7 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
     }
     case 1:
     {
-      for (i=0; i < (long) *bytes_per_line; i++)
+      for (i=0; i < (ssize_t) *bytes_per_line; i++)
       {
         *q++=(*p >> 7) & 0x01;
         *q++=(*p >> 6) & 0x01;
@@ -422,16 +425,12 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
 }
 
 static unsigned char *DecodeImage(Image *blob,Image *image,
-  unsigned long bytes_per_line,const unsigned int bits_per_pixel,size_t *extent)
+  size_t bytes_per_line,const unsigned int bits_per_pixel,size_t *extent)
 {
-  long
-    j,
-    y;
-
   MagickSizeType
     number_pixels;
 
-  register long
+  register ssize_t
     i;
 
   register unsigned char
@@ -439,21 +438,21 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
     *q;
 
   size_t
+    bytes_per_pixel,
     length,
-    row_bytes;
+    row_bytes,
+    scanline_length,
+    width;
 
   ssize_t
-    count;
+    count,
+    j,
+    y;
 
   unsigned char
     *pixels,
     *scanline;
 
-  unsigned long
-    bytes_per_pixel,
-    scanline_length,
-    width;
-
   /*
     Determine pixel buffer size.
   */
@@ -491,11 +490,12 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
       /*
         Pixels are already uncompressed.
       */
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
         q=pixels+y*width;
         number_pixels=bytes_per_line;
         count=ReadBlob(blob,(size_t) number_pixels,scanline);
+        (void) count;
         p=ExpandBuffer(scanline,&number_pixels,bits_per_pixel);
         if ((q+number_pixels) > (pixels+(*extent)))
           {
@@ -512,7 +512,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
   /*
     Uncompress RLE pixels into uncompressed pixel buffer.
   */
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     q=pixels+y*width;
     if (bytes_per_line > 200)
@@ -526,7 +526,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
         break;
       }
     count=ReadBlob(blob,scanline_length,scanline);
-    for (j=0; j < (long) scanline_length; )
+    for (j=0; j < (ssize_t) scanline_length; )
       if ((scanline[j] & 0x80) == 0)
         {
           length=(size_t) ((scanline[j] & 0xff)+1);
@@ -535,20 +535,20 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           if ((q-pixels+number_pixels) <= *extent)
             (void) CopyMagickMemory(q,p,(size_t) number_pixels);
           q+=number_pixels;
-          j+=(long) (length*bytes_per_pixel+1);
+          j+=(ssize_t) (length*bytes_per_pixel+1);
         }
       else
         {
           length=(size_t) (((scanline[j] ^ 0xff) & 0xff)+2);
           number_pixels=bytes_per_pixel;
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
-          for (i=0; i < (long) length; i++)
+          for (i=0; i < (ssize_t) length; i++)
           {
             if ((q-pixels+number_pixels) <= *extent)
               (void) CopyMagickMemory(q,p,(size_t) number_pixels);
             q+=number_pixels;
           }
-          j+=bytes_per_pixel+1;
+          j+=(ssize_t) bytes_per_pixel+1;
         }
   }
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -572,7 +572,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
 %  The format of the EncodeImage method is:
 %
 %      size_t EncodeImage(Image *image,const unsigned char *scanline,
-%        const unsigned long bytes_per_line,unsigned char *pixels)
+%        const size_t bytes_per_line,unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
@@ -585,23 +585,17 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
 %    o pixels: A pointer to an array of characters where the packed
 %      characters are stored.
 %
-%
 */
 static size_t EncodeImage(Image *image,const unsigned char *scanline,
-  const unsigned long bytes_per_line,unsigned char *pixels)
+  const size_t bytes_per_line,unsigned char *pixels)
 {
 #define MaxCount  128
 #define MaxPackbitsRunlength  128
 
-  long
-    count,
-    repeat_count,
-    runlength;
-
   register const unsigned char
     *p;
 
-  register long
+  register ssize_t
     i;
 
   register unsigned char
@@ -610,6 +604,11 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   size_t
     length;
 
+  ssize_t
+    count,
+    repeat_count,
+    runlength;
+
   unsigned char
     index;
 
@@ -627,7 +626,7 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   p=scanline+(bytes_per_line-1);
   q=pixels;
   index=(*p);
-  for (i=(long) bytes_per_line-1; i >= 0; i--)
+  for (i=(ssize_t) bytes_per_line-1; i >= 0; i--)
   {
     if (index == *p)
       runlength++;
@@ -736,12 +735,10 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsPICT(const unsigned char *magick,const size_t length)
 {
@@ -781,8 +778,8 @@ static MagickBooleanType IsPICT(const unsigned char *magick,const size_t length)
 %
 */
 
-static inline unsigned long MagickMax(const unsigned long x,
-  const unsigned long y)
+static inline size_t MagickMax(const size_t x,
+  const size_t y)
 {
   if (x > y)
     return(x);
@@ -805,12 +802,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     c,
     code;
 
-  long
-    flags,
-    j,
-    version,
-    y;
-
   MagickBooleanType
     jpeg,
     status;
@@ -824,13 +815,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
-  register long
+  register ssize_t
     i;
 
   size_t
@@ -838,7 +829,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     length;
 
   ssize_t
-    count;
+    count,
+    flags,
+    j,
+    version,
+    y;
 
   StringInfo
     *profile;
@@ -896,11 +891,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   image->x_resolution=DefaultResolution;
   image->y_resolution=DefaultResolution;
   image->units=UndefinedResolution;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   /*
     Interpret PICT opcodes.
   */
@@ -934,7 +924,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             length=ReadBlobMSBShort(image);
             if (length != 0x000a)
               {
-                for (i=0; i < (long) (length-2); i++)
+                for (i=0; i < (ssize_t) (length-2); i++)
                   (void) ReadBlobByte(image);
                 break;
               }
@@ -950,10 +940,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           case 0x13:
           case 0x14:
           {
-            long
+            ssize_t
               pattern;
 
-            unsigned long
+            size_t
               height,
               width;
 
@@ -981,7 +971,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             (void) ReadBlobMSBLong(image);
             flags=1L*ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
-            for (i=0; i <= (long) length; i++)
+            for (i=0; i <= (ssize_t) length; i++)
               (void) ReadBlobMSBLong(image);
             width=1UL*(frame.bottom-frame.top);
             height=1UL*(frame.right-frame.left);
@@ -993,16 +983,16 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               length=width;
             if (length < 8)
               {
-                for (i=0; i < (long) (length*height); i++)
+                for (i=0; i < (ssize_t) (length*height); i++)
                   (void) ReadBlobByte(image);
               }
             else
               for (j=0; j < (int) height; j++)
                 if (length > 200)
-                  for (j=0; j < (long) ReadBlobMSBShort(image); j++)
+                  for (j=0; j < (ssize_t) ReadBlobMSBShort(image); j++)
                     (void) ReadBlobByte(image);
                 else
-                  for (j=0; j < (long) ReadBlobByte(image); j++)
+                  for (j=0; j < (ssize_t) ReadBlobByte(image); j++)
                     (void) ReadBlobByte(image);
             break;
           }
@@ -1032,7 +1022,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               Skip polygon or region.
             */
             length=ReadBlobMSBShort(image);
-            for (i=0; i < (long) (length-2); i++)
+            for (i=0; i < (ssize_t) (length-2); i++)
               (void) ReadBlobByte(image);
             break;
           }
@@ -1043,7 +1033,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           case 0x9a:
           case 0x9b:
           {
-            long
+            ssize_t
               bytes_per_line;
 
             PICTRectangle
@@ -1082,8 +1072,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               1UL*(frame.bottom-frame.top),MagickTrue,exception);
             if (tile_image == (Image *) NULL)
               return((Image *) NULL);
-            DestroyBlob(tile_image);
-            tile_image->blob=CloneBlobInfo((BlobInfo *) NULL);
             if ((code == 0x9a) || (code == 0x9b) ||
                 ((bytes_per_line & 0x8000) != 0))
               {
@@ -1118,7 +1106,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   }
                 if ((bytes_per_line & 0x8000) != 0)
                   {
-                    for (i=0; i < (long) tile_image->colors; i++)
+                    for (i=0; i < (ssize_t) tile_image->colors; i++)
                     {
                       j=ReadBlobMSBShort(image) % tile_image->colors;
                       if ((flags & 0x8000) != 0)
@@ -1133,7 +1121,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   }
                 else
                   {
-                    for (i=0; i < (long) tile_image->colors; i++)
+                    for (i=0; i < (ssize_t) tile_image->colors; i++)
                     {
                       tile_image->colormap[i].red=(Quantum) (QuantumRange-
                         tile_image->colormap[i].red);
@@ -1153,7 +1141,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   Skip region.
                 */
                 length=ReadBlobMSBShort(image);
-                for (i=0; i <= (long) (length-2); i++)
+                for (i=0; i < (ssize_t) (length-2); i++)
                   (void) ReadBlobByte(image);
               }
             if ((code != 0x9a) && (code != 0x9b) &&
@@ -1172,23 +1160,27 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               Convert PICT tile image to pixel packets.
             */
             p=pixels;
-            for (y=0; y < (long) tile_image->rows; y++)
+            for (y=0; y < (ssize_t) tile_image->rows; y++)
             {
               if (p > (pixels+extent+image->columns))
                 ThrowReaderException(CorruptImageError,"NotEnoughPixelData");
-              q=SetImagePixels(tile_image,0,y,tile_image->columns,1);
+              q=QueueAuthenticPixels(tile_image,0,y,tile_image->columns,1,
+                exception);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(tile_image);
-              for (x=0; x < (long) tile_image->columns; x++)
+              indexes=GetAuthenticIndexQueue(tile_image);
+              for (x=0; x < (ssize_t) tile_image->columns; x++)
               {
                 if (tile_image->storage_class == PseudoClass)
                   {
                     index=ConstrainColormapIndex(tile_image,*p);
-                    indexes[x]=index;
-                    q->red=tile_image->colormap[(long) index].red;
-                    q->green=tile_image->colormap[(long) index].green;
-                    q->blue=tile_image->colormap[(long) index].blue;
+                    SetIndexPixelComponent(indexes+x,index);
+                    SetRedPixelComponent(q,
+                      tile_image->colormap[(ssize_t) index].red);
+                    SetGreenPixelComponent(q,
+                      tile_image->colormap[(ssize_t) index].green);
+                    SetBluePixelComponent(q,
+                      tile_image->colormap[(ssize_t) index].blue);
                   }
                 else
                   {
@@ -1196,12 +1188,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       {
                         i=(*p++);
                         j=(*p);
-                        q->red=ScaleCharToQuantum((unsigned char)
-                          ((i & 0x7c) << 1));
-                        q->green=ScaleCharToQuantum((unsigned char)
-                          (((i & 0x03) << 6) | ((j & 0xe0) >> 2)));
-                        q->blue=ScaleCharToQuantum((unsigned char)
-                          ((j & 0x1f) << 3));
+                        SetRedPixelComponent(q,ScaleCharToQuantum(
+                          (unsigned char) ((i & 0x7c) << 1)));
+                        SetGreenPixelComponent(q,ScaleCharToQuantum(
+                          (unsigned char) (((i & 0x03) << 6) |
+                          ((j & 0xe0) >> 2))));
+                        SetBluePixelComponent(q,ScaleCharToQuantum(
+                          (unsigned char) ((j & 0x1f) << 3)));
                       }
                     else
                       if (tile_image->matte == MagickFalse)
@@ -1209,30 +1202,31 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                           if (p > (pixels+extent+2*image->columns))
                             ThrowReaderException(CorruptImageError,
                               "NotEnoughPixelData");
-                          q->red=ScaleCharToQuantum(*p);
-                          q->green=ScaleCharToQuantum(
-                            *(p+tile_image->columns));
-                          q->blue=ScaleCharToQuantum(
-                            *(p+2*tile_image->columns));
+                          SetRedPixelComponent(q,ScaleCharToQuantum(*p));
+                          SetGreenPixelComponent(q,ScaleCharToQuantum(
+                            *(p+tile_image->columns)));
+                          SetBluePixelComponent(q,ScaleCharToQuantum(
+                            *(p+2*tile_image->columns)));
                         }
                       else
                         {
                           if (p > (pixels+extent+3*image->columns))
                             ThrowReaderException(CorruptImageError,
                               "NotEnoughPixelData");
-                          q->opacity=(Quantum) (QuantumRange-
-                            ScaleCharToQuantum(*p));
-                          q->red=ScaleCharToQuantum(*(p+tile_image->columns));
-                          q->green=(Quantum) ScaleCharToQuantum(
-                            *(p+2*tile_image->columns));
-                          q->blue=ScaleCharToQuantum(
-                            *(p+3*tile_image->columns));
+                          SetOpacityPixelComponent(q,(Quantum) (QuantumRange-
+                            ScaleCharToQuantum(*p)));
+                          SetRedPixelComponent(q,ScaleCharToQuantum(
+                            *(p+tile_image->columns)));
+                          SetGreenPixelComponent(q,ScaleCharToQuantum(
+                            *(p+2*tile_image->columns)));
+                          SetBluePixelComponent(q,ScaleCharToQuantum(
+                            *(p+3*tile_image->columns)));
                         }
                   }
                 p++;
                 q++;
               }
-              if (SyncImagePixels(tile_image) == MagickFalse)
+              if (SyncAuthenticPixels(tile_image,exception) == MagickFalse)
                 break;
               if ((tile_image->storage_class == DirectClass) &&
                   (pixmap.bits_per_pixel != 16))
@@ -1259,7 +1253,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             unsigned char
               *info;
 
-            unsigned long
+            size_t
               type;
 
             /*
@@ -1277,6 +1271,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if (info == (unsigned char *) NULL)
               break;
             count=ReadBlob(image,length,info);
+            (void) count;
             switch (type)
             {
               case 0xe0:
@@ -1319,7 +1314,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if (codes[code].length == -1)
               (void) ReadBlobMSBShort(image);
             else
-              for (i=0; i < (long) codes[code].length; i++)
+              for (i=0; i < (ssize_t) codes[code].length; i++)
                 (void) ReadBlobByte(image);
           }
         }
@@ -1359,7 +1354,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         file=(FILE *) NULL;
         unique_file=AcquireUniqueFileResource(read_info->filename);
         if (unique_file != -1)
-          file=OpenMagickStream(read_info->filename,"wb");
+          file=fdopen(unique_file,"wb");
         if ((unique_file == -1) || (file == (FILE *) NULL))
           {
             (void) CopyMagickString(image->filename,read_info->filename,
@@ -1375,7 +1370,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         ReadRectangle(image,frame);
         for (i=0; i < 122; i++)
           (void) ReadBlobByte(image);
-        for (i=0; i < (long) (length-154); i++)
+        for (i=0; i < (ssize_t) (length-154); i++)
         {
           c=ReadBlobByte(image);
           (void) fputc(c,file);
@@ -1386,12 +1381,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         read_info=DestroyImageInfo(read_info);
         if (tile_image == (Image *) NULL)
           continue;
-        (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
+        (void) FormatMagickString(geometry,MaxTextExtent,"%.20gx%.20g",
+          (double) MagickMax(image->columns,tile_image->columns),
+          (double) MagickMax(image->rows,tile_image->rows));
+        (void) SetImageExtent(image,
           MagickMax(image->columns,tile_image->columns),
           MagickMax(image->rows,tile_image->rows));
-        (void) TransformImage(&image,(char *) NULL,geometry);
         if (image->colorspace != RGBColorspace)
-          (void) SetImageColorspace(image,tile_image->colorspace);
+          (void) TransformImageColorspace(image,tile_image->colorspace);
         (void) CompositeImage(image,CopyCompositeOp,tile_image,frame.left,
           frame.right);
         image->compression=tile_image->compression;
@@ -1407,7 +1404,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           Skip reserved.
         */
         length=ReadBlobMSBShort(image);
-        for (i=0; i < (long) length; i++)
+        for (i=0; i < (ssize_t) length; i++)
           (void) ReadBlobByte(image);
         continue;
       }
@@ -1417,7 +1414,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           Skip reserved.
         */
         length=(size_t) ((code >> 7) & 0xff);
-        for (i=0; i < (long) length; i++)
+        for (i=0; i < (ssize_t) length; i++)
           (void) ReadBlobByte(image);
         continue;
       }
@@ -1447,10 +1444,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterPICTImage method is:
 %
-%      unsigned long RegisterPICTImage(void)
+%      size_t RegisterPICTImage(void)
 %
 */
-ModuleExport unsigned long RegisterPICTImage(void)
+ModuleExport size_t RegisterPICTImage(void)
 {
   MagickInfo
     *entry;
@@ -1459,6 +1456,7 @@ ModuleExport unsigned long RegisterPICTImage(void)
   entry->decoder=(DecodeImageHandler *) ReadPICTImage;
   entry->encoder=(EncodeImageHandler *) WritePICTImage;
   entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Apple Macintosh QuickDraw/PICT");
   entry->magick=(IsImageFormatHandler *) IsPICT;
   entry->module=ConstantString("PICT");
@@ -1467,6 +1465,7 @@ ModuleExport unsigned long RegisterPICTImage(void)
   entry->decoder=(DecodeImageHandler *) ReadPICTImage;
   entry->encoder=(EncodeImageHandler *) WritePICTImage;
   entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Apple Macintosh QuickDraw/PICT");
   entry->magick=(IsImageFormatHandler *) IsPICT;
   entry->module=ConstantString("PICT");
@@ -1515,7 +1514,8 @@ ModuleExport void UnregisterPICTImage(void)
 %
 %  The format of the WritePICTImage method is:
 %
-%      MagickBooleanType WritePICTImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePICTImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -1523,7 +1523,6 @@ ModuleExport void UnregisterPICTImage(void)
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   Image *image)
@@ -1545,9 +1544,6 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     x_resolution,
     y_resolution;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -1565,27 +1561,29 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     size_rectangle,
     source_rectangle;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
-  register long
+  register ssize_t
     i,
     x;
 
   size_t
-    count;
+    bytes_per_line,
+    count,
+    storage_class;
+
+  ssize_t
+    y;
 
   unsigned char
     *buffer,
     *packed_scanline,
     *scanline;
 
-  unsigned long
-    bytes_per_line,
-    storage_class;
 
   unsigned short
     base_address,
@@ -1607,7 +1605,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Initialize image info.
   */
@@ -1699,18 +1697,22 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0x1f2);
-      (void) WriteBlobMSBShort(image,(unsigned short) (GetStringInfoLength(profile)+4));
+      (void) WriteBlobMSBShort(image,(unsigned short)
+        (GetStringInfoLength(profile)+4));
       (void) WriteBlobString(image,"8BIM");
-      (void) WriteBlob(image,GetStringInfoLength(profile),GetStringInfoDatum(profile));
+      (void) WriteBlob(image,GetStringInfoLength(profile),
+        GetStringInfoDatum(profile));
     }
   profile=GetImageProfile(image,"icc");
   if (profile != (StringInfo *) NULL)
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
-      (void) WriteBlobMSBShort(image,(unsigned short) (GetStringInfoLength(profile)+4));
+      (void) WriteBlobMSBShort(image,(unsigned short)
+        (GetStringInfoLength(profile)+4));
       (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlob(image,GetStringInfoLength(profile),GetStringInfoDatum(profile));
+      (void) WriteBlob(image,GetStringInfoLength(profile),
+        GetStringInfoDatum(profile));
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,4);
@@ -1745,8 +1747,6 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
           (void) CloseBlob(image);
           return(MagickFalse);
         }
-      DestroyBlob(jpeg_image);
-      jpeg_image->blob=CloneBlobInfo((BlobInfo *) NULL);
       jpeg_info=CloneImageInfo(image_info);
       (void) CopyMagickString(jpeg_info->magick,"JPEG",MaxTextExtent);
       length=0;
@@ -1757,7 +1757,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         return(MagickFalse);
       jpeg_image=DestroyImage(jpeg_image);
       (void) WriteBlobMSBShort(image,PictJPEGOp);
-      (void) WriteBlobMSBLong(image,(unsigned long) length+154);
+      (void) WriteBlobMSBLong(image,(unsigned int) length+154);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBLong(image,0x00010000UL);
       (void) WriteBlobMSBLong(image,0x00000000UL);
@@ -1816,7 +1816,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   else
     {
       (void) WriteBlobMSBShort(image,PictPixmapOp);
-      (void) WriteBlobMSBLong(image,(unsigned long) base_address);
+      (void) WriteBlobMSBLong(image,(size_t) base_address);
     }
   (void) WriteBlobMSBShort(image,(unsigned short) (row_bytes | 0x8000));
   (void) WriteBlobMSBShort(image,(unsigned short) bounds.top);
@@ -1828,7 +1828,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   */
   (void) WriteBlobMSBShort(image,(unsigned short) pixmap.version);
   (void) WriteBlobMSBShort(image,(unsigned short) pixmap.pack_type);
-  (void) WriteBlobMSBLong(image,pixmap.pack_size);
+  (void) WriteBlobMSBLong(image,(unsigned int) pixmap.pack_size);
   (void) WriteBlobMSBShort(image,(unsigned short) (x_resolution+0.5));
   (void) WriteBlobMSBShort(image,0x0000);
   (void) WriteBlobMSBShort(image,(unsigned short) (y_resolution+0.5));
@@ -1840,9 +1840,9 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   */
   (void) WriteBlobMSBShort(image,(unsigned short) pixmap.component_count);
   (void) WriteBlobMSBShort(image,(unsigned short) pixmap.component_size);
-  (void) WriteBlobMSBLong(image,(unsigned long) pixmap.plane_bytes);
-  (void) WriteBlobMSBLong(image,(unsigned long) pixmap.table);
-  (void) WriteBlobMSBLong(image,(unsigned long) pixmap.reserved);
+  (void) WriteBlobMSBLong(image,(unsigned int) pixmap.plane_bytes);
+  (void) WriteBlobMSBLong(image,(unsigned int) pixmap.table);
+  (void) WriteBlobMSBLong(image,(unsigned int) pixmap.reserved);
   if (storage_class == PseudoClass)
     {
       /*
@@ -1851,7 +1851,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBLong(image,0x00000000L);  /* color seed */
       (void) WriteBlobMSBShort(image,0L);  /* color flags */
       (void) WriteBlobMSBShort(image,(unsigned short) (image->colors-1));
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (ssize_t) image->colors; i++)
       {
         (void) WriteBlobMSBShort(image,(unsigned short) i);
         (void) WriteBlobMSBShort(image,ScaleQuantumToShort(
@@ -1879,21 +1879,20 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   */
   count=0;
   if ((storage_class == PseudoClass) && (image->matte == MagickFalse))
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
-      for (x=0; x < (long) image->columns; x++)
-        scanline[x]=(unsigned char) indexes[x];
-      count+=EncodeImage(image,scanline,(unsigned long) (row_bytes & 0x7FFF),
+      indexes=GetVirtualIndexQueue(image);
+      for (x=0; x < (ssize_t) image->columns; x++)
+        scanline[x]=(unsigned char) GetIndexPixelComponent(indexes+x);
+      count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
         packed_scanline);
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
+      if (image->previous == (Image *) NULL)
         {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
+          status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
@@ -1902,9 +1901,9 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     if (image_info->compression == JPEGCompression)
       {
         (void) ResetMagickMemory(scanline,0,row_bytes);
-        for (y=0; y < (long) image->rows; y++)
-          count+=EncodeImage(image,scanline,(unsigned long)
-            (row_bytes & 0x7FFF),packed_scanline);
+        for (y=0; y < (ssize_t) image->rows; y++)
+          count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
+            packed_scanline);
       }
     else
       {
@@ -1918,9 +1917,9 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         green=scanline+image->columns;
         blue=scanline+2*image->columns;
         opacity=scanline+3*image->columns;
-        for (y=0; y < (long) image->rows; y++)
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
           red=scanline;
@@ -1933,23 +1932,22 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
               green=scanline+2*image->columns;
               blue=scanline+3*image->columns;
             }
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            *red++=ScaleQuantumToChar(p->red);
-            *green++=ScaleQuantumToChar(p->green);
-            *blue++=ScaleQuantumToChar(p->blue);
+            *red++=ScaleQuantumToChar(GetRedPixelComponent(p));
+            *green++=ScaleQuantumToChar(GetGreenPixelComponent(p));
+            *blue++=ScaleQuantumToChar(GetBluePixelComponent(p));
             if (image->matte != MagickFalse)
               *opacity++=ScaleQuantumToChar((Quantum)
-                (QuantumRange-p->opacity));
+                (GetAlphaPixelComponent(p)));
             p++;
           }
           count+=EncodeImage(image,scanline,bytes_per_line & 0x7FFF,
             packed_scanline);
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
+          if (image->previous == (Image *) NULL)
             {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
+              status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
               if (status == MagickFalse)
                 break;
             }
