@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,8 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -91,9 +93,6 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -105,20 +104,21 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
   size_t
-    length;
-
-  unsigned long
     bits_per_pixel,
     height,
+    length,
     width;
 
+  ssize_t
+    y;
+
   /*
     Open image file.
   */
@@ -163,23 +163,18 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Convert PIX raster image to pixel packets.
     */
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
     red=(Quantum) 0;
     green=(Quantum) 0;
     blue=(Quantum) 0;
     index=(IndexPacket) 0;
     length=0;
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      q=SetImagePixels(image,0,y,image->columns,1);
+      q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
-      for (x=0; x < (long) image->columns; x++)
+      indexes=GetAuthenticIndexQueue(image);
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
         if (length == 0)
           {
@@ -194,18 +189,19 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
           }
         if (image->storage_class == PseudoClass)
-          indexes[x]=index;
-        q->blue=blue;
-        q->green=green;
-        q->red=red;
+          SetIndexPixelComponent(indexes+x,index);
+        SetBluePixelComponent(q,blue);
+        SetGreenPixelComponent(q,green);
+        SetRedPixelComponent(q,red);
         length--;
         q++;
       }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
@@ -273,10 +269,10 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPIXImage method is:
 %
-%      unsigned long RegisterPIXImage(void)
+%      size_t RegisterPIXImage(void)
 %
 */
-ModuleExport unsigned long RegisterPIXImage(void)
+ModuleExport size_t RegisterPIXImage(void)
 {
   MagickInfo
     *entry;
