@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,9 +42,11 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/fx.h"
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/list.h"
@@ -53,6 +55,7 @@
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/random_.h"
+#include "magick/random-private.h"
 #include "magick/signature-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -93,25 +96,31 @@ static inline size_t MagickMax(const size_t x,const size_t y)
   return(y);
 }
 
-static inline void PlasmaPixel(Image *image,double x,double y)
+static inline void PlasmaPixel(Image *image,RandomInfo *random_info,double x,
+  double y)
 {
+  ExceptionInfo
+    *exception;
+
   QuantumAny
-    scale;
+    range;
 
   register PixelPacket
     *q;
 
-  q=GetImagePixels(image,(long) ceil(x-0.5),(long) ceil(y-0.5),1,1);
+  exception=(&image->exception);
+  q=GetAuthenticPixels(image,(ssize_t) ceil(x-0.5),(ssize_t) ceil(y-0.5),1,1,
+    exception);
   if (q == (PixelPacket *) NULL)
     return;
-  scale=GetQuantumScale(16UL);
-  q->red=ScaleAnyToQuantum((unsigned long) (65535.0*GetPseudoRandomValue()+
-    0.5),16UL,scale);
-  q->green=ScaleAnyToQuantum((unsigned long) (65535.0*GetPseudoRandomValue()+
-    0.5),16UL,scale);
-  q->blue=ScaleAnyToQuantum((unsigned long) (65535.0*GetPseudoRandomValue()+
-    0.5),16UL,scale);
-  (void) SyncImagePixels(image);
+  range=GetQuantumRange(16UL);
+  SetRedPixelComponent(q,ScaleAnyToQuantum((size_t) (65535.0*
+    GetPseudoRandomValue(random_info)+0.5),range));
+  SetGreenPixelComponent(q,ScaleAnyToQuantum((size_t) (65535.0*
+    GetPseudoRandomValue(random_info)+0.5),range));
+  SetBluePixelComponent(q,ScaleAnyToQuantum((size_t) (65535.0*
+    GetPseudoRandomValue(random_info)+0.5),range));
+  (void) SyncAuthenticPixels(image,exception);
 }
 
 static Image *ReadPlasmaImage(const ImageInfo *image_info,
@@ -123,28 +132,28 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
   ImageInfo
     *read_info;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
-  register unsigned long
+  register size_t
     i;
 
   SegmentInfo
     segment_info;
 
-  unsigned long
+  size_t
     depth,
     max_depth;
 
+  ssize_t
+    y;
+
   /*
     Recursively apply plasma to the image.
   */
@@ -157,17 +166,17 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
   if (image == (Image *) NULL)
     return((Image *) NULL);
   image->storage_class=DirectClass;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      q->opacity=(Quantum) (QuantumRange/2);
+      SetOpacityPixelComponent(q,QuantumRange/2);
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   segment_info.x1=0;
@@ -176,28 +185,38 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
   segment_info.y2=(double) image->rows-1;
   if (LocaleCompare(image_info->filename,"fractal") == 0)
     {
+      RandomInfo
+        *random_info;
+
       /*
         Seed pixels before recursion.
       */
-      PlasmaPixel(image,segment_info.x1,segment_info.y1);
-      PlasmaPixel(image,segment_info.x1,(segment_info.y1+segment_info.y2)/2);
-      PlasmaPixel(image,segment_info.x1,segment_info.y2);
-      PlasmaPixel(image,(segment_info.x1+segment_info.x2)/2,segment_info.y1);
-      PlasmaPixel(image,(segment_info.x1+segment_info.x2)/2,
+      random_info=AcquireRandomInfo();
+      PlasmaPixel(image,random_info,segment_info.x1,segment_info.y1);
+      PlasmaPixel(image,random_info,segment_info.x1,(segment_info.y1+
+        segment_info.y2)/2);
+      PlasmaPixel(image,random_info,segment_info.x1,segment_info.y2);
+      PlasmaPixel(image,random_info,(segment_info.x1+segment_info.x2)/2,
+        segment_info.y1);
+      PlasmaPixel(image,random_info,(segment_info.x1+segment_info.x2)/2,
         (segment_info.y1+segment_info.y2)/2);
-      PlasmaPixel(image,(segment_info.x1+segment_info.x2)/2,segment_info.y2);
-      PlasmaPixel(image,segment_info.x2,segment_info.y1);
-      PlasmaPixel(image,segment_info.x2,(segment_info.y1+segment_info.y2)/2);
-      PlasmaPixel(image,segment_info.x2,segment_info.y2);
+      PlasmaPixel(image,random_info,(segment_info.x1+segment_info.x2)/2,
+        segment_info.y2);
+      PlasmaPixel(image,random_info,segment_info.x2,segment_info.y1);
+      PlasmaPixel(image,random_info,segment_info.x2,(segment_info.y1+
+        segment_info.y2)/2);
+      PlasmaPixel(image,random_info,segment_info.x2,segment_info.y2);
+      random_info=DestroyRandomInfo(random_info);
     }
-  i=(unsigned long) MagickMax(image->columns,image->rows)/2;
+  i=(size_t) MagickMax(image->columns,image->rows)/2;
   for (max_depth=0; i != 0; max_depth++)
     i>>=1;
   for (depth=1; ; depth++)
   {
     if (PlasmaImage(image,&segment_info,0,depth) != MagickFalse)
       break;
-    status=SetImageProgress(image,LoadImageTag,depth,max_depth);
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) depth,
+      max_depth);
     if (status == MagickFalse)
       break;
   }
@@ -225,10 +244,10 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterPLASMAImage method is:
 %
-%      unsigned long RegisterPLASMAImage(void)
+%      size_t RegisterPLASMAImage(void)
 %
 */
-ModuleExport unsigned long RegisterPLASMAImage(void)
+ModuleExport size_t RegisterPLASMAImage(void)
 {
   MagickInfo
     *entry;
@@ -236,14 +255,14 @@ ModuleExport unsigned long RegisterPLASMAImage(void)
   entry=SetMagickInfo("PLASMA");
   entry->decoder=(DecodeImageHandler *) ReadPlasmaImage;
   entry->adjoin=MagickFalse;
-  entry->format_type=ExplicitFormatType;
+  entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Plasma fractal image");
   entry->module=ConstantString("PLASMA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FRACTAL");
   entry->decoder=(DecodeImageHandler *) ReadPlasmaImage;
   entry->adjoin=MagickFalse;
-  entry->format_type=ExplicitFormatType;
+  entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Plasma fractal image");
   entry->module=ConstantString("PLASMA");
   (void) RegisterMagickInfo(entry);
