@@ -10,14 +10,14 @@
 %                            R  R   LLLLL  A   A                              %
 %                                                                             %
 %                                                                             %
-%                      Read Alias/Wavefront Image Format.                     %
+%                      Read Alias/Wavefront Image Format                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,6 +43,7 @@
 #include "magick/property.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -51,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -79,11 +81,10 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -117,7 +118,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       blue_primary[24],
       white_point[24];
 
-    long
+    ssize_t
       job_number;
 
     char
@@ -149,7 +150,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       auxiliary[32],
       space[36];
 
-    long
+    ssize_t
       next;
   } RLAInfo;
 
@@ -161,19 +162,13 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     length,
     runlength;
 
-  long
-    y;
-
-  long
-    *scanlines;
-
   MagickBooleanType
     status;
 
   MagickOffsetType
     offset;
 
-  register long
+  register ssize_t
     i,
     x;
 
@@ -181,7 +176,9 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *q;
 
   ssize_t
-    count;
+    count,
+    *scanlines,
+    y;
 
   RLAInfo
     rla_info;
@@ -199,7 +196,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -228,7 +225,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   count=ReadBlob(image,24,(unsigned char *) rla_info.green_primary);
   count=ReadBlob(image,24,(unsigned char *) rla_info.blue_primary);
   count=ReadBlob(image,24,(unsigned char *) rla_info.white_point);
-  rla_info.job_number=(long) ReadBlobMSBLong(image);
+  rla_info.job_number=(int) ReadBlobMSBLong(image);
   count=ReadBlob(image,128,(unsigned char *) rla_info.name);
   count=ReadBlob(image,128,(unsigned char *) rla_info.description);
   count=ReadBlob(image,64,(unsigned char *) rla_info.program);
@@ -248,7 +245,9 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   rla_info.auxiliary_bits=(short) ReadBlobMSBShort(image);
   count=ReadBlob(image,32,(unsigned char *) rla_info.auxiliary);
   count=ReadBlob(image,36,(unsigned char *) rla_info.space);
-  rla_info.next=(long) ReadBlobMSBLong(image);
+  if ((size_t) count != 36)
+    ThrowReaderException(CorruptImageError,"UnableToReadImageData");
+  rla_info.next=(int) ReadBlobMSBLong(image);
   /*
     Initialize image structure.
   */
@@ -257,29 +256,24 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->rows=1UL*rla_info.active_window.top-rla_info.active_window.bottom+1;
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
-  scanlines=(long *) AcquireQuantumMemory(image->rows,sizeof(*scanlines));
-  if (scanlines == (long *) NULL)
+  scanlines=(ssize_t *) AcquireQuantumMemory(image->rows,sizeof(*scanlines));
+  if (scanlines == (ssize_t *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (*rla_info.description != '\0')
     (void) SetImageProperty(image,"comment",rla_info.description);
   /*
     Read offsets to each scanline data.
   */
-  for (i=0; i < (long) image->rows; i++)
-    scanlines[i]=(long) ReadBlobMSBLong(image);
+  for (i=0; i < (ssize_t) image->rows; i++)
+    scanlines[i]=(int) ReadBlobMSBLong(image);
   /*
     Read image data.
   */
   x=0;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     offset=SeekBlob(image,scanlines[image->rows-y-1],SEEK_SET);
     if (offset < 0)
@@ -300,8 +294,8 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             while (runlength < 0)
             {
-              q=GetImagePixels(image,(long) (x % image->columns),
-                (long) (y % image->rows),1,1);
+              q=GetAuthenticPixels(image,(ssize_t) (x % image->columns),
+                (ssize_t) (y % image->rows),1,1,exception);
               if (q == (PixelPacket *) NULL)
                 break;
               byte=(unsigned char) ReadBlobByte(image);
@@ -310,27 +304,28 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 case 0:
                 {
-                  q->red=ScaleCharToQuantum(byte);
+                  SetRedPixelComponent(q,ScaleCharToQuantum(byte));
                   break;
                 }
                 case 1:
                 {
-                  q->green=ScaleCharToQuantum(byte);
+                  SetGreenPixelComponent(q,ScaleCharToQuantum(byte));
                   break;
                 }
                 case 2:
                 {
-                  q->blue=ScaleCharToQuantum(byte);
+                  SetBluePixelComponent(q,ScaleCharToQuantum(byte));
                   break;
                 }
                 case 3:
                 default:
                 {
-                  q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(byte));
+                  SetOpacityPixelComponent(q,QuantumRange-
+                    ScaleCharToQuantum(byte));
                   break;
                 }
               }
-              if (SyncImagePixels(image) == MagickFalse)
+              if (SyncAuthenticPixels(image,exception) == MagickFalse)
                 break;
               x++;
               runlength++;
@@ -342,35 +337,35 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         runlength++;
         do
         {
-          q=GetImagePixels(image,(long) (x % image->columns),
-            (long) (y % image->rows),1,1);
+          q=GetAuthenticPixels(image,(ssize_t) (x % image->columns),
+            (ssize_t) (y % image->rows),1,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
           switch (channel)
           {
             case 0:
             {
-              q->red=ScaleCharToQuantum(byte);
+              SetRedPixelComponent(q,ScaleCharToQuantum(byte));
               break;
             }
             case 1:
             {
-              q->green=ScaleCharToQuantum(byte);
+              SetGreenPixelComponent(q,ScaleCharToQuantum(byte));
               break;
             }
             case 2:
             {
-              q->blue=ScaleCharToQuantum(byte);
+              SetBluePixelComponent(q,ScaleCharToQuantum(byte));
               break;
             }
             case 3:
             default:
             {
-              q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(byte));
+              SetOpacityPixelComponent(q,QuantumRange-ScaleCharToQuantum(byte));
               break;
             }
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
           x++;
           runlength--;
@@ -378,20 +373,15 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         while (runlength > 0);
       }
     }
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(LoadImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
+      break;
   }
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -415,10 +405,10 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterRLAImage method is:
 %
-%      unsigned long RegisterRLAImage(void)
+%      size_t RegisterRLAImage(void)
 %
 */
-ModuleExport unsigned long RegisterRLAImage(void)
+ModuleExport size_t RegisterRLAImage(void)
 {
   MagickInfo
     *entry;
@@ -426,6 +416,7 @@ ModuleExport unsigned long RegisterRLAImage(void)
   entry=SetMagickInfo("RLA");
   entry->decoder=(DecodeImageHandler *) ReadRLAImage;
   entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Alias/Wavefront image");
   entry->module=ConstantString("RLA");
   (void) RegisterMagickInfo(entry);
