@@ -17,7 +17,7 @@
 %                               October 2003                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -105,21 +106,21 @@ static Image *ReadSCRImage(const ImageInfo *image_info,ExceptionInfo *exception)
     int bright;
 
   unsigned char colour_palette[] = {
-    000,000,000,
-    000,000,192,
-    192,000,000,
-    192,000,192,
-    000,192,000,
-    000,192,192,
-    192,192,000,
+      0,  0,  0,
+      0,  0,192,
+    192,  0,  0,
+    192,  0,192,
+      0,192,  0,
+      0,192,192,
+    192,192,  0,
     192,192,192,
-    000,000,000,
-    000,000,255,
-    255,000,000,
-    255,000,255,
-    000,255,000,
-    000,255,255,
-    255,255,000,
+      0,  0,  0,
+      0,  0,255,
+    255,  0,  0,
+    255,  0,255,
+      0,255,  0,
+      0,255,255,
+    255,255,  0,
     255,255,255
   };
 
@@ -154,12 +155,8 @@ static Image *ReadSCRImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   image->columns = 256;
   image->rows = 192;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   count=ReadBlob(image,6144,(unsigned char *) zxscr);
+  (void) count;
   count=ReadBlob(image,768,(unsigned char *) zxattr);
   for(zonenr=0;zonenr<3;zonenr++)
   {
@@ -214,17 +211,23 @@ static Image *ReadSCRImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
             for(z=7;z>-1;z--)
           {
-              q=SetImagePixels(image,pix,piy,1,1);
+              q=QueueAuthenticPixels(image,pix,piy,1,1,exception);
 
               if(binar[z])
             {
-                q->red=(Quantum) QuantumRange*(colour_palette[3*ink]);
-                q->green=(Quantum) QuantumRange*(colour_palette[1+(3*ink)]);
-                q->blue=(Quantum) QuantumRange*(colour_palette[2+(3*ink)]);
+                SetRedPixelComponent(q,ScaleCharToQuantum(
+                  colour_palette[3*ink]));
+                SetGreenPixelComponent(q,ScaleCharToQuantum(
+                  colour_palette[1+(3*ink)]));
+                SetBluePixelComponent(q,ScaleCharToQuantum(
+                  colour_palette[2+(3*ink)]));
             } else {
-                q->red=(Quantum) QuantumRange*(colour_palette[3*paper]);
-                q->green=(Quantum) QuantumRange*(colour_palette[1+(3*paper)]);
-                q->blue=(Quantum) QuantumRange*(colour_palette[2+(3*paper)]);
+                SetRedPixelComponent(q,ScaleCharToQuantum(
+                  colour_palette[3*paper]));
+                SetGreenPixelComponent(q,ScaleCharToQuantum(
+                  colour_palette[1+(3*paper)]));
+                SetBluePixelComponent(q,ScaleCharToQuantum(
+                  colour_palette[2+(3*paper)]));
             }
 
               pix++;
@@ -257,10 +260,10 @@ static Image *ReadSCRImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterSCRImage method is:
 %
-%      unsigned long RegisterSCRImage(void)
+%      size_t RegisterSCRImage(void)
 %
 */
-ModuleExport unsigned long RegisterSCRImage(void)
+ModuleExport size_t RegisterSCRImage(void)
 {
   MagickInfo
     *entry;
