@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,8 +42,10 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -78,7 +80,7 @@ typedef struct _SGIInfo
     rows,
     depth;
 
-  unsigned long
+  size_t
     minimum_value,
     maximum_value,
     sans;
@@ -86,7 +88,7 @@ typedef struct _SGIInfo
   char
     name[80];
 
-  unsigned long
+  size_t
     pixel_format;
 
   unsigned char
@@ -118,12 +120,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsSGI(const unsigned char *magick,const size_t length)
 {
@@ -169,19 +169,19 @@ static inline size_t MagickMin(const size_t x,const size_t y)
 }
 
 static MagickBooleanType SGIDecode(const size_t bytes_per_pixel,
-  long number_packets,unsigned char *packets,long number_pixels,
+  ssize_t number_packets,unsigned char *packets,ssize_t number_pixels,
   unsigned char *pixels)
 {
   register unsigned char
     *p,
     *q;
 
+  size_t
+    pixel;
+
   ssize_t
     count;
 
-  unsigned long
-    pixel;
-
   p=packets;
   q=pixels;
   if (bytes_per_pixel == 2)
@@ -190,7 +190,7 @@ static MagickBooleanType SGIDecode(const size_t bytes_per_pixel,
       {
         if (number_packets-- == 0)
           return(MagickFalse);
-        pixel=(unsigned long) (*p++) << 8;
+        pixel=(size_t) (*p++) << 8;
         pixel|=(*p++);
         count=(ssize_t) (pixel & 0x7f);
         if (count == 0)
@@ -209,7 +209,7 @@ static MagickBooleanType SGIDecode(const size_t bytes_per_pixel,
           }
         else
           {
-            pixel=(unsigned long) (*p++) << 8;
+            pixel=(size_t) (*p++) << 8;
             pixel|=(*p++);
             for ( ; count != 0; count--)
             {
@@ -227,7 +227,7 @@ static MagickBooleanType SGIDecode(const size_t bytes_per_pixel,
   {
     if (number_packets-- == 0)
       return(MagickFalse);
-    pixel=(unsigned long) (*p++);
+    pixel=(size_t) (*p++);
     count=(ssize_t) (pixel & 0x7f);
     if (count == 0)
       break;
@@ -246,7 +246,7 @@ static MagickBooleanType SGIDecode(const size_t bytes_per_pixel,
       {
         if (number_packets-- == 0)
           return(MagickFalse);
-        pixel=(unsigned long) (*p++);
+        pixel=(size_t) (*p++);
         for ( ; count != 0; count--)
         {
           *q=(unsigned char) pixel;
@@ -262,10 +262,6 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  long
-    y,
-    z;
-
   MagickBooleanType
     status;
 
@@ -275,31 +271,31 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register long
-    i,
-    x;
-
   register PixelPacket
     *q;
 
+  register ssize_t
+    i,
+    x;
+
   register unsigned char
     *p;
 
-  ssize_t
-    count;
-
   SGIInfo
     iris_info;
 
   size_t
-    bytes_per_pixel;
+    bytes_per_pixel,
+    quantum;
+
+  ssize_t
+    count,
+    y,
+    z;
 
   unsigned char
     *iris_pixels;
 
-  unsigned long
-    quantum;
-
   /*
     Open image file.
   */
@@ -357,25 +353,21 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (iris_info.pixel_format != 0)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     count=ReadBlob(image,sizeof(iris_info.filler),iris_info.filler);
+    (void) count;
     image->columns=iris_info.columns;
     image->rows=iris_info.rows;
-    image->depth=(unsigned long) MagickMin(iris_info.depth,MAGICKCORE_QUANTUM_DEPTH);
+    image->depth=(size_t) MagickMin(iris_info.depth,MAGICKCORE_QUANTUM_DEPTH);
     if (iris_info.pixel_format == 0)
-      image->depth=(unsigned long) MagickMin((size_t) 8*
+      image->depth=(size_t) MagickMin((size_t) 8*
         iris_info.bytes_per_pixel,MAGICKCORE_QUANTUM_DEPTH);
     if (iris_info.depth < 3)
       {
         image->storage_class=PseudoClass;
-        image->colors=256;
+        image->colors=iris_info.bytes_per_pixel > 1 ? 65535 : 256;
       }
     if ((image_info->ping != MagickFalse)  && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
     /*
       Allocate SGI pixels.
     */
@@ -400,23 +392,23 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           bytes_per_pixel*sizeof(*scanline));
         if (scanline == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        for (z=0; z < (long) iris_info.depth; z++)
+        for (z=0; z < (ssize_t) iris_info.depth; z++)
         {
           p=iris_pixels+bytes_per_pixel*z;
-          for (y=0; y < (long) iris_info.rows; y++)
+          for (y=0; y < (ssize_t) iris_info.rows; y++)
           {
             count=ReadBlob(image,bytes_per_pixel*iris_info.columns,scanline);
             if (EOFBlob(image) != MagickFalse)
               break;
             if (bytes_per_pixel == 2)
-              for (x=0; x < (long) iris_info.columns; x++)
+              for (x=0; x < (ssize_t) iris_info.columns; x++)
               {
                 *p=scanline[2*x];
                 *(p+1)=scanline[2*x+1];
                 p+=8;
               }
             else
-              for (x=0; x < (long) iris_info.columns; x++)
+              for (x=0; x < (ssize_t) iris_info.columns; x++)
               {
                 *p=scanline[x];
                 p+=4;
@@ -427,6 +419,9 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     else
       {
+        size_t
+          *runlength;
+
         ssize_t
           offset,
           *offsets;
@@ -437,9 +432,6 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         unsigned int
           data_order;
 
-        unsigned long
-          *runlength;
-
         /*
           Read runlength-encoded image format.
         */
@@ -447,15 +439,15 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           iris_info.depth*sizeof(*offsets));
         packets=(unsigned char *) AcquireQuantumMemory((size_t)
           iris_info.columns+10UL,4UL*sizeof(*packets));
-        runlength=(unsigned long *) AcquireQuantumMemory(iris_info.rows,
+        runlength=(size_t *) AcquireQuantumMemory(iris_info.rows,
           iris_info.depth*sizeof(*runlength));
         if ((offsets == (ssize_t *) NULL) ||
             (packets == (unsigned char *) NULL) ||
-            (runlength == (unsigned long *) NULL))
+            (runlength == (size_t *) NULL))
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
-          offsets[i]=(ssize_t) ReadBlobMSBLong(image);
-        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (ssize_t) (iris_info.rows*iris_info.depth); i++)
+          offsets[i]=(int) ReadBlobMSBLong(image);
+        for (i=0; i < (ssize_t) (iris_info.rows*iris_info.depth); i++)
         {
           runlength[i]=ReadBlobMSBLong(image);
           if (runlength[i] > (4*(size_t) iris_info.columns+10))
@@ -466,8 +458,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         offset=0;
         data_order=0;
-        for (y=0; ((y < (long) iris_info.rows) && (data_order == 0)); y++)
-          for (z=0; ((z < (long) iris_info.depth) && (data_order == 0)); z++)
+        for (y=0; ((y < (ssize_t) iris_info.rows) && (data_order == 0)); y++)
+          for (z=0; ((z < (ssize_t) iris_info.depth) && (data_order == 0)); z++)
           {
             if (offsets[y+z*iris_info.rows] < offset)
               data_order=1;
@@ -476,22 +468,22 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         offset=(ssize_t) TellBlob(image);
         if (data_order == 1)
           {
-            for (z=0; z < (long) iris_info.depth; z++)
+            for (z=0; z < (ssize_t) iris_info.depth; z++)
             {
               p=iris_pixels;
-              for (y=0; y < (long) iris_info.rows; y++)
+              for (y=0; y < (ssize_t) iris_info.rows; y++)
               {
                 if (offset != offsets[y+z*iris_info.rows])
                   {
                     offset=offsets[y+z*iris_info.rows];
-                    offset=(ssize_t) SeekBlob(image,(long) offset,SEEK_SET);
+                    offset=(ssize_t) SeekBlob(image,(ssize_t) offset,SEEK_SET);
                   }
                 count=ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
                   packets);
                 if (EOFBlob(image) != MagickFalse)
                   break;
-                offset+=runlength[y+z*iris_info.rows];
-                status=SGIDecode(bytes_per_pixel,(long)
+                offset+=(ssize_t) runlength[y+z*iris_info.rows];
+                status=SGIDecode(bytes_per_pixel,(ssize_t)
                   (runlength[y+z*iris_info.rows]/bytes_per_pixel),packets,
                   1L*iris_info.columns,p+bytes_per_pixel*z);
                 if (status == MagickFalse)
@@ -507,21 +499,21 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
            
             position=TellBlob(image);
             p=iris_pixels;
-            for (y=0; y < (long) iris_info.rows; y++)
+            for (y=0; y < (ssize_t) iris_info.rows; y++)
             {
-              for (z=0; z < (long) iris_info.depth; z++)
+              for (z=0; z < (ssize_t) iris_info.depth; z++)
               {
                 if (offset != offsets[y+z*iris_info.rows])
                   {
                     offset=offsets[y+z*iris_info.rows];
-                    offset=(ssize_t) SeekBlob(image,(long) offset,SEEK_SET);
+                    offset=(ssize_t) SeekBlob(image,(ssize_t) offset,SEEK_SET);
                   }
                 count=ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
                   packets);
                 if (EOFBlob(image) != MagickFalse)
                   break;
-                offset+=runlength[y+z*iris_info.rows];
-                status=SGIDecode(bytes_per_pixel,(long)
+                offset+=(ssize_t) runlength[y+z*iris_info.rows];
+                status=SGIDecode(bytes_per_pixel,(ssize_t)
                   (runlength[y+z*iris_info.rows]/bytes_per_pixel),packets,
                   1L*iris_info.columns,p+bytes_per_pixel*z);
                 if (status == MagickFalse)
@@ -531,7 +523,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
             offset=(ssize_t) SeekBlob(image,position,SEEK_SET);
           }
-        runlength=(unsigned long *) RelinquishMagickMemory(runlength);
+        runlength=(size_t *) RelinquishMagickMemory(runlength);
         packets=(unsigned char *) RelinquishMagickMemory(packets);
         offsets=(ssize_t *) RelinquishMagickMemory(offsets);
       }
@@ -551,60 +543,63 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (bytes_per_pixel == 2)
           {
-            for (y=0; y < (long) image->rows; y++)
+            for (y=0; y < (ssize_t) image->rows; y++)
             {
               p=iris_pixels+(image->rows-y-1)*8*image->columns;
-              q=SetImagePixels(image,0,y,image->columns,1);
+              q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
               if (q == (PixelPacket *) NULL)
                 break;
-              for (x=0; x < (long) image->columns; x++)
+              for (x=0; x < (ssize_t) image->columns; x++)
               {
-                q->red=ScaleShortToQuantum((unsigned short)
-                  ((*(p+0) << 8) | (*(p+1))));
-                q->green=ScaleShortToQuantum((unsigned short)
-                  ((*(p+2) << 8) | (*(p+3))));
-                q->blue=ScaleShortToQuantum((unsigned short)
-                  ((*(p+4) << 8) | (*(p+5))));
-                q->opacity=OpaqueOpacity;
+                SetRedPixelComponent(q,ScaleShortToQuantum((unsigned short)
+                  ((*(p+0) << 8) | (*(p+1)))));
+                SetGreenPixelComponent(q,ScaleShortToQuantum((unsigned short)
+                  ((*(p+2) << 8) | (*(p+3)))));
+                SetBluePixelComponent(q,ScaleShortToQuantum((unsigned short)
+                  ((*(p+4) << 8) | (*(p+5)))));
+                SetOpacityPixelComponent(q,OpaqueOpacity);
                 if (image->matte != MagickFalse)
-                  q->opacity=(Quantum) (QuantumRange-ScaleShortToQuantum(
+                  SetOpacityPixelComponent(q,QuantumRange-ScaleShortToQuantum(
                     (unsigned short) ((*(p+6) << 8) | (*(p+7)))));
                 p+=8;
                 q++;
               }
-              if (SyncImagePixels(image) == MagickFalse)
+              if (SyncAuthenticPixels(image,exception) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
                 {
-                  status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                  status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
+                    y,image->rows);
                   if (status == MagickFalse)
                     break;
                 }
             }
           }
         else
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
             p=iris_pixels+(image->rows-y-1)*4*image->columns;
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
-            for (x=0; x < (long) image->columns; x++)
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
-              q->red=ScaleCharToQuantum(*p);
+              SetRedPixelComponent(q,ScaleCharToQuantum(*p));
               q->green=ScaleCharToQuantum(*(p+1));
               q->blue=ScaleCharToQuantum(*(p+2));
-              q->opacity=OpaqueOpacity;
+              SetOpacityPixelComponent(q,OpaqueOpacity);
               if (image->matte != MagickFalse)
-                q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*(p+3)));
+                SetOpacityPixelComponent(q,QuantumRange-
+                  ScaleCharToQuantum(*(p+3)));
               p+=4;
               q++;
             }
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
               {
-                status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
                 if (status == MagickFalse)
                   break;
               }
@@ -622,50 +617,52 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (bytes_per_pixel == 2)
           {
-            for (y=0; y < (long) image->rows; y++)
+            for (y=0; y < (ssize_t) image->rows; y++)
             {
               p=iris_pixels+(image->rows-y-1)*8*image->columns;
-              q=SetImagePixels(image,0,y,image->columns,1);
+              q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
-              for (x=0; x < (long) image->columns; x++)
+              indexes=GetAuthenticIndexQueue(image);
+              for (x=0; x < (ssize_t) image->columns; x++)
               {
                 quantum=(*p << 8);
                 quantum|=(*(p+1));
-                indexes[x]=(IndexPacket) quantum;
+                SetIndexPixelComponent(indexes+x,quantum);
                 p+=8;
                 q++;
               }
-              if (SyncImagePixels(image) == MagickFalse)
+              if (SyncAuthenticPixels(image,exception) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
                 {
-                  status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                  status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
+                    y,image->rows);
                   if (status == MagickFalse)
                     break;
                 }
             }
           }
         else
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
             p=iris_pixels+(image->rows-y-1)*4*image->columns;
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
-            for (x=0; x < (long) image->columns; x++)
+            indexes=GetAuthenticIndexQueue(image);
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
-              indexes[x]=(IndexPacket) (*p);
+              SetIndexPixelComponent(indexes+x,*p);
               p+=4;
               q++;
             }
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
               {
-                status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
                 if (status == MagickFalse)
                   break;
               }
@@ -728,10 +725,10 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterSGIImage method is:
 %
-%      unsigned long RegisterSGIImage(void)
+%      size_t RegisterSGIImage(void)
 %
 */
-ModuleExport unsigned long RegisterSGIImage(void)
+ModuleExport size_t RegisterSGIImage(void)
 {
   MagickInfo
     *entry;
@@ -857,10 +854,6 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
   const char
     *value;
 
-  long
-    y,
-    z;
-
   MagickBooleanType
     status;
 
@@ -876,13 +869,17 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     i,
     x;
 
   register unsigned char
     *q;
 
+  ssize_t
+    y,
+    z;
+
   unsigned char
     *iris_pixels,
     *packets;
@@ -908,10 +905,12 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
       Initialize SGI raster file header.
     */
     if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
+      (void) TransformImageColorspace(image,RGBColorspace);
     (void) ResetMagickMemory(&iris_info,0,sizeof(iris_info));
     iris_info.magic=0x01DA;
-    compression=image_info->compression;
+    compression=image->compression;
+    if (image_info->compression != UndefinedCompression)
+      compression=image_info->compression;
     if (image->depth > 8)
       compression=NoCompression;
     if (compression == NoCompression)
@@ -936,7 +935,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           iris_info.depth=3;
       }
     iris_info.minimum_value=0;
-    iris_info.maximum_value=(unsigned long) (image->depth <= 8 ?
+    iris_info.maximum_value=(size_t) (image->depth <= 8 ?
       1UL*ScaleQuantumToChar((Quantum) QuantumRange) :
       1UL*ScaleQuantumToShort((Quantum) QuantumRange));
     /*
@@ -949,15 +948,15 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobMSBShort(image,iris_info.columns);
     (void) WriteBlobMSBShort(image,iris_info.rows);
     (void) WriteBlobMSBShort(image,iris_info.depth);
-    (void) WriteBlobMSBLong(image,iris_info.minimum_value);
-    (void) WriteBlobMSBLong(image,iris_info.maximum_value);
-    (void) WriteBlobMSBLong(image,iris_info.sans);
+    (void) WriteBlobMSBLong(image,(unsigned int) iris_info.minimum_value);
+    (void) WriteBlobMSBLong(image,(unsigned int) iris_info.maximum_value);
+    (void) WriteBlobMSBLong(image,(unsigned int) iris_info.sans);
     value=GetImageProperty(image,"label");
     if (value != (const char *) NULL)
       (void) CopyMagickString(iris_info.name,value,sizeof(iris_info.name));
     (void) WriteBlob(image,sizeof(iris_info.name),(unsigned char *)
       iris_info.name);
-    (void) WriteBlobMSBLong(image,iris_info.pixel_format);
+    (void) WriteBlobMSBLong(image,(unsigned int) iris_info.pixel_format);
     (void) WriteBlob(image,sizeof(iris_info.filler),iris_info.filler);
     /*
       Allocate SGI pixels.
@@ -973,42 +972,43 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     /*
       Convert image pixels to uncompressed SGI pixels.
     */
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
       if (image->depth <= 8)
-        for (x=0; x < (long) image->columns; x++)
+        for (x=0; x < (ssize_t) image->columns; x++)
         {
           register unsigned char
             *q;
 
           q=(unsigned char *) iris_pixels;
           q+=((iris_info.rows-1)-y)*(4*iris_info.columns)+4*x;
-          *q++=ScaleQuantumToChar(p->red);
-          *q++=ScaleQuantumToChar(p->green);
-          *q++=ScaleQuantumToChar(p->blue);
-          *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
+          *q++=ScaleQuantumToChar(GetRedPixelComponent(p));
+          *q++=ScaleQuantumToChar(GetGreenPixelComponent(p));
+          *q++=ScaleQuantumToChar(GetBluePixelComponent(p));
+          *q++=ScaleQuantumToChar(GetAlphaPixelComponent(p));
           p++;
         }
       else
-        for (x=0; x < (long) image->columns; x++)
+        for (x=0; x < (ssize_t) image->columns; x++)
         {
           register unsigned short
             *q;
 
           q=(unsigned short *) iris_pixels;
           q+=((iris_info.rows-1)-y)*(4*iris_info.columns)+4*x;
-          *q++=ScaleQuantumToShort(p->red);
-          *q++=ScaleQuantumToShort(p->green);
-          *q++=ScaleQuantumToShort(p->blue);
-          *q++=ScaleQuantumToShort((Quantum) (QuantumRange-p->opacity));
+          *q++=ScaleQuantumToShort(GetRedPixelComponent(p));
+          *q++=ScaleQuantumToShort(GetGreenPixelComponent(p));
+          *q++=ScaleQuantumToShort(GetBluePixelComponent(p));
+          *q++=ScaleQuantumToShort(GetAlphaPixelComponent(p));
           p++;
         }
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
@@ -1020,12 +1020,12 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
         /*
           Write uncompressed SGI pixels.
         */
-        for (z=0; z < (long) iris_info.depth; z++)
+        for (z=0; z < (ssize_t) iris_info.depth; z++)
         {
-          for (y=0; y < (long) iris_info.rows; y++)
+          for (y=0; y < (ssize_t) iris_info.rows; y++)
           {
             if (image->depth <= 8)
-              for (x=0; x < (long) iris_info.columns; x++)
+              for (x=0; x < (ssize_t) iris_info.columns; x++)
               {
                 register unsigned char
                   *q;
@@ -1035,7 +1035,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
                 (void) WriteBlobByte(image,*q);
               }
             else
-              for (x=0; x < (long) iris_info.columns; x++)
+              for (x=0; x < (ssize_t) iris_info.columns; x++)
               {
                 register unsigned short
                   *q;
@@ -1050,17 +1050,15 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
       }
       default:
       {
+        size_t
+          length,
+          number_packets,
+          *runlength;
+
         ssize_t
           offset,
           *offsets;
 
-        size_t
-          length,
-          number_packets;
-
-        unsigned long
-          *runlength;
-
         /*
           Convert SGI uncompressed pixels.
         */
@@ -1068,24 +1066,24 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           sizeof(*offsets));
         packets=(unsigned char *) AcquireQuantumMemory((2*(size_t)
           iris_info.columns+10)*image->rows,4*sizeof(*packets));
-        runlength=(unsigned long *) AcquireQuantumMemory(iris_info.rows,
+        runlength=(size_t *) AcquireQuantumMemory(iris_info.rows,
           iris_info.depth*sizeof(*runlength));
         if ((offsets == (ssize_t *) NULL) ||
             (packets == (unsigned char *) NULL) ||
-            (runlength == (unsigned long *) NULL))
+            (runlength == (size_t *) NULL))
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         offset=512+4*2*((ssize_t) iris_info.rows*iris_info.depth);
         number_packets=0;
         q=iris_pixels;
-        for (y=0; y < (long) iris_info.rows; y++)
+        for (y=0; y < (ssize_t) iris_info.rows; y++)
         {
-          for (z=0; z < (long) iris_info.depth; z++)
+          for (z=0; z < (ssize_t) iris_info.depth; z++)
           {
             length=SGIEncode(q+z,(size_t) iris_info.columns,packets+
               number_packets);
             number_packets+=length;
             offsets[y+z*iris_info.rows]=offset;
-            runlength[y+z*iris_info.rows]=(unsigned long) length;
+            runlength[y+z*iris_info.rows]=(size_t) length;
             offset+=(ssize_t) length;
           }
           q+=(iris_info.columns*4);
@@ -1093,15 +1091,15 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
         /*
           Write out line start and length tables and runlength-encoded pixels.
         */
-        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
-          (void) WriteBlobMSBLong(image,(unsigned long) offsets[i]);
-        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
-          (void) WriteBlobMSBLong(image,runlength[i]);
+        for (i=0; i < (ssize_t) (iris_info.rows*iris_info.depth); i++)
+          (void) WriteBlobMSBLong(image,(unsigned int) offsets[i]);
+        for (i=0; i < (ssize_t) (iris_info.rows*iris_info.depth); i++)
+          (void) WriteBlobMSBLong(image,(unsigned int) runlength[i]);
         (void) WriteBlob(image,number_packets,packets);
         /*
           Relinquish resources.
         */
-        runlength=(unsigned long *) RelinquishMagickMemory(runlength);
+        runlength=(size_t *) RelinquishMagickMemory(runlength);
         packets=(unsigned char *) RelinquishMagickMemory(packets);
         offsets=(ssize_t *) RelinquishMagickMemory(offsets);
         break;
