@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,8 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -85,8 +87,8 @@
 %
 */
 
-static inline unsigned long MagickMin(const unsigned long x,
-  const unsigned long y)
+static inline size_t MagickMin(const size_t x,
+  const size_t y)
 {
   if (x < y)
     return(x);
@@ -96,11 +98,11 @@ static inline unsigned long MagickMin(const unsigned long x,
 static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define GetBit(alpha,i) MagickMin((((unsigned long) (alpha) >> (unsigned long) \
+#define GetBit(alpha,i) MagickMin((((size_t) (alpha) >> (size_t) \
   (i)) & 0x01),16)
-#define SetBit(alpha,i,set) (alpha)=(IndexPacket) ((set) != 0 ? \
-  (unsigned long) (alpha) | (1UL << (unsigned long) (i)) : (unsigned long) \
-  (alpha) & ~(1UL << (unsigned long) (i)))
+#define SetBit(indexes,i,set) SetIndexPixelComponent(indexes,((set) != 0 ? \
+  (size_t) GetIndexPixelComponent(indexes) | (one << (size_t) (i)) : (size_t) \
+  GetIndexPixelComponent(indexes) & ~(one << (size_t) (i))))
 
   Image
     *image,
@@ -109,12 +111,8 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   ImageInfo
     *read_info;
 
-  long
-    c,
-    i,
-    j,
-    k,
-    y;
+  int
+    c;
 
   MagickBooleanType
     status;
@@ -125,12 +123,22 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   register IndexPacket
     *indexes;
 
-  register long
-    x;
-
   register PixelPacket
     *q;
 
+  register ssize_t
+    x;
+
+  size_t
+    depth,
+    one;
+
+  ssize_t
+    i,
+    j,
+    k,
+    y;
+
   /*
     Initialize Image structure.
   */
@@ -141,6 +149,7 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
+  one=1;
   image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
@@ -165,51 +174,54 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   c=0;
   i=0;
   j=0;
-  i=MAGICKCORE_QUANTUM_DEPTH-1;
-  for (k=image->offset; (i >= 0) && (j < MAGICKCORE_QUANTUM_DEPTH); i--)
+  i=(ssize_t) (watermark->depth-1);
+  depth=watermark->depth;
+  for (k=image->offset; (i >= 0) && (j < (ssize_t) depth); i--)
   {
-    for (y=0; (y < (long) image->rows) && (j < MAGICKCORE_QUANTUM_DEPTH); y++)
+    for (y=0; (y < (ssize_t) image->rows) && (j < (ssize_t) depth); y++)
     {
       x=0;
-      for (; (x < (long) image->columns) && (j < MAGICKCORE_QUANTUM_DEPTH); x++)
+      for ( ; (x < (ssize_t) image->columns) && (j < (ssize_t) depth); x++)
       {
-        (void) AcquireOnePixel(watermark,k % (long) watermark->columns,
-          k/(long) watermark->columns,&pixel,exception);
-        q=GetImagePixels(image,x,y,1,1);
+        if ((k/(ssize_t) watermark->columns) >= (ssize_t) watermark->rows)
+          break;
+        (void) GetOneVirtualPixel(watermark,k % (ssize_t) watermark->columns,
+          k/(ssize_t) watermark->columns,&pixel,exception);
+        q=GetAuthenticPixels(image,x,y,1,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         switch (c)
         {
           case 0:
           {
-            SetBit(*indexes,i,GetBit(pixel.red,j));
+            SetBit(indexes,i,GetBit(pixel.red,j));
             break;
           }
           case 1:
           {
-            SetBit(*indexes,i,GetBit(pixel.green,j));
+            SetBit(indexes,i,GetBit(pixel.green,j));
             break;
           }
           case 2:
           {
-            SetBit(*indexes,i,GetBit(pixel.blue,j));
+            SetBit(indexes,i,GetBit(pixel.blue,j));
             break;
           }
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         c++;
         if (c == 3)
           c=0;
         k++;
-        if (k == (long) (watermark->columns*watermark->columns))
+        if (k == (ssize_t) (watermark->columns*watermark->columns))
           k=0;
         if (k == image->offset)
           j++;
       }
     }
-    status=SetImageProgress(image,LoadImagesTag,i,MAGICKCORE_QUANTUM_DEPTH);
+    status=SetImageProgress(image,LoadImagesTag,(MagickOffsetType) i,depth);
     if (status == MagickFalse)
       break;
   }
@@ -238,17 +250,17 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterSTEGANOImage method is:
 %
-%      unsigned long RegisterSTEGANOImage(void)
+%      size_t RegisterSTEGANOImage(void)
 %
 */
-ModuleExport unsigned long RegisterSTEGANOImage(void)
+ModuleExport size_t RegisterSTEGANOImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("STEGANO");
   entry->decoder=(DecodeImageHandler *) ReadSTEGANOImage;
-  entry->format_type=ExplicitFormatType;
+  entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Steganographic image");
   entry->module=ConstantString("STEGANO");
   (void) RegisterMagickInfo(entry);
