@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -87,8 +87,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -98,7 +97,7 @@ static MagickBooleanType IsPFA(const unsigned char *magick,const size_t length)
 {
   if (length < 14)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"%!PS-AdobeFont-1.0",14) == 0)
+  if (LocaleNCompare((char *) magick,"%!PS-AdobeFont",14) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -123,8 +122,7 @@ static MagickBooleanType IsPFA(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -136,7 +134,7 @@ static MagickBooleanType IsTTF(const unsigned char *magick,const size_t length)
     return(MagickFalse);
   if (((int) magick[0] == 0x00) && ((int) magick[1] == 0x01) &&
       ((int) magick[2] == 0x00) && ((int) magick[3] == 0x00) &&
-      ((int) magick[4] == 0x0))
+      ((int) magick[4] == 0x00))
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -163,7 +161,7 @@ static MagickBooleanType IsTTF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -171,14 +169,14 @@ static MagickBooleanType IsTTF(const unsigned char *magick,const size_t length)
 static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
-    buffer[MaxTextExtent];
+    buffer[MaxTextExtent],
+    *text;
 
   const char
     *Text = (char *)
-      "The quick brown fox jumps over the lazy dog 0123456789\n"
-      "ABCDEFGHIJKLMNOPQRSTUVWXYZ &#~\"\\'(-`_^@)=+\260\n"
-      "$\243^\250*\265\371%!\247:/;.,?<> "
-      "\342\352\356\373\364\344\353\357\366\374\377\340\371\351\350\347 ";
+      "abcdefghijklmnopqrstuvwxyz\n"
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
+      "0123456789.:,;(*!?}^)#${%^&-+@\n";
 
   const TypeInfo
     *type_info;
@@ -189,22 +187,22 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
   PixelPacket
     background_color;
 
-  register long
+  register ssize_t
     i,
     x;
 
   register PixelPacket
     *q;
 
+  ssize_t
+    y;
+
   /*
     Open image file.
   */
@@ -215,7 +213,7 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   image->columns=800;
   image->rows=480;
   type_info=GetTypeInfo(image_info->filename,exception);
@@ -231,20 +229,15 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Color canvas with background color
   */
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   background_color=image_info->background_color;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
       *q++=background_color;
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   (void) CopyMagickString(image->magick,image_info->magick,MaxTextExtent);
@@ -256,26 +249,30 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   draw_info->font=AcquireString(image->filename);
   ConcatenateString(&draw_info->primitive,"push graphic-context\n");
-  (void) FormatMagickString(buffer,MaxTextExtent," viewbox 0 0 %lu %lu\n",
-    image->columns,image->rows);
+  (void) FormatMagickString(buffer,MaxTextExtent," viewbox 0 0 %.20g %.20g\n",
+    (double) image->columns,(double) image->rows);
   ConcatenateString(&draw_info->primitive,buffer);
   ConcatenateString(&draw_info->primitive," font-size 18\n");
-  (void) FormatMagickString(buffer,MaxTextExtent," text 10,%ld '",y);
+  (void) FormatMagickString(buffer,MaxTextExtent," text 10,%.20g '",(double) y);
   ConcatenateString(&draw_info->primitive,buffer);
-  ConcatenateString(&draw_info->primitive,Text);
+  text=EscapeString(Text,'"');
+  ConcatenateString(&draw_info->primitive,text);
+  text=DestroyString(text);
   (void) FormatMagickString(buffer,MaxTextExtent,"'\n");
   ConcatenateString(&draw_info->primitive,buffer);
-  y+=20*MultilineCensus((char *) Text)+20;
+  y+=20*(ssize_t) MultilineCensus((char *) Text)+20;
   for (i=12; i <= 72; i+=6)
   {
     y+=i+12;
     ConcatenateString(&draw_info->primitive," font-size 18\n");
-    (void) FormatMagickString(buffer,MaxTextExtent," text 10,%ld '%ld'\n",y,i);
+    (void) FormatMagickString(buffer,MaxTextExtent," text 10,%.20g '%.20g'\n",
+      (double) y,(double) i);
     ConcatenateString(&draw_info->primitive,buffer);
-    (void) FormatMagickString(buffer,MaxTextExtent," font-size %ld\n",i);
+    (void) FormatMagickString(buffer,MaxTextExtent," font-size %.20g\n",
+      (double) i);
     ConcatenateString(&draw_info->primitive,buffer);
-    (void) FormatMagickString(buffer,MaxTextExtent," text 50,%ld "
-      "'That which does not destroy me, only makes me stronger.'\n",y);
+    (void) FormatMagickString(buffer,MaxTextExtent," text 50,%.20g "
+      "'That which does not destroy me, only makes me stronger.'\n",(double) y);
     ConcatenateString(&draw_info->primitive,buffer);
     if (i >= 24)
       i+=6;
@@ -283,10 +280,10 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ConcatenateString(&draw_info->primitive,"pop graphic-context");
   (void) DrawImage(image,draw_info);
   /*
-    Free resources.
+    Relinquish resources.
   */
   draw_info=DestroyDrawInfo(draw_info);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 #endif /* MAGICKCORE_FREETYPE_DELEGATE */
@@ -311,10 +308,10 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterTTFImage method is:
 %
-%      unsigned long RegisterTTFImage(void)
+%      size_t RegisterTTFImage(void)
 %
 */
-ModuleExport unsigned long RegisterTTFImage(void)
+ModuleExport size_t RegisterTTFImage(void)
 {
   char
     version[MaxTextExtent];
