@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -85,10 +86,10 @@ static MagickBooleanType
 %
 %  The format of the RegisterUILImage method is:
 %
-%      unsigned long RegisterUILImage(void)
+%      size_t RegisterUILImage(void)
 %
 */
-ModuleExport unsigned long RegisterUILImage(void)
+ModuleExport size_t RegisterUILImage(void)
 {
   MagickInfo
     *entry;
@@ -150,7 +151,6 @@ ModuleExport void UnregisterUILImage(void)
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
 {
@@ -160,15 +160,14 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
     basename[MaxTextExtent],
     buffer[MaxTextExtent],
     name[MaxTextExtent],
-    symbol[MaxTextExtent];
+    *symbol;
+
+  ExceptionInfo
+    *exception;
 
   int
     j;
 
-  long
-    k,
-    y;
-
   MagickBooleanType
     status,
     transparent;
@@ -179,24 +178,28 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
   MagickSizeType
     number_pixels;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
-  register long
+  register ssize_t
     i,
     x;
 
+  size_t
+    characters_per_pixel,
+    colors;
+
+  ssize_t
+    k,
+    y;
+
   static const char
     Cixel[MaxCixels+1] = " .XoO+@#$%&*=-;:>,<1234567890qwertyuipasdfghjk"
                          "lzxcvbnmMNBVCZASDFGHJKLPIUYTREWQ!~^/()_`'][{}|";
 
-  unsigned long
-    characters_per_pixel,
-    colors;
-
   /*
     Open output image file.
   */
@@ -210,7 +213,8 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
+  exception=(&image->exception);
   transparent=MagickFalse;
   i=0;
   p=(const PixelPacket *) NULL;
@@ -237,15 +241,15 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
             image->rows*sizeof(*matte_image));
           if (matte_image == (unsigned char *) NULL)
             ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            for (x=0; x < (long) image->columns; x++)
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
-              matte_image[i]=(unsigned char)
-              (p->opacity == (Quantum) TransparentOpacity ? 1 : 0);
+              matte_image[i]=(unsigned char) (GetOpacityPixelComponent(p) ==
+                (Quantum) TransparentOpacity ? 1 : 0);
               if (matte_image[i] != 0)
                 transparent=MagickTrue;
               i++;
@@ -255,19 +259,25 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
         }
       (void) SetImageType(image,PaletteType);
       colors=image->colors;
-      if (transparent)
+      if (transparent != MagickFalse)
         {
+          register IndexPacket
+            *indexes;
+
+          register PixelPacket
+            *q;
+
           colors++;
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-            if (p == (const PixelPacket *) NULL)
+            q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+            if (q == (PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
-            for (x=0; x < (long) image->columns; x++)
+            indexes=GetAuthenticIndexQueue(image);
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
-              if (matte_image[i])
-                indexes[x]=(IndexPacket) image->colors;
+              if (matte_image[i] != 0)
+                SetIndexPixelComponent(indexes+x,image->colors);
               p++;
             }
           }
@@ -279,18 +289,19 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
     Compute the character per pixel.
   */
   characters_per_pixel=1;
-  for (k=MaxCixels; (long) colors > k; k*=MaxCixels)
+  for (k=MaxCixels; (ssize_t) colors > k; k*=MaxCixels)
     characters_per_pixel++;
   /*
     UIL header.
   */
+  symbol=AcquireString("");
   (void) WriteBlobString(image,"/* UIL */\n");
   GetPathComponent(image->filename,BasePath,basename);
   (void) FormatMagickString(buffer,MaxTextExtent,
     "value\n  %s_ct : color_table(\n",basename);
   (void) WriteBlobString(image,buffer);
   GetMagickPixelPacket(image,&pixel);
-  for (i=0; i < (long) colors; i++)
+  for (i=0; i < (ssize_t) colors; i++)
   {
     /*
       Define UIL color.
@@ -301,7 +312,7 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
     pixel.opacity=(MagickRealType) OpaqueOpacity;
     GetColorTuple(&pixel,MagickTrue,name);
     if (transparent != MagickFalse)
-      if (i == (long) (colors-1))
+      if (i == (ssize_t) (colors-1))
         (void) CopyMagickString(name,"None",MaxTextExtent);
     /*
       Write UIL color.
@@ -314,6 +325,7 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
       symbol[j]=Cixel[k];
     }
     symbol[j]='\0';
+    (void) SubstituteString(&symbol,"'","''");
     if (LocaleCompare(name,"None") == 0)
       (void) FormatMagickString(buffer,MaxTextExtent,
         "    background color = '%s'",symbol);
@@ -324,7 +336,7 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
         ((Quantum) QuantumRange/2) ? "background" : "foreground",symbol);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"%s",
-      (i == (long) (colors-1) ? ");\n" : ",\n"));
+      (i == (ssize_t) (colors-1) ? ");\n" : ",\n"));
     (void) WriteBlobString(image,buffer);
   }
   /*
@@ -334,20 +346,20 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
   (void) FormatMagickString(buffer,MaxTextExtent,
     "  %s_icon : icon(color_table = %s_ct,\n",basename,basename);
   (void) WriteBlobString(image,buffer);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     (void) WriteBlobString(image,"    \"");
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      k=((long) indexes[x] % MaxCixels);
+      k=((ssize_t) GetIndexPixelComponent(indexes+x) % MaxCixels);
       symbol[0]=Cixel[k];
       for (j=1; j < (int) characters_per_pixel; j++)
       {
-        k=(((int) indexes[x]-k)/MaxCixels) % MaxCixels;
+        k=(((int) GetIndexPixelComponent(indexes+x)-k)/MaxCixels) % MaxCixels;
         symbol[j]=Cixel[k];
       }
       symbol[j]='\0';
@@ -356,12 +368,14 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
       p++;
     }
     (void) FormatMagickString(buffer,MaxTextExtent,"\"%s\n",
-      (y == (long) (image->rows-1) ? ");" : ","));
+      (y == (ssize_t) (image->rows-1) ? ");" : ","));
     (void) WriteBlobString(image,buffer);
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
     if (status == MagickFalse)
       break;
   }
+  symbol=DestroyString(symbol);
   (void) CloseBlob(image);
   return(MagickTrue);
 }
