@@ -16,7 +16,7 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,8 @@
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
+#include "magick/colormap-private.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -222,12 +224,10 @@ const RGB_Record WPG1_Palette[256]={
 %
 %    o status:  Method IsWPG returns True if the image format type is WPG.
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static unsigned int IsWPG(const unsigned char *magick,const size_t length)
 {
@@ -239,7 +239,7 @@ static unsigned int IsWPG(const unsigned char *magick,const size_t length)
 }
 
 
-static void Rd_WP_DWORD(Image *image,unsigned long *d)
+static void Rd_WP_DWORD(Image *image,size_t *d)
 {
   unsigned char
     b;
@@ -249,25 +249,28 @@ static void Rd_WP_DWORD(Image *image,unsigned long *d)
   if (b < 0xFFU)
     return;
   b=ReadBlobByte(image);
-  *d=(unsigned long) b;
+  *d=(size_t) b;
   b=ReadBlobByte(image);
-  *d+=(unsigned long) b*256l;
+  *d+=(size_t) b*256l;
   if (*d < 0x8000)
     return;
   *d=(*d & 0x7FFF) << 16;
   b=ReadBlobByte(image);
-  *d+=(unsigned long) b;
+  *d+=(size_t) b;
   b=ReadBlobByte(image);
-  *d+=(unsigned long) b*256l;
+  *d+=(size_t) b*256l;
   return;
 }
 
-static void InsertRow(unsigned char *p,long y,Image *image, int bpp)
+static void InsertRow(unsigned char *p,ssize_t y,Image *image, int bpp)
 {
+  ExceptionInfo
+    *exception;
+
   int
     bit;
 
-  long
+  ssize_t
     x;
 
   register PixelPacket
@@ -279,143 +282,187 @@ static void InsertRow(unsigned char *p,long y,Image *image, int bpp)
   register IndexPacket
     *indexes;
 
+  exception=(&image->exception);
   switch (bpp)
     {
     case 1:  /* Convert bitmap scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-7); x+=8)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-7); x+=8)
           {
             for (bit=0; bit < 8; bit++)
               {
                 index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
-                indexes[x+bit]=index;
-                *q++=image->colormap[(int) index];
+                SetIndexPixelComponent(indexes+x+bit,index);
+                SetRedPixelComponent(q,image->colormap[(int) index].red);
+                SetGreenPixelComponent(q,image->colormap[(int) index].green);
+                SetBluePixelComponent(q,image->colormap[(int) index].blue);
+                q++;
               }
             p++;
           }
         if ((image->columns % 8) != 0)
           {
-            for (bit=0; bit < (long) (image->columns % 8); bit++)
+            for (bit=0; bit < (ssize_t) (image->columns % 8); bit++)
               {
                 index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
-                indexes[x+bit]=index;
-                *q++=image->colormap[(int) index];
+                SetIndexPixelComponent(indexes+x+bit,index);
+                SetRedPixelComponent(q,image->colormap[(int) index].red);
+                SetGreenPixelComponent(q,image->colormap[(int) index].green);
+                SetBluePixelComponent(q,image->colormap[(int) index].blue);
+                q++;
               }
             p++;
           }
-        if (!SyncImagePixels(image))
+        if (!SyncAuthenticPixels(image,exception))
           break;
         break;
       }
     case 2:  /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-1); x+=2)
         {
             index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+            q++;
             index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+            q++;
             index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+            q++;
             index=ConstrainColormapIndex(image,(*p) & 0x3);
-            indexes[x+1]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x+1,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
             p++;
+            q++;
         }
        if ((image->columns % 4) != 0)
           {
             index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+            q++;
             if ((image->columns % 4) >= 1)
 
               {
                 index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
-                indexes[x]=index;
-                *q++=image->colormap[(long) index];
+                SetIndexPixelComponent(indexes+x,index);
+                SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+                SetGreenPixelComponent(q,image->colormap[(ssize_t)
+                  index].green);
+                SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+                q++;
                 if ((image->columns % 4) >= 2)
 
                   {
                     index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
-                    indexes[x]=index;
-                    *q++=image->colormap[(long) index];
+                    SetIndexPixelComponent(indexes+x,index);
+                    SetRedPixelComponent(q,image->colormap[(ssize_t)
+                      index].red);
+                    SetGreenPixelComponent(q,image->colormap[(ssize_t)
+                      index].green);
+                    SetBluePixelComponent(q,image->colormap[(ssize_t)
+                      index].blue);
+                    q++;
                   }
               }
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         break;
       }
  
     case 4:  /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-1); x+=2)
           { 
             index=ConstrainColormapIndex(image,(*p >> 4) & 0x0f);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+            q++;
             index=ConstrainColormapIndex(image,(*p) & 0x0f);
-            indexes[x+1]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x+1,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
             p++;
+            q++;
           }
         if ((image->columns % 2) != 0)
           {
             index=ConstrainColormapIndex(image,(*p >> 4) & 0x0f);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
             p++;
+            q++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         break;
       }
     case 8: /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL) break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
 
-        for (x=0; x < (long) image->columns; x++)
+        for (x=0; x < (ssize_t) image->columns; x++)
           {
             index=ConstrainColormapIndex(image,*p);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
             p++;
+            q++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
       }
       break;
      
     case 24:     /*  Convert DirectColor scanline.  */
-      q=SetImagePixels(image,0,y,image->columns,1);
+      q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      for (x=0; x < (long) image->columns; x++)
+      for (x=0; x < (ssize_t) image->columns; x++)
         {
-          q->red=ScaleCharToQuantum(*p++);
-          q->green=ScaleCharToQuantum(*p++);
-          q->blue=ScaleCharToQuantum(*p++);
+          SetRedPixelComponent(q,ScaleCharToQuantum(*p++));
+          SetGreenPixelComponent(q,ScaleCharToQuantum(*p++));
+          SetBluePixelComponent(q,ScaleCharToQuantum(*p++));
           q++;
         }
-      if (!SyncImagePixels(image))
+      if (!SyncAuthenticPixels(image,exception))
         break;
       break;     
     }
@@ -427,9 +474,9 @@ static void InsertRow(unsigned char *p,long y,Image *image, int bpp)
 { \
   BImgBuff[x]=b; \
   x++; \
-  if((long) x>=ldblk) \
+  if((ssize_t) x>=ldblk) \
   { \
-    InsertRow(BImgBuff,(long) y,image,bpp); \
+    InsertRow(BImgBuff,(ssize_t) y,image,bpp); \
     x=0; \
     y++; \
     } \
@@ -447,18 +494,18 @@ static int UnpackWPGRaster(Image *image,int bpp)
     *BImgBuff,
     RunCount;
 
-  long
+  ssize_t
     ldblk;
 
   x=0;
   y=0;
 
-  ldblk=(long) ((bpp*image->columns+7)/8);
+  ldblk=(ssize_t) ((bpp*image->columns+7)/8);
   BImgBuff=(unsigned char *) AcquireQuantumMemory((size_t) ldblk,
     sizeof(*BImgBuff));
   if(BImgBuff==NULL) return(-2);
 
-  while(y<(long) image->rows)
+  while(y<(ssize_t) image->rows)
     {
       bbuf=ReadBlobByte(image);
 
@@ -496,7 +543,7 @@ static int UnpackWPGRaster(Image *image,int bpp)
               x=0;
               y++;    /* Here I need to duplicate previous row RUNCOUNT* */
               if(y<2) continue;
-              if(y>(long) image->rows)
+              if(y>(ssize_t) image->rows)
                 {
                   BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
                   return(-4);
@@ -519,9 +566,9 @@ static int UnpackWPGRaster(Image *image,int bpp)
   else\
     BImgBuff[x] = b;\
   x++; \
-  if((long) x >= ldblk) \
+  if((ssize_t) x >= ldblk) \
   { \
-    InsertRow(BImgBuff,(long) y,image,bpp); \
+    InsertRow(BImgBuff,(ssize_t) y,image,bpp); \
     x=0; \
     y++; \
    } \
@@ -529,6 +576,15 @@ static int UnpackWPGRaster(Image *image,int bpp)
 /* WPG2 raster reader. */
 static int UnpackWPG2Raster(Image *image,int bpp)
 {
+  size_t
+    x,
+    y;
+
+  ssize_t
+    ldblk;
+
+  int XorMe = 0;
+
   unsigned int
     SampleSize=1;
 
@@ -538,21 +594,12 @@ static int UnpackWPG2Raster(Image *image,int bpp)
     RunCount,
     SampleBuffer[8];
 
-  unsigned long
-    x,
-    y;
-
   unsigned int
     i;
 
-  long
-    ldblk;
-
-  int XorMe = 0;
-
   x=0;
   y=0;
-  ldblk=(long) ((bpp*image->columns+7)/8);
+  ldblk=(ssize_t) ((bpp*image->columns+7)/8);
   BImgBuff=(unsigned char *) AcquireQuantumMemory((size_t) ldblk,
     sizeof(*BImgBuff));
   if(BImgBuff==NULL)
@@ -593,15 +640,15 @@ static int UnpackWPG2Raster(Image *image,int bpp)
           if(x!=0)
             {
               (void) fprintf(stderr,
-                             "\nUnsupported WPG2 unaligned token RST x=%lu, please report!\n"
-                             ,x);
+                             "\nUnsupported WPG2 unaligned token RST x=%.20g, please report!\n"
+                             ,(double) x);
               return(-3);
             }
           {
             /* duplicate the previous row RunCount x */
             for(i=0;i<=RunCount;i++)
               {      
-                InsertRow(BImgBuff,(long) (image->rows >= y ? y : image->rows-1),
+                InsertRow(BImgBuff,(ssize_t) (image->rows >= y ? y : image->rows-1),
                           image,bpp);
                 y++;
               }    
@@ -644,7 +691,7 @@ typedef float tCTM[3][3];
 static unsigned LoadWPG2Flags(Image *image,char Precision,float *Angle,tCTM *CTM)
 {
 const unsigned char TPR=1,TRN=2,SKW=4,SCL=8,ROT=0x10,OID=0x20,LCK=0x80;
-long x;
+ssize_t x;
 unsigned DenX;
 unsigned Flags;
 
@@ -703,7 +750,7 @@ unsigned Flags;
 
 
 static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
-  MagickOffsetType PS_Offset,long PS_Size,ExceptionInfo *exception)
+  MagickOffsetType PS_Offset,ssize_t PS_Size,ExceptionInfo *exception)
 {
   char
     postscript_file[MaxTextExtent];
@@ -756,8 +803,8 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   (void) CopyMagickMemory(clone_info->magick,magic_info->name,MaxTextExtent);
   
     /* Read nested image */
-  /*FormatString(clone_info->filename,"%s:%.1024s",magic_info->name,postscript_file);*/
-  FormatMagickString(clone_info->filename,MaxTextExtent,"%.1024s",postscript_file);
+  /*FormatString(clone_info->filename,"%s:%s",magic_info->name,postscript_file);*/
+  FormatMagickString(clone_info->filename,MaxTextExtent,"%s",postscript_file);
   image2=ReadImage(clone_info,exception);
 
   if (!image2)
@@ -821,7 +868,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
 {
   typedef struct
   {
-    unsigned long FileId;
+    size_t FileId;
     MagickOffsetType DataOffset;
     unsigned int ProductType;
     unsigned int FileType;
@@ -834,15 +881,15 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   typedef struct
   {
     unsigned char RecType;
-    unsigned long RecordLength;
+    size_t RecordLength;
   } WPGRecord;
 
   typedef struct
   {
     unsigned char Class;
     unsigned char RecType;
-    unsigned long Extension;
-    unsigned long RecordLength;
+    size_t Extension;
+    size_t RecordLength;
   } WPG2Record;
 
   typedef struct
@@ -890,7 +937,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   } WPGColorMapRec;
 
   typedef struct {
-    unsigned long PS_unknown1;
+    size_t PS_unknown1;
     unsigned int PS_unknown2;
     unsigned int PS_unknown3;
   } WPGPSl1Record;  
@@ -930,9 +977,12 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     bpp,
     WPG2Flags;
 
-  long
+  ssize_t
     ldblk;
 
+  size_t
+    one;
+
   unsigned char
     *BImgBuff;
 
@@ -945,6 +995,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
+  one=1;
   image=AcquireImage(image_info);
   image->depth=8;
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
@@ -1038,7 +1089,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if(Rec.RecordLength > 8)
                 image=ExtractPostscript(image,image_info,
                   TellBlob(image)+8,   /* skip PS header in the wpg */
-                  (long) Rec.RecordLength-8,exception);
+                  (ssize_t) Rec.RecordLength-8,exception);
               break;     
 
             case 0x14:  /* bitmap type 2 */
@@ -1072,7 +1123,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
             UnpackRaster:      
               if ((image->colors == 0) && (bpp != 24))
                 {
-                  image->colors=1 << bpp;
+                  image->colors=one << bpp;
                   if (!AcquireImageColormap(image,image->colors))
                     {
                     NoMemory:
@@ -1090,9 +1141,9 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               else
                 {
                   if (bpp < 24)
-                    if ( (image->colors < (1UL<<bpp)) && (bpp != 24) )
+                    if ( (image->colors < (one << bpp)) && (bpp != 24) )
                       image->colormap=(PixelPacket *) ResizeQuantumMemory(
-                        image->colormap,(size_t) (1UL << bpp),
+                        image->colormap,(size_t) (one << bpp),
                         sizeof(*image->colormap));
                 }
           
@@ -1125,7 +1176,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     {
                       rotated_image = FlopImage(image, exception);
                       rotated_image->blob = image->blob;
-                      image->blob = NULL;
+                      DuplicateBlob(rotated_image,image);
                       (void) RemoveLastImageFromList(&image);
                       AppendImageToList(&image,rotated_image);
                     }
@@ -1134,7 +1185,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     {
                       rotated_image = FlipImage(image, exception);
                       rotated_image->blob = image->blob;
-                      image->blob = NULL;
+                      DuplicateBlob(rotated_image,image);
                       (void) RemoveLastImageFromList(&image);
                       AppendImageToList(&image,rotated_image);    
                     }
@@ -1144,7 +1195,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     {
                       rotated_image = RotateImage(image, (BitmapHeader2.RotAngle & 0x0FFF), exception);
                       rotated_image->blob = image->blob;
-                      image->blob = NULL;
+                      DuplicateBlob(rotated_image,image);
                       (void) RemoveLastImageFromList(&image);
                       AppendImageToList(&image,rotated_image);    
                     }                
@@ -1164,7 +1215,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if(Rec.RecordLength>0x3C)
                 image=ExtractPostscript(image,image_info,
                   TellBlob(image)+0x3C,   /* skip PS l2 header in the wpg */
-                  (long) Rec.RecordLength-0x3C,exception);
+                  (ssize_t) Rec.RecordLength-0x3C,exception);
               break;
             }
         }
@@ -1252,16 +1303,20 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
 
               if ((image->colors == 0) && (bpp != 24))
                 {
-                  image->colors=1 << bpp;
+                  size_t
+                    one;
+
+                  one=1;
+                  image->colors=one << bpp;
                   if (!AcquireImageColormap(image,image->colors))
                     goto NoMemory;
                 }
               else
                 {
                   if(bpp < 24)
-                    if( image->colors<(1UL<<bpp) && bpp!=24 )
+                    if( image->colors<(one << bpp) && bpp!=24 )
                       image->colormap=(PixelPacket *) ResizeQuantumMemory(
-                       image->colormap,(size_t) (1UL << bpp),
+                       image->colormap,(size_t) (one << bpp),
                        sizeof(*image->colormap));
                 }
 
@@ -1270,13 +1325,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                 {
                 case 0:    /*Uncompressed raster*/
                   {
-                    ldblk=(long) ((bpp*image->columns+7)/8);
+                    ldblk=(ssize_t) ((bpp*image->columns+7)/8);
                     BImgBuff=(unsigned char *) AcquireQuantumMemory((size_t)
                       ldblk,sizeof(*BImgBuff));
                     if (BImgBuff == (unsigned char *) NULL)
                       goto NoMemory;
 
-                    for(i=0; i< (long) image->rows; i++)
+                    for(i=0; i< (ssize_t) image->rows; i++)
                       {
                         (void) ReadBlob(image,ldblk,BImgBuff);
                         InsertRow(BImgBuff,i,image,bpp);
@@ -1298,7 +1353,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     {    /*?? RotAngle=360-RotAngle;*/
       rotated_image = FlopImage(image, exception);
       rotated_image->blob = image->blob;
-      image->blob = NULL;
+      DuplicateBlob(rotated_image,image);
       (void) RemoveLastImageFromList(&image);
       AppendImageToList(&image,rotated_image);
                   /* Try to change CTM according to Flip - I am not sure, must be checked.      
@@ -1311,7 +1366,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     {    /*?? RotAngle=360-RotAngle;*/
       rotated_image = FlipImage(image, exception);
       rotated_image->blob = image->blob;
-      image->blob = NULL;
+      DuplicateBlob(rotated_image,image);
       (void) RemoveLastImageFromList(&image);
       AppendImageToList(&image,rotated_image);
                   /* Try to change CTM according to Flip - I am not sure, must be checked.
@@ -1338,11 +1393,12 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if(Rec2.RecordLength > (unsigned int) i)
                 image=ExtractPostscript(image,image_info,
                   TellBlob(image)+i,    /*skip PS header in the wpg2*/
-                  (long) (Rec2.RecordLength-i-2),exception);
+                  (ssize_t) (Rec2.RecordLength-i-2),exception);
               break;
 
       case 0x1B:          /*bitmap rectangle*/
               WPG2Flags = LoadWPG2Flags(image,StartWPG.PosSizePrecision,NULL,&CTM);
+              (void) WPG2Flags;
               break;
             }
         }
@@ -1362,7 +1418,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     Image
       *p;
 
-    long
+    ssize_t
       scene=0;
 
     /*
@@ -1385,7 +1441,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
       Fix scene numbers.
     */
     for (p=image; p != (Image *) NULL; p=p->next)
-      p->scene=(unsigned long) scene++;
+      p->scene=(size_t) scene++;
   }
   if (image == (Image *) NULL)
     ThrowReaderException(CorruptImageError,
@@ -1413,10 +1469,10 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterWPGImage method is:
 %
-%      unsigned long RegisterWPGImage(void)
+%      size_t RegisterWPGImage(void)
 %
 */
-ModuleExport unsigned long RegisterWPGImage(void)
+ModuleExport size_t RegisterWPGImage(void)
 {
   MagickInfo
     *entry;
