@@ -17,7 +17,7 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/composite.h"
 #include "magick/exception.h"
@@ -51,6 +52,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/pixel.h"
 #include "magick/quantize.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -106,7 +108,7 @@ typedef enum
 
 typedef struct
 {
-  unsigned long
+  size_t
     width,
     height,
     image_type,
@@ -130,7 +132,7 @@ typedef struct
   unsigned int
     active;
 
-  unsigned long
+  size_t
     width,
     height,
     type,
@@ -147,7 +149,7 @@ typedef struct
     offset_x,
     offset_y;
 
-  unsigned long
+  size_t
     mode,
     tattoo;
 
@@ -187,8 +189,7 @@ typedef struct
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -231,7 +232,7 @@ typedef enum
   ImageMagick compositing operators
 */
 static CompositeOperator GIMPBlendModeToCompositeOperator(
-  unsigned long blendMode)
+  size_t blendMode)
 {
   switch ( blendMode )
   {
@@ -272,7 +273,7 @@ static CompositeOperator GIMPBlendModeToCompositeOperator(
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  ReadBlobStringWithLongSize reads characters from a blob or file
-%  starting with a long length byte and then characters to that length
+%  starting with a ssize_t length byte and then characters to that length
 %
 %  The format of the ReadBlobStringWithLongSize method is:
 %
@@ -301,10 +302,10 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max)
   MagickOffsetType
     offset;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     length;
 
   assert(image != (Image *) NULL);
@@ -313,7 +314,7 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max)
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   length=ReadBlobMSBLong(image);
-  for (i=0; i < (long) MagickMin(length,max-1); i++)
+  for (i=0; i < (ssize_t) MagickMin(length,max-1); i++)
   {
     c=ReadBlobByte(image);
     if (c == EOF)
@@ -331,10 +332,13 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max)
 static MagickBooleanType load_tile(Image *image,Image *tile_image,
   XCFDocInfo *inDocInfo,XCFLayerInfo *inLayerInfo,size_t data_length)
 {
-  long
+  ExceptionInfo
+    *exception;
+
+  ssize_t
     y;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
@@ -360,19 +364,21 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
   if (count != (ssize_t) data_length)
     ThrowBinaryException(CorruptImageError,"NotEnoughPixelData",
       image->filename);
-  for (y=0; y < (long) tile_image->rows; y++)
+  exception=(&image->exception);
+  for (y=0; y < (ssize_t) tile_image->rows; y++)
   {
-    q=SetImagePixels(tile_image,0,y,tile_image->columns,1);
+    q=QueueAuthenticPixels(tile_image,0,y,tile_image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
     if (inDocInfo->image_type == GIMP_GRAY)
       {
-        for (x=0; x < (long) tile_image->columns; x++)
+        for (x=0; x < (ssize_t) tile_image->columns; x++)
         {
-          q->red=ScaleCharToQuantum(*graydata);
-          q->green=q->red;
-          q->blue=q->red;
-          q->opacity=ScaleCharToQuantum(255-inLayerInfo->opacity);
+          SetRedPixelComponent(q,ScaleCharToQuantum(*graydata));
+          SetGreenPixelComponent(q,GetRedPixelComponent(q));
+          SetBluePixelComponent(q,GetRedPixelComponent(q));
+          SetOpacityPixelComponent(q,ScaleCharToQuantum(
+            (unsigned char) (255-inLayerInfo->opacity)));
           graydata++;
           q++;
         }
@@ -380,18 +386,19 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
     else
       if (inDocInfo->image_type == GIMP_RGB)
         {
-          for (x=0; x < (long) tile_image->columns; x++)
+          for (x=0; x < (ssize_t) tile_image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(xcfdata->red);
-            q->green=ScaleCharToQuantum(xcfdata->green);
-            q->blue=ScaleCharToQuantum(xcfdata->blue);
-            q->opacity=(Quantum) (xcfdata->opacity == 0U ? TransparentOpacity :
-              ScaleCharToQuantum(255-inLayerInfo->opacity));
+            SetRedPixelComponent(q,ScaleCharToQuantum(xcfdata->red));
+            SetGreenPixelComponent(q,ScaleCharToQuantum(xcfdata->green));
+            SetBluePixelComponent(q,ScaleCharToQuantum(xcfdata->blue));
+            SetOpacityPixelComponent(q,xcfdata->opacity == 0U ?
+              TransparentOpacity : ScaleCharToQuantum((unsigned char) (255-
+              inLayerInfo->opacity)));
             xcfdata++;
             q++;
           }
         }
-     if (SyncImagePixels(tile_image) == MagickFalse)
+     if (SyncAuthenticPixels(tile_image,exception) == MagickFalse)
        break;
   }
   xcfodata=(XCFPixelPacket *) RelinquishMagickMemory(xcfodata);
@@ -401,9 +408,8 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
 static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
   XCFDocInfo *inDocInfo,XCFLayerInfo *inLayerInfo,size_t data_length)
 {
-  long
-    i,
-    j;
+  ExceptionInfo
+    *exception;
 
   MagickOffsetType
     size;
@@ -411,13 +417,15 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
   register PixelPacket
     *q;
 
-  ssize_t
-    bytes_per_pixel,
-    count;
-
   size_t
     length;
 
+  ssize_t
+    bytes_per_pixel,
+    count,
+    i,
+    j;
+
   unsigned char
     data,
     pixel,
@@ -433,9 +441,11 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
   xcfodata=xcfdata;
   count=ReadBlob(image, (size_t) data_length, xcfdata);
   xcfdatalimit = xcfodata+count-1;
-  for (i=0; i < (long) bytes_per_pixel; i++)
+  exception=(&image->exception);
+  for (i=0; i < (ssize_t) bytes_per_pixel; i++)
   {
-    q=GetImagePixels(tile_image,0,0,tile_image->columns,tile_image->rows);
+    q=GetAuthenticPixels(tile_image,0,0,tile_image->columns,tile_image->rows,
+      exception);
     size=(MagickOffsetType) tile_image->rows*tile_image->columns;
     while (size > 0)
     {
@@ -465,35 +475,38 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
               {
                 case 0:
                 {
-                  q->red=ScaleCharToQuantum(data);
+                  SetRedPixelComponent(q,ScaleCharToQuantum(data));
                   if (inDocInfo->image_type == GIMP_GRAY)
                     {
-                      q->green=ScaleCharToQuantum(data);
-                      q->blue=ScaleCharToQuantum(data);
-                      q->opacity=ScaleCharToQuantum(255-inLayerInfo->opacity);
+                      SetGreenPixelComponent(q,ScaleCharToQuantum(data));
+                      SetBluePixelComponent(q,ScaleCharToQuantum(data));
+                      SetOpacityPixelComponent(q,ScaleCharToQuantum(
+                        (unsigned char) (255-inLayerInfo->opacity)));
                     }
                   else
                     {
-                      q->green= q->red;
-                      q->blue= q->red;
-                      q->opacity=ScaleCharToQuantum(255-inLayerInfo->opacity);
+                      SetGreenPixelComponent(q,GetRedPixelComponent(q));
+                      SetBluePixelComponent(q,GetRedPixelComponent(q));
+                      SetOpacityPixelComponent(q,ScaleCharToQuantum(
+                        (unsigned char) (255-inLayerInfo->opacity)));
                     }
                   break;
                 }
                 case 1:
                 {
-                  q->green=ScaleCharToQuantum(data);
+                  SetGreenPixelComponent(q,ScaleCharToQuantum(data));
                   break;
                 }
                 case 2:
                 {
-                  q->blue=ScaleCharToQuantum(data);
+                  SetBluePixelComponent(q,ScaleCharToQuantum(data));
                   break;
                 }
                 case 3:
                 {
-                  q->opacity=(Quantum) (data == 0 ? TransparentOpacity :
-                    ScaleCharToQuantum(255-inLayerInfo->opacity));
+                  SetOpacityPixelComponent(q,data == 0 ? TransparentOpacity :
+                    ScaleCharToQuantum((unsigned char) (255-
+                    inLayerInfo->opacity)));
                   break;
                 }
               }
@@ -516,42 +529,45 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
             if (xcfdata > xcfdatalimit)
               goto bogus_rle;
             pixel=(*xcfdata++);
-            for (j= 0; j < (long) length; j++)
+            for (j= 0; j < (ssize_t) length; j++)
             {
               data=pixel;
               switch (i)
               {
                 case 0:
                 {
-                  q->red=ScaleCharToQuantum(data);
+                  SetRedPixelComponent(q,ScaleCharToQuantum(data));
                   if (inDocInfo->image_type == GIMP_GRAY)
                     {
-                      q->green=ScaleCharToQuantum(data);
-                      q->blue=ScaleCharToQuantum(data);
-                      q->opacity=ScaleCharToQuantum(255-inLayerInfo->opacity);
+                      SetGreenPixelComponent(q,ScaleCharToQuantum(data));
+                      SetBluePixelComponent(q,ScaleCharToQuantum(data));
+                      SetOpacityPixelComponent(q,ScaleCharToQuantum(
+                        (unsigned char) (255-inLayerInfo->opacity)));
                     }
                   else
                     {
-                      q->green=q->red;
-                      q->blue=q->red;
-                      q->opacity=ScaleCharToQuantum(255-inLayerInfo->opacity);
+                      SetGreenPixelComponent(q,GetRedPixelComponent(q));
+                      SetBluePixelComponent(q,GetRedPixelComponent(q));
+                      SetOpacityPixelComponent(q,ScaleCharToQuantum(
+                        (unsigned char) (255-inLayerInfo->opacity)));
                     }
                   break;
                 }
                 case 1:
                 {
-                  q->green=ScaleCharToQuantum(data);
+                  SetGreenPixelComponent(q,ScaleCharToQuantum(data));
                   break;
                 }
                 case 2:
                 {
-                  q->blue=ScaleCharToQuantum(data);
+                  SetBluePixelComponent(q,ScaleCharToQuantum(data));
                   break;
                 }
                 case 3:
                 {
-                  q->opacity=(Quantum) (data == 0 ? TransparentOpacity :
-                    ScaleCharToQuantum(255-inLayerInfo->opacity));
+                  SetOpacityPixelComponent(q,data == 0 ? TransparentOpacity :
+                    ScaleCharToQuantum((unsigned char) (255-
+                    inLayerInfo->opacity)));
                   break;
                 }
               }
@@ -559,7 +575,7 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
             }
           }
       }
-      if (SyncImagePixels(tile_image) == MagickFalse)
+      if (SyncAuthenticPixels(tile_image,exception) == MagickFalse)
         break;
     }
   xcfodata=(unsigned char *) RelinquishMagickMemory(xcfodata);
@@ -592,10 +608,10 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     offset,
     offset2;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     width,
     height,
     ntiles,
@@ -621,7 +637,7 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
   ntile_rows=(height+TILE_HEIGHT-1)/TILE_HEIGHT;
   ntile_cols=(width+TILE_WIDTH-1)/TILE_WIDTH;
   ntiles=ntile_rows*ntile_cols;
-  for (i = 0; i < (long) ntiles; i++)
+  for (i = 0; i < (ssize_t) ntiles; i++)
   {
     status=MagickFalse;
     if (offset == 0)
@@ -643,10 +659,10 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
       /* allocate the image for the tile
         NOTE: the last tile in a row or column may not be a full tile!
       */
-      tile_image_width=(unsigned long) (destLeft == (int) ntile_cols-1 ?
+      tile_image_width=(size_t) (destLeft == (int) ntile_cols-1 ?
         (int) width % TILE_WIDTH : TILE_WIDTH);
       if (tile_image_width == 0) tile_image_width=TILE_WIDTH;
-      tile_image_height = (unsigned long) (destTop == (int) ntile_rows-1 ?
+      tile_image_height = (size_t) (destTop == (int) ntile_rows-1 ?
         (int) height % TILE_HEIGHT : TILE_HEIGHT);
       if (tile_image_height == 0) tile_image_height=TILE_HEIGHT;
       tile_image=CloneImage(inLayerInfo->image,tile_image_width,
@@ -706,14 +722,17 @@ static MagickBooleanType load_hierarchy(Image *image,XCFDocInfo *inDocInfo,
     offset,
     junk;
 
-  unsigned long
+  size_t
     width,
     height,
     bytes_per_pixel;
 
   width=ReadBlobMSBLong(image);
+  (void) width;
   height=ReadBlobMSBLong(image);
+  (void) height;
   bytes_per_pixel=inDocInfo->bytes_per_pixel=ReadBlobMSBLong(image);
+  (void) bytes_per_pixel;
 
   /* load in the levels...we make sure that the number of levels
    *  calculated when the TileManager was created is the same
@@ -750,16 +769,13 @@ static MagickBooleanType load_hierarchy(Image *image,XCFDocInfo *inDocInfo,
 static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
   XCFLayerInfo *outLayer )
 {
-  long
-    i;
-
   MagickOffsetType
     offset;
 
   unsigned int
     foundPropEnd = 0;
 
-  unsigned long
+  size_t
     hierarchy_offset,
     layer_mask_offset;
 
@@ -780,7 +796,7 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
   foundPropEnd = 0;
   while ( (foundPropEnd == MagickFalse) && (EOFBlob(image) == MagickFalse) ) {
   PropType    prop_type = (PropType) ReadBlobMSBLong(image);
-  unsigned long  prop_size = ReadBlobMSBLong(image);
+  size_t  prop_size = ReadBlobMSBLong(image);
     switch (prop_type)
     {
     case PROP_END:
@@ -814,8 +830,8 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
       outLayer->show_mask = ReadBlobMSBLong(image);
       break;
     case PROP_OFFSETS:
-      outLayer->offset_x = (long) ReadBlobMSBLong(image);
-      outLayer->offset_y = (long) ReadBlobMSBLong(image);
+      outLayer->offset_x = (int) ReadBlobMSBLong(image);
+      outLayer->offset_y = (int) ReadBlobMSBLong(image);
       break;
     case PROP_MODE:
       outLayer->mode = ReadBlobMSBLong(image);
@@ -825,11 +841,12 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
       break;
      case PROP_PARASITES:
      {
-        for (i=0; i < (long) prop_size; i++ )
-          (void) ReadBlobByte(image);
+       if (DiscardBlobBytes(image,prop_size) == MagickFalse)
+         ThrowFileException(&image->exception,CorruptImageError,
+           "UnexpectedEndOfFile",image->filename);
 
         /*
-       long base = info->cp;
+       ssize_t base = info->cp;
        GimpParasite *p;
        while (info->cp - base < prop_size)
        {
@@ -858,7 +875,7 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
         if (!amount)
           ThrowBinaryException(CorruptImageError,"CorruptImage",
             image->filename);
-        prop_size -= (unsigned long) MagickMin(16, (size_t) amount);
+        prop_size -= (size_t) MagickMin(16, (size_t) amount);
         }
       }
       break;
@@ -972,18 +989,16 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickOffsetType
     offset;
 
-  register long
+  register ssize_t
     i;
 
   size_t
+    image_type,
     length;
 
   ssize_t
     count;
 
-  unsigned long
-    image_type;
-
   XCFDocInfo
     doc_info;
 
@@ -1032,11 +1047,6 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     else
       if (image_type == GIMP_INDEXED)
         ThrowReaderException(CoderError,"ColormapTypeNotSupported");
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   (void) SetImageBackgroundColor(image);
   image->matte=MagickTrue;
   /*
@@ -1045,7 +1055,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   while ((foundPropEnd == MagickFalse) && (EOFBlob(image) == MagickFalse))
   {
     PropType prop_type = (PropType) ReadBlobMSBLong(image);
-    unsigned long prop_size = ReadBlobMSBLong(image);
+    size_t prop_size = ReadBlobMSBLong(image);
 
     switch (prop_type)
     {
@@ -1057,9 +1067,10 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /* Cannot rely on prop_size here--the value is set incorrectly
            by some Gimp versions.
         */
-        unsigned long num_colours = ReadBlobMSBLong(image);
-        for (i=0; i < (long) (3L*num_colours); i++ )
-          (void) ReadBlobByte(image);
+        size_t num_colours = ReadBlobMSBLong(image);
+        if (DiscardBlobBytes(image,3*num_colours) == MagickFalse)
+          ThrowFileException(&image->exception,CorruptImageError,
+            "UnexpectedEndOfFile",image->filename);
     /*
       if (info->file_version == 0)
       {
@@ -1105,10 +1116,9 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       case PROP_GUIDES:
       {
          /* just skip it - we don't care about guides */
-        for (i=0; i < (long) prop_size; i++ )
-          if (ReadBlobByte(image) == EOF)
-            ThrowFileException(exception,CorruptImageError,
-              "UnexpectedEndOfFile",image->filename);
+        if (DiscardBlobBytes(image,prop_size) == MagickFalse)
+          ThrowFileException(&image->exception,CorruptImageError,
+            "UnexpectedEndOfFile",image->filename);
       }
       break;
 
@@ -1139,20 +1149,18 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     case PROP_TATTOO:
       {
         /* we need to read it, even if we ignore it */
-        /*unsigned long  tattoo_state = */ (void) ReadBlobMSBLong(image);
+        /*size_t  tattoo_state = */ (void) ReadBlobMSBLong(image);
       }
       break;
 
     case PROP_PARASITES:
       {
         /* BOGUS: we may need these for IPTC stuff */
-        for (i=0; i < (long) prop_size; i++ )
-          if (ReadBlobByte(image) == EOF)
-            ThrowFileException(exception,CorruptImageError,
-              "UnexpectedEndOfFile",image->filename);
-
+        if (DiscardBlobBytes(image,prop_size) == MagickFalse)
+          ThrowFileException(&image->exception,CorruptImageError,
+            "UnexpectedEndOfFile",image->filename);
         /*
-      glong         base = info->cp;
+      gssize_t         base = info->cp;
       GimpParasite *p;
 
       while (info->cp - base < prop_size)
@@ -1170,17 +1178,16 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     case PROP_UNIT:
       {
         /* BOGUS: ignore for now... */
-      /*unsigned long unit =  */ (void) ReadBlobMSBLong(image);
+      /*size_t unit =  */ (void) ReadBlobMSBLong(image);
       }
       break;
 
     case PROP_PATHS:
       {
       /* BOGUS: just skip it for now */
-        for (i=0; i< (long) prop_size; i++ )
-          if (ReadBlobByte(image) == EOF)
-            ThrowFileException(exception,CorruptImageError,
-              "UnexpectedEndOfFile",image->filename);
+        if (DiscardBlobBytes(image,prop_size) == MagickFalse)
+          ThrowFileException(&image->exception,CorruptImageError,
+            "UnexpectedEndOfFile",image->filename);
 
         /*
       PathList *paths = xcf_load_bzpaths (gimage, info);
@@ -1194,7 +1201,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         char  unit_string[1000];
         /*BOGUS: ignored for now */
         /*float  factor = (float) */ (void) ReadBlobMSBLong(image);
-        /* unsigned long digits =  */ (void) ReadBlobMSBLong(image);
+        /* size_t digits =  */ (void) ReadBlobMSBLong(image);
         for (i=0; i<5; i++)
          (void) ReadBlobStringWithLongSize(image, unit_string,
            sizeof(unit_string));
@@ -1204,16 +1211,16 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       default:
       {
         int buf[16];
-        long amount;
+        ssize_t amount;
 
       /* read over it... */
       while ((prop_size > 0) && (EOFBlob(image) == MagickFalse))
       {
-        amount=(long) MagickMin(16, prop_size);
-        amount=(long) ReadBlob(image,(size_t) amount,(unsigned char *) &buf);
+        amount=(ssize_t) MagickMin(16, prop_size);
+        amount=(ssize_t) ReadBlob(image,(size_t) amount,(unsigned char *) &buf);
         if (!amount)
           ThrowReaderException(CorruptImageError,"CorruptImage");
-        prop_size -= (unsigned long) MagickMin(16,(size_t) amount);
+        prop_size -= (size_t) MagickMin(16,(size_t) amount);
       }
     }
     break;
@@ -1244,7 +1251,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       do
       {
-        long offset = (long) ReadBlobMSBLong(image);
+        ssize_t offset = (int) ReadBlobMSBLong(image);
         if (offset == 0)
           foundAllLayers=MagickTrue;
         else
@@ -1442,10 +1449,10 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterXCFImage method is:
 %
-%      unsigned long RegisterXCFImage(void)
+%      size_t RegisterXCFImage(void)
 %
 */
-ModuleExport unsigned long RegisterXCFImage(void)
+ModuleExport size_t RegisterXCFImage(void)
 {
   MagickInfo
     *entry;
