@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,7 +42,10 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
+#include "magick/colormap-private.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -95,12 +98,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 {
@@ -147,7 +148,7 @@ static MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define CheckOverflowException(length,width,height) \
-   (((height) != 0) && ((length)/((size_t) height) != ((size_t) width)))
+  (((height) != 0) && ((length)/((size_t) height) != ((size_t) width)))
 
   char
     *comment;
@@ -161,8 +162,8 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   int
     x_status;
 
-  long
-    y;
+  MagickBooleanType
+    authentic_colormap;
 
   MagickStatusType
     status;
@@ -170,26 +171,25 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
-  register long
+  register ssize_t
     i;
 
-  register unsigned long
+  register size_t
     pixel;
 
   size_t
-    length;
+    length,
+    lsb_first;
 
   ssize_t
-    count;
-
-  unsigned long
-    lsb_first;
+    count,
+    y;
 
   XColor
     *colors;
@@ -247,7 +247,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   comment[length]='\0';
   (void) SetImageProperty(image,"comment",comment);
   comment=DestroyString(comment);
-  if (count == 0)
+  if (count != (ssize_t) length)
     ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
   /*
     Initialize the X image.
@@ -281,6 +281,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read colormap.
   */
+  authentic_colormap=MagickFalse;
   colors=(XColor *) NULL;
   if (header.ncolors != 0)
     {
@@ -291,7 +292,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       colors=(XColor *) AcquireQuantumMemory(length,sizeof(*colors));
       if (colors == (XColor *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      for (i=0; i < (long) header.ncolors; i++)
+      for (i=0; i < (ssize_t) header.ncolors; i++)
       {
         count=ReadBlob(image,sz_XWDColor,(unsigned char *) &color);
         if (count == 0)
@@ -301,13 +302,15 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         colors[i].green=color.green;
         colors[i].blue=color.blue;
         colors[i].flags=(char) color.flags;
+        if (color.flags != 0)
+         authentic_colormap=MagickTrue;
       }
       /*
         Ensure the header byte-order is most-significant byte first.
       */
       lsb_first=1;
       if ((int) (*(char *) &lsb_first) != 0)
-        for (i=0; i < (long) header.ncolors; i++)
+        for (i=0; i < (ssize_t) header.ncolors; i++)
         {
           MSBOrderLong((unsigned char *) &colors[i].pixel,
             sizeof(colors[i].pixel));
@@ -340,18 +343,13 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Convert image to MIFF format.
   */
-  image->columns=(unsigned long) ximage->width;
-  image->rows=(unsigned long) ximage->height;
+  image->columns=(size_t) ximage->width;
+  image->rows=(size_t) ximage->height;
   if ((colors == (XColor *) NULL) || (ximage->red_mask != 0) ||
       (ximage->green_mask != 0) || (ximage->blue_mask != 0))
     image->storage_class=DirectClass;
   else
     image->storage_class=PseudoClass;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   image->colors=header.ncolors;
   if (image_info->ping == MagickFalse)
     switch (image->storage_class)
@@ -359,10 +357,10 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       case DirectClass:
       default:
       {
-        register unsigned long
+        register size_t
           color;
 
-        unsigned long
+        size_t
           blue_mask,
           blue_shift,
           green_mask,
@@ -397,52 +395,60 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Convert X image to DirectClass packets.
         */
-        if (image->colors != 0)
-          for (y=0; y < (long) image->rows; y++)
+        if ((image->colors != 0) && (authentic_colormap != MagickFalse))
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
-            for (x=0; x < (long) image->columns; x++)
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
               index=(IndexPacket) ((pixel >> red_shift) & red_mask);
-              q->red=ScaleShortToQuantum(colors[(long) index].red);
+              SetRedPixelComponent(q,ScaleShortToQuantum(colors[(ssize_t)
+                index].red));
               index=(IndexPacket) ((pixel >> green_shift) & green_mask);
-              q->green=ScaleShortToQuantum(colors[(long) index].green);
+              SetGreenPixelComponent(q,ScaleShortToQuantum(colors[(ssize_t)
+                index].green));
               index=(IndexPacket) ((pixel >> blue_shift) & blue_mask);
-              q->blue=ScaleShortToQuantum(colors[(long) index].blue);
+              SetBluePixelComponent(q,ScaleShortToQuantum(colors[(ssize_t)
+                index].blue));
               q++;
             }
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
-            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+              image->rows);
             if (status == MagickFalse)
               break;
           }
         else
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
-            for (x=0; x < (long) image->columns; x++)
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
               color=(pixel >> red_shift) & red_mask;
               color=(color*65535UL)/red_mask;
-              q->red=ScaleShortToQuantum((unsigned short) color);
+              SetRedPixelComponent(q,ScaleShortToQuantum((unsigned short)
+                color));
               color=(pixel >> green_shift) & green_mask;
               color=(color*65535UL)/green_mask;
-              q->green=ScaleShortToQuantum((unsigned short) color);
+              SetGreenPixelComponent(q,ScaleShortToQuantum((unsigned short)
+                color));
               color=(pixel >> blue_shift) & blue_mask;
               color=(color*65535UL)/blue_mask;
-              q->blue=ScaleShortToQuantum((unsigned short) color);
+              SetBluePixelComponent(q,ScaleShortToQuantum((unsigned short)
+                color));
               q++;
             }
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
-            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
             if (status == MagickFalse)
               break;
           }
@@ -455,28 +461,32 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (AcquireImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        for (i=0; i < (long) image->colors; i++)
+        for (i=0; i < (ssize_t) image->colors; i++)
         {
           image->colormap[i].red=ScaleShortToQuantum(colors[i].red);
           image->colormap[i].green=ScaleShortToQuantum(colors[i].green);
           image->colormap[i].blue=ScaleShortToQuantum(colors[i].blue);
         }
-        for (y=0; y < (long) image->rows; y++)
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
+          indexes=GetAuthenticIndexQueue(image);
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             index=ConstrainColormapIndex(image,XGetPixel(ximage,(int) x,
               (int) y));
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            SetIndexPixelComponent(indexes+x,index);
+            SetRedPixelComponent(q,image->colormap[(ssize_t) index].red);
+            SetGreenPixelComponent(q,image->colormap[(ssize_t) index].green);
+            SetBluePixelComponent(q,image->colormap[(ssize_t) index].blue);
+            q++;
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
@@ -518,10 +528,10 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterXWDImage method is:
 %
-%      unsigned long RegisterXWDImage(void)
+%      size_t RegisterXWDImage(void)
 %
 */
-ModuleExport unsigned long RegisterXWDImage(void)
+ModuleExport size_t RegisterXWDImage(void)
 {
   MagickInfo
     *entry;
@@ -594,39 +604,37 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
   const char
     *value;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
-  register long
+  register ssize_t
     x;
 
-  register long
+  register ssize_t
     i;
 
   register unsigned char
     *q;
 
   size_t
-    length;
+    bits_per_pixel,
+    bytes_per_line,
+    length,
+    lsb_first,
+    scanline_pad;
+
+  ssize_t
+    y;
 
   unsigned char
     *pixels;
 
-  unsigned long
-    bits_per_pixel,
-    bytes_per_line,
-    lsb_first,
-    scanline_pad;
-
   XWDFileHeader
     xwd_info;
 
@@ -643,10 +651,11 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Initialize XWD file header.
   */
+  (void) ResetMagickMemory(&xwd_info,0,sizeof(xwd_info));
   xwd_info.header_size=(CARD32) sz_XWDheader;
   value=GetImageProperty(image,"comment");
   if (value != (const char *) NULL)
@@ -662,7 +671,7 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
   xwd_info.bitmap_unit=(CARD32) (image->storage_class == DirectClass ? 32 : 8);
   xwd_info.bitmap_bit_order=(CARD32) MSBFirst;
   xwd_info.bitmap_pad=(CARD32) (image->storage_class == DirectClass ? 32 : 8);
-  bits_per_pixel=(unsigned long) (image->storage_class == DirectClass ? 24 : 8);
+  bits_per_pixel=(size_t) (image->storage_class == DirectClass ? 24 : 8);
   xwd_info.bits_per_pixel=(CARD32) bits_per_pixel;
   bytes_per_line=(CARD32) ((((xwd_info.bits_per_pixel*
     xwd_info.pixmap_width)+((xwd_info.bitmap_pad)-1))/
@@ -706,13 +715,14 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
       /*
         Dump colormap to file.
       */
+      (void) ResetMagickMemory(&color,0,sizeof(color));
       colors=(XColor *) AcquireQuantumMemory((size_t) image->colors,
         sizeof(*colors));
       if (colors == (XColor *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (ssize_t) image->colors; i++)
       {
-        colors[i].pixel=(unsigned long) i;
+        colors[i].pixel=(size_t) i;
         colors[i].red=ScaleQuantumToShort(image->colormap[i].red);
         colors[i].green=ScaleQuantumToShort(image->colormap[i].green);
         colors[i].blue=ScaleQuantumToShort(image->colormap[i].blue);
@@ -726,7 +736,7 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
               3*sizeof(colors[i].red));
           }
       }
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (ssize_t) image->colors; i++)
       {
         color.pixel=(CARD32) colors[i].pixel;
         color.red=colors[i].red;
@@ -751,30 +761,31 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
     Convert MIFF to XWD raster pixels.
   */
   scanline_pad=(bytes_per_line-((image->columns*bits_per_pixel) >> 3));
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
     q=pixels;
     if (image->storage_class == PseudoClass)
       {
-        indexes=GetIndexes(image);
-        for (x=0; x < (long) image->columns; x++)
-          *q++=(unsigned char) indexes[x];
+        indexes=GetVirtualIndexQueue(image);
+        for (x=0; x < (ssize_t) image->columns; x++)
+          *q++=(unsigned char) GetIndexPixelComponent(indexes+x);
       }
     else
-      for (x=0; x < (long) image->columns; x++)
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
-        *q++=ScaleQuantumToChar(p->red);
-        *q++=ScaleQuantumToChar(p->green);
-        *q++=ScaleQuantumToChar(p->blue);
+        *q++=ScaleQuantumToChar(GetRedPixelComponent(p));
+        *q++=ScaleQuantumToChar(GetGreenPixelComponent(p));
+        *q++=ScaleQuantumToChar(GetBluePixelComponent(p));
         p++;
       }
-    for (x=0; x < (long) scanline_pad; x++)
+    for (x=0; x < (ssize_t) scanline_pad; x++)
       *q++='\0';
     (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
     if (status == MagickFalse)
       break;
   }
