@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
@@ -104,31 +105,27 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   InterlaceType
     interlace;
 
-  long
-    horizontal_factor,
-    vertical_factor,
-    y;
-
   MagickBooleanType
     status;
 
   register const PixelPacket
     *chroma_pixels;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
-  register long
-    i;
-
   register unsigned char
     *p;
 
   ssize_t
-    count;
+    count,
+    horizontal_factor,
+    quantum,
+    vertical_factor,
+    y;
 
   unsigned char
     *scanline;
@@ -146,7 +143,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
-  image->depth=8;
+  quantum=image->depth <= 8 ? 1 : 2;
   interlace=image_info->interlace;
   horizontal_factor=2;
   vertical_factor=2;
@@ -159,8 +156,8 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
-      horizontal_factor=(long) geometry_info.rho;
-      vertical_factor=(long) geometry_info.sigma;
+      horizontal_factor=(ssize_t) geometry_info.rho;
+      vertical_factor=(ssize_t) geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
@@ -185,23 +182,19 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image=DestroyImageList(image);
           return((Image *) NULL);
         }
-      for (i=0; i < image->offset; i++)
-        if (ReadBlobByte(image) == EOF)
-          {
-            ThrowFileException(exception,CorruptImageError,
-              "UnexpectedEndOfFile",image->filename);
-            break;
-          }
+      if (DiscardBlobBytes(image,image->offset) == MagickFalse)
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
     }
   /*
     Allocate memory for a scanline.
   */
   if (interlace == NoInterlace)
     scanline=(unsigned char *) AcquireQuantumMemory((size_t) 2UL*
-      image->columns+2UL,sizeof(*scanline));
+      image->columns+2UL,quantum*sizeof(*scanline));
   else
     scanline=(unsigned char *) AcquireQuantumMemory((size_t) image->columns,
-      sizeof(*scanline));
+      quantum*sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   do
@@ -216,11 +209,6 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
     if (interlace == PartitionInterlace)
       {
         AppendImageFormat("Y",image->filename);
@@ -231,7 +219,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             return((Image *) NULL);
           }
       }
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
       register PixelPacket
         *chroma_pixels;
@@ -239,27 +227,55 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (interlace == NoInterlace)
         {
           if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
-            count=ReadBlob(image,(size_t) (2*image->columns),scanline);
+            count=ReadBlob(image,(size_t) (2*quantum*image->columns),scanline);
           p=scanline;
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          chroma_pixels=SetImagePixels(chroma_image,0,y,
-            chroma_image->columns,1);
+          chroma_pixels=QueueAuthenticPixels(chroma_image,0,y,
+            chroma_image->columns,1,exception);
           if (chroma_pixels == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x+=2)
+          for (x=0; x < (ssize_t) image->columns; x+=2)
           {
-            chroma_pixels->red=(Quantum) 0;
-            chroma_pixels->green=ScaleCharToQuantum(*p++);
-            q->red=ScaleCharToQuantum(*p++);
-            q->green=(Quantum) 0;
-            q->blue=(Quantum) 0;
+            SetRedPixelComponent(chroma_pixels,0);
+            if (quantum == 1)
+              SetGreenPixelComponent(chroma_pixels,ScaleCharToQuantum(*p++));
+            else
+              {
+                SetGreenPixelComponent(chroma_pixels,ScaleShortToQuantum(
+                  ((*p) << 8) | *(p+1)));
+                p+=2;
+              }
+            if (quantum == 1)
+              SetRedPixelComponent(q,ScaleCharToQuantum(*p++));
+            else
+              {
+                SetRedPixelComponent(q,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)));
+                p+=2;
+              }
+            SetGreenPixelComponent(q,0);
+            SetBluePixelComponent(q,0);
             q++;
-            q->green=0;
-            q->blue=0;
-            chroma_pixels->blue=ScaleCharToQuantum(*p++);
-            q->red=ScaleCharToQuantum(*p++);
+            SetGreenPixelComponent(q,0);
+            SetBluePixelComponent(q,0);
+            if (quantum == 1)
+              SetBluePixelComponent(chroma_pixels,ScaleCharToQuantum(*p++));
+            else
+              {
+                SetBluePixelComponent(chroma_pixels,ScaleShortToQuantum(
+                  ((*p) << 8) | *(p+1)));
+                p+=2;
+              }
+            if (quantum == 1)
+              SetRedPixelComponent(q,ScaleCharToQuantum(*p++));
+            else
+              {
+                SetRedPixelComponent(q,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)));
+                p+=2;
+              }
             chroma_pixels++;
             q++;
           }
@@ -267,27 +283,35 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       else
         {
           if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
-            count=ReadBlob(image,(size_t) image->columns,scanline);
+            count=ReadBlob(image,(size_t) quantum*image->columns,scanline);
           p=scanline;
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(*p++);
-            q->green=0;
-            q->blue=0;
+            if (quantum == 1)
+              SetRedPixelComponent(q,ScaleCharToQuantum(*p++));
+            else
+              {
+                SetRedPixelComponent(q,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)));
+                p+=2;
+              }
+            SetGreenPixelComponent(q,0);
+            SetBluePixelComponent(q,0);
             q++;
           }
         }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
       if (interlace == NoInterlace)
-        if (SyncImagePixels(chroma_image) == MagickFalse)
+        if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
           break;
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
@@ -305,21 +329,29 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     if (interlace != NoInterlace)
       {
-        for (y=0; y < (long) chroma_image->rows; y++)
+        for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
-          count=ReadBlob(image,(size_t) chroma_image->columns,scanline);
+          count=ReadBlob(image,(size_t) quantum*chroma_image->columns,scanline);
           p=scanline;
-          q=SetImagePixels(chroma_image,0,y,chroma_image->columns,1);
+          q=QueueAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,
+            exception);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) chroma_image->columns; x++)
+          for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
-            q->red=(Quantum) 0;
-            q->green=ScaleCharToQuantum(*p++);
-            q->blue=(Quantum) 0;
+            SetRedPixelComponent(q,0);
+            if (quantum == 1)
+              SetGreenPixelComponent(q,ScaleCharToQuantum(*p++));
+            else
+              {
+                SetGreenPixelComponent(q,ScaleShortToQuantum(((*p) << 8) |
+                  *(p+1)));
+                p+=2;
+              }
+            SetBluePixelComponent(q,0);
             q++;
           }
-          if (SyncImagePixels(chroma_image) == MagickFalse)
+          if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
             break;
         }
       if (interlace == PartitionInterlace)
@@ -333,19 +365,27 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
               return((Image *) NULL);
             }
         }
-      for (y=0; y < (long) chroma_image->rows; y++)
+      for (y=0; y < (ssize_t) chroma_image->rows; y++)
       {
-        count=ReadBlob(image,(size_t) chroma_image->columns,scanline);
+        count=ReadBlob(image,(size_t) quantum*chroma_image->columns,scanline);
         p=scanline;
-        q=GetImagePixels(chroma_image,0,y,chroma_image->columns,1);
+        q=GetAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,
+          exception);
         if (q == (PixelPacket *) NULL)
           break;
-        for (x=0; x < (long) chroma_image->columns; x++)
+        for (x=0; x < (ssize_t) chroma_image->columns; x++)
         {
-          q->blue=ScaleCharToQuantum(*p++);
+          if (quantum == 1)
+            SetBluePixelComponent(q,ScaleCharToQuantum(*p++));
+          else
+            {
+              SetBluePixelComponent(q,ScaleShortToQuantum(((*p) << 8) |
+                *(p+1)));
+              p+=2;
+            }
           q++;
         }
-        if (SyncImagePixels(chroma_image) == MagickFalse)
+        if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
           break;
       }
     }
@@ -357,22 +397,22 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     chroma_image=DestroyImage(chroma_image);
     if (resize_image == (Image *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      q=GetImagePixels(image,0,y,image->columns,1);
-      chroma_pixels=AcquireImagePixels(resize_image,0,y,resize_image->columns,1,
+      q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+      chroma_pixels=GetVirtualPixels(resize_image,0,y,resize_image->columns,1,
         &resize_image->exception);
       if ((q == (PixelPacket *) NULL) ||
           (chroma_pixels == (const PixelPacket *) NULL))
         break;
-      for (x=0; x < (long) image->columns; x++)
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
-        q->green=chroma_pixels->green;
-        q->blue=chroma_pixels->blue;
+        SetGreenPixelComponent(q,GetGreenPixelComponent(chroma_pixels));
+        SetBluePixelComponent(q,GetBluePixelComponent(chroma_pixels));
         chroma_pixels++;
         q++;
       }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
     }
     resize_image=DestroyImage(resize_image);
@@ -393,9 +433,9 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     if (interlace == NoInterlace)
-      count=ReadBlob(image,(size_t) (2*image->columns),scanline);
+      count=ReadBlob(image,(size_t) (2*quantum*image->columns),scanline);
     else
-      count=ReadBlob(image,(size_t) image->columns,scanline);
+      count=ReadBlob(image,(size_t) quantum*image->columns,scanline);
     if (count != 0)
       {
         /*
@@ -439,10 +479,10 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterYUVImage method is:
 %
-%      unsigned long RegisterYUVImage(void)
+%      size_t RegisterYUVImage(void)
 %
 */
-ModuleExport unsigned long RegisterYUVImage(void)
+ModuleExport size_t RegisterYUVImage(void)
 {
   MagickInfo
     *entry;
@@ -507,7 +547,6 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
 {
@@ -518,11 +557,6 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
   InterlaceType
     interlace;
 
-  long
-    horizontal_factor,
-    vertical_factor,
-    y;
-
   MagickBooleanType
     status;
 
@@ -533,19 +567,26 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
     *p,
     *s;
 
-  register long
+  register ssize_t
     x;
 
-  unsigned long
+  size_t
     height,
+    quantum,
     width;
 
+  ssize_t
+    horizontal_factor,
+    vertical_factor,
+    y;
+
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  quantum=image->depth <= 8 ? 1 : 2;
   interlace=image->interlace;
   horizontal_factor=2;
   vertical_factor=2;
@@ -558,8 +599,8 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
-      horizontal_factor=(long) geometry_info.rho;
-      vertical_factor=(long) geometry_info.sigma;
+      horizontal_factor=(ssize_t) geometry_info.rho;
+      vertical_factor=(ssize_t) geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
@@ -595,14 +636,14 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
     /*
       Sample image to an even width and height, if necessary.
     */
-    image->depth=8;
+    image->depth=quantum == 1 ? 8 : 16;
     width=image->columns+(image->columns & (horizontal_factor-1));
     height=image->rows+(image->rows & (vertical_factor-1));
     yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
       &image->exception);
     if (yuv_image == (Image *) NULL)
       ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) SetImageColorspace(yuv_image,YCbCrColorspace);
+    (void) TransformImageColorspace(yuv_image,YCbCrColorspace);
     /*
       Downsample image.
     */
@@ -610,36 +651,56 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       height/vertical_factor,TriangleFilter,1.0,&image->exception);
     if (chroma_image == (Image *) NULL)
       ThrowWriterException(ResourceLimitError,image->exception.reason);
-    (void) SetImageColorspace(chroma_image,YCbCrColorspace);
+    (void) TransformImageColorspace(chroma_image,YCbCrColorspace);
     if (interlace == NoInterlace)
       {
         /*
           Write noninterlaced YUV.
         */
-        for (y=0; y < (long) yuv_image->rows; y++)
+        for (y=0; y < (ssize_t) yuv_image->rows; y++)
         {
-          p=AcquireImagePixels(yuv_image,0,y,yuv_image->columns,1,
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
             &yuv_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          s=AcquireImagePixels(chroma_image,0,y,chroma_image->columns,1,
+          s=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
             &chroma_image->exception);
           if (s == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) yuv_image->columns; x++)
+          for (x=0; x < (ssize_t) yuv_image->columns; x++)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(s->green));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
-            p++;
-            (void) WriteBlobByte(image,ScaleQuantumToChar(s->blue));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
+            if (quantum == 1)
+              {
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetGreenPixelComponent(s)));
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetRedPixelComponent(p)));
+                p++;
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetBluePixelComponent(s)));
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetRedPixelComponent(p)));
+              }
+            else
+              {
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetGreenPixelComponent(s)));
+                (void) WriteBlobShort(image,ScaleQuantumToShort(
+                  GetRedPixelComponent(p)));
+                p++;
+                (void) WriteBlobByte(image,ScaleQuantumToChar(
+                  GetBluePixelComponent(s)));
+                (void) WriteBlobShort(image,ScaleQuantumToShort(
+                  GetRedPixelComponent(p)));
+              }
             p++;
             s++;
             x++;
           }
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -651,20 +712,26 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         /*
           Initialize Y channel.
         */
-        for (y=0; y < (long) yuv_image->rows; y++)
+        for (y=0; y < (ssize_t) yuv_image->rows; y++)
         {
-          p=AcquireImagePixels(yuv_image,0,y,yuv_image->columns,1,
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
             &yuv_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) yuv_image->columns; x++)
+          for (x=0; x < (ssize_t) yuv_image->columns; x++)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
+            if (quantum == 1)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetRedPixelComponent(p)));
+            else
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetRedPixelComponent(p)));
             p++;
           }
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -688,15 +755,20 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             if (status == MagickFalse)
               return(status);
           }
-        for (y=0; y < (long) chroma_image->rows; y++)
+        for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
-          p=AcquireImagePixels(chroma_image,0,y,chroma_image->columns,1,
+          p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
             &chroma_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) chroma_image->columns; x++)
+          for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->green));
+            if (quantum == 1)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetGreenPixelComponent(p)));
+            else
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetGreenPixelComponent(p)));
             p++;
           }
         }
@@ -718,15 +790,20 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             if (status == MagickFalse)
               return(status);
           }
-        for (y=0; y < (long) chroma_image->rows; y++)
+        for (y=0; y < (ssize_t) chroma_image->rows; y++)
         {
-          p=AcquireImagePixels(chroma_image,0,y,chroma_image->columns,1,
+          p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
             &chroma_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) chroma_image->columns; x++)
+          for (x=0; x < (ssize_t) chroma_image->columns; x++)
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(p->blue));
+            if (quantum == 1)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetBluePixelComponent(p)));
+            else
+              (void) WriteBlobShort(image,ScaleQuantumToShort(
+                GetBluePixelComponent(p)));
             p++;
           }
         }
