@@ -16,7 +16,7 @@
 %                               March  2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,6 +39,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/cache.h"
 #include "magick/cipher.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -77,11 +78,11 @@ typedef struct _AESInfo
     *encipher_key,
     *decipher_key;
 
-  long
+  ssize_t
     rounds,
     timestamp;
 
-  unsigned long
+  size_t
     signature;
 } AESInfo;
 
@@ -203,7 +204,7 @@ static AESInfo *AcquireAESInfo(void)
       (aes_info->encipher_key == (unsigned int *) NULL) ||
       (aes_info->decipher_key == (unsigned int *) NULL))
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  aes_info->timestamp=(long) time(0);
+  aes_info->timestamp=(ssize_t) time(0);
   aes_info->signature=MagickSignature;
   return(aes_info);
 }
@@ -213,181 +214,6 @@ static AESInfo *AcquireAESInfo(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%     D e c i p h e r I m a g e                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DecipherImage() converts cipher pixels to plain pixels.
-%
-%  The format of the DecipherImage method is:
-%
-%      MagickBooleanType DecipherImage(Image *image,const char *passphrase,
-%        ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image.
-%
-%    o passphrase: decipher cipher pixels with this passphrase.
-%
-%    o exception: return any errors or warnings in this structure.
-%
-*/
-
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
-MagickExport MagickBooleanType DecipherImage(Image *image,
-  const char *passphrase,ExceptionInfo *exception)
-{
-#define DecipherImageTag  "Decipher/Image "
-
-  AESInfo
-    *aes_info;
-
-  const unsigned char
-    *digest;
-
-  IndexPacket
-    *indexes;
-
-  long
-    y;
-
-  MagickBooleanType
-    proceed;
-
-  QuantumInfo
-    *quantum_info;
-
-  QuantumType
-    quantum_type;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  register unsigned char
-    *p;
-
-  SignatureInfo
-    *signature_info;
-
-  size_t
-    length;
-
-  StringInfo
-    *key,
-    *nonce;
-
-  unsigned char
-    input_block[AESBlocksize],
-    output_block[AESBlocksize],
-    *pixels;
-
-  /*
-    Generate decipher key and nonce.
-  */
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  if (passphrase == (const char *) NULL)
-    return(MagickTrue);
-  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
-  if (quantum_info == (QuantumInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  aes_info=AcquireAESInfo();
-  key=StringToStringInfo(passphrase);
-  if (key == (StringInfo *) NULL)
-    {
-      aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
-      return(MagickFalse);
-    }
-  nonce=SplitStringInfo(key,GetStringInfoLength(key)/2);
-  if (nonce == (StringInfo *) NULL)
-    {
-      key=DestroyStringInfo(key);
-      aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
-      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-        image->filename);
-    }
-  SetAESKey(aes_info,key);
-  key=DestroyStringInfo(key);
-  signature_info=AcquireSignatureInfo();
-  UpdateSignature(signature_info,nonce);
-  SetStringInfoLength(nonce,sizeof(quantum_info->extent));
-  SetStringInfoDatum(nonce,(const unsigned char *) &quantum_info->extent);
-  UpdateSignature(signature_info,nonce);
-  FinalizeSignature(signature_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
-  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
-    GetSignatureDigestsize(signature_info))*sizeof(*input_block));
-  nonce=DestroyStringInfo(nonce);
-  signature_info=DestroySignatureInfo(signature_info);
-  /*
-    Convert cipher pixels to plain pixels.
-  */
-  quantum_type=GetQuantumType(image,exception);
-  pixels=GetQuantumPixels(quantum_info);
-  for (y=0; y < (long) image->rows; y++)
-  {
-    q=GetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
-    length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
-    p=pixels;
-    for (x=0; x < (long) length; x++)
-    {
-      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
-        sizeof(*output_block));
-      EncipherAESBlock(aes_info,output_block,output_block); 
-      (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
-        sizeof(*input_block));
-      input_block[AESBlocksize-1]=(*p);
-      *p++^=(*output_block);
-    }
-    (void) ImportQuantumPixels(image,quantum_info,quantum_type,pixels);
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    proceed=SetImageProgress(image,DecipherImageTag,y,image->rows);
-    if (proceed == MagickFalse)
-      break;
-  }
-  (void) DeleteImageProperty(image,"cipher:type");
-  (void) DeleteImageProperty(image,"cipher:mode");
-  (void) DeleteImageProperty(image,"cipher:nonce");
-  image->taint=MagickFalse;
-  /*
-    Free resources.
-  */
-  quantum_info=DestroyQuantumInfo(quantum_info);
-  aes_info=DestroyAESInfo(aes_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
-  return(y == (long) image->rows ? MagickTrue : MagickFalse);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   D e s t r o y A E S I n f o                                               %
 %                                                                             %
 %                                                                             %
@@ -428,180 +254,6 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%     E n c i p h e r I m a g e                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  EncipherImage() converts pixels to cipher-pixels.
-%
-%  The format of the EncipherImage method is:
-%
-%      MagickBooleanType EncipherImage(Image *image,const char *passphrase,
-%        ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image.
-%
-%    o passphrase: encipher pixels with this passphrase.
-%
-%    o exception: return any errors or warnings in this structure.
-%
-*/
-MagickExport MagickBooleanType EncipherImage(Image *image,
-  const char *passphrase,ExceptionInfo *exception)
-{
-#define EncipherImageTag  "Encipher/Image "
-
-  AESInfo
-    *aes_info;
-
-  char
-    *signature;
-
-  const unsigned char
-    *digest;
-
-  IndexPacket
-    *indexes;
-
-  long
-    y;
-
-  MagickBooleanType
-    proceed;
-
-  QuantumInfo
-    *quantum_info;
-
-  QuantumType
-    quantum_type;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  register unsigned char
-    *p;
-
-  SignatureInfo
-    *signature_info;
-
-  size_t
-    length;
-
-  StringInfo
-    *key,
-    *nonce;
-
-  unsigned char
-    input_block[AESBlocksize],
-    output_block[AESBlocksize],
-    *pixels;
-
-  /*
-    Generate encipher key and nonce.
-  */
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  if (passphrase == (const char *) NULL)
-    return(MagickTrue);
-  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
-    return(MagickFalse);
-  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
-  if (quantum_info == (QuantumInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  aes_info=AcquireAESInfo();
-  key=StringToStringInfo(passphrase);
-  if (key == (StringInfo *) NULL)
-    {
-      aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
-      return(MagickFalse);
-    }
-  nonce=SplitStringInfo(key,GetStringInfoLength(key)/2);
-  if (nonce == (StringInfo *) NULL)
-    {
-      key=DestroyStringInfo(key);
-      aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
-      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-        image->filename);
-    }
-  SetAESKey(aes_info,key);
-  key=DestroyStringInfo(key);
-  signature_info=AcquireSignatureInfo();
-  UpdateSignature(signature_info,nonce);
-  SetStringInfoLength(nonce,sizeof(quantum_info->extent));
-  SetStringInfoDatum(nonce,(const unsigned char *) &quantum_info->extent);
-  UpdateSignature(signature_info,nonce);
-  nonce=DestroyStringInfo(nonce);
-  FinalizeSignature(signature_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
-  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
-    GetSignatureDigestsize(signature_info))*sizeof(*input_block));
-  signature=StringInfoToHexString(GetSignatureDigest(signature_info));
-  (void) SetImageProperty(image,"cipher:type","AES");
-  (void) SetImageProperty(image,"cipher:mode","CFB");
-  (void) SetImageProperty(image,"cipher:nonce",signature);
-  signature=DestroyString(signature);
-  signature_info=DestroySignatureInfo(signature_info);
-  /*
-    Convert plain pixels to cipher pixels.
-  */
-  quantum_type=GetQuantumType(image,exception);
-  pixels=GetQuantumPixels(quantum_info);
-  for (y=0; y < (long) image->rows; y++)
-  {
-    q=GetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
-    length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
-    p=pixels;
-    for (x=0; x < (long) length; x++)
-    {
-      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
-        sizeof(*output_block));
-      EncipherAESBlock(aes_info,output_block,output_block); 
-      *p^=(*output_block);
-      (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
-        sizeof(*input_block));
-      input_block[AESBlocksize-1]=(*p++);
-    }
-    (void) ImportQuantumPixels(image,quantum_info,quantum_type,pixels);
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    proceed=SetImageProgress(image,EncipherImageTag,y,image->rows);
-    if (proceed == MagickFalse)
-      break;
-  }
-  image->taint=MagickFalse;
-  /*
-    Free resources.
-  */
-  quantum_info=DestroyQuantumInfo(quantum_info);
-  aes_info=DestroyAESInfo(aes_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
-  return(y == (long) image->rows ? MagickTrue : MagickFalse);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   E n c i p h e r A E S B l o c k                                           %
 %                                                                             %
 %                                                                             %
@@ -629,7 +281,7 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
 static inline void AddRoundKey(const unsigned int *ciphertext,
   const unsigned int *key,unsigned int *plaintext)
 {
-  register long
+  register ssize_t
     i;
 
   /*
@@ -728,7 +380,7 @@ static inline unsigned int RotateLeft(const unsigned int x)
 static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
   unsigned char *ciphertext)
 {
-  register long
+  register ssize_t
     i,
     j;
 
@@ -840,6 +492,412 @@ static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%     P a s s k e y D e c i p h e r I m a g e                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PasskeyDecipherImage() converts cipher pixels to plain pixels.
+%
+%  The format of the PasskeyDecipherImage method is:
+%
+%      MagickBooleanType PasskeyDecipherImage(Image *image,
+%        const StringInfo *passkey,ExceptionInfo *exception)
+%      MagickBooleanType DecipherImage(Image *image,const char *passphrase,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o passphrase: decipher cipher pixels with this passphrase.
+%
+%    o passkey: decrypt cipher pixels with this passkey.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
+MagickExport MagickBooleanType DecipherImage(Image *image,
+  const char *passphrase,ExceptionInfo *exception)
+{
+  MagickBooleanType
+    status;
+
+  StringInfo
+    *passkey;
+
+  if (passphrase == (const char *) NULL)
+    return(MagickTrue);
+  passkey=StringToStringInfo(passphrase);
+  if (passkey == (StringInfo *) NULL)
+    return(MagickFalse);
+  status=PasskeyDecipherImage(image,passkey,exception);
+  passkey=DestroyStringInfo(passkey);
+  return(status);
+}
+
+MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
+  const StringInfo *passkey,ExceptionInfo *exception)
+{
+#define DecipherImageTag  "Decipher/Image "
+
+  AESInfo
+    *aes_info;
+
+  CacheView
+    *image_view;
+
+  const unsigned char
+    *digest;
+
+  MagickBooleanType
+    proceed;
+
+  QuantumInfo
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
+
+  SignatureInfo
+    *signature_info;
+
+  register unsigned char
+    *p;
+
+  size_t
+    length;
+
+  ssize_t
+    y;
+
+  StringInfo
+    *key,
+    *nonce;
+
+  unsigned char
+    input_block[AESBlocksize],
+    output_block[AESBlocksize],
+    *pixels;
+
+  /*
+    Generate decipher key and nonce.
+  */
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  if (passkey == (const StringInfo *) NULL)
+    return(MagickTrue);
+  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  aes_info=AcquireAESInfo();
+  key=CloneStringInfo(passkey);
+  if (key == (StringInfo *) NULL)
+    {
+      aes_info=DestroyAESInfo(aes_info);
+      quantum_info=DestroyQuantumInfo(quantum_info);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
+  nonce=SplitStringInfo(key,GetStringInfoLength(key)/2);
+  if (nonce == (StringInfo *) NULL)
+    {
+      key=DestroyStringInfo(key);
+      aes_info=DestroyAESInfo(aes_info);
+      quantum_info=DestroyQuantumInfo(quantum_info);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
+  SetAESKey(aes_info,key);
+  key=DestroyStringInfo(key);
+  signature_info=AcquireSignatureInfo();
+  UpdateSignature(signature_info,nonce);
+  SetStringInfoLength(nonce,sizeof(quantum_info->extent));
+  SetStringInfoDatum(nonce,(const unsigned char *) &quantum_info->extent);
+  UpdateSignature(signature_info,nonce);
+  FinalizeSignature(signature_info);
+  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
+  digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
+  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
+    GetSignatureDigestsize(signature_info))*sizeof(*input_block));
+  nonce=DestroyStringInfo(nonce);
+  signature_info=DestroySignatureInfo(signature_info);
+  /*
+    Convert cipher pixels to plain pixels.
+  */
+  quantum_type=GetQuantumType(image,exception);
+  pixels=GetQuantumPixels(quantum_info);
+  image_view=AcquireCacheView(image);
+  for (y=0; y < (ssize_t) image->rows; y++)
+  {
+    register ssize_t
+      x;
+
+    register PixelPacket
+      *restrict q;
+
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
+    if (q == (PixelPacket *) NULL)
+      break;
+    length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
+    p=pixels;
+    for (x=0; x < (ssize_t) length; x++)
+    {
+      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
+        sizeof(*output_block));
+      EncipherAESBlock(aes_info,output_block,output_block);
+      (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
+        sizeof(*input_block));
+      input_block[AESBlocksize-1]=(*p);
+      *p++^=(*output_block);
+    }
+    (void) ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      break;
+    proceed=SetImageProgress(image,DecipherImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (proceed == MagickFalse)
+      break;
+  }
+  image_view=DestroyCacheView(image_view);
+  (void) DeleteImageProperty(image,"cipher:type");
+  (void) DeleteImageProperty(image,"cipher:mode");
+  (void) DeleteImageProperty(image,"cipher:nonce");
+  image->taint=MagickFalse;
+  /*
+    Free resources.
+  */
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  aes_info=DestroyAESInfo(aes_info);
+  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
+  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
+  return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%     P a s s k e y E n c i p h e r I m a g e                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PasskeyEncipherImage() converts pixels to cipher-pixels.
+%
+%  The format of the PasskeyEncipherImage method is:
+%
+%      MagickBooleanType PasskeyEncipherImage(Image *image,
+%        const StringInfo *passkey,ExceptionInfo *exception)
+%      MagickBooleanType EncipherImage(Image *image,const char *passphrase,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o passphrase: encipher pixels with this passphrase.
+%
+%    o passkey: decrypt cipher pixels with this passkey.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+
+MagickExport MagickBooleanType EncipherImage(Image *image,
+  const char *passphrase,ExceptionInfo *exception)
+{
+  MagickBooleanType
+    status;
+
+  StringInfo
+    *passkey;
+
+  if (passphrase == (const char *) NULL)
+    return(MagickTrue);
+  passkey=StringToStringInfo(passphrase);
+  if (passkey == (StringInfo *) NULL)
+    return(MagickFalse);
+  status=PasskeyEncipherImage(image,passkey,exception);
+  passkey=DestroyStringInfo(passkey);
+  return(status);
+}
+
+MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
+  const StringInfo *passkey,ExceptionInfo *exception)
+{
+#define EncipherImageTag  "Encipher/Image "
+
+  AESInfo
+    *aes_info;
+
+  CacheView
+    *image_view;
+
+  char
+    *signature;
+
+  const unsigned char
+    *digest;
+
+  MagickBooleanType
+    proceed;
+
+  QuantumInfo
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
+
+  register unsigned char
+    *p;
+
+  SignatureInfo
+    *signature_info;
+
+  size_t
+    length;
+
+  ssize_t
+    y;
+
+  StringInfo
+    *key,
+    *nonce;
+
+  unsigned char
+    input_block[AESBlocksize],
+    output_block[AESBlocksize],
+    *pixels;
+
+  /*
+    Generate encipher key and nonce.
+  */
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  if (passkey == (const StringInfo *) NULL)
+    return(MagickTrue);
+  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+    return(MagickFalse);
+  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  aes_info=AcquireAESInfo();
+  key=CloneStringInfo(passkey);
+  if (key == (StringInfo *) NULL)
+    {
+      aes_info=DestroyAESInfo(aes_info);
+      quantum_info=DestroyQuantumInfo(quantum_info);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
+  nonce=SplitStringInfo(key,GetStringInfoLength(key)/2);
+  if (nonce == (StringInfo *) NULL)
+    {
+      key=DestroyStringInfo(key);
+      aes_info=DestroyAESInfo(aes_info);
+      quantum_info=DestroyQuantumInfo(quantum_info);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
+  SetAESKey(aes_info,key);
+  key=DestroyStringInfo(key);
+  signature_info=AcquireSignatureInfo();
+  UpdateSignature(signature_info,nonce);
+  SetStringInfoLength(nonce,sizeof(quantum_info->extent));
+  SetStringInfoDatum(nonce,(const unsigned char *) &quantum_info->extent);
+  UpdateSignature(signature_info,nonce);
+  nonce=DestroyStringInfo(nonce);
+  FinalizeSignature(signature_info);
+  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
+  digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
+  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
+    GetSignatureDigestsize(signature_info))*sizeof(*input_block));
+  signature=StringInfoToHexString(GetSignatureDigest(signature_info));
+  (void) SetImageProperty(image,"cipher:type","AES");
+  (void) SetImageProperty(image,"cipher:mode","CFB");
+  (void) SetImageProperty(image,"cipher:nonce",signature);
+  signature=DestroyString(signature);
+  signature_info=DestroySignatureInfo(signature_info);
+  /*
+    Convert plain pixels to cipher pixels.
+  */
+  quantum_type=GetQuantumType(image,exception);
+  pixels=GetQuantumPixels(quantum_info);
+  image_view=AcquireCacheView(image);
+  for (y=0; y < (ssize_t) image->rows; y++)
+  {
+    register ssize_t
+      x;
+
+    register PixelPacket
+      *restrict q;
+
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
+    if (q == (PixelPacket *) NULL)
+      break;
+    length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
+    p=pixels;
+    for (x=0; x < (ssize_t) length; x++)
+    {
+      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
+        sizeof(*output_block));
+      EncipherAESBlock(aes_info,output_block,output_block);
+      *p^=(*output_block);
+      (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
+        sizeof(*input_block));
+      input_block[AESBlocksize-1]=(*p++);
+    }
+    (void) ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,exception);
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      break;
+    proceed=SetImageProgress(image,EncipherImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (proceed == MagickFalse)
+      break;
+  }
+  image_view=DestroyCacheView(image_view);
+  image->taint=MagickFalse;
+  /*
+    Free resources.
+  */
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  aes_info=DestroyAESInfo(aes_info);
+  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
+  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
+  return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S e t A E S K e y                                                         %
 %                                                                             %
 %                                                                             %
@@ -898,13 +956,13 @@ static inline unsigned int RotateRight(const unsigned int x)
 
 static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
 {
-  long
+  register ssize_t
+    i;
+
+  ssize_t
     bytes,
     n;
 
-  register long
-    i;
-
   unsigned char
     *datum;
 
@@ -982,16 +1040,18 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+     D e c i p h e r I m a g e                                               %
+%     P a s s k e y D e c i p h e r I m a g e                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DecipherImage() converts cipher pixels to plain pixels.
+%  PasskeyDecipherImage() converts cipher pixels to plain pixels.
 %
-%  The format of the DecipherImage method is:
+%  The format of the PasskeyDecipherImage method is:
 %
+%      MagickBooleanType PasskeyDecipherImage(Image *image,
+%        const StringInfo *passkey,ExceptionInfo *exception)
 %      MagickBooleanType DecipherImage(Image *image,const char *passphrase,
 %        ExceptionInfo *exception)
 %
@@ -1001,9 +1061,12 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
 %
 %    o passphrase: decipher cipher pixels with this passphrase.
 %
+%    o passkey: decrypt cipher pixels with this passkey.
+%
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
 MagickExport MagickBooleanType DecipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
 {
@@ -1016,34 +1079,50 @@ MagickExport MagickBooleanType DecipherImage(Image *image,
   (void) passphrase;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
+
+MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
+  const StringInfo *passkey,ExceptionInfo *exception)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  (void) passkey;
+  ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+     E n c i p h e r I m a g e                                               %
+%     P a s s k e y E n c i p h e r I m a g e                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  EncipherImage() converts pixels to cipher-pixels.
+%  PasskeyEncipherImage() converts pixels to cipher-pixels.
 %
-%  The format of the EncipherImage method is:
+%  The format of the PasskeyEncipherImage method is:
 %
+%      MagickBooleanType PasskeyEncipherImage(Image *image,
+%        const StringInfo *passkey,ExceptionInfo *exception)
 %      MagickBooleanType EncipherImage(Image *image,const char *passphrase,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o image: the image.
+%    o passphrase: decipher cipher pixels with this passphrase.
 %
-%    o passphrase: encipher pixels with this passphrase.
+%    o passkey: decrypt cipher pixels with this passkey.
 %
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
 MagickExport MagickBooleanType EncipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
 {
@@ -1056,4 +1135,17 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
   (void) passphrase;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
+
+MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
+  const StringInfo *passkey,ExceptionInfo *exception)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  (void) passkey;
+  ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
+}
 #endif
