@@ -1,12 +1,12 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
-  
+
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
-  
+
     http://www.imagemagick.org/script/license.php
-  
+
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -29,21 +29,11 @@ extern "C" {
 static inline MagickBooleanType IsColorEqual(const PixelPacket *p,
   const PixelPacket *q)
 {
-  if ((p->red == q->red) && (p->green == q->green) && (p->blue == q->blue))
+  if ((GetRedPixelComponent(p) == q->red) && (GetGreenPixelComponent(p) == q->green) && (GetBluePixelComponent(p) == q->blue))
     return(MagickTrue);
   return(MagickFalse);
 }
 
-static inline IndexPacket ConstrainColormapIndex(Image *image,
-  const unsigned long index)
-{
-  if (index < image->colors)
-    return((IndexPacket) index);
-  (void) ThrowMagickException(&image->exception,GetMagickModule(),
-    CorruptImageError,"InvalidColormapIndex","`%s'",image->filename);
-  return((IndexPacket) 0);
-}
-
 static inline MagickBooleanType IsGray(const PixelPacket *pixel)
 {
   if ((pixel->red == pixel->green) && (pixel->green == pixel->blue))
@@ -54,27 +44,51 @@ static inline MagickBooleanType IsGray(const PixelPacket *pixel)
 static inline MagickBooleanType IsMagickColorEqual(const MagickPixelPacket *p,
   const MagickPixelPacket *q)
 {
+#if !defined(MAGICKCORE_HDRI_SUPPORT)
   if ((p->matte != MagickFalse) && (q->matte == MagickFalse) &&
-      (p->opacity != OpaqueOpacity))
+      (GetOpacityPixelComponent(p) != OpaqueOpacity))
     return(MagickFalse);
   if ((q->matte != MagickFalse) && (p->matte == MagickFalse) &&
       (q->opacity != OpaqueOpacity))
     return(MagickFalse);
   if ((p->matte != MagickFalse) && (q->matte != MagickFalse))
     {
-      if (p->opacity != q->opacity)
+      if (GetOpacityPixelComponent(p) != q->opacity)
         return(MagickFalse);
-      if (p->opacity == TransparentOpacity)
+      if (GetOpacityPixelComponent(p) == TransparentOpacity)
         return(MagickTrue);
     }
-  if (p->red != q->red)
+  if (GetRedPixelComponent(p) != q->red)
     return(MagickFalse);
-  if (p->green != q->green)
+  if (GetGreenPixelComponent(p) != q->green)
     return(MagickFalse);
-  if (p->blue != q->blue)
+  if (GetBluePixelComponent(p) != q->blue)
     return(MagickFalse);
   if ((p->colorspace == CMYKColorspace) && (p->index != q->index))
     return(MagickFalse);
+#else
+  if ((p->matte != MagickFalse) && (q->matte == MagickFalse) &&
+      (fabs(GetOpacityPixelComponent(p)-OpaqueOpacity) > 0.5))
+    return(MagickFalse);
+  if ((q->matte != MagickFalse) && (p->matte == MagickFalse) &&
+      (fabs(q->opacity-OpaqueOpacity)) > 0.5)
+    return(MagickFalse);
+  if ((p->matte != MagickFalse) && (q->matte != MagickFalse))
+    {
+      if (fabs(GetOpacityPixelComponent(p)-q->opacity) > 0.5)
+        return(MagickFalse);
+      if (fabs(GetOpacityPixelComponent(p)-TransparentOpacity) <= 0.5)
+        return(MagickTrue);
+    }
+  if (fabs(GetRedPixelComponent(p)-q->red) > 0.5)
+    return(MagickFalse);
+  if (fabs(GetGreenPixelComponent(p)-q->green) > 0.5)
+    return(MagickFalse);
+  if (fabs(GetBluePixelComponent(p)-q->blue) > 0.5)
+    return(MagickFalse);
+  if ((p->colorspace == CMYKColorspace) && (fabs(p->index-q->index) > 0.5))
+    return(MagickFalse);
+#endif
   return(MagickTrue);
 }
 
@@ -90,11 +104,7 @@ static inline MagickBooleanType IsMagickGray(const MagickPixelPacket *pixel)
 static inline MagickRealType MagickPixelIntensity(
   const MagickPixelPacket *pixel)
 {
-  MagickRealType
-    intensity;
-
-  intensity=0.299*pixel->red+0.587*pixel->green+0.114*pixel->blue;
-  return(intensity);
+  return((MagickRealType) (0.299*pixel->red+0.587*pixel->green+0.114*pixel->blue));
 }
 
 static inline Quantum MagickPixelIntensityToQuantum(
