@@ -17,7 +17,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/module.h"
+#include "magick/policy.h"
 #include "magick/semaphore.h"
 #include "magick/splay-tree.h"
 #include "magick/static.h"
@@ -100,7 +101,6 @@ static const ModuleInfo
 
 static MagickBooleanType
   GetMagickModulePath(const char *,MagickModuleType,char *,ExceptionInfo *),
-  InitializeModuleList(ExceptionInfo *),
   UnregisterModule(const ModuleInfo *,ExceptionInfo *);
 
 static void
@@ -175,16 +175,19 @@ MagickExport void DestroyModuleList(void)
   /*
     Destroy magick modules.
   */
-  AcquireSemaphoreInfo(&module_semaphore);
+  LockSemaphoreInfo(module_semaphore);
 #if defined(MAGICKCORE_MODULES_SUPPORT)
   if (module_list != (SplayTreeInfo *) NULL)
     module_list=DestroySplayTree(module_list);
   if (instantiate_module != MagickFalse)
-    (void) lt_dlexit();
+    {
+#if !defined(MAGICKCORE_JP2_DELEGATE)
+      (void) lt_dlexit();  /* Jasper has an errant atexit() handler */
 #endif
-  instantiate_module=MagickFalse;
-  RelinquishSemaphoreInfo(module_semaphore);
-  DestroySemaphoreInfo(&module_semaphore);
+      instantiate_module=MagickFalse;
+    }
+#endif
+  UnlockSemaphoreInfo(module_semaphore);
 }
 
 /*
@@ -205,42 +208,41 @@ MagickExport void DestroyModuleList(void)
 %
 %  The format of the GetModuleInfo module is:
 %
-%      const ModuleInfo *GetModuleInfo(const char *tag,ExceptionInfo *exception)
+%      ModuleInfo *GetModuleInfo(const char *tag,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o tag: a character string that represents the image format we are
 %      looking for.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
-  ExceptionInfo *exception)
+MagickExport ModuleInfo *GetModuleInfo(const char *tag,ExceptionInfo *exception)
 {
   if ((module_list == (SplayTreeInfo *) NULL) ||
       (instantiate_module == MagickFalse))
     if (InitializeModuleList(exception) == MagickFalse)
-      return((const ModuleInfo *) NULL);
+      return((ModuleInfo *) NULL);
   if ((module_list == (SplayTreeInfo *) NULL) ||
       (GetNumberOfNodesInSplayTree(module_list) == 0))
-    return((const ModuleInfo *) NULL);
+    return((ModuleInfo *) NULL);
   if ((tag == (const char *) NULL) || (LocaleCompare(tag,"*") == 0))
     {
-      const ModuleInfo
+      ModuleInfo
         *p;
 
 #if defined(MAGICKCORE_MODULES_SUPPORT)
       if (LocaleCompare(tag,"*") == 0)
         (void) OpenModules(exception);
 #endif
-      AcquireSemaphoreInfo(&module_semaphore);
+      LockSemaphoreInfo(module_semaphore);
       ResetSplayTreeIterator(module_list);
-      p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
-      RelinquishSemaphoreInfo(module_semaphore);
+      p=(ModuleInfo *) GetNextValueInSplayTree(module_list);
+      UnlockSemaphoreInfo(module_semaphore);
       return(p);
     }
-  return((const ModuleInfo *) GetValueFromSplayTree(module_list,tag));
+  return((ModuleInfo *) GetValueFromSplayTree(module_list,tag));
 }
 
 /*
@@ -259,7 +261,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
 %  The format of the GetModuleInfoList function is:
 %
 %      const ModuleInfo **GetModuleInfoList(const char *pattern,
-%        unsigned long *number_modules,ExceptionInfo *exception)
+%        size_t *number_modules,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -267,7 +269,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
 %
 %    o number_modules:  This integer returns the number of modules in the list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -293,7 +295,7 @@ static int ModuleInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
-  unsigned long *number_modules,ExceptionInfo *exception)
+  size_t *number_modules,ExceptionInfo *exception)
 {
   const ModuleInfo
     **modules;
@@ -301,7 +303,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   register const ModuleInfo
     *p;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -309,7 +311,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   */
   assert(pattern != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
-  assert(number_modules != (unsigned long *) NULL);
+  assert(number_modules != (size_t *) NULL);
   *number_modules=0;
   p=GetModuleInfo("*",exception);
   if (p == (const ModuleInfo *) NULL)
@@ -321,7 +323,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   /*
     Generate module list.
   */
-  AcquireSemaphoreInfo(&module_semaphore);
+  LockSemaphoreInfo(module_semaphore);
   ResetSplayTreeIterator(module_list);
   p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   for (i=0; p != (const ModuleInfo *) NULL; )
@@ -331,10 +333,10 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
       modules[i++]=p;
     p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   }
-  RelinquishSemaphoreInfo(module_semaphore);
+  UnlockSemaphoreInfo(module_semaphore);
   qsort((void *) modules,(size_t) i,sizeof(*modules),ModuleInfoCompare);
   modules[i]=(ModuleInfo *) NULL;
-  *number_modules=(unsigned long) i;
+  *number_modules=(size_t) i;
   return(modules);
 }
 
@@ -354,17 +356,19 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
 %
 %  The format of the GetModuleList function is:
 %
-%      char **GetModuleList(const char *pattern,unsigned long *number_modules,
-%        ExceptionInfo *exception)
+%      char **GetModuleList(const char *pattern,const MagickModuleType type,
+%        size_t *number_modules,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
+%    o type: choose from MagickImageCoderModule or MagickImageFilterModule.
+%
 %    o number_modules:  This integer returns the number of modules in the
 %      list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -387,8 +391,21 @@ static int ModuleCompare(const void *x,const void *y)
 }
 #endif
 
+static inline int MagickReadDirectory(DIR *directory,struct dirent *entry,
+  struct dirent **result)
+{
+#if defined(MAGICKCORE_HAVE_READDIR_R)
+  return(readdir_r(directory,entry,result));
+#else
+  (void) entry;
+  errno=0;
+  *result=readdir(directory);
+  return(errno);
+#endif
+}
+
 MagickExport char **GetModuleList(const char *pattern,
-  unsigned long *number_modules,ExceptionInfo *exception)
+  const MagickModuleType type,size_t *number_modules,ExceptionInfo *exception)
 {
   char
     **modules,
@@ -399,25 +416,41 @@ MagickExport char **GetModuleList(const char *pattern,
   DIR
     *directory;
 
-  register long
+  MagickBooleanType
+    status;
+
+  register ssize_t
     i;
 
   size_t
-    length;
-
-  struct dirent
-    *entry;
-
-  unsigned long
     max_entries;
 
+  struct dirent
+    *buffer,
+    *entry;
+
   /*
-    Locate all modules in the coder path.
+    Locate all modules in the image coder or filter path.
   */
-  TagToCoderModuleName("magick",filename);
-  length=GetMagickModulePath(filename,MagickImageCoderModule,module_path,
-    exception);
-  if (length == 0)
+  switch (type)
+  {
+    case MagickImageCoderModule:
+    default:
+    {
+      TagToCoderModuleName("magick",filename);
+      status=GetMagickModulePath(filename,MagickImageCoderModule,module_path,
+        exception);
+      break;
+    }
+    case MagickImageFilterModule:
+    {
+      TagToFilterModuleName("analyze",filename);
+      status=GetMagickModulePath(filename,MagickImageFilterModule,module_path,
+        exception);
+      break;
+    }
+  }
+  if (status == MagickFalse)
     return((char **) NULL);
   GetPathComponent(module_path,HeadPath,path);
   max_entries=255;
@@ -428,19 +461,24 @@ MagickExport char **GetModuleList(const char *pattern,
   *modules=(char *) NULL;
   directory=opendir(path);
   if (directory == (DIR *) NULL)
-    return((char **) NULL);
+    {
+      modules=(char **) RelinquishMagickMemory(modules);
+      return((char **) NULL);
+    }
+  buffer=(struct dirent *) AcquireMagickMemory(sizeof(*buffer)+
+    FILENAME_MAX+1);
+  if (buffer == (struct dirent *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   i=0;
-  entry=readdir(directory);
-  while (entry != (struct dirent *) NULL)
+  while ((MagickReadDirectory(directory,buffer,&entry) == 0) &&
+         (entry != (struct dirent *) NULL))
   {
-    if (GlobExpression(entry->d_name,ModuleGlobExpression,MagickFalse) == MagickFalse)
-      {
-        entry=readdir(directory);
-        continue;
-      }
+    status=GlobExpression(entry->d_name,ModuleGlobExpression,MagickFalse);
+    if (status == MagickFalse)
+      continue;
     if (GlobExpression(entry->d_name,pattern,MagickFalse) == MagickFalse)
       continue;
-    if (i >= (long) max_entries)
+    if (i >= (ssize_t) max_entries)
       {
         modules=(char **) NULL;
         if (~max_entries > max_entries)
@@ -455,25 +493,24 @@ MagickExport char **GetModuleList(const char *pattern,
     */
     modules[i]=AcquireString((char *) NULL);
     GetPathComponent(entry->d_name,BasePath,modules[i]);
-    LocaleUpper(modules[i]);
     if (LocaleNCompare("IM_MOD_",modules[i],7) == 0)
       {
         (void) CopyMagickString(modules[i],modules[i]+10,MaxTextExtent);
         modules[i][strlen(modules[i])-1]='\0';
       }
     i++;
-    entry=readdir(directory);
   }
+  buffer=(struct dirent *) RelinquishMagickMemory(buffer);
   (void) closedir(directory);
   if (modules == (char **) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-        "MemoryAllocationFauled","`%s'",pattern);
+        "MemoryAllocationFailed","`%s'",pattern);
       return((char **) NULL);
     }
   qsort((void *) modules,(size_t) i,sizeof(*modules),ModuleCompare);
   modules[i]=(char *) NULL;
-  *number_modules=(unsigned long) i;
+  *number_modules=(size_t) i;
   return(modules);
 }
 
@@ -505,7 +542,7 @@ MagickExport char **GetModuleList(const char *pattern,
 %
 %    o path: the path associated with the filename.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType GetMagickModulePath(const char *filename,
@@ -572,7 +609,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       module_path=DestroyString(module_path);
     }
 #if defined(MAGICKCORE_INSTALLED_SUPPORT)
-   else
+  else
 #if defined(MAGICKCORE_CODER_PATH)
     {
       const char
@@ -605,7 +642,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       return(MagickTrue);
     }
 #else
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
     {
       const char
         *registery_key;
@@ -639,7 +676,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
         }
       (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",(char *) key_value,
         DirectorySeparator,filename);
-      key_value=(unsigned char *) RelinquishMagickMemory(key_value); 
+      key_value=(unsigned char *) RelinquishMagickMemory(key_value);
       if (IsPathAccessible(path) == MagickFalse)
         {
           ThrowFileException(exception,ConfigureWarning,
@@ -650,8 +687,8 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     }
 #endif
 #endif
-#if !defined(MAGICKCORE_CODER_PATH) && !defined(__WINDOWS__)
-# error MAGICKCORE_CODER_PATH or __WINDOWS__ must be defined when MAGICKCORE_INSTALLED_SUPPORT is defined
+#if !defined(MAGICKCORE_CODER_PATH) && !defined(MAGICKCORE_WINDOWS_SUPPORT)
+# error MAGICKCORE_CODER_PATH or MAGICKCORE_WINDOWS_SUPPORT must be defined when MAGICKCORE_INSTALLED_SUPPORT is defined
 #endif
 #else
   {
@@ -731,17 +768,20 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       if (IsPathAccessible(path) != MagickFalse)
         return(MagickTrue);
     }
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
     /*
       Search module path.
     */
-    if (NTGetModulePath("CORE_RL_magick_.dll",path) != MagickFalse)
-      if (IsPathAccessible(path) != MagickFalse)
-        return(MagickTrue);
-    if (NTGetModulePath("Magick.dll",path) != MagickFalse)
-      if (IsPathAccessible(path) != MagickFalse)
-        return(MagickTrue);
+    if ((NTGetModulePath("CORE_RL_magick_.dll",path) != MagickFalse) ||
+        (NTGetModulePath("CORE_DB_magick_.dll",path) != MagickFalse) ||
+        (NTGetModulePath("Magick.dll",path) != MagickFalse))
+      {
+        (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
+        (void) ConcatenateMagickString(path,filename,MaxTextExtent);
+        if (IsPathAccessible(path) != MagickFalse)
+          return(MagickTrue);
+      }
   }
 #endif
   {
@@ -781,7 +821,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   I n i t i a l i z e M o d u l e L i s t                                   %
+%   I n i t i a l i z e M o d u l e L i s t                                   %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -795,7 +835,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 %
 %  A description of each parameter follows.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -819,13 +859,15 @@ static void *DestroyModuleNode(void *module_info)
   return(RelinquishMagickMemory(p));
 }
 
-static MagickBooleanType InitializeModuleList(
+MagickExport MagickBooleanType InitializeModuleList(
   ExceptionInfo *magick_unused(exception))
 {
   if ((module_list == (SplayTreeInfo *) NULL) &&
       (instantiate_module == MagickFalse))
     {
-      AcquireSemaphoreInfo(&module_semaphore);
+      if (module_semaphore == (SemaphoreInfo *) NULL)
+        AcquireSemaphoreInfo(&module_semaphore);
+      LockSemaphoreInfo(module_semaphore);
       if ((module_list == (SplayTreeInfo *) NULL) &&
           (instantiate_module == MagickFalse))
         {
@@ -851,7 +893,7 @@ static MagickBooleanType InitializeModuleList(
               "UnableToInitializeModuleLoader");
           instantiate_module=MagickTrue;
         }
-      RelinquishSemaphoreInfo(module_semaphore);
+      UnlockSemaphoreInfo(module_semaphore);
     }
   return(module_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -872,7 +914,7 @@ static MagickBooleanType InitializeModuleList(
 %  The format of the InvokeDynamicImageFilter module is:
 %
 %      MagickBooleanType InvokeDynamicImageFilter(const char *tag,Image **image,
-%        const int argc,char **argv,ExceptionInfo *exception)
+%        const int argc,const char **argv,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -890,7 +932,7 @@ static MagickBooleanType InitializeModuleList(
 %
 */
 MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
-  Image **images,const int argc,char **argv,ExceptionInfo *exception)
+  Image **images,const int argc,const char **argv,ExceptionInfo *exception)
 {
   char
     name[MaxTextExtent],
@@ -899,11 +941,14 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
   ImageFilterHandler
     *image_filter;
 
+  MagickBooleanType
+    status;
+
   ModuleHandle
     handle;
 
-  size_t
-    length;
+  PolicyRights
+    rights;
 
   /*
     Find the module.
@@ -923,10 +968,22 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
       return(status);
   }
 #endif
+  rights=ReadPolicyRights;
+  if (IsRightsAuthorized(FilterPolicyDomain,rights,tag) == MagickFalse)
+    {
+      errno=EPERM;
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",tag);
+      return(MagickFalse);
+    }
   TagToFilterModuleName(tag,name);
-  length=GetMagickModulePath(name,MagickImageFilterModule,path,exception);
-  if (length == 0)
-    return(MagickFalse);
+  status=GetMagickModulePath(name,MagickImageFilterModule,path,exception);
+  if (status == MagickFalse)
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
+        "UnableToLoadModule","`%s': %s",name,path);
+      return(MagickFalse);
+    }
   /*
     Open the module.
   */
@@ -949,13 +1006,14 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
   /*
     Execute the module.
   */
+  ClearMagickException(exception);
   image_filter=(ImageFilterHandler *) lt_dlsym(handle,name);
   if (image_filter == (ImageFilterHandler *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
       "UnableToLoadModule","`%s': %s",name,lt_dlerror());
   else
     {
-      unsigned long
+      size_t
         signature;
 
       if ((*images)->debug != MagickFalse)
@@ -966,23 +1024,17 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
         (void) LogMagickEvent(ModuleEvent,GetMagickModule(),"\"%s\" completes",
           tag);
       if (signature != MagickImageFilterSignature)
-        {
-          (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-            "ImageFilterSignatureMismatch","`%s': %8lx != %8lx",tag,signature,
-            MagickImageFilterSignature);
-          return(MagickFalse);
-        }
+        (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
+          "ImageFilterSignatureMismatch","`%s': %8lx != %8lx",tag,
+          (unsigned long) signature,(unsigned long) MagickImageFilterSignature);
     }
   /*
     Close the module.
   */
   if (lt_dlclose(handle) != 0)
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ModuleWarning,
-        "UnableToCloseModule","`%s': %s",name,lt_dlerror());
-      return(MagickFalse);
-    }
-  return(MagickTrue);
+    (void) ThrowMagickException(exception,GetMagickModule(),ModuleWarning,
+      "UnableToCloseModule","`%s': %s",name,lt_dlerror());
+  return(exception->severity < ErrorException ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -1006,47 +1058,78 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
 %
 %    o file:  An pointer to a FILE.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType ListModuleInfo(FILE *file,
   ExceptionInfo *exception)
 {
-  const ModuleInfo
-    **module_info;
+  char
+    filename[MaxTextExtent],
+    module_path[MaxTextExtent],
+    **modules,
+    path[MaxTextExtent];
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_modules;
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  module_info=GetModuleInfoList("*",&number_modules,exception);
-  if (module_info == (const ModuleInfo **) NULL)
+  /*
+    List image coders.
+  */
+  modules=GetModuleList("*",MagickImageCoderModule,&number_modules,exception);
+  if (modules == (char **) NULL)
     return(MagickFalse);
-  if (module_info[0]->path != (char *) NULL)
-    {
-      char
-        path[MaxTextExtent];
-
-      GetPathComponent(module_info[0]->path,HeadPath,path);
-      (void) fprintf(file,"\nPath: %s\n\n",path);
-    }
-  (void) fprintf(file,"Module\n");
+  TagToCoderModuleName("magick",filename);
+  (void) GetMagickModulePath(filename,MagickImageCoderModule,module_path,
+    exception);
+  GetPathComponent(module_path,HeadPath,path);
+  (void) fprintf(file,"\nPath: %s\n\n",path);
+  (void) fprintf(file,"Image Coder\n");
   (void) fprintf(file,"-------------------------------------------------"
     "------------------------------\n");
-  for (i=0; i < (long) number_modules; i++)
+  for (i=0; i < (ssize_t) number_modules; i++)
   {
-    if (module_info[i]->stealth != MagickFalse)
-      continue;
-    (void) fprintf(file,"%s",module_info[i]->tag);
+    (void) fprintf(file,"%s",modules[i]);
     (void) fprintf(file,"\n");
   }
   (void) fflush(file);
-  module_info=(const ModuleInfo **)
-    RelinquishMagickMemory((void *) module_info);
+  /*
+    Relinquish resources.
+  */
+  for (i=0; i < (ssize_t) number_modules; i++)
+    modules[i]=DestroyString(modules[i]);
+  modules=(char **) RelinquishMagickMemory(modules);
+  /*
+    List image filters.
+  */
+  modules=GetModuleList("*",MagickImageFilterModule,&number_modules,exception);
+  if (modules == (char **) NULL)
+    return(MagickFalse);
+  TagToFilterModuleName("analyze",filename);
+  (void) GetMagickModulePath(filename,MagickImageFilterModule,module_path,
+    exception);
+  GetPathComponent(module_path,HeadPath,path);
+  (void) fprintf(file,"\nPath: %s\n\n",path);
+  (void) fprintf(file,"Image Filter\n");
+  (void) fprintf(file,"-------------------------------------------------"
+    "------------------------------\n");
+  for (i=0; i < (ssize_t) number_modules; i++)
+  {
+    (void) fprintf(file,"%s",modules[i]);
+    (void) fprintf(file,"\n");
+  }
+  (void) fflush(file);
+  /*
+    Relinquish resources.
+  */
+  for (i=0; i < (ssize_t) number_modules; i++)
+    modules[i]=DestroyString(modules[i]);
+  modules=(char **) RelinquishMagickMemory(modules);
   return(MagickTrue);
 }
 
@@ -1055,6 +1138,65 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *file,
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   M o d u l e C o m p o n e n t G e n e s i s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ModuleComponentGenesis() instantiates the module component.
+%
+%  The format of the ModuleComponentGenesis method is:
+%
+%      MagickBooleanType ModuleComponentGenesis(void)
+%
+*/
+MagickExport MagickBooleanType ModuleComponentGenesis(void)
+{
+  ExceptionInfo
+    *exception;
+
+  MagickBooleanType
+    status;
+
+  AcquireSemaphoreInfo(&module_semaphore);
+  exception=AcquireExceptionInfo();
+  status=InitializeModuleList(exception);
+  exception=DestroyExceptionInfo(exception);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   M o d u l e C o m p o n e n t T e r m i n u s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ModuleComponentTerminus() destroys the module component.
+%
+%  The format of the ModuleComponentTerminus method is:
+%
+%      ModuleComponentTerminus(void)
+%
+*/
+MagickExport void ModuleComponentTerminus(void)
+{
+  if (module_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&module_semaphore);
+  DestroyModuleList();
+  DestroySemaphoreInfo(&module_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   O p e n M o d u l e                                                       %
 %                                                                             %
 %                                                                             %
@@ -1072,7 +1214,7 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *file,
 %
 %    o module: a character string that indicates the module to load.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType OpenModule(const char *module,
@@ -1084,6 +1226,9 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     name[MaxTextExtent],
     path[MaxTextExtent];
 
+  MagickBooleanType
+    status;
+
   ModuleHandle
     handle;
 
@@ -1094,9 +1239,6 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     *p;
 
   size_t
-    length;
-
-  unsigned long
     signature;
 
   /*
@@ -1120,8 +1262,8 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Searching for module \"%s\" using filename \"%s\"",module_name,filename);
   *path='\0';
-  length=GetMagickModulePath(filename,MagickImageCoderModule,path,exception);
-  if (length == 0)
+  status=GetMagickModulePath(filename,MagickImageCoderModule,path,exception);
+  if (status == MagickFalse)
     return(MagickFalse);
   /*
     Load module
@@ -1146,8 +1288,8 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     Define RegisterFORMATImage method.
   */
   TagToModuleName(module_name,"Register%sImage",name);
-  module_info->register_module=(unsigned long (*)(void)) lt_dlsym(handle,name);
-  if (module_info->register_module == (unsigned long (*)(void)) NULL)
+  module_info->register_module=(size_t (*)(void)) lt_dlsym(handle,name);
+  if (module_info->register_module == (size_t (*)(void)) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
@@ -1175,7 +1317,7 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "ImageCoderSignatureMismatch","`%s': %8lx != %8lx",module_name,
-        signature,MagickImageCoderSignature);
+        (unsigned long) signature,(unsigned long) MagickImageCoderSignature);
       return(MagickFalse);
     }
   return(MagickTrue);
@@ -1200,7 +1342,7 @@ MagickExport MagickBooleanType OpenModule(const char *module,
 %
 %  A description of each parameter follows:
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
@@ -1208,10 +1350,10 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   char
     **modules;
 
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_modules;
 
   /*
@@ -1219,15 +1361,15 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   */
   (void) GetMagickInfo((char *) NULL,exception);
   number_modules=0;
-  modules=GetModuleList("*",&number_modules,exception);
+  modules=GetModuleList("*",MagickImageCoderModule,&number_modules,exception);
   if (modules == (char **) NULL)
     return(MagickFalse);
-  for (i=0; i < (long) number_modules; i++)
+  for (i=0; i < (ssize_t) number_modules; i++)
     (void) OpenModule(modules[i],exception);
   /*
     Relinquish resources.
   */
-  for (i=0; i < (long) number_modules; i++)
+  for (i=0; i < (ssize_t) number_modules; i++)
     modules[i]=DestroyString(modules[i]);
   modules=(char **) RelinquishMagickMemory(modules);
   return(MagickTrue);
@@ -1258,7 +1400,7 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
 %
 %    o module_info: a pointer to the ModuleInfo structure to register.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
@@ -1313,7 +1455,7 @@ static void TagToCoderModuleName(const char *tag,char *name)
   (void) FormatMagickString(name,MaxTextExtent,"%s.la",tag);
   (void) LocaleLower(name);
 #else
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   if (LocaleNCompare("IM_MOD_",tag,7) == 0)
     (void) CopyMagickString(name,tag,MaxTextExtent);
   else
@@ -1362,7 +1504,6 @@ static void TagToFilterModuleName(const char *tag,char *name)
   (void) FormatMagickString(name,MaxTextExtent,"%s.dll",tag);
 #else
   (void) FormatMagickString(name,MaxTextExtent,"%s.la",tag);
-  (void) LocaleLower(name);
 #endif
 }
 
@@ -1442,7 +1583,7 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
 %
 %    o module_info: the module info.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
@@ -1473,29 +1614,59 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *magick_unused(file),
 }
 
 MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
-  Image **image,const int argc,char **argv,ExceptionInfo *exception)
+  Image **image,const int argc,const char **argv,ExceptionInfo *exception)
 {
-#if !defined(MAGICKCORE_BUILD_MODULES)
+  PolicyRights
+    rights;
+
+  assert(image != (Image **) NULL);
+  assert((*image)->signature == MagickSignature);
+  if ((*image)->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
+  rights=ReadPolicyRights;
+  if (IsRightsAuthorized(FilterPolicyDomain,rights,tag) == MagickFalse)
+    {
+      errno=EPERM;
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",tag);
+      return(MagickFalse);
+    }
+#if defined(MAGICKCORE_BUILD_MODULES)
+  (void) tag;
+  (void) argc;
+  (void) argv;
+  (void) exception;
+#else
   {
-    extern unsigned long
-      analyzeImage(Image **,const int,char **,ExceptionInfo *);
+    extern size_t
+      analyzeImage(Image **,const int,const char **,ExceptionInfo *);
 
     ImageFilterHandler
       *image_filter;
 
     image_filter=(ImageFilterHandler *) NULL;
     if (LocaleCompare("analyze",tag) == 0)
-      image_filter=analyzeImage;
-    if (image_filter != (ImageFilterHandler *) NULL)
+      image_filter=(ImageFilterHandler *) analyzeImage;
+    if (image_filter == (ImageFilterHandler *) NULL)
+      (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
+        "UnableToLoadModule","`%s'",tag);
+    else
       {
-        unsigned long
+        size_t
           signature;
 
+        if ((*image)->debug != MagickFalse)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "Invoking \"%s\" static image filter",tag);
         signature=image_filter(image,argc,argv,exception);
+        if ((*image)->debug != MagickFalse)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"\"%s\" completes",
+            tag);
         if (signature != MagickImageFilterSignature)
           {
             (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-              "ImageFilterSignatureMismatch","`%s': %8lx != %8lx",tag,signature,
+              "ImageFilterSignatureMismatch","`%s': %8lx != %8lx",tag,
+              (unsigned long) signature,(unsigned long)
               MagickImageFilterSignature);
             return(MagickFalse);
           }
