@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -209,7 +209,6 @@ MagickExport MontageInfo *DestroyMontageInfo(MontageInfo *montage_info)
 %
 %    o montage_info: Specifies a pointer to a MontageInfo structure.
 %
-%
 */
 MagickExport void GetMontageInfo(const ImageInfo *image_info,
   MontageInfo *montage_info)
@@ -265,35 +264,35 @@ MagickExport void GetMontageInfo(const ImageInfo *image_info,
 %
 %    o images: Specifies a pointer to an array of Image structures.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
-static void GetMontageGeometry(char *geometry,const unsigned long number_images,
-  long *x_offset,long *y_offset,unsigned long *tiles_per_column,
-  unsigned long *tiles_per_row)
+static void GetMontageGeometry(char *geometry,const size_t number_images,
+  ssize_t *x_offset,ssize_t *y_offset,size_t *tiles_per_column,
+  size_t *tiles_per_row)
 {
   *tiles_per_column=0;
   *tiles_per_row=0;
   (void) GetGeometry(geometry,x_offset,y_offset,tiles_per_row,tiles_per_column);
   if ((*tiles_per_column == 0) && (*tiles_per_row == 0))
-    *tiles_per_column=(unsigned long) sqrt((double) number_images);
+    *tiles_per_column=(size_t) sqrt((double) number_images);
   if (*tiles_per_column == 0)
-    *tiles_per_column=(unsigned long)
+    *tiles_per_column=(size_t)
       ceil((double) number_images/(*tiles_per_row));
   if (*tiles_per_row == 0)
-    *tiles_per_row=(unsigned long)
+    *tiles_per_row=(size_t)
       ceil((double) number_images/(*tiles_per_column));
 }
 
-static inline long MagickMax(const long x,const long y)
+static inline ssize_t MagickMax(const ssize_t x,const ssize_t y)
 {
   if (x > y)
     return(x);
   return(y);
 }
 
-static inline long MagickMin(const long x,const long y)
+static inline ssize_t MagickMin(const ssize_t x,const ssize_t y)
 {
   if (x < y)
     return(x);
@@ -365,13 +364,6 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   ImageInfo
     *clone_info;
 
-  long
-    tile,
-    x,
-    x_offset,
-    y,
-    y_offset;
-
   MagickBooleanType
     concatenate,
     proceed,
@@ -380,13 +372,13 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   MagickOffsetType
     tiles;
 
-  MagickStatusType
-    flags;
-
   MagickProgressMonitor
     progress_monitor;
 
-  register long
+  MagickStatusType
+    flags;
+
+  register ssize_t
     i;
 
   RectangleInfo
@@ -394,15 +386,11 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     geometry,
     extract_info;
 
+
   size_t
-    extent;
-
-  TypeMetric
-    metrics;
-
-  unsigned long
     bevel_width,
     border_width,
+    extent,
     height,
     images_per_page,
     max_height,
@@ -416,6 +404,16 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     total_tiles,
     width;
 
+  ssize_t
+    tile,
+    x,
+    x_offset,
+    y,
+    y_offset;
+
+  TypeMetric
+    metrics;
+
   /*
     Create image tiles.
   */
@@ -434,7 +432,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   if (master_list == (Image **) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
   thumbnail=NewImageList();
-  for (i=0; i < (long) number_images; i++)
+  for (i=0; i < (ssize_t) number_images; i++)
   {
     image=CloneImage(image_list[i],0,0,MagickTrue,exception);
     if (image == (Image *) NULL)
@@ -442,22 +440,23 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     (void) ParseAbsoluteGeometry("0x0+0+0",&image->page);
     progress_monitor=SetImageProgressMonitor(image,(MagickProgressMonitor) NULL,
       image->client_data);
-    flags=ParseSizeGeometry(image,montage_info->geometry,&geometry);
+    flags=ParseRegionGeometry(image,montage_info->geometry,&geometry,exception);
     thumbnail=ThumbnailImage(image,geometry.width,geometry.height,exception);
     if (thumbnail == (Image *) NULL)
       break;
     image_list[i]=thumbnail;
     (void) SetImageProgressMonitor(image,progress_monitor,image->client_data);
-    proceed=SetImageProgress(image,TileImageTag,i,number_images);
+    proceed=SetImageProgress(image,TileImageTag,(MagickOffsetType) i,
+      number_images);
     if (proceed == MagickFalse)
       break;
     image=DestroyImage(image);
   }
-  if (i < (long) number_images)
+  if (i < (ssize_t) number_images)
     {
       if (thumbnail == (Image *) NULL)
         i--;
-      for (tile=0; (long) tile <= i; tile++)
+      for (tile=0; (ssize_t) tile <= i; tile++)
         if (image_list[tile] != (Image *) NULL)
           image_list[tile]=DestroyImage(image_list[tile]);
       master_list=(Image **) RelinquishMagickMemory(master_list);
@@ -466,17 +465,17 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   /*
     Sort image list by increasing tile number.
   */
-  for (i=0; i < (long) number_images; i++)
+  for (i=0; i < (ssize_t) number_images; i++)
     if (image_list[i]->scene == 0)
       break;
-  if (i == (long) number_images)
+  if (i == (ssize_t) number_images)
     qsort((void *) image_list,(size_t) number_images,sizeof(*image_list),
       SceneCompare);
   /*
     Determine tiles per row and column.
   */
-  tiles_per_column=(unsigned long) sqrt((double) number_images);
-  tiles_per_row=(unsigned long) ceil((double) number_images/tiles_per_column);
+  tiles_per_column=(size_t) sqrt((double) number_images);
+  tiles_per_row=(size_t) ceil((double) number_images/tiles_per_column);
   x_offset=0;
   y_offset=0;
   if (montage_info->tile != (char *) NULL)
@@ -487,8 +486,8 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   */
   concatenate=MagickFalse;
   SetGeometry(image_list[0],&extract_info);
-  extract_info.x=(long) montage_info->border_width;
-  extract_info.y=(long) montage_info->border_width;
+  extract_info.x=(ssize_t) montage_info->border_width;
+  extract_info.y=(ssize_t) montage_info->border_width;
   if (montage_info->geometry != (char *) NULL)
     {
       /*
@@ -517,17 +516,17 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       if ((flags & HeightValue) == 0)
         frame_info.height=frame_info.width;
       if ((flags & XiValue) == 0)
-        frame_info.outer_bevel=(long) frame_info.width/2;
+        frame_info.outer_bevel=(ssize_t) frame_info.width/2;
       if ((flags & PsiValue) == 0)
         frame_info.inner_bevel=frame_info.outer_bevel;
-      frame_info.x=(long) frame_info.width;
-      frame_info.y=(long) frame_info.height;
-      bevel_width=(unsigned long) MagickMax(frame_info.inner_bevel,
+      frame_info.x=(ssize_t) frame_info.width;
+      frame_info.y=(ssize_t) frame_info.height;
+      bevel_width=(size_t) MagickMax(frame_info.inner_bevel,
         frame_info.outer_bevel);
-      border_width=(unsigned long) MagickMax((long) frame_info.width,
-        (long) frame_info.height);
+      border_width=(size_t) MagickMax((ssize_t) frame_info.width,
+        (ssize_t) frame_info.height);
     }
-  for (i=0; i < (long) number_images; i++)
+  for (i=0; i < (ssize_t) number_images; i++)
   {
     if (image_list[i]->columns > extract_info.width)
       extract_info.width=image_list[i]->columns;
@@ -563,10 +562,10 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   title=InterpretImageProperties(clone_info,image_list[0],montage_info->title);
   title_offset=0;
   if (montage_info->title != (char *) NULL)
-    title_offset=(unsigned long) (2*(metrics.ascent-metrics.descent)*
+    title_offset=(size_t) (2*(metrics.ascent-metrics.descent)*
       MultilineCensus(title)+2*extract_info.y);
   number_lines=0;
-  for (i=0; i < (long) number_images; i++)
+  for (i=0; i < (ssize_t) number_images; i++)
   {
     value=GetImageProperty(image_list[i],"label");
     if (value == (const char *) NULL)
@@ -579,11 +578,12 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   */
   tile_image=AcquireImage(NULL);
   montage=AcquireImage(clone_info);
+  montage->background_color=montage_info->background_color;
   montage->scene=0;
   images_per_page=(number_images-1)/(tiles_per_row*tiles_per_column)+1;
   tiles=0;
-  total_tiles=(unsigned long) number_images;
-  for (i=0; i < (long) images_per_page; i++)
+  total_tiles=(size_t) number_images;
+  for (i=0; i < (ssize_t) images_per_page; i++)
   {
     /*
       Determine bounding box.
@@ -595,24 +595,24 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       GetMontageGeometry(montage_info->tile,number_images,&x_offset,&y_offset,
         &sans,&sans);
     tiles_per_page=tiles_per_row*tiles_per_column;
-    y_offset+=(long) title_offset;
+    y_offset+=(ssize_t) title_offset;
     max_height=0;
     bounds.width=0;
     bounds.height=0;
     width=0;
-    for (tile=0; tile < (long) tiles_per_page; tile++)
+    for (tile=0; tile < (ssize_t) tiles_per_page; tile++)
     {
-      if (tile < (long) number_images)
+      if (tile < (ssize_t) number_images)
         {
           width=concatenate != MagickFalse ? image_list[tile]->columns :
             extract_info.width;
           if (image_list[tile]->rows > max_height)
             max_height=image_list[tile]->rows;
         }
-      x_offset+=width+(extract_info.x+border_width)*2;
-      if (x_offset > (long) bounds.width)
-        bounds.width=(unsigned long) x_offset;
-      if (((tile+1) == (long) tiles_per_page) ||
+      x_offset+=(ssize_t) (width+2*(extract_info.x+border_width));
+      if (x_offset > (ssize_t) bounds.width)
+        bounds.width=(size_t) x_offset;
+      if (((tile+1) == (ssize_t) tiles_per_page) ||
           (((tile+1) % tiles_per_row) == 0))
         {
           x_offset=0;
@@ -620,11 +620,11 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
             GetMontageGeometry(montage_info->tile,number_images,&x_offset,&y,
               &sans,&sans);
           height=concatenate != MagickFalse ? max_height : extract_info.height;
-          y_offset+=(unsigned long) (height+(extract_info.y+border_width)*2+
+          y_offset+=(ssize_t) (height+(extract_info.y+(ssize_t) border_width)*2+
             (metrics.ascent-metrics.descent+4)*number_lines+
             (montage_info->shadow != MagickFalse ? 4 : 0));
-          if (y_offset > (long) bounds.height)
-            bounds.height=(unsigned long) y_offset;
+          if (y_offset > (ssize_t) bounds.height)
+            bounds.height=(size_t) y_offset;
           max_height=0;
         }
     }
@@ -644,7 +644,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     montage->montage=AcquireString((char *) NULL);
     tile=0;
     extent=1;
-    while (tile < MagickMin((long) tiles_per_page,(long) number_images))
+    while (tile < MagickMin((ssize_t) tiles_per_page,(ssize_t) number_images))
     {
       extent+=strlen(image_list[tile]->filename)+1;
       tile++;
@@ -659,15 +659,16 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     if (montage_info->tile != (char *) NULL)
       GetMontageGeometry(montage_info->tile,number_images,&x_offset,&y_offset,
         &sans,&sans);
-    y_offset+=(long) title_offset;
-    (void) FormatMagickString(montage->montage,MaxTextExtent,"%ldx%ld%+ld%+ld",
-      (long) (extract_info.width+(extract_info.x+border_width)*2),
-      (long) (extract_info.height+(extract_info.y+border_width)*2+
-      (metrics.ascent-metrics.descent+4)*number_lines+
-      (montage_info->shadow != MagickFalse ? 4 : 0)),x_offset,y_offset);
+    y_offset+=(ssize_t) title_offset;
+    (void) FormatMagickString(montage->montage,MaxTextExtent,
+      "%.20gx%.20g%+.20g%+.20g",(double) (extract_info.width+
+      (extract_info.x+border_width)*2),(double) (extract_info.height+
+      (extract_info.y+border_width)*2+(double) ((metrics.ascent-
+      metrics.descent+4)*number_lines+(montage_info->shadow != MagickFalse ? 4 :
+      0))),(double) x_offset,(double) y_offset);
     *montage->directory='\0';
     tile=0;
-    while (tile < MagickMin((long) tiles_per_page,(long) number_images))
+    while (tile < MagickMin((ssize_t) tiles_per_page,(ssize_t) number_images))
     {
       (void) ConcatenateMagickString(montage->directory,
         image_list[tile]->filename,extent);
@@ -686,16 +687,21 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
         DrawInfo
           *clone_info;
 
+        TypeMetric
+          metrics;
+
         /*
           Annotate composite image with title.
         */
         clone_info=CloneDrawInfo(image_info,draw_info);
-        (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
-          montage->columns,(unsigned long) (2*(metrics.ascent-metrics.descent)),
-          0L,(long) (extract_info.y+4));
+        clone_info->gravity=CenterGravity;
+        clone_info->pointsize*=2.0;
+        (void) GetTypeMetrics(image_list[0],clone_info,&metrics);
+        (void) FormatMagickString(geometry,MaxTextExtent,
+          "%.20gx%.20g%+.20g%+.20g",(double) montage->columns,(double)
+          (metrics.ascent-metrics.descent),0.0,(double) extract_info.y+4);
         (void) CloneString(&clone_info->geometry,geometry);
         (void) CloneString(&clone_info->text,title);
-        clone_info->pointsize*=2.0;
         (void) AnnotateImage(montage,clone_info);
         clone_info=DestroyDrawInfo(clone_info);
       }
@@ -710,9 +716,10 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       GetMontageGeometry(montage_info->tile,number_images,&x_offset,&y_offset,
         &sans,&sans);
     x_offset+=extract_info.x;
-    y_offset+=(long) title_offset+extract_info.y;
+    y_offset+=(ssize_t) title_offset+extract_info.y;
     max_height=0;
-    for (tile=0; tile < MagickMin((long) tiles_per_page,(long) number_images); tile++)
+    status=MagickTrue;
+    for (tile=0; tile < MagickMin((ssize_t) tiles_per_page,(ssize_t) number_images); tile++)
     {
       /*
         Copy this tile to the composite.
@@ -760,11 +767,11 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       tile_image->gravity=montage_info->gravity;
       if (image->gravity != UndefinedGravity)
         tile_image->gravity=image->gravity;
-      (void) FormatMagickString(tile_geometry,MaxTextExtent,"%lux%lu+0+0",
-        image->columns,image->rows);
-      flags=ParseGravityGeometry(tile_image,tile_geometry,&geometry);
-      x=(long) (geometry.x+border_width);
-      y=(long) (geometry.y+border_width);
+      (void) FormatMagickString(tile_geometry,MaxTextExtent,"%.20gx%.20g+0+0",
+        (double) image->columns,(double) image->rows);
+      flags=ParseGravityGeometry(tile_image,tile_geometry,&geometry,exception);
+      x=(ssize_t) (geometry.x+border_width);
+      y=(ssize_t) (geometry.y+border_width);
       if ((montage_info->frame != (char *) NULL) && (bevel_width != 0))
         {
           FrameInfo
@@ -781,7 +788,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
           extract_info.height=height+2*frame_info.height;
           value=GetImageProperty(image,"label");
           if (value != (const char *) NULL)
-            extract_info.height+=(unsigned long) ((metrics.ascent-
+            extract_info.height+=(size_t) ((metrics.ascent-
               metrics.descent+4)*MultilineCensus(value));
           frame_image=FrameImage(image,&extract_info,exception);
           if (frame_image != (Image *) NULL)
@@ -816,7 +823,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
                   image=shadow_image;
                 }
           }
-          (void) CompositeImage(montage,OverCompositeOp,image,x_offset+x,
+          (void) CompositeImage(montage,image->compose,image,x_offset+x,
             y_offset+y);
           value=GetImageProperty(image,"label");
           if (value != (const char *) NULL)
@@ -828,40 +835,43 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
                 Annotate composite tile with label.
               */
               (void) FormatMagickString(geometry,MaxTextExtent,
-                "%lux%lu%+ld%+ld",(montage_info->frame ? image->columns :
-                width)-2*border_width,(unsigned long) (metrics.ascent-
-                metrics.descent+4)*MultilineCensus(value),x_offset+
-                border_width,(montage_info->frame ? y_offset+height+
-                border_width+4 : y_offset+extract_info.height+border_width+
-                (montage_info->shadow != MagickFalse ? 4 : 0)));
+                "%.20gx%.20g%+.20g%+.20g",(double) ((montage_info->frame ?
+                image->columns : width)-2*border_width),(double)
+                (metrics.ascent-metrics.descent+4)*MultilineCensus(value),
+                (double) (x_offset+border_width),(double)
+                ((montage_info->frame ? y_offset+height+border_width+4 :
+                y_offset+extract_info.height+border_width+
+                (montage_info->shadow != MagickFalse ? 4 : 0))+bevel_width));
               (void) CloneString(&draw_info->geometry,geometry);
               (void) CloneString(&draw_info->text,value);
               (void) AnnotateImage(montage,draw_info);
             }
         }
-      x_offset+=width+(extract_info.x+border_width)*2;
-      if (((tile+1) == (long) tiles_per_page) ||
+      x_offset+=(ssize_t) (width+2*(extract_info.x+border_width));
+      if (((tile+1) == (ssize_t) tiles_per_page) ||
           (((tile+1) % tiles_per_row) == 0))
         {
           x_offset=extract_info.x;
-          y_offset+=(unsigned long) (height+(extract_info.y+border_width)*2+
+          y_offset+=(ssize_t) (height+(extract_info.y+border_width)*2+
             (metrics.ascent-metrics.descent+4)*number_lines+
             (montage_info->shadow != MagickFalse ? 4 : 0));
           max_height=0;
         }
-      if ((images->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(tiles,total_tiles) != MagickFalse))
+      if (images->progress_monitor != (MagickProgressMonitor) NULL)
         {
-          status=images->progress_monitor(MontageImageTag,tiles,total_tiles,
-            images->client_data);
-          if (status == MagickFalse)
-            break;
+          MagickBooleanType
+            proceed;
+
+          proceed=SetImageProgress(image,MontageImageTag,tiles,total_tiles);
+          if (proceed == MagickFalse)
+            status=MagickFalse;
         }
-      image=DestroyImage(image);
       image_list[tile]=DestroyImage(image_list[tile]);
+      image=DestroyImage(image);
       tiles++;
     }
-    if ((i+1) < (long) images_per_page)
+    (void) status;
+    if ((i+1) < (ssize_t) images_per_page)
       {
         /*
           Allocate next image structure.
@@ -873,6 +883,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
             return((Image *) NULL);
           }
         montage=GetNextImageInList(montage);
+        montage->background_color=montage_info->background_color;
         image_list+=tiles_per_page;
         number_images-=tiles_per_page;
       }
@@ -883,7 +894,5 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   master_list=(Image **) RelinquishMagickMemory(master_list);
   draw_info=DestroyDrawInfo(draw_info);
   clone_info=DestroyImageInfo(clone_info);
-  while (GetPreviousImageInList(montage) != (Image *) NULL)
-    montage=GetPreviousImageInList(montage);
-  return(montage);
+  return(GetFirstImageInList(montage));
 }
