@@ -17,7 +17,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,7 +39,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
 #include "magick/client.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -54,7 +54,7 @@
 #endif
 #include "magick/nt-base.h"
 #if defined(MAGICKCORE_CIPHER_SUPPORT)
-#include <NTSecAPI.h>
+#include <ntsecapi.h>
 #include <wincrypt.h>
 #endif
 
@@ -73,16 +73,16 @@ static char
   *lt_slsearchpath = (char *) NULL;
 #endif
 
-static GhostscriptVectors
-  gs_vectors;
+static GhostInfo
+  ghost_info;
 
 static void
-  *gs_dll_handle = (void *) NULL;
+  *ghost_handle = (void *) NULL;
 
 /*
   External declarations.
 */
-#if !defined(__WINDOWS__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT)
 extern "C" BOOL WINAPI
   DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved);
 #endif
@@ -236,6 +236,73 @@ MagickExport int Exit(int status)
   return(0);
 }
 
+#if !defined(__MINGW32__)
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   g e t t i m e o f d a y                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  The gettimeofday() method get the time of day.
+%
+%  The format of the gettimeofday method is:
+%
+%      int gettimeofday(struct timeval *time_value,struct timezone *time_zone)
+%
+%  A description of each parameter follows:
+%
+%    o time_value: the time value.
+%
+%    o time_zone: the time zone.
+%
+*/
+MagickExport int gettimeofday (struct timeval *time_value,
+  struct timezone *time_zone)
+{
+#define EpochFiletime  MagickLLConstant(116444736000000000)
+
+  static int
+    is_tz_set;
+
+  if (time_value != (struct timeval *) NULL)
+    {
+      FILETIME
+        file_time;
+
+      __int64
+        time;
+
+      LARGE_INTEGER
+        date_time;
+
+      GetSystemTimeAsFileTime(&file_time);
+      date_time.LowPart=file_time.dwLowDateTime;
+      date_time.HighPart=file_time.dwHighDateTime;
+      time=date_time.QuadPart;
+      time-=EpochFiletime;
+      time/=10;
+      time_value->tv_sec=(ssize_t) (time / 1000000);
+      time_value->tv_usec=(ssize_t) (time % 1000000);
+    }
+  if (time_zone != (struct timezone *) NULL)
+    {
+      if (is_tz_set == 0)
+        {
+          _tzset();
+          is_tz_set++;
+        }
+      time_zone->tz_minuteswest=_timezone/60;
+      time_zone->tz_dsttime=_daylight;
+    }
+  return(0);
+}
+#endif
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -349,7 +416,8 @@ MagickExport int NTCloseLibrary(void *handle)
 
 static BOOL ControlHandler(DWORD type)
 {
-  AsynchronousDestroyMagickResources();
+  (void) type;
+  AsynchronousResourceComponentTerminus();
   return(FALSE);
 }
 
@@ -411,12 +479,12 @@ MagickExport double NTElapsedTime(void)
 %
 %  The format of the NTErrorHandler method is:
 %
-%      void NTErrorHandler(const ExceptionType error,const char *reason,
+%      void NTErrorHandler(const ExceptionType severity,const char *reason,
 %        const char *description)
 %
 %  A description of each parameter follows:
 %
-%    o error: Specifies the numeric error category.
+%    o severity: Specifies the numeric error category.
 %
 %    o reason: Specifies the reason to display before terminating the
 %      program.
@@ -424,13 +492,14 @@ MagickExport double NTElapsedTime(void)
 %    o description: Specifies any description to the reason.
 %
 */
-MagickExport void NTErrorHandler(const ExceptionType error,const char *reason,
-  const char *description)
+MagickExport void NTErrorHandler(const ExceptionType severity,
+  const char *reason,const char *description)
 {
   char
     buffer[3*MaxTextExtent],
     *message;
 
+  (void) severity;
   if (reason == (char *) NULL)
     {
       MagickCoreTerminus();
@@ -509,7 +578,7 @@ MagickExport int NTExitLibrary(void)
 MagickExport MagickBooleanType NTGatherRandomData(const size_t length,
   unsigned char *random)
 {
-#if defined(MAGICKCORE_CIPHER_SUPPORT)
+#if defined(MAGICKCORE_CIPHER_SUPPORT) && defined(_MSC_VER) && (_MSC_VER > 1200)
   HCRYPTPROV
     handle;
 
@@ -533,6 +602,9 @@ MagickExport MagickBooleanType NTGatherRandomData(const size_t length,
   status=CryptReleaseContext(handle,0);
   if (status == 0)
     return(MagickFalse);
+#else
+  (void) random;
+  (void) length;
 #endif
   return(MagickTrue);
 }
@@ -714,7 +786,7 @@ MagickExport MagickBooleanType NTGetModulePath(const char *module,char *path)
   HMODULE
     handle;
 
-  long
+  ssize_t
     length;
 
   *path='\0';
@@ -738,302 +810,216 @@ MagickExport MagickBooleanType NTGetModulePath(const char *module,char *path)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  NTGhostscriptDLL() obtains the path to the latest Ghostscript DLL.  The
-%  method returns MagickFalse if a value is not obtained.
+%  NTGhostscriptDLL() returns the path to the most recent Ghostscript version
+%  DLL.  The method returns TRUE on success otherwise FALSE.
 %
 %  The format of the NTGhostscriptDLL method is:
 %
-%      int NTGhostscriptDLL(char *path, int length)
+%      int NTGhostscriptDLL(char *path,int length)
 %
 %  A description of each parameter follows:
 %
-%    o path: Pointer to buffer in which to return result.
+%    o path: return the Ghostscript DLL path here.
 %
-%    o length: Length of buffer
+%    o length: the buffer length.
 %
 */
 
-#define GS_PRODUCT_AFPL "AFPL Ghostscript"
-#define GS_PRODUCT_ALADDIN "Aladdin Ghostscript"
-#define GS_PRODUCT_GNU "GNU Ghostscript"
-#define GS_PRODUCT_GPL "GPL Ghostscript"
-#define GS_MINIMUM_VERSION 550
-
-/*
-  Get Ghostscript versions for given product.
-  Store results starting at pver + 1 + offset.
-  Returns total number of versions in pver.
-*/
-static int NTGhostscriptProductVersions(int *pver,int offset,
-  const char *gs_productfamily)
+static int NTGetRegistryValue(HKEY root,const char *key,const char *name,
+  char *value,int *length)
 {
-  HKEY
-    hkey,
-    hkeyroot;
-
-  DWORD
-    cbData;
-
-  char
-    key[MaxTextExtent],
+  BYTE
+    byte,
     *p;
 
-  int
-    n = 0,
-    ver;
+  DWORD
+    extent,
+    type;
 
-  (void) FormatMagickString(key,MaxTextExtent,"Software\\%s",gs_productfamily);
-  hkeyroot = HKEY_LOCAL_MACHINE;
-  if (RegOpenKeyExA(hkeyroot,key,0,KEY_READ,&hkey) == ERROR_SUCCESS)
-    {
-      /*
-        Now enumerate the keys.
-      */
-      cbData = sizeof(key) / sizeof(char);
-      while (RegEnumKeyA(hkey,n,key,cbData) == ERROR_SUCCESS)
-      {
-        n++;
-        ver = 0;
-        p = key;
-        while (*p && (*p!='.')) {
-          ver = (ver * 10) + (*p - '0')*100;
-          p++;
-        }
-        if (*p == '.')
-          p++;
-        if (*p)
-          {
-            ver+=10*(*p-'0');
-            p++;
-          }
-        if (*p)
-          ver+=(*p - '0');
-        if ((n+offset) < pver[0])
-          pver[n+offset]=ver;
-      }
-      RegCloseKey(hkey);
-    }
-  return(n+offset);
-}
-
-/* Query registry to find which versions of Ghostscript are installed.
- * Return version numbers in an integer array.
- * On entry, the first element in the array must be the array size
- * in elements.
- * If all is well, TRUE is returned.
- * On exit, the first element is set to the number of Ghostscript
- * versions installed, and subsequent elements to the version
- * numbers of Ghostscript.
- * e.g. on entry {5, 0, 0, 0, 0}, on exit {3, 550, 600, 596, 0}
- * Returned version numbers may not be sorted.
- *
- * If Ghostscript is not installed at all, return FALSE
- * and set pver[0] to 0.
- * If the array is not large enough, return FALSE
- * and set pver[0] to the number of Ghostscript versions installed.
- */
-
-static int NTGhostscriptEnumerateVersions(int *pver)
-{
-  int
-    n;
-
-  assert(pver != (int *) NULL);
-  n=NTGhostscriptProductVersions(pver,0,GS_PRODUCT_AFPL);
-  n=NTGhostscriptProductVersions(pver,n,GS_PRODUCT_ALADDIN);
-  n=NTGhostscriptProductVersions(pver,n,GS_PRODUCT_GNU);
-  n=NTGhostscriptProductVersions(pver,n,GS_PRODUCT_GPL);
-  if (n >= pver[0])
-    {
-      pver[0]=n;
-      return(FALSE);  /* too small */
-    }
-  if (n == 0)
-    {
-      pver[0] = 0;
-      return(FALSE);  /* not installed */
-    }
-  pver[0]=n;
-  return(TRUE);
-}
-
-/*
- Get a named registry value.
- Key = hkeyroot\\key, named value = name.
- name, ptr, plen and return values are the same as in gp_getenv();
-*/
-static int NTGetRegistryValue(HKEY hkeyroot,const char *key,const char *name,
-  char *ptr,int *plen)
-{
   HKEY
     hkey;
 
-  DWORD
-    cbData,
-    keytype;
-
-  BYTE
-    b,
-    *bptr = (BYTE *)ptr;
-
   LONG
-    rc;
+    status;
 
-  if (RegOpenKeyExA(hkeyroot,key,0,KEY_READ,&hkey) == ERROR_SUCCESS)
+  /*
+    Get a registry value: key = root\\key, named value = name.
+  */
+  if (RegOpenKeyExA(root,key,0,KEY_READ,&hkey) != ERROR_SUCCESS)
+    return(1);  /* no match */
+  p=(BYTE *) value;
+  type=REG_SZ;
+  extent=(*length);
+  if (p == (BYTE *) NULL)
+    p=(&byte);  /* ERROR_MORE_DATA only if value is NULL */
+  status=RegQueryValueExA(hkey,(char *) name,0,&type,p,&extent);
+  RegCloseKey(hkey);
+  if (status == ERROR_SUCCESS)
     {
-      keytype = REG_SZ;
-      cbData = *plen;
-      if (bptr == (BYTE *)NULL)
-        bptr=(&b);  /* Registry API won't return ERROR_MORE_DATA */
-      /* if ptr is NULL */
-      rc=RegQueryValueExA(hkey,(char *) name,0,&keytype,bptr,&cbData);
-      RegCloseKey(hkey);
-      if (rc == ERROR_SUCCESS)
-        {
-          *plen = cbData;
-          return 0;  /* found environment variable and copied it */
-        } else if (rc == ERROR_MORE_DATA) {
-        /* buffer wasn't large enough */
-          *plen = cbData;
-          return -1;
-        }
+      *length=extent;
+      return(0);  /* return the match */
+    }
+  if (status == ERROR_MORE_DATA)
+    {
+      *length=extent;
+      return(-1);  /* buffer not large enough */
     }
   return(1);  /* not found */
 }
 
-static int NTGhostscriptGetProductString(int gs_revision,const char *name,
-  char *ptr,int len,const char *gs_productfamily)
+static int NTLocateGhostscript(const char **product_family,int *major_version,
+  int *minor_version)
 {
-  /* If using Win32, look in the registry for a value with
-   * the given name.  The registry value will be under the key
-   * HKEY_CURRENT_USER\Software\AFPL Ghostscript\N.NN
-   * or if that fails under the key
-   * HKEY_LOCAL_MACHINE\Software\AFPL Ghostscript\N.NN
-   * where "AFPL Ghostscript" is actually gs_productfamily
-   * and N.NN is obtained from gs_revision.
-   */
+  int
+    i;
 
+  MagickBooleanType
+    status;
+
+  static const char
+    *products[4] =
+    {
+      "GPL Ghostscript",
+      "GNU Ghostscript",
+      "AFPL Ghostscript",
+      "Aladdin Ghostscript"
+    };
+
+  /*
+    Find the most recent version of Ghostscript.
+  */
+  status=FALSE;
+  *product_family=NULL;
+  *major_version=5;
+  *minor_version=49; /* min version of Ghostscript is 5.50 */
+  for (i=0; i < (ssize_t) (sizeof(products)/sizeof(products[0])); i++)
+  {
+    char
+      key[MaxTextExtent];
+
+    HKEY
+      hkey,
+      root;
+
+    REGSAM
+      mode;
+
+    (void) FormatMagickString(key,MaxTextExtent,"SOFTWARE\\%s",products[i]);
+    root=HKEY_LOCAL_MACHINE;
+    mode=KEY_READ;
+#if defined(KEY_WOW64_32KEY)
+    mode|=KEY_WOW64_32KEY;
+#endif
+    if (RegOpenKeyExA(root,key,0,mode,&hkey) == ERROR_SUCCESS)
+      {
+        DWORD
+          extent;
+
+        int
+          j;
+
+        /*
+          Now enumerate the keys.
+        */
+        extent=sizeof(key)/sizeof(char);
+        for (j=0; RegEnumKeyA(hkey,j,key,extent) == ERROR_SUCCESS; j++)
+        {
+          int
+            major,
+            minor;
+
+          major=0;
+          minor=0;
+          if (sscanf(key,"%d.%d",&major,&minor) != 2)
+            continue;
+          if ((major > *major_version) || ((major == *major_version) &&
+              (minor > *minor_version)))
+            {
+              *product_family=products[i];
+              *major_version=major;
+              *minor_version=minor;
+              status=TRUE;
+            }
+       }
+       (void) RegCloseKey(hkey);
+     }
+  }
+  if (status == FALSE)
+    {
+      *major_version=0;
+      *minor_version=0;
+    }
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),"Ghostscript (%s) "
+    "version %d.%02d",*product_family,*major_version,*minor_version);
+  return(status);
+}
+
+static int NTGhostscriptGetString(const char *name,char *value,
+  const size_t length)
+{
   char
-    dotversion[MaxTextExtent],
     key[MaxTextExtent];
 
   int
-    code,
-    length;
+    i,
+    extent;
 
-  DWORD version = GetVersion();
+  static const char
+    *product_family = (const char *) NULL;
 
-  if (((HIWORD(version) & 0x8000) != 0) && ((HIWORD(version) & 0x4000) == 0))
-    {
-      /* Win32s */
-      return FALSE;
-    }
-  (void) FormatMagickString(dotversion,MaxTextExtent,"%d.%02d",(int)
-    (gs_revision/100),(int) (gs_revision % 100));
-  (void) FormatMagickString(key,MaxTextExtent,"Software\\%s\\%s",
-    gs_productfamily,dotversion);
-  length = len;
-  code = NTGetRegistryValue(HKEY_CURRENT_USER, key, name, ptr, &length);
-  if ( code == 0 )
-    return TRUE;  /* found it */
-  length = len;
-  code = NTGetRegistryValue(HKEY_LOCAL_MACHINE, key, name, ptr, &length);
-  if ( code == 0 )
-    return TRUE;  /* found it */
-  return FALSE;
-}
+  static int
+    major_version=0,
+    minor_version=0;
 
-static int NTGhostscriptGetString(int gs_revision,const char *name,char *ptr,
-  int len)
-{
-  if (NTGhostscriptGetProductString(gs_revision,name,ptr,len,GS_PRODUCT_AFPL))
-    return(TRUE);
-  if (NTGhostscriptGetProductString(gs_revision,name,ptr,len,GS_PRODUCT_ALADDIN))
-    return(TRUE);
-  if (NTGhostscriptGetProductString(gs_revision,name,ptr,len,GS_PRODUCT_GNU))
-    return(TRUE);
-  if (NTGhostscriptGetProductString(gs_revision,name,ptr,len,GS_PRODUCT_GPL))
-    return(TRUE);
+  struct
+  {
+    const HKEY
+      hkey;
+
+    const char
+      *name;
+  }
+  hkeys[2] =
+  {
+    { HKEY_CURRENT_USER,  "HKEY_CURRENT_USER"  },
+    { HKEY_LOCAL_MACHINE, "HKEY_LOCAL_MACHINE" }
+  };
+
+  /*
+    Get a string from the installed Ghostscript.
+  */
+  *value='\0';
+  if (product_family == NULL)
+    (void) NTLocateGhostscript(&product_family,&major_version,&minor_version);
+  if (product_family == NULL)
+    return(FALSE);
+  (void) FormatMagickString(key,MaxTextExtent,"SOFTWARE\\%s\\%d.%02d",
+    product_family,major_version,minor_version);
+  for (i=0; i < (ssize_t) (sizeof(hkeys)/sizeof(hkeys[0])); i++)
+  {
+    extent=(int) length;
+    if (NTGetRegistryValue(hkeys[i].hkey,key,name,value,&extent) == 0)
+      {
+        (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+          "registry: \"%s\\%s\\%s\"=\"%s\"",hkeys[i].name,key,name,value);
+        return(TRUE);
+      }
+    (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+      "registry: \"%s\\%s\\%s\" (failed)",hkeys[i].name,key,name);
+  }
   return(FALSE);
 }
 
-static int NTGetLatestGhostscript( void )
+MagickExport int NTGhostscriptDLL(char *path,int length)
 {
-  int
-    count,
-    i,
-    gsver,
-    *ver;
-
-  DWORD version = GetVersion();
-  if ( ((HIWORD(version) & 0x8000)!=0) && ((HIWORD(version) & 0x4000)==0) )
-    return FALSE;  /* win32s */
-
-  count = 1;
-  NTGhostscriptEnumerateVersions(&count);
-  if (count < 1)
-    return FALSE;
-  ver=(int *) AcquireQuantumMemory(count+1UL,sizeof(*ver));
-  if (ver == (int *)NULL)
-    return(FALSE);
-  ver[0]=count+1;
-  if (!NTGhostscriptEnumerateVersions(ver))
-    {
-      ver=(int *) RelinquishMagickMemory(ver);
-      return FALSE;
-    }
-  gsver = 0;
-  for (i=1; i<=ver[0]; i++) {
-    if (ver[i] > gsver)
-      gsver = ver[i];
-  }
-  ver=(int *) RelinquishMagickMemory(ver);
-  return(gsver);
-}
-
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   N T G h o s t s c r i p t D L L                                           %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  NTGhostscriptDLL() obtains the path to the latest Ghostscript DLL.  The
-%  method returns MagickFalse if a value is not obtained.
-%
-%  The format of the NTGhostscriptDLL method is:
-%
-%      int NTGhostscriptDLL( char *path, int length)
-%
-%  A description of each parameter follows:
-%
-%    o path: Pointer to path buffer to update
-%
-%    o length: Allocation size of path buffer.
-%
-*/
-MagickExport int NTGhostscriptDLL(char *path, int length)
-{
-  int
-    gsver;
-
-  char
-    buf[256];
+  static char
+    dll[MaxTextExtent] = { "" };
 
   *path='\0';
-  gsver = NTGetLatestGhostscript();
-  if ((gsver == FALSE) || (gsver < GS_MINIMUM_VERSION))
-    return FALSE;
-
-  if (!NTGhostscriptGetString(gsver, "GS_DLL", buf, sizeof(buf)))
-    return FALSE;
-
-  (void) CopyMagickString(path,buf,length+1);
+  if ((*dll == '\0') &&
+      (NTGhostscriptGetString("GS_DLL",dll,sizeof(dll)) == FALSE))
+    return(FALSE);
+  (void) CopyMagickString(path,dll,length);
   return(TRUE);
 }
 
@@ -1048,21 +1034,21 @@ MagickExport int NTGhostscriptDLL(char *path, int length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  NTGhostscriptDLLVectors() returns a GhostscriptVectors structure containing
+%  NTGhostscriptDLLVectors() returns a GhostInfo structure that includes
 %  function vectors to invoke Ghostscript DLL functions. A null pointer is
-%  returned if there is an error with loading the DLL or retrieving the
+%  returned if there is an error when loading the DLL or retrieving the
 %  function vectors.
 %
 %  The format of the NTGhostscriptDLLVectors method is:
 %
-%      const GhostscriptVectors *NTGhostscriptDLLVectors(void)
+%      const GhostInfo *NTGhostscriptDLLVectors(void)
 %
 */
-MagickExport const GhostscriptVectors *NTGhostscriptDLLVectors( void )
+MagickExport const GhostInfo *NTGhostscriptDLLVectors(void)
 {
-  if (NTGhostscriptLoadDLL())
-    return(&gs_vectors);
-  return((GhostscriptVectors*) NULL);
+  if (NTGhostscriptLoadDLL() == FALSE)
+    return((GhostInfo *) NULL);
+  return(&ghost_info);
 }
 
 /*
@@ -1077,44 +1063,41 @@ MagickExport const GhostscriptVectors *NTGhostscriptDLLVectors( void )
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  NTGhostscriptEXE() obtains the path to the latest Ghostscript executable.
-%  The method returns MagickFalse if a value is not obtained.
+%  The method returns FALSE if a full path value is not obtained and returns
+%  a default path of gswin32c.exe.
 %
 %  The format of the NTGhostscriptEXE method is:
 %
-%      int NTGhostscriptEXE(char *path, int length)
+%      int NTGhostscriptEXE(char *path,int length)
 %
 %  A description of each parameter follows:
 %
-%    o path: Pointer to buffer in which to return result.
+%    o path: return the Ghostscript executable path here.
 %
-%    o length: Length of buffer
+%    o length: length of buffer.
 %
 */
 MagickExport int NTGhostscriptEXE(char *path,int length)
 {
-  int
-    gsver;
-
-  char
-    buf[256],
+  register char
     *p;
 
-  (void) CopyMagickString(path,"gswin32c.exe",length);
-  gsver=NTGetLatestGhostscript();
-  if ((gsver == FALSE) || (gsver < GS_MINIMUM_VERSION))
-    return(FALSE);
-  if (!NTGhostscriptGetString(gsver, "GS_DLL", buf, sizeof(buf)))
-    return(FALSE);
-  p=strrchr(buf, '\\');
-  if (p) {
-    p++;
-    *p = 0;
-    (void) CopyMagickString(p,"gswin32c.exe",sizeof(buf));
-    (void) CopyMagickString(path,buf,length+1);
-    return TRUE;
-  }
+  static char
+    program[MaxTextExtent] = { "" };
 
-  return FALSE;
+  (void) CopyMagickString(path,"gswin32c.exe",length);
+  if ((*program == '\0') &&
+      (NTGhostscriptGetString("GS_DLL",program,sizeof(program)) == FALSE))
+    return(FALSE);
+  p=strrchr(program,'\\');
+  if (p != (char *) NULL)
+    {
+      p++;
+      *p='\0';
+      (void) ConcatenateMagickString(program,"gswin32c.exe",sizeof(program));
+    }
+  (void) CopyMagickString(path,program,length);
+  return(TRUE);
 }
 
 /*
@@ -1128,18 +1111,18 @@ MagickExport int NTGhostscriptEXE(char *path,int length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  NTGhostscriptFonts() gets the path to the Ghostscript fonts.  The method
-%  returns MagickFalse if an error occurs otherwise MagickTrue.
+%  NTGhostscriptFonts() obtains the path to the Ghostscript fonts.  The method
+%  returns FALSE if it cannot determine the font path.
 %
 %  The format of the NTGhostscriptFonts method is:
 %
-%      int NTGhostscriptFonts(char *path,int length)
+%      int NTGhostscriptFonts(char *path, int length)
 %
 %  A description of each parameter follows:
 %
-%    o path: Pointer to buffer in which to return result.
+%    o path: return the font path here.
 %
-%    o length: Length of buffer.
+%    o length: length of the path buffer.
 %
 */
 MagickExport int NTGhostscriptFonts(char *path,int length)
@@ -1148,18 +1131,12 @@ MagickExport int NTGhostscriptFonts(char *path,int length)
     buffer[MaxTextExtent],
     filename[MaxTextExtent];
 
-  int
-    version;
-
   register char
     *p,
     *q;
 
   *path='\0';
-  version=NTGetLatestGhostscript();
-  if ((version == FALSE) || (version < GS_MINIMUM_VERSION))
-    return(FALSE);
-  if (!NTGhostscriptGetString(version,"GS_LIB",buffer,MaxTextExtent))
+  if (NTGhostscriptGetString("GS_LIB",buffer,MaxTextExtent) == FALSE)
     return(FALSE);
   for (p=buffer-1; p != (char *) NULL; p=strchr(p+1,DirectoryListSeparator))
   {
@@ -1167,7 +1144,7 @@ MagickExport int NTGhostscriptFonts(char *path,int length)
     q=strchr(path,DirectoryListSeparator);
     if (q != (char *) NULL)
       *q='\0';
-    FormatMagickString(filename,MaxTextExtent,"%s%sfonts.dir",path,
+    (void) FormatMagickString(filename,MaxTextExtent,"%s%sfonts.dir",path,
       DirectorySeparator);
     if (IsPathAccessible(filename) != MagickFalse)
       return(TRUE);
@@ -1187,7 +1164,7 @@ MagickExport int NTGhostscriptFonts(char *path,int length)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  NTGhostscriptLoadDLL() attempts to load the Ghostscript DLL and returns
-%  MagickTrue if it succeeds.
+%  TRUE if it succeeds.
 %
 %  The format of the NTGhostscriptLoadDLL method is:
 %
@@ -1197,35 +1174,31 @@ MagickExport int NTGhostscriptFonts(char *path,int length)
 MagickExport int NTGhostscriptLoadDLL(void)
 {
   char
-    module_path[MaxTextExtent];
+    path[MaxTextExtent];
 
-  if (gs_dll_handle != (void *) NULL)
-    return(MagickTrue);
-  if (NTGhostscriptDLL(module_path,sizeof(module_path)) == MagickFalse)
-    return(MagickFalse);
-  gs_dll_handle=NTOpenLibrary(module_path);
-  if (gs_dll_handle == (void *) NULL)
-    return(MagickFalse);
-  (void) ResetMagickMemory((void *) &gs_vectors,0,sizeof(GhostscriptVectors));
-  gs_vectors.exit=(int (MagickDLLCall *)
-    (gs_main_instance *)) NTGetLibrarySymbol(gs_dll_handle,"gsapi_exit");
-  gs_vectors.init_with_args=(int (MagickDLLCall *)
-    (gs_main_instance *,int,char **))
-    (NTGetLibrarySymbol(gs_dll_handle,"gsapi_init_with_args"));
-  gs_vectors.new_instance=(int (MagickDLLCall *) (gs_main_instance **,void *))
-    (NTGetLibrarySymbol(gs_dll_handle,"gsapi_new_instance"));
-  gs_vectors.run_string=(int (MagickDLLCall *)
-    (gs_main_instance *,const char *,int,int *))
-    (NTGetLibrarySymbol(gs_dll_handle,"gsapi_run_string"));
-  gs_vectors.delete_instance=(void (MagickDLLCall *)(gs_main_instance *))
-    (NTGetLibrarySymbol(gs_dll_handle,"gsapi_delete_instance"));
-  if ((gs_vectors.exit == NULL) ||
-      (gs_vectors.init_with_args == NULL) ||
-      (gs_vectors.new_instance == NULL) ||
-      (gs_vectors.run_string == NULL) ||
-      (gs_vectors.delete_instance == NULL))
-    return(MagickFalse);
-  return(MagickTrue);
+  if (ghost_handle != (void *) NULL)
+    return(TRUE);
+  if (NTGhostscriptDLL(path,sizeof(path)) == FALSE)
+    return(FALSE);
+  ghost_handle=lt_dlopen(path);
+  if (ghost_handle == (void *) NULL)
+    return(FALSE);
+  (void) ResetMagickMemory((void *) &ghost_info,0,sizeof(GhostInfo));
+  ghost_info.exit=(int (MagickDLLCall *)(gs_main_instance*))
+    lt_dlsym(ghost_handle,"gsapi_exit");
+  ghost_info.init_with_args=(int (MagickDLLCall *)(gs_main_instance *,int,
+    char **)) (lt_dlsym(ghost_handle,"gsapi_init_with_args"));
+  ghost_info.new_instance=(int (MagickDLLCall *)(gs_main_instance **,void *)) (
+    lt_dlsym(ghost_handle,"gsapi_new_instance"));
+  ghost_info.run_string=(int (MagickDLLCall *)(gs_main_instance *,const char *,
+    int,int *)) (lt_dlsym(ghost_handle,"gsapi_run_string"));
+  ghost_info.delete_instance=(void (MagickDLLCall *) (gs_main_instance *)) (
+    lt_dlsym(ghost_handle,"gsapi_delete_instance"));
+  if ((ghost_info.exit == NULL) || (ghost_info.init_with_args == NULL) ||
+      (ghost_info.new_instance == NULL) || (ghost_info.run_string == NULL) ||
+      (ghost_info.delete_instance == NULL))
+    return(FALSE);
+  return(TRUE);
 }
 
 /*
@@ -1239,7 +1212,8 @@ MagickExport int NTGhostscriptLoadDLL(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  NTGhostscriptUnLoadDLL() unloads the Ghostscript DLL if it is loaded.
+%  NTGhostscriptUnLoadDLL() unloads the Ghostscript DLL and returns TRUE if
+%  it succeeds.
 %
 %  The format of the NTGhostscriptUnLoadDLL method is:
 %
@@ -1248,12 +1222,15 @@ MagickExport int NTGhostscriptLoadDLL(void)
 */
 MagickExport int NTGhostscriptUnLoadDLL(void)
 {
-  if (gs_dll_handle == (void *) NULL)
-    return(MagickFalse);
-  NTCloseLibrary(gs_dll_handle);
-  gs_dll_handle=(void *) NULL;
-  (void) ResetMagickMemory((void *) &gs_vectors,0,sizeof(GhostscriptVectors));
-  return(MagickTrue);
+  int
+    status;
+
+  if (ghost_handle == (void *) NULL)
+    return(FALSE);
+  status=lt_dlclose(ghost_handle);
+  ghost_handle=(void *) NULL;
+  (void) ResetMagickMemory((void *) &ghost_info,0,sizeof(GhostInfo));
+  return(status);
 }
 
 /*
@@ -1316,6 +1293,7 @@ MagickExport void *NTMapMemory(char *address,size_t length,int protection,
   void
     *map;
 
+  (void) address;
   access_mode=0;
   file_handle=INVALID_HANDLE_VALUE;
   low_length=(DWORD) (length & 0xFFFFFFFFUL);
@@ -1692,7 +1670,7 @@ MagickExport MagickBooleanType NTReportEvent(const char *event,
   WORD
     type;
 
-  handle=RegisterEventSource(NULL,"ImageMagick");
+  handle=RegisterEventSource(NULL,MAGICKCORE_PACKAGE_NAME);
   if (handle == NULL)
     return(MagickFalse);
   events[0]=event;
@@ -1799,7 +1777,7 @@ MagickExport unsigned char *NTResourceToBlob(const char *id)
 %
 %  The format of the NTSeekDirectory method is:
 %
-%      void NTSeekDirectory(DIR *entry,long position)
+%      void NTSeekDirectory(DIR *entry,ssize_t position)
 %
 %  A description of each parameter follows:
 %
@@ -1809,8 +1787,9 @@ MagickExport unsigned char *NTResourceToBlob(const char *id)
 %      stream.
 %
 */
-MagickExport void NTSeekDirectory(DIR *entry,long position)
+MagickExport void NTSeekDirectory(DIR *entry,ssize_t position)
 {
+  (void) position;
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(entry != (DIR *) NULL);
 }
@@ -1880,6 +1859,7 @@ MagickExport int NTSetSearchPath(const char *path)
 */
 MagickExport int NTSyncMemory(void *address,size_t length,int flags)
 {
+  (void) flags;
   if (FlushViewOfFile(address,length) == MagickFalse)
     return(-1);
   return(0);
@@ -1899,9 +1879,9 @@ MagickExport int NTSyncMemory(void *address,size_t length,int flags)
 %  NTSystemCommand() executes the specified command and waits until it
 %  terminates.  The returned value is the exit status of the command.
 %
-%  The format of the NTSystemComman method is:
+%  The format of the NTSystemCommand method is:
 %
-%      int NTSystemComman(const char *command)
+%      int NTSystemCommand(MagickFalse,const char *command)
 %
 %  A description of each parameter follows:
 %
@@ -1952,7 +1932,7 @@ MagickExport int NTSystemCommand(const char *command)
     return(status == 0);
   status=WaitForSingleObject(process_info.hProcess,INFINITE);
   if (status != WAIT_OBJECT_0)
-    return (status);
+    return(status);
   status=GetExitCodeProcess(process_info.hProcess,&child_status);
   if (status == 0)
     return(-1);
@@ -1977,14 +1957,14 @@ MagickExport int NTSystemCommand(const char *command)
 %
 %  The format of the exit method is:
 %
-%      long NTSystemConfiguration(int name)
+%      ssize_t NTSystemConfiguration(int name)
 %
 %  A description of each parameter follows:
 %
 %    o name: _SC_PAGE_SIZE or _SC_PHYS_PAGES.
 %
 */
-MagickExport long NTSystemConfiguration(int name)
+MagickExport ssize_t NTSystemConfiguration(int name)
 {
   switch (name)
   {
@@ -2021,12 +2001,12 @@ MagickExport long NTSystemConfiguration(int name)
             status;
 
           GlobalMemoryStatus(&status);
-          return((long) status.dwTotalPhys/system_info.dwPageSize);
+          return((ssize_t) status.dwTotalPhys/system_info.dwPageSize);
         }
       status.dwLength=sizeof(status);
       if (module(&status) == 0)
         return(0L);
-      return((long) status.ullTotalPhys/system_info.dwPageSize);
+      return((ssize_t) status.ullTotalPhys/system_info.dwPageSize);
     }
     case _SC_OPEN_MAX:
       return(2048);
@@ -2035,7 +2015,6 @@ MagickExport long NTSystemConfiguration(int name)
   }
   return(-1);
 }
-
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -2053,14 +2032,14 @@ MagickExport long NTSystemConfiguration(int name)
 %
 %  The format of the NTTellDirectory method is:
 %
-%      long NTTellDirectory(DIR *entry)
+%      ssize_t NTTellDirectory(DIR *entry)
 %
 %  A description of each parameter follows:
 %
 %    o entry: Specifies a pointer to a DIR structure.
 %
 */
-MagickExport long NTTellDirectory(DIR *entry)
+MagickExport ssize_t NTTellDirectory(DIR *entry)
 {
   assert(entry != (DIR *) NULL);
   return(0);
@@ -2139,6 +2118,7 @@ MagickExport int NTTruncateFile(int file,off_t length)
 */
 MagickExport int NTUnmapMemory(void *map,size_t length)
 {
+  (void) length;
   if (UnmapViewOfFile(map) == 0)
     return(-1);
   return(0);
@@ -2219,12 +2199,12 @@ MagickExport double NTUserTime(void)
 %
 %  The format of the NTWarningHandler method is:
 %
-%      void NTWarningHandler(const ExceptionType warning,const char *reason,
+%      void NTWarningHandler(const ExceptionType severity,const char *reason,
 %        const char *description)
 %
 %  A description of each parameter follows:
 %
-%    o warning: Specifies the numeric warning category.
+%    o severity: Specifies the numeric warning category.
 %
 %    o reason: Specifies the reason to display before terminating the
 %      program.
@@ -2232,12 +2212,13 @@ MagickExport double NTUserTime(void)
 %    o description: Specifies any description to the reason.
 %
 */
-MagickExport void NTWarningHandler(const ExceptionType warning,
+MagickExport void NTWarningHandler(const ExceptionType severity,
   const char *reason,const char *description)
 {
   char
     buffer[2*MaxTextExtent];
 
+  (void) severity;
   if (reason == (char *) NULL)
     return;
   if (description == (char *) NULL)
