@@ -2,6 +2,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %                      PPPP    AAA   IIIII  N   N  TTTTT                      %
 %                      P   P  A   A    I    NN  N    T                        %
 %                      PPPP   AAAAA    I    N N N    T                        %
@@ -16,7 +17,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,11 +55,8 @@
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
 #include "magick/string_.h"
+#include "magick/thread-private.h"
 
-static inline double MagickMax(const double x,const double y)
-{
-    return( x > y ? x : y);
-}
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -86,8 +84,8 @@ static inline double MagickMax(const double x,const double y)
 %
 %      MagickBooleanType FloodfillPaintImage(Image *image,
 %        const ChannelType channel,const DrawInfo *draw_info,
-%        const MagickPixelPacket target,const long x_offset,const long y_offset,
-%        const MagickBooleanType invert)
+%        const MagickPixelPacket target,const ssize_t x_offset,
+%        const ssize_t y_offset,const MagickBooleanType invert)
 %
 %  A description of each parameter follows:
 %
@@ -106,7 +104,7 @@ static inline double MagickMax(const double x,const double y)
 */
 MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   const ChannelType channel,const DrawInfo *draw_info,
-  const MagickPixelPacket *target,const long x_offset,const long y_offset,
+  const MagickPixelPacket *target,const ssize_t x_offset,const ssize_t y_offset,
   const MagickBooleanType invert)
 {
 #define MaxStacksize  (1UL << 15)
@@ -116,7 +114,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     ThrowBinaryException(DrawError,"SegmentStackOverflow",image->filename) \
   else \
     { \
-      if ((((up)+(delta)) >= 0) && (((up)+(delta)) < (long) image->rows)) \
+      if ((((up)+(delta)) >= 0) && (((up)+(delta)) < (ssize_t) image->rows)) \
         { \
           s->x1=(double) (left); \
           s->y1=(double) (up); \
@@ -127,16 +125,16 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     } \
 }
 
+  CacheView
+    *floodplane_view,
+    *image_view;
+
+  ExceptionInfo
+    *exception;
+
   Image
     *floodplane_image;
 
-  long
-    offset,
-    start,
-    x1,
-    x2,
-    y;
-
   MagickBooleanType
     skip;
 
@@ -147,24 +145,20 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   PixelPacket
     fill_color;
 
-  register const PixelPacket
-    *p;
-
-  register IndexPacket
-    *indexes;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
   register SegmentInfo
     *s;
 
   SegmentInfo
     *segment_stack;
 
+  ssize_t
+    offset,
+    start,
+    x,
+    x1,
+    x2,
+    y;
+
   /*
     Check boundary conditions.
   */
@@ -174,9 +168,9 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->signature == MagickSignature);
-  if ((x_offset < 0) || (x_offset >= (long) image->columns))
+  if ((x_offset < 0) || (x_offset >= (ssize_t) image->columns))
     return(MagickFalse);
-  if ((y_offset < 0) || (y_offset >= (long) image->rows))
+  if ((y_offset < 0) || (y_offset >= (ssize_t) image->rows))
     return(MagickFalse);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
@@ -185,8 +179,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   /*
     Set floodfill state.
   */
-  floodplane_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-    &image->exception);
+  floodplane_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (floodplane_image == (Image *) NULL)
     return(MagickFalse);
   (void) SetImageAlphaChannel(floodplane_image,OpaqueAlphaChannel);
@@ -201,6 +194,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   /*
     Push initial segment on stack.
   */
+  exception=(&image->exception);
   x=x_offset;
   y=y_offset;
   start=0;
@@ -209,24 +203,39 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   PushSegmentStack(y+1,x,x,-1);
   GetMagickPixelPacket(image,&fill);
   GetMagickPixelPacket(image,&pixel);
+  image_view=AcquireCacheView(image);
+  floodplane_view=AcquireCacheView(floodplane_image);
   while (s > segment_stack)
   {
+    register const IndexPacket
+      *restrict indexes;
+
+    register const PixelPacket
+      *restrict p;
+
+    register ssize_t
+      x;
+
+    register PixelPacket
+      *restrict q;
+
     /*
       Pop segment off stack.
     */
     s--;
-    x1=(long) s->x1;
-    x2=(long) s->x2;
-    offset=(long) s->y2;
-    y=(long) s->y1+offset;
+    x1=(ssize_t) s->x1;
+    x2=(ssize_t) s->x2;
+    offset=(ssize_t) s->y2;
+    y=(ssize_t) s->y1+offset;
     /*
       Recolor neighboring pixels.
     */
-    p=AcquireImagePixels(image,0,y,(unsigned long) (x1+1),1,&image->exception);
-    q=GetImagePixels(floodplane_image,0,y,(unsigned long) (x1+1),1);
+    p=GetCacheViewVirtualPixels(image_view,0,y,(size_t) (x1+1),1,exception);
+    q=GetCacheViewAuthenticPixels(floodplane_view,0,y,(size_t) (x1+1),1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetIndexes(image);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
     p+=x1;
     q+=x1;
     for (x=x1; x >= 0; x--)
@@ -240,7 +249,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       p--;
       q--;
     }
-    if (SyncImagePixels(floodplane_image) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(floodplane_view,exception) == MagickFalse)
       break;
     skip=x >= x1 ? MagickTrue : MagickFalse;
     if (skip == MagickFalse)
@@ -254,16 +263,17 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     {
       if (skip == MagickFalse)
         {
-          if (x < (long) image->columns)
+          if (x < (ssize_t) image->columns)
             {
-              p=AcquireImagePixels(image,x,y,image->columns-x,1,
-                &image->exception);
-              q=GetImagePixels(floodplane_image,x,y,image->columns-x,1);
+              p=GetCacheViewVirtualPixels(image_view,x,y,image->columns-x,1,
+                exception);
+              q=GetCacheViewAuthenticPixels(floodplane_view,x,y,
+                image->columns-x,1,exception);
               if ((p == (const PixelPacket *) NULL) ||
                   (q == (PixelPacket *) NULL))
                 break;
-              indexes=GetIndexes(image);
-              for ( ; x < (long) image->columns; x++)
+              indexes=GetCacheViewVirtualIndexQueue(image_view);
+              for ( ; x < (ssize_t) image->columns; x++)
               {
                 if (q->opacity == (Quantum) TransparentOpacity)
                   break;
@@ -274,7 +284,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
                 p++;
                 q++;
               }
-              if (SyncImagePixels(floodplane_image) == MagickFalse)
+              if (SyncCacheViewAuthenticPixels(floodplane_view,exception) == MagickFalse)
                 break;
             }
           PushSegmentStack(y,start,x-1,offset);
@@ -285,12 +295,13 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       x++;
       if (x <= x2)
         {
-          p=AcquireImagePixels(image,x,y,(unsigned long) (x2-x+1),1,
-            &image->exception);
-          q=GetImagePixels(floodplane_image,x,y,(unsigned long) (x2-x+1),1);
+          p=GetCacheViewVirtualPixels(image_view,x,y,(size_t) (x2-x+1),1,
+            exception);
+          q=GetCacheViewAuthenticPixels(floodplane_view,x,y,(size_t) (x2-x+1),1,
+            exception);
           if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
-          indexes=GetIndexes(image);
+          indexes=GetCacheViewVirtualIndexQueue(image_view);
           for ( ; x <= x2; x++)
           {
             if (q->opacity == (Quantum) TransparentOpacity)
@@ -305,46 +316,60 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       start=x;
     } while (x <= x2);
   }
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
+    register const PixelPacket
+      *restrict p;
+
+    register IndexPacket
+      *restrict indexes;
+
+    register ssize_t
+      x;
+
+    register PixelPacket
+      *restrict q;
+
     /*
       Tile fill color onto floodplane.
     */
-    p=AcquireImagePixels(floodplane_image,0,y,image->columns,1,
-      &image->exception);
-    q=GetImagePixels(image,0,y,image->columns,1);
+    p=GetCacheViewVirtualPixels(floodplane_view,0,y,image->columns,1,
+      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetIndexes(image);
-    for (x=0; x < (long) image->columns; x++)
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if (p->opacity != OpaqueOpacity)
+      if (GetOpacityPixelComponent(p) != OpaqueOpacity)
         {
           (void) GetFillColor(draw_info,x,y,&fill_color);
           SetMagickPixelPacket(image,&fill_color,(IndexPacket *) NULL,&fill);
           if (image->colorspace == CMYKColorspace)
             ConvertRGBToCMYK(&fill);
           if ((channel & RedChannel) != 0)
-            q->red=RoundToQuantum(fill.red);
+            q->red=ClampToQuantum(fill.red);
           if ((channel & GreenChannel) != 0)
-            q->green=RoundToQuantum(fill.green);
+            q->green=ClampToQuantum(fill.green);
           if ((channel & BlueChannel) != 0)
-            q->blue=RoundToQuantum(fill.blue);
+            q->blue=ClampToQuantum(fill.blue);
           if ((channel & OpacityChannel) != 0)
-            q->opacity=RoundToQuantum(fill.opacity);
+            q->opacity=ClampToQuantum(fill.opacity);
           if (((channel & IndexChannel) != 0) &&
               (image->colorspace == CMYKColorspace))
-            indexes[x]=RoundToQuantum(fill.index);
+            indexes[x]=ClampToQuantum(fill.index);
         }
       p++;
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
   }
+  floodplane_view=DestroyCacheView(floodplane_view);
+  image_view=DestroyCacheView(image_view);
   segment_stack=(SegmentInfo *) RelinquishMagickMemory(segment_stack);
   floodplane_image=DestroyImage(floodplane_image);
-  return(y == (long) image->rows ? MagickTrue : MagickFalse);
+  return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -385,6 +410,12 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
 % This provides a good example of making use of the DrawGradientImage
 % function and the gradient structure in draw_info.
 */
+
+static inline double MagickMax(const double x,const double y)
+{
+  return(x > y ? x : y);
+}
+
 MagickExport MagickBooleanType GradientImage(Image *image,
   const GradientType type,const SpreadMethod method,
   const PixelPacket *start_color,const PixelPacket *stop_color)
@@ -398,7 +429,7 @@ MagickExport MagickBooleanType GradientImage(Image *image,
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -434,7 +465,7 @@ MagickExport MagickBooleanType GradientImage(Image *image,
       image->filename);
   (void) ResetMagickMemory(gradient->stops,0,gradient->number_stops*
     sizeof(*gradient->stops));
-  for (i=0; i < (long) gradient->number_stops; i++)
+  for (i=0; i < (ssize_t) gradient->number_stops; i++)
     GetMagickPixelPacket(image,&gradient->stops[i].color);
   SetMagickPixelPacket(image,start_color,(IndexPacket *) NULL,
     &gradient->stops[0].color);
@@ -482,41 +513,43 @@ MagickExport MagickBooleanType GradientImage(Image *image,
 %
 %    o radius: the radius of the circular neighborhood.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
-static unsigned long **DestroyHistogramThreadSet(unsigned long **histogram)
+static size_t **DestroyHistogramThreadSet(size_t **histogram)
 {
-  register long
+  register ssize_t
     i;
 
-  assert(histogram != (unsigned long **) NULL);
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
-    if (histogram[i] != (unsigned long *) NULL)
-      histogram[i]=(unsigned long *) RelinquishMagickMemory(histogram[i]);
-  return((unsigned long **) RelinquishMagickMemory(histogram));
+  assert(histogram != (size_t **) NULL);
+  for (i=0; i < (ssize_t) GetOpenMPMaximumThreads(); i++)
+    if (histogram[i] != (size_t *) NULL)
+      histogram[i]=(size_t *) RelinquishMagickMemory(histogram[i]);
+  histogram=(size_t **) RelinquishMagickMemory(histogram);
+  return(histogram);
 }
 
-static unsigned long **AcquireHistogramThreadSet(const size_t count)
+static size_t **AcquireHistogramThreadSet(const size_t count)
 {
-  register long
+  register ssize_t
     i;
 
-  unsigned long
-    **histogram;
+  size_t
+    **histogram,
+    number_threads;
 
-  histogram=(unsigned long **) AcquireQuantumMemory(
-    GetCacheViewMaximumThreads(),sizeof(*histogram));
-  if (histogram == (unsigned long **) NULL)
-    return((unsigned long **) NULL);
-  (void) ResetMagickMemory(histogram,0,GetCacheViewMaximumThreads()*
+  number_threads=GetOpenMPMaximumThreads();
+  histogram=(size_t **) AcquireQuantumMemory(number_threads,
     sizeof(*histogram));
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  if (histogram == (size_t **) NULL)
+    return((size_t **) NULL);
+  (void) ResetMagickMemory(histogram,0,number_threads*sizeof(*histogram));
+  for (i=0; i < (ssize_t) number_threads; i++)
   {
-    histogram[i]=(unsigned long *) AcquireQuantumMemory(count,
+    histogram[i]=(size_t *) AcquireQuantumMemory(count,
       sizeof(**histogram));
-    if (histogram[i] == (unsigned long *) NULL)
+    if (histogram[i] == (size_t *) NULL)
       return(DestroyHistogramThreadSet(histogram));
   }
   return(histogram);
@@ -528,23 +561,25 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
 #define NumberPaintBins  256
 #define OilPaintImageTag  "OilPaint/Image"
 
+  CacheView
+    *image_view,
+    *paint_view;
+
   Image
     *paint_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     status;
 
-  unsigned long
-    **histogram,
+  MagickOffsetType
+    progress;
+
+  size_t
+    **restrict histograms,
     width;
 
-  ViewInfo
-    **image_view,
-    **paint_view;
+  ssize_t
+    y;
 
   /*
     Initialize painted image attributes.
@@ -556,8 +591,6 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   width=GetOptimalKernelWidth2D(radius,0.5);
-  if ((image->columns < width) || (image->rows < width))
-    ThrowImageException(OptionError,"ImageSmallerThanRadius");
   paint_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
   if (paint_image == (Image *) NULL)
     return((Image *) NULL);
@@ -567,8 +600,8 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       paint_image=DestroyImage(paint_image);
       return((Image *) NULL);
     }
-  histogram=AcquireHistogramThreadSet(NumberPaintBins);
-  if (histogram == (unsigned long **) NULL)
+  histograms=AcquireHistogramThreadSet(NumberPaintBins);
+  if (histograms == (size_t **) NULL)
     {
       paint_image=DestroyImage(paint_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
@@ -578,77 +611,79 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  paint_view=AcquireCacheViewThreadSet(paint_image);
+  image_view=AcquireCacheView(image);
+  paint_view=AcquireCacheView(paint_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,64) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    const IndexPacket
-      *indexes;
-
-    IndexPacket
-      *paint_indexes;
+    register const IndexPacket
+      *restrict indexes;
 
     register const PixelPacket
-      *p;
+      *restrict p;
 
-    register long
-      id,
+    register IndexPacket
+      *restrict paint_indexes;
+
+    register ssize_t
       x;
 
     register PixelPacket
-      *q;
+      *restrict q;
+
+    register size_t
+      *histogram;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],-((long) width/2L),y-(long)
+    p=GetCacheViewVirtualPixels(image_view,-((ssize_t) width/2L),y-(ssize_t)
       (width/2L),image->columns+width,width,exception);
-    q=SetCacheViewPixels(paint_view[id],0,y,paint_image->columns,1);
+    q=QueueCacheViewAuthenticPixels(paint_view,0,y,paint_image->columns,1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    paint_indexes=GetCacheViewIndexes(paint_view[id]);
-    for (x=0; x < (long) image->columns; x++)
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    paint_indexes=GetCacheViewAuthenticIndexQueue(paint_view);
+    histogram=histograms[GetOpenMPThreadId()];
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      long
-        j,
-        k,
-        v;
-
-      register long
+      register ssize_t
         i,
         u;
 
-      unsigned long
+      size_t
         count;
 
+      ssize_t
+        j,
+        k,
+        v;
+
       /*
         Assign most frequent color.
       */
       i=0;
       j=0;
       count=0;
-      (void) ResetMagickMemory(histogram[id],0,NumberPaintBins*
-        sizeof(**histogram));
-      for (v=0; v < (long) width; v++)
+      (void) ResetMagickMemory(histogram,0,NumberPaintBins*sizeof(*histogram));
+      for (v=0; v < (ssize_t) width; v++)
       {
-        for (u=0; u < (long) width; u++)
+        for (u=0; u < (ssize_t) width; u++)
         {
-          k=(long) ScaleQuantumToChar(PixelIntensityToQuantum(p+u+i));
-          histogram[id][k]++;
-          if (histogram[id][k] > count)
+          k=(ssize_t) ScaleQuantumToChar(PixelIntensityToQuantum(p+u+i));
+          histogram[k]++;
+          if (histogram[k] > count)
             {
               j=i+u;
-              count=histogram[id][k];
+              count=histogram[k];
             }
         }
-        i+=image->columns+width;
+        i+=(ssize_t) (image->columns+width);
       }
       *q=(*(p+j));
       if (image->colorspace == CMYKColorspace)
@@ -656,7 +691,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       p++;
       q++;
     }
-    if (SyncCacheViewPixels(paint_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(paint_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -664,16 +699,16 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_OilPaintImage)
 #endif
         proceed=SetImageProgress(image,OilPaintImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  paint_view=DestroyCacheViewThreadSet(paint_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
-  histogram=DestroyHistogramThreadSet(histogram);
+  paint_view=DestroyCacheView(paint_view);
+  image_view=DestroyCacheView(image_view);
+  histograms=DestroyHistogramThreadSet(histograms);
   if (status == MagickFalse)
     paint_image=DestroyImage(paint_image);
   return(paint_image);
@@ -726,7 +761,7 @@ MagickExport MagickBooleanType OpaquePaintImage(Image *image,
   const MagickPixelPacket *target,const MagickPixelPacket *fill,
   const MagickBooleanType invert)
 {
-  return(OpaquePaintImageChannel(image,AllChannels,target,fill,invert));
+  return(OpaquePaintImageChannel(image,CompositeChannels,target,fill,invert));
 }
 
 MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
@@ -735,18 +770,23 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
 {
 #define OpaquePaintImageTag  "Opaque/Image"
 
-  long
-    progress,
-    y;
+  CacheView
+    *image_view;
+
+  ExceptionInfo
+    *exception;
 
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
   MagickPixelPacket
     zero;
 
-  ViewInfo
-    **image_view;
+  ssize_t
+    y;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -761,57 +801,56 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
   */
   status=MagickTrue;
   progress=0;
+  exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,64) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     MagickPixelPacket
       pixel;
 
     register IndexPacket
-      *indexes;
+      *restrict indexes;
 
-    register long
-      id,
+    register ssize_t
       x;
 
     register PixelPacket
-      *q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     pixel=zero;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       SetMagickPixelPacket(image,q,indexes+x,&pixel);
       if (IsMagickColorSimilar(&pixel,target) != invert)
         {
           if ((channel & RedChannel) != 0)
-            q->red=RoundToQuantum(fill->red);
+            q->red=ClampToQuantum(fill->red);
           if ((channel & GreenChannel) != 0)
-            q->green=RoundToQuantum(fill->green);
+            q->green=ClampToQuantum(fill->green);
           if ((channel & BlueChannel) != 0)
-            q->blue=RoundToQuantum(fill->blue);
+            q->blue=ClampToQuantum(fill->blue);
           if ((channel & OpacityChannel) != 0)
-            q->opacity=RoundToQuantum(fill->opacity);
+            q->opacity=ClampToQuantum(fill->opacity);
           if (((channel & IndexChannel) != 0) &&
               (image->colorspace == CMYKColorspace))
-            indexes[x]=RoundToQuantum(fill->index);
+            indexes[x]=ClampToQuantum(fill->index);
         }
       q++;
     }
-    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -819,7 +858,7 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_OpaquePaintImageChannel)
 #endif
         proceed=SetImageProgress(image,OpaquePaintImageTag,progress++,
           image->rows);
@@ -827,7 +866,7 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -874,18 +913,23 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
 {
 #define TransparentPaintImageTag  "Transparent/Image"
 
-  long
-    progress,
-    y;
+  CacheView
+    *image_view;
+
+  ExceptionInfo
+    *exception;
 
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
   MagickPixelPacket
     zero;
 
-  ViewInfo
-    **image_view;
+  ssize_t
+    y;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -901,45 +945,44 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
   */
   status=MagickTrue;
   progress=0;
+  exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,64) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     MagickPixelPacket
       pixel;
 
     register IndexPacket
-      *indexes;
+      *restrict indexes;
 
-    register long
-      id,
+    register ssize_t
       x;
 
     register PixelPacket
-      *q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     pixel=zero;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       SetMagickPixelPacket(image,q,indexes+x,&pixel);
       if (IsMagickColorSimilar(&pixel,target) != invert)
         q->opacity=opacity;
       q++;
     }
-    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -947,7 +990,7 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_TransparentPaintImage)
 #endif
         proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
           image->rows);
@@ -955,6 +998,145 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%     T r a n s p a r e n t P a i n t I m a g e C h r o m a                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransparentPaintImageChroma() changes the opacity value associated with any
+%  pixel that matches color to the value defined by opacity.
+%
+%  As there is one fuzz value for the all the channels, the
+%  TransparentPaintImage() API is not suitable for the operations like chroma,
+%  where the tolerance for similarity of two color component (RGB) can be
+%  different, Thus we define this method take two target pixels (one
+%  low and one hight) and all the pixels of an image which are lying between
+%  these two pixels are made transparent.
+%
+%  The format of the TransparentPaintImage method is:
+%
+%      MagickBooleanType TransparentPaintImage(Image *image,
+%        const MagickPixelPacket *low,const MagickPixelPacket *hight,
+%        const Quantum opacity,const MagickBooleanType invert)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o low: the low target color.
+%
+%    o high: the high target color.
+%
+%    o opacity: the replacement opacity value.
+%
+%    o invert: paint any pixel that does not match the target color.
+%
+*/
+MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
+  const MagickPixelPacket *low,const MagickPixelPacket *high,
+  const Quantum opacity,const MagickBooleanType invert)
+{
+#define TransparentPaintImageTag  "Transparent/Image"
+
+  CacheView
+    *image_view;
+
+  ExceptionInfo
+    *exception;
+
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    progress;
+
+  ssize_t
+    y;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  assert(high != (MagickPixelPacket *) NULL);
+  assert(low != (MagickPixelPacket *) NULL);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+    return(MagickFalse);
+  if (image->matte == MagickFalse)
+    (void) SetImageAlphaChannel(image,ResetAlphaChannel);
+  /*
+    Make image color transparent.
+  */
+  status=MagickTrue;
+  progress=0;
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
+  for (y=0; y < (ssize_t) image->rows; y++)
+  {
+    MagickBooleanType
+      match;
+
+    MagickPixelPacket
+      pixel;
+
+    register IndexPacket
+      *restrict indexes;
+
+    register ssize_t
+      x;
+
+    register PixelPacket
+      *restrict q;
+
+    if (status == MagickFalse)
+      continue;
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
+    if (q == (PixelPacket *) NULL)
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    GetMagickPixelPacket(image,&pixel);
+    for (x=0; x < (ssize_t) image->columns; x++)
+    {
+      SetMagickPixelPacket(image,q,indexes+x,&pixel);
+      match=((pixel.red >= low->red) && (pixel.red <= high->red) &&
+        (pixel.green >= low->green) && (pixel.green <= high->green) &&
+        (pixel.blue  >= low->blue) && (pixel.blue <= high->blue)) ?
+        MagickTrue : MagickFalse;
+      if (match != invert)
+        q->opacity=opacity;
+      q++;
+    }
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_TransparentPaintImageChroma)
+#endif
+        proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
+  }
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
