@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -28,22 +28,14 @@ extern "C" {
 #include <magick/image-private.h>
 #include <magick/quantum-private.h>
 
-typedef struct _RealPixelPacket
-{
-  MagickRealType
-    red,
-    green,
-    blue,
-    opacity;
-} RealPixelPacket;
-
 static inline MagickPixelPacket *CloneMagickPixelPacket(
   const MagickPixelPacket *pixel)
 {
   MagickPixelPacket
     *clone_pixel;
 
-  clone_pixel=(MagickPixelPacket *) AcquireMagickMemory(sizeof(*clone_pixel));
+  clone_pixel=(MagickPixelPacket *) AcquireAlignedMemory(1,
+    sizeof(*clone_pixel));
   if (clone_pixel == (MagickPixelPacket *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   *clone_pixel=(*pixel);
@@ -86,23 +78,33 @@ static inline void SetMagickPixelPacket(const Image *image,
   pixel->green=(MagickRealType) color->green;
   pixel->blue=(MagickRealType) color->blue;
   pixel->opacity=(MagickRealType) color->opacity;
-  if (((image->colorspace == CMYKColorspace) ||
-       (image->storage_class == PseudoClass)) &&
+  if ((image->colorspace == CMYKColorspace) &&
       (index != (const IndexPacket *) NULL))
     pixel->index=(MagickRealType) *index;
 }
 
+/* This function is obsoleted by MorphologyApply() -- Anthony Thyssen */
+static inline void SetMagickPixelPacketBias(const Image *image,
+  MagickPixelPacket *pixel)
+{
+  pixel->red=image->bias;
+  pixel->green=image->bias;
+  pixel->blue=image->bias;
+  pixel->opacity=image->bias;
+  pixel->index=image->bias;
+}
+
 static inline void SetPixelPacket(const Image *image,
   const MagickPixelPacket *pixel,PixelPacket *color,IndexPacket *index)
 {
-  color->red=RoundToQuantum(pixel->red);
-  color->green=RoundToQuantum(pixel->green);
-  color->blue=RoundToQuantum(pixel->blue);
-  color->opacity=RoundToQuantum(pixel->opacity);
+  color->red=ClampToQuantum(pixel->red);
+  color->green=ClampToQuantum(pixel->green);
+  color->blue=ClampToQuantum(pixel->blue);
+  color->opacity=ClampToQuantum(pixel->opacity);
   if (((image->colorspace == CMYKColorspace) ||
        (image->storage_class == PseudoClass)) &&
       (index != (const IndexPacket *) NULL))
-    *index=RoundToQuantum(pixel->index);
+    *index=ClampToQuantum(pixel->index);
 }
 
 #if defined(__cplusplus) || defined(c_plusplus)
