@@ -17,7 +17,7 @@
 %                                October 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -79,17 +79,20 @@
 */
 MagickExport double GetImageTotalInkDensity(Image *image)
 {
+  CacheView
+    *image_view;
+
   double
     total_ink_density;
 
-  long
-    y;
+  ExceptionInfo
+    *exception;
 
   MagickBooleanType
     status;
 
-  ViewInfo
-    **image_view;
+  ssize_t
+    y;
 
   assert(image != (Image *) NULL);
   if (image->debug != MagickFalse)
@@ -103,11 +106,12 @@ MagickExport double GetImageTotalInkDensity(Image *image)
     }
   status=MagickTrue;
   total_ink_density=0.0;
-  image_view=AcquireCacheViewThreadSet(image);
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,64) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     double
       density;
@@ -118,25 +122,22 @@ MagickExport double GetImageTotalInkDensity(Image *image)
     register const PixelPacket
       *p;
 
-    register long
-      id,
+    register ssize_t
       x;
 
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,
-      &image->exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    for (x=0; x < (long) image->columns; x++)
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      density=(double) p->red+p->green+p->blue+indexes[x];
+      density=(double) GetRedPixelComponent(p)+GetGreenPixelComponent(p)+GetBluePixelComponent(p)+indexes[x];
       if (density > total_ink_density)
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_GetImageTotalInkDensity)
 #endif
         {
           if (density > total_ink_density)
@@ -145,7 +146,7 @@ MagickExport double GetImageTotalInkDensity(Image *image)
       p++;
     }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   if (status == MagickFalse)
     total_ink_density=0.0;
   return(total_ink_density);
