@@ -16,7 +16,7 @@
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -56,6 +56,8 @@
 #include "magick/resource_.h"
 #include "magick/signature-private.h"
 #include "magick/string_.h"
+#include "magick/thread_.h"
+#include "magick/thread-private.h"
 #include "magick/utility.h"
 /*
   Define declarations.
@@ -82,6 +84,12 @@ struct _RandomInfo
   size_t
     i;
 
+  unsigned long
+    seed[4];
+
+  double
+    normalize;
+
   unsigned short
     protocol_major,
     protocol_minor;
@@ -89,17 +97,17 @@ struct _RandomInfo
   SemaphoreInfo
     *semaphore;
 
-  long
+  ssize_t
     timestamp;
 
-  unsigned long
+  size_t
     signature;
 };
 
 /*
   External declarations.
 */
-#if defined(__APPLE__)
+#if defined(__APPLE__) && !defined(TARGET_OS_IPHONE)
 #include <crt_externs.h>
 #define environ (*_NSGetEnviron())
 #endif
@@ -114,7 +122,7 @@ static SemaphoreInfo
   *random_semaphore = (SemaphoreInfo *) NULL;
 
 static unsigned long
-  random_seed[4] = { ~0UL, 0x50a7f451UL, 0x5365417eUL, 0xc3a4171aUL };
+  random_seed = ~0UL;
 
 static MagickBooleanType
   gather_true_random = MagickFalse;
@@ -143,56 +151,99 @@ static StringInfo
 %      RandomInfo *AcquireRandomInfo(void)
 %
 */
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 MagickExport RandomInfo *AcquireRandomInfo(void)
 {
+  const StringInfo
+    *digest;
+
   RandomInfo
     *random_info;
 
   StringInfo
-    *entropy;
+    *entropy,
+    *key,
+    *nonce;
 
   random_info=(RandomInfo *) AcquireMagickMemory(sizeof(*random_info));
   if (random_info == (RandomInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) ResetMagickMemory(random_info,0,sizeof(*random_info));
   random_info->signature_info=AcquireSignatureInfo();
-  random_info->nonce=AcquireStringInfo(GetSignatureDigestsize(
+  random_info->nonce=AcquireStringInfo(2*GetSignatureDigestsize(
     random_info->signature_info));
   ResetStringInfo(random_info->nonce);
   random_info->reservoir=AcquireStringInfo(GetSignatureDigestsize(
     random_info->signature_info));
   ResetStringInfo(random_info->reservoir);
+  random_info->normalize=1.0/(~0UL);
   random_info->semaphore=AllocateSemaphoreInfo();
   random_info->protocol_major=RandomProtocolMajorVersion;
   random_info->protocol_minor=RandomProtocolMinorVersion;
-  random_info->timestamp=(long) time(0);
+  random_info->timestamp=(ssize_t) time(0);
   random_info->signature=MagickSignature;
+  /*
+    Seed random nonce.
+  */
+  nonce=GenerateEntropicChaos(random_info);
+  if (nonce == (StringInfo *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  InitializeSignature(random_info->signature_info);
+  UpdateSignature(random_info->signature_info,nonce);
+  FinalizeSignature(random_info->signature_info);
+  SetStringInfoLength(nonce,(GetSignatureDigestsize(
+    random_info->signature_info)+1)/2);
+  SetStringInfo(nonce,GetSignatureDigest(random_info->signature_info));
+  SetStringInfo(random_info->nonce,nonce);
+  nonce=DestroyStringInfo(nonce);
   /*
     Seed random reservoir with entropic data.
   */
   entropy=GenerateEntropicChaos(random_info);
   if (entropy == (StringInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  if (GetStringInfoLength(entropy) > GetStringInfoLength(random_info->nonce))
-    {
-      StringInfo
-        *nonce;
-
-      nonce=SplitStringInfo(entropy,GetStringInfoLength(random_info->nonce)/2);
-      if (nonce != (StringInfo *) NULL)
-        {
-          /*
-            Use some of the entropy to set a random nonce.
-          */
-          SetStringInfo(random_info->nonce,nonce);
-          nonce=DestroyStringInfo(nonce);
-        }
-    }
   UpdateSignature(random_info->signature_info,entropy);
   FinalizeSignature(random_info->signature_info);
   SetStringInfo(random_info->reservoir,GetSignatureDigest(
     random_info->signature_info));
   entropy=DestroyStringInfo(entropy);
+  /*
+    Seed pseudo random number generator.
+  */
+  if (random_seed == ~0UL)
+    {
+      key=GetRandomKey(random_info,sizeof(random_seed));
+      (void) CopyMagickMemory(random_info->seed,GetStringInfoDatum(key),
+        GetStringInfoLength(key));
+      key=DestroyStringInfo(key);
+    }
+  else
+    {
+      SignatureInfo
+        *signature_info;
+
+      signature_info=AcquireSignatureInfo();
+      key=AcquireStringInfo(sizeof(random_seed));
+      SetStringInfoDatum(key,(unsigned char *) &random_seed);
+      UpdateSignature(signature_info,key);
+      key=DestroyStringInfo(key);
+      FinalizeSignature(signature_info);
+      digest=GetSignatureDigest(signature_info);
+      (void) CopyMagickMemory(random_info->seed,GetStringInfoDatum(digest),
+        MagickMin(GetSignatureDigestsize(signature_info),
+        sizeof(*random_info->seed)));
+      signature_info=DestroySignatureInfo(signature_info);
+    }
+  random_info->seed[1]=0x50a7f451UL;
+  random_info->seed[2]=0x5365417eUL;
+  random_info->seed[3]=0xc3a4171aUL;
   return(random_info);
 }
 
@@ -224,7 +275,7 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(random_info != (RandomInfo *) NULL);
   assert(random_info->signature == MagickSignature);
-  (void) LockSemaphoreInfo(random_info->semaphore);
+  LockSemaphoreInfo(random_info->semaphore);
   if (random_info->reservoir != (StringInfo *) NULL)
     random_info->reservoir=DestroyStringInfo(random_info->reservoir);
   if (random_info->nonce != (StringInfo *) NULL)
@@ -232,8 +283,9 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
   if (random_info->signature_info != (SignatureInfo *) NULL)
     random_info->signature_info=DestroySignatureInfo(
       random_info->signature_info);
+  (void) ResetMagickMemory(random_info->seed,0,sizeof(*random_info->seed));
   random_info->signature=(~MagickSignature);
-  (void) UnlockSemaphoreInfo(random_info->semaphore);
+  UnlockSemaphoreInfo(random_info->semaphore);
   DestroySemaphoreInfo(&random_info->semaphore);
   random_info=(RandomInfo *) RelinquishMagickMemory(random_info);
   return(random_info);
@@ -244,32 +296,6 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   D e s t r o y R a n d o m R e s e r v i o r                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DestroyRandomReservoir() deallocates memory associated with the random
-%  reservoir.
-%
-%  The format of the DestroyRandomReservoir method is:
-%
-%      DestroyRandomReservoir(void)
-%
-*/
-MagickExport void DestroyRandomReservoir(void)
-{
-  AcquireSemaphoreInfo(&random_semaphore);
-  (void) UnlockSemaphoreInfo(random_semaphore);
-  DestroySemaphoreInfo(&random_semaphore);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 +   G e n e r a t e E n t r o p i c C h a o s                                 %
 %                                                                             %
 %                                                                             %
@@ -289,6 +315,7 @@ MagickExport void DestroyRandomReservoir(void)
 %
 */
 
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT)
 static ssize_t ReadRandom(int file,unsigned char *source,size_t length)
 {
   register unsigned char
@@ -314,36 +341,56 @@ static ssize_t ReadRandom(int file,unsigned char *source,size_t length)
   }
   return(offset);
 }
+#endif
 
 static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 {
 #define MaxEntropyExtent  64
 
-  long
-    pid;
+  MagickThreadType
+    tid;
 
   StringInfo
     *chaos,
     *entropy;
 
-  unsigned long
+  size_t
     nanoseconds,
     seconds;
 
+  ssize_t
+    pid;
+
   /*
     Initialize random reservoir.
   */
   entropy=AcquireStringInfo(0);
-  (void) LockSemaphoreInfo(random_info->semaphore);
+  LockSemaphoreInfo(random_info->semaphore);
   chaos=AcquireStringInfo(sizeof(unsigned char *));
   SetStringInfoDatum(chaos,(unsigned char *) &entropy);
   ConcatenateStringInfo(entropy,chaos);
   SetStringInfoDatum(chaos,(unsigned char *) entropy);
   ConcatenateStringInfo(entropy,chaos);
-  pid=(long) getpid();
+  pid=(ssize_t) getpid();
   SetStringInfoLength(chaos,sizeof(pid));
   SetStringInfoDatum(chaos,(unsigned char *) &pid);
   ConcatenateStringInfo(entropy,chaos);
+  tid=GetMagickThreadId();
+  SetStringInfoLength(chaos,sizeof(tid));
+  SetStringInfoDatum(chaos,(unsigned char *) &tid);
+  ConcatenateStringInfo(entropy,chaos);
+#if defined(MAGICKCORE_HAVE_GETRUSAGE) && defined(RUSAGE_SELF)
+  {
+    struct rusage
+      usage;
+
+    if (getrusage(RUSAGE_SELF,&usage) == 0)
+      {
+        SetStringInfoLength(chaos,sizeof(usage));
+        SetStringInfoDatum(chaos,(unsigned char *) &usage);
+      }
+  }
+#endif
   seconds=time((time_t *) 0);
   nanoseconds=0;
 #if defined(MAGICKCORE_HAVE_GETTIMEOFDAY)
@@ -351,19 +398,19 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     struct timeval
       timer;
 
-    if (gettimeofday(&timer,0) == 0)
+    if (gettimeofday(&timer,(struct timezone *) NULL) == 0)
       {
         seconds=timer.tv_sec;
         nanoseconds=1000UL*timer.tv_usec;
       }
   }
 #endif
-#if defined(MAGICKCORE_HAVE_CLOCK_GETTIME) && defined(CLOCK_HIGHRES)
+#if defined(MAGICKCORE_HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME_HR)
   {
     struct timespec
       timer;
 
-    if (clock_gettime(CLOCK_HIGHRES,&timer) == 0)
+    if (clock_gettime(CLOCK_REALTIME_HR,&timer) == 0)
       {
         seconds=timer.tv_sec;
         nanoseconds=timer.tv_nsec;
@@ -402,6 +449,9 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 
     filename=ConstantString("magickXXXXXX");
     file=mkstemp(filename);
+#if defined(__OS2__)
+    setmode(file,O_BINARY);
+#endif
     if (file != -1)
       (void) close(file);
     (void) remove(filename);
@@ -411,7 +461,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     filename=DestroyString(filename);
   }
 #endif
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
     double
       seconds;
@@ -461,7 +511,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     */
     if (environ != (char **) NULL)
       {
-        register long
+        register ssize_t
           i;
 
         /*
@@ -516,7 +566,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
   }
 #endif
   chaos=DestroyStringInfo(chaos);
-  (void) UnlockSemaphoreInfo(random_info->semaphore);
+  UnlockSemaphoreInfo(random_info->semaphore);
   return(entropy);
 }
 
@@ -537,34 +587,31 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 %
 %  The format of the GetPseudoRandomValue method is:
 %
-%      double GetPseudoRandomValue(void)
+%      double GetPseudoRandomValue(RandomInfo *randon_info)
+%
+%  A description of each parameter follows:
+%
+%    o random_info: the random info.
 %
 */
-MagickExport double GetPseudoRandomValue(void)
+MagickExport double GetPseudoRandomValue(RandomInfo *random_info)
 {
-  double
-    random;
+  register unsigned long
+    *seed;
 
   unsigned long
-    alpha,
-    range;
+    alpha;
 
-  AcquireSemaphoreInfo(&random_semaphore);
-  if (random_seed[0] == ~0UL)
-    SeedPseudoRandomGenerator(~0UL);
-  range=(~0UL);
+  seed=random_info->seed;
   do
   {
-    alpha=(unsigned long) (random_seed[1] ^ (random_seed[1] << 11));
-    random_seed[1]=random_seed[2];
-    random_seed[2]=random_seed[3];
-    random_seed[3]=random_seed[0];
-    random_seed[0]=(random_seed[0] ^ (random_seed[0] >> 19)) ^ (alpha ^
-      (alpha >> 8));
-  } while (random_seed[0] == range);
-  random=(double) random_seed[0]/range;
-  (void) UnlockSemaphoreInfo(random_semaphore);
-  return(random);
+    alpha=(unsigned long) (seed[1] ^ (seed[1] << 11));
+    seed[1]=seed[2];
+    seed[2]=seed[3];
+    seed[3]=seed[0];
+    seed[0]=(seed[0] ^ (seed[0] >> 19)) ^ (alpha ^ (alpha >> 8));
+  } while (seed[0] == ~0UL);
+  return(random_info->normalize*seed[0]);
 }
 
 /*
@@ -642,6 +689,55 @@ MagickExport double GetRandomValue(RandomInfo *random_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   R a n d o m C o m p o n e n t G e n e s i s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  RandomComponentGenesis() instantiates the random component.
+%
+%  The format of the RandomComponentGenesis method is:
+%
+%      MagickBooleanType RandomComponentGenesis(void)
+%
+*/
+MagickExport MagickBooleanType RandomComponentGenesis(void)
+{
+  AcquireSemaphoreInfo(&random_semaphore);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   R a n d o m C o m p o n e n t T e r m i n u s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  RandomComponentTerminus() destroys the random component.
+%
+%  The format of the RandomComponentTerminus method is:
+%
+%      RandomComponentTerminus(void)
+%
+*/
+MagickExport void RandomComponentTerminus(void)
+{
+  if (random_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&random_semaphore);
+  DestroySemaphoreInfo(&random_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S e e d P s e u d o R a n d o m G e n e r a t o r                         %
 %                                                                             %
 %                                                                             %
@@ -660,59 +756,9 @@ MagickExport double GetRandomValue(RandomInfo *random_info)
 %    o seed: the seed.
 %
 */
-
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 MagickExport void SeedPseudoRandomGenerator(const unsigned long seed)
 {
-  const StringInfo
-    *digest;
-
-  SignatureInfo
-    *signature_info;
-
-  StringInfo
-    *key;
-
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  if (seed == ~0UL)
-    {
-      RandomInfo
-        *random_info;
-
-      StringInfo
-        *key;
-
-
-      /*
-        Initialize pseudo random number generator with random seed.
-      */
-      random_info=AcquireRandomInfo();
-      key=GetRandomKey(random_info,sizeof(random_seed));
-      (void) CopyMagickMemory(random_seed,GetStringInfoDatum(key),
-        GetStringInfoLength(key));
-      key=DestroyStringInfo(key);
-      random_info=DestroyRandomInfo(random_info);
-      return;
-    }
-  /*
-    Initialize pseudo random number generator with a user-supplied seed.
-  */
-  signature_info=AcquireSignatureInfo();
-  key=AcquireStringInfo(sizeof(seed));
-  SetStringInfoDatum(key,(unsigned char *) &seed);
-  UpdateSignature(signature_info,key);
-  key=DestroyStringInfo(key);
-  FinalizeSignature(signature_info);
-  digest=GetSignatureDigest(signature_info);
-  (void) CopyMagickMemory(random_seed,GetStringInfoDatum(digest),
-    MagickMin(GetSignatureDigestsize(signature_info),sizeof(random_seed)));
-  signature_info=DestroySignatureInfo(signature_info);
+  random_seed=seed;
 }
 
 /*
@@ -745,14 +791,14 @@ MagickExport void SeedPseudoRandomGenerator(const unsigned long seed)
 
 static inline void IncrementRandomNonce(StringInfo *nonce)
 {
-  register long
+  register ssize_t
     i;
 
   unsigned char
     *datum;
 
   datum=GetStringInfoDatum(nonce);
-  for (i=(long) (GetStringInfoLength(nonce)-1); i != 0; i--)
+  for (i=(ssize_t) (GetStringInfoLength(nonce)-1); i != 0; i--)
   {
     datum[i]++;
     if (datum[i] != 0)
@@ -779,7 +825,7 @@ MagickExport void SetRandomKey(RandomInfo *random_info,const size_t length,
   assert(random_info != (RandomInfo *) NULL);
   if (length == 0)
     return;
-  (void) LockSemaphoreInfo(random_info->semaphore);
+  LockSemaphoreInfo(random_info->semaphore);
   signature_info=random_info->signature_info;
   datum=GetStringInfoDatum(random_info->reservoir);
   i=length;
@@ -813,7 +859,7 @@ MagickExport void SetRandomKey(RandomInfo *random_info,const size_t length,
       while (i-- != 0)
         p[i]=datum[i];
     }
-  (void) UnlockSemaphoreInfo(random_info->semaphore);
+  UnlockSemaphoreInfo(random_info->semaphore);
 }
 
 /*
