@@ -1,12 +1,12 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
-  
+
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
-  
+
     http://www.imagemagick.org/script/license.php
-  
+
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -22,38 +22,10 @@
 extern "C" {
 #endif
 
-#if defined(__CYGWIN32__)
-#  if !defined(__CYGWIN__)
-#    define __CYGWIN__ __CYGWIN32__
-#  endif
-#endif
-
-#if defined(_WIN32) || defined(WIN32)
-#  if !defined(__WINDOWS__)
-#    if defined(_WIN32)
-#      define __WINDOWS__ _WIN32
-#    else
-#      if defined(WIN32)
-#        define __WINDOWS__ WIN32
-#      endif
-#    endif
-#  endif
-#endif
-
-#if defined(_WIN64) || defined(WIN64)
-#  if !defined(__WINDOWS__)
-#    if defined(_WIN64)
-#      define __WINDOWS__ _WIN64
-#    else
-#      if defined(WIN64)
-#        define __WINDOWS__ WIN64
-#      endif
-#    endif
-#  endif
-#endif
-
-#if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
-# define MAGICKCORE_POSIX_SUPPORT
+#if defined(WIN32) || defined(WIN64)
+#  define MAGICKCORE_WINDOWS_SUPPORT
+#else
+#  define MAGICKCORE_POSIX_SUPPORT
 #endif
 
 #define MAGICKCORE_IMPLEMENTATION  1
@@ -69,14 +41,17 @@ extern "C" {
 # define _FILE_OFFSET_BITS MAGICKCORE__FILE_OFFSET_BITS
 #endif
 #if defined(_magickcore_const) && !defined(const)
-# define const _magickcore_const
+# define const  _magickcore_const
 #endif
 #if defined(_magickcore_inline) && !defined(inline)
-# define inline _magickcore_inline
+# define inline  _magickcore_inline
 #endif
 # if defined(__cplusplus) || defined(c_plusplus)
 #  undef inline
 # endif
+#if defined(_magickcore_restrict) && !defined(restrict)
+# define restrict  _magickcore_restrict
+#endif
 #endif
 
 #if defined(MAGICKCORE_NAMESPACE_PREFIX)
@@ -95,7 +70,7 @@ extern "C" {
 #  undef MAGICKCORE_BUILD_MODULES
 #endif
 
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__CYGWIN__) && !defined(__MINGW32__)
 # if defined(_MT) && defined(_DLL) && !defined(_MAGICKDLL_) && !defined(_LIB)
 #  define _MAGICKDLL_
 # endif
@@ -156,23 +131,45 @@ extern "C" {
 
 #include <stdarg.h>
 #include <stdio.h>
-#if defined(__WINDOWS__) && defined(_DEBUG)
+#if defined(MAGICKCORE_HAVE_SYS_STAT_H)
+# include <sys/stat.h>
+#endif
+#if defined(MAGICKCORE_STDC_HEADERS)
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# if defined(MAGICKCORE_HAVE_STDLIB_H)
+#  include <stdlib.h>
+# endif
+#endif
+#if defined(MAGICKCORE_HAVE_STRING_H)
+# if !defined(STDC_HEADERS) && defined(MAGICKCORE_HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#if defined(MAGICKCORE_HAVE_STRINGS_H)
+# include <strings.h>
+#endif
+#if defined(MAGICKCORE_HAVE_INTTYPES_H)
+# include <inttypes.h>
+#endif
+#if defined(MAGICKCORE_HAVE_STDINT_H)
+# include <stdint.h>
+#endif
+#if defined(MAGICKCORE_HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_DEBUG)
 #define _CRTDBG_MAP_ALLOC
 #endif
-#include <stdlib.h>
-#if !defined(__WINDOWS__)
-# include <unistd.h>
-#else
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
 # include <direct.h>
 # if !defined(MAGICKCORE_HAVE_STRERROR)
 #  define HAVE_STRERROR
 # endif
 #endif
 
-#if defined(MAGICKCORE_HAVE_STRINGS_H)
-# include <strings.h>
-#endif
-#include <string.h>
 #include <ctype.h>
 #include <locale.h>
 #include <errno.h>
@@ -183,8 +180,11 @@ extern "C" {
 #include <signal.h>
 #include <assert.h>
 
-#if defined(MAGICKCORE_HAVE_PTHREAD )
+#if defined(MAGICKCORE_THREAD_SUPPORT)
 # include <pthread.h>
+#elif defined(MAGICKCORE_WINDOWS_SUPPORT)
+#  define MAGICKCORE_HAVE_WINTHREADS  1
+#include <windows.h>
 #endif
 #if defined(MAGICKCORE_HAVE_SYS_SYSLIMITS_H)
 # include <sys/syslimits.h>
@@ -193,9 +193,19 @@ extern "C" {
 # include <arm/limits.h>
 #endif
 
+#if defined(MAGICKCORE__OPENCL)
+#if defined(MAGICKCORE_HAVE_CL_CL_H)
+#  include <CL/cl.h>
+#endif
+#if defined(MAGICKCORE_HAVE_OPENCL_CL_H)
+#  include <OpenCL/cl.h>
+#endif
+#  define MAGICKCORE_OPENCL_SUPPORT  1
+#endif
+
 #if defined(_OPENMP) && (_OPENMP >= 200203)
 #  include <omp.h>
-#  define MAGICKCORE_OPENMP_SUPPORT 1
+#  define MAGICKCORE_OPENMP_SUPPORT  1
 #endif
 
 #if defined(MAGICKCORE_HAVE_PREAD) && defined(MAGICKCORE_HAVE_DECL_PREAD) && !MAGICKCORE_HAVE_DECL_PREAD
@@ -214,25 +224,17 @@ extern size_t strlcpy(char *,const char *,size_t);
 extern int vsnprintf(char *,size_t,const char *,va_list);
 #endif
 
-#if !defined(magick_attribute)
-#  if !defined(__GNUC__)
-#    define magick_attribute(x)  /* nothing */
-#  else
-#    define magick_attribute  __attribute__
-#  endif
+#if defined(MAGICKCORE_HAVE___ATTRIBUTE__)
+#  define magick_aligned(x)  __attribute__((aligned(x)))
+#  define magick_attribute  __attribute__
+#  define magick_unused(x)  magick_unused_ ## x __attribute__((unused))
+#else
+#  define magick_aligned(x)  /* nothing */
+#  define magick_attribute(x)  /* nothing */
+#  define magick_unused(x) x
 #endif
 
-#if !defined(magick_unused)
-#  if defined(__GNUC__)
-#     define magick_unused(x)  magick_unused_ ## x __attribute__((unused))
-#  elif defined(__LCLINT__)
-#    define magick_unused(x) /*@unused@*/ x
-#  else
-#    define magick_unused(x) x
-#  endif
-#endif
-
-#if defined(__WINDOWS__) || defined(MAGICKCORE_POSIX_SUPPORT)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(MAGICKCORE_POSIX_SUPPORT)
 # include <sys/types.h>
 # include <sys/stat.h>
 # if defined(MAGICKCORE_HAVE_FTIME)
@@ -265,7 +267,7 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
 # endif
 # include "magick/magick-type.h"
-# if !defined(__WINDOWS__)
+# if !defined(MAGICKCORE_WINDOWS_SUPPORT)
 #  include <sys/time.h>
 # if defined(MAGICKCORE_HAVE_SYS_TIMES_H)
 #  include <sys/times.h>
@@ -289,13 +291,13 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 
 #if defined(S_IRUSR) && defined(S_IWUSR)
 # define S_MODE (S_IRUSR | S_IWUSR)
-#elif defined (__WINDOWS__)
+#elif defined (MAGICKCORE_WINDOWS_SUPPORT)
 # define S_MODE (_S_IREAD | _S_IWRITE)
 #else
 # define S_MODE  0600
 #endif
 
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
 # include "magick/nt-base.h"
 #endif
 #if defined(macintosh)
@@ -305,13 +307,16 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 # include "magick/vms.h"
 #endif
 
+#undef HAVE_CONFIG_H
+#undef gamma
 #undef index
 #undef pipe
+#undef y1
 
 /*
   Review these platform specific definitions.
 */
-#if defined(MAGICKCORE_POSIX_SUPPORT)
+#if defined(MAGICKCORE_POSIX_SUPPORT) && !defined(__OS2__)
 # define DirectorySeparator  "/"
 # define DirectoryListSeparator  ':'
 # define EditorOptions  " -title \"Edit Image Comment\" -e vi"
@@ -339,6 +344,18 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers
 # endif
+# if defined(__OS2__)
+#   define DirectorySeparator  "\\"
+#   define DirectoryListSeparator  ';'
+# define EditorOptions  " -title \"Edit Image Comment\" -e vi"
+# define Exit  exit
+#  define IsBasenameSeparator(c) \
+  (((c) == '/') || ((c) == '\\') ? MagickTrue : MagickFalse)
+# define PreferencesDefaults  "~\."
+# define ProcessPendingEvents(text)
+# define ReadCommandlLine(argc,argv)
+# define SetNotifyHandlers
+#endif
 # if defined(macintosh)
 #  define X11_APPLICATION_PATH  "/usr/lib/X11/app-defaults/"
 #  define DirectorySeparator  ":"
@@ -363,7 +380,7 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
      SetWarningHandler(MACWarningHandler)
 #  endif
 # endif
-# if defined(__WINDOWS__)
+# if defined(MAGICKCORE_WINDOWS_SUPPORT)
 #  define DirectorySeparator  "\\"
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions ""
@@ -397,7 +414,11 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #define O_BINARY  0x00
 #endif
 
-#if defined(MAGICKCORE_LTDL_DELEGATE) || (defined(__WINDOWS__) && defined(_DLL) && !defined(_LIB))
+#if !defined(PATH_MAX)
+#define PATH_MAX  4096
+#endif
+
+#if defined(MAGICKCORE_LTDL_DELEGATE) || (defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_DLL) && !defined(_LIB))
 #  define MAGICKCORE_MODULES_SUPPORT
 #endif
 
@@ -406,17 +427,6 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 # define MAGICKCORE_BUILD_MODULES
 #endif
 
-/*
-  I/O defines.
-*/
-#if defined(__WINDOWS__) && !defined(Windows95) && !defined(__BORLANDC__)
-#define MagickSeek(file,offset,whence)  _lseeki64(file,offset,whence)
-#define MagickTell(file)  _telli64(file)
-#else
-#define MagickSeek(file,offset,whence)  lseek(file,offset,whence)
-#define MagickTell(file) tell(file)
-#endif
-
 /*
   Magick defines.
 */
