@@ -23,7 +23,7 @@
 %                               December 2004                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,8 @@
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
+#include "magick/token-private.h"
 #include "magick/xml-tree.h"
 #include "magick/utility.h"
 
@@ -90,7 +92,7 @@ struct _XMLTreeInfo
   SemaphoreInfo
     *semaphore;
 
-  unsigned long
+  size_t
     signature;
 };
 
@@ -119,7 +121,7 @@ struct _XMLTreeRoot
   SemaphoreInfo
     *semaphore;
 
-  unsigned long
+  size_t
     signature;
 };
 
@@ -213,19 +215,19 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
     subnode[MaxTextExtent],
     tag[MaxTextExtent];
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
+  size_t
+    number_components;
+
+  ssize_t
+    j;
+
   XMLTreeInfo
     *child,
     *node;
 
-  unsigned long
-    number_components;
-
   assert(xml_info != (XMLTreeInfo *) NULL);
   assert((xml_info->signature == MagickSignature) ||
          (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
@@ -234,7 +236,7 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
   components=GetPathComponents(path,&number_components);
   if (components == (char **) NULL)
     return((XMLTreeInfo *) NULL);
-  for (i=0; i < (long) number_components; i++)
+  for (i=0; i < (ssize_t) number_components; i++)
   {
     GetPathComponent(components[i],SubimagePath,subnode);
     GetPathComponent(components[i],CanonicalPath,tag);
@@ -244,7 +246,7 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
     node=child;
     if (node == (XMLTreeInfo *) NULL)
       break;
-    for (j=atol(subnode)-1; j > 0; j--)
+    for (j=(ssize_t) StringToLong(subnode)-1; j > 0; j--)
     {
       node=GetXMLTreeOrdered(node);
       if (node == (XMLTreeInfo *) NULL)
@@ -254,7 +256,7 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
       break;
     components[i]=DestroyString(components[i]);
   }
-  for ( ; i < (long) number_components; i++)
+  for ( ; i < (ssize_t) number_components; i++)
     components[i]=DestroyString(components[i]);
   components=(char **) RelinquishMagickMemory(components);
   return(node);
@@ -289,48 +291,6 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
 %      entities.
 %
 */
-
-static unsigned char *ConvertLatin1ToUTF8(const unsigned char *content)
-{
-  register const unsigned char
-    *p;
-
-  register unsigned char
-    *q;
-
-  size_t
-    length;
-
-  unsigned char
-    *utf8;
-
-  unsigned int
-    c;
-
-  length=0;
-  for (p=content; *p != '\0'; p++)
-    length+=(*p & 0x80) != 0 ? 2 : 1;
-  utf8=(unsigned char *) NULL;
-  if (~length >= 1)
-    utf8=(unsigned char *) AcquireQuantumMemory(length+1UL,sizeof(*utf8));
-  if (utf8 == (unsigned char *) NULL)
-    return((unsigned char *) NULL);
-  q=utf8;
-  for (p=content; *p != '\0'; p++)
-  {
-    c=(*p);
-    if ((c & 0x80) == 0)
-      *q++=c;
-    else
-      {
-        *q++=0xc0 | ((c >> 6) & 0x3f);
-        *q++=0x80 | (c & 0x3f);
-      }
-  }
-  *q='\0';
-  return(utf8);
-}
-
 MagickExport char *CanonicalXMLContent(const char *content,
   const MagickBooleanType pedantic)
 {
@@ -341,7 +301,7 @@ MagickExport char *CanonicalXMLContent(const char *content,
   register const unsigned char
     *p;
 
-  register long
+  register ssize_t
     i;
 
   size_t
@@ -380,7 +340,7 @@ MagickExport char *CanonicalXMLContent(const char *content,
   extent=MaxTextExtent;
   for (p=utf8; *p != '\0'; p++)
   {
-    if ((i+MaxTextExtent) > (long) extent)
+    if ((i+MaxTextExtent) > (ssize_t) extent)
       {
         extent+=MaxTextExtent;
         canonical_content=(char *) ResizeQuantumMemory(canonical_content,extent,
@@ -472,7 +432,7 @@ MagickExport char *CanonicalXMLContent(const char *content,
 
 static char **DestroyXMLTreeAttributes(char **attributes)
 {
-  register long
+  register ssize_t
     i;
 
   /*
@@ -499,12 +459,12 @@ MagickExport XMLTreeInfo *DestroyXMLTree(XMLTreeInfo *xml_info)
   char
     **attributes;
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
+  ssize_t
+    j;
+
   XMLTreeRoot
     *root;
 
@@ -625,12 +585,12 @@ MagickExport XMLTreeInfo *GetNextXMLTreeTag(XMLTreeInfo *xml_info)
 MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
   const char *tag)
 {
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
+  ssize_t
+    j;
+
   XMLTreeRoot
     *root;
 
@@ -693,12 +653,12 @@ MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
 MagickExport MagickBooleanType GetXMLTreeAttributes(const XMLTreeInfo *xml_info,
   SplayTreeInfo *attributes)
 {
-  register long
+  register ssize_t
     i;
 
   assert(xml_info != (XMLTreeInfo *) NULL);
   assert((xml_info->signature == MagickSignature) ||
-         (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
+         (((const XMLTreeRoot *) xml_info)->signature == MagickSignature));
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(attributes != (SplayTreeInfo *) NULL);
   if (xml_info->attributes == (char **) NULL)
@@ -848,18 +808,18 @@ MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
     subnode[MaxTextExtent],
     tag[MaxTextExtent];
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
+  size_t
+    number_components;
+
+  ssize_t
+    j;
+
   XMLTreeInfo
     *node;
 
-  unsigned long
-    number_components;
-
   assert(xml_info != (XMLTreeInfo *) NULL);
   assert((xml_info->signature == MagickSignature) ||
          (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
@@ -868,14 +828,14 @@ MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
   components=GetPathComponents(path,&number_components);
   if (components == (char **) NULL)
     return((XMLTreeInfo *) NULL);
-  for (i=0; i < (long) number_components; i++)
+  for (i=0; i < (ssize_t) number_components; i++)
   {
     GetPathComponent(components[i],SubimagePath,subnode);
     GetPathComponent(components[i],CanonicalPath,tag);
     node=GetXMLTreeChild(node,tag);
     if (node == (XMLTreeInfo *) NULL)
       break;
-    for (j=atol(subnode)-1; j > 0; j--)
+    for (j=(ssize_t) StringToLong(subnode)-1; j > 0; j--)
     {
       node=GetXMLTreeOrdered(node);
       if (node == (XMLTreeInfo *) NULL)
@@ -885,7 +845,7 @@ MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
       break;
     components[i]=DestroyString(components[i]);
   }
-  for ( ; i < (long) number_components; i++)
+  for ( ; i < (ssize_t) number_components; i++)
     components[i]=DestroyString(components[i]);
   components=(char **) RelinquishMagickMemory(components);
   return(node);
@@ -918,7 +878,7 @@ MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
 MagickExport const char **GetXMLTreeProcessingInstructions(
   XMLTreeInfo *xml_info,const char *target)
 {
-  register long
+  register ssize_t
     i;
 
   XMLTreeRoot
@@ -1119,7 +1079,7 @@ MagickExport XMLTreeInfo *InsertTagIntoXMLTree(XMLTreeInfo *xml_info,
 %
 %    o xml:  The XML string.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -1134,15 +1094,15 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
     c,
     encoding;
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
   size_t
     extent;
 
+  ssize_t
+    j;
+
   utf8=(char *) AcquireQuantumMemory(*length,sizeof(*utf8));
   if (utf8 == (char *) NULL)
     return((char *) NULL);
@@ -1157,11 +1117,11 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
     }
   j=0;
   extent=(*length);
-  for (i=2; i < (long) (*length-1); i+=2)
+  for (i=2; i < (ssize_t) (*length-1); i+=2)
   {
     c=(encoding != 0) ? ((content[i] & 0xff) << 8) | (content[i+1] & 0xff) :
       ((content[i+1] & 0xff) << 8) | (content[i] & 0xff);
-    if ((c >= 0xd800) && (c <= 0xdfff) && ((i+=2) < (long) (*length-1)))
+    if ((c >= 0xd800) && (c <= 0xdfff) && ((i+=2) < (ssize_t) (*length-1)))
       {
         byte=(encoding != 0) ? ((content[i] & 0xff) << 8) |
           (content[i+1] & 0xff) : ((content[i+1] & 0xff) << 8) |
@@ -1213,7 +1173,7 @@ static char *ParseEntities(char *xml,char **entities,int state)
     *p,
     *q;
 
-  register long
+  register ssize_t
     i;
 
   size_t
@@ -1351,7 +1311,7 @@ static char *ParseEntities(char *xml,char **entities,int state)
       */
       for (xml=p; *xml != '\0'; xml++)
       {
-        i=(long) strspn(xml," ");
+        i=(ssize_t) strspn(xml," ");
         if (i != 0)
           (void) CopyMagickMemory(xml,xml+i,strlen(xml+i)+1);
         while ((*xml != '\0') && (*xml != ' '))
@@ -1405,7 +1365,7 @@ static XMLTreeInfo *ParseCloseTag(XMLTreeRoot *root,char *tag,
 
 static MagickBooleanType ValidateEntities(char *tag,char *xml,char **entities)
 {
-  register long
+  register ssize_t
     i;
 
   /*
@@ -1436,12 +1396,12 @@ static void ParseProcessingInstructions(XMLTreeRoot *root,char *xml,
   char
     *target;
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
+  ssize_t
+    j;
+
   target=xml;
   xml[length]='\0';
   xml+=strcspn(xml,XMLWhitespace);
@@ -1519,12 +1479,12 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
     *t,
     *v;
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
+  ssize_t
+    j;
+
   n=(char *) NULL;
   predefined_entitites=(char **) AcquireMagickMemory(sizeof(sentinel));
   if (predefined_entitites == (char **) NULL)
@@ -1611,9 +1571,10 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
             *xml='\0';
             i=0;
             while ((root->attributes[i] != (char **) NULL) &&
-                    (strcmp(n,root->attributes[i][0]) != 0))
+                   (strcmp(n,root->attributes[i][0]) != 0))
               i++;
-            while (*(n=(++xml)+strspn(xml,XMLWhitespace)) && (*n != '>'))
+            while ((*(n=xml+strspn(xml+1,XMLWhitespace)+1) != '\0') &&
+                   (*n != '>'))
             {
               xml=n+strcspn(n,XMLWhitespace);
               if (*xml != '\0')
@@ -1751,21 +1712,21 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
     c,
     terminal;
 
-  long
-    j,
-    l;
+  MagickBooleanType
+    status;
 
   register char
     *p;
 
-  register long
+  register ssize_t
     i;
 
   size_t
     length;
 
-  MagickBooleanType
-    status;
+  ssize_t
+    j,
+    l;
 
   XMLTreeRoot
     *root;
@@ -1806,7 +1767,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
     attributes=(char **) sentinel;
     tag=p;
     if ((isalpha((int) ((unsigned char) *p)) !=0) || (*p == '_') ||
-        (*p == ':') || (*p < '\0'))
+        (*p == ':') || (((int) *p) < '\0'))
       {
         /*
           Tag.
@@ -2005,7 +1966,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                 for (l=0; (*p != '\0') && (((l == 0) && (*p != '>')) ||
                      ((l != 0) && ((*p != ']') ||
                      (*(p+strspn(p+1,XMLWhitespace)+1) != '>'))));
-                  l=(*p == '[') ? 1 : l)
+                  l=(ssize_t) ((*p == '[') ? 1 : l))
                 p+=strcspn(p+1,"[]>")+1;
                 if ((*p == '\0') && (terminal != '>'))
                   {
@@ -2140,8 +2101,8 @@ MagickExport XMLTreeInfo *NewXMLTreeTag(const char *tag)
   (void) CopyMagickMemory(root->entities,predefined_entities,
     sizeof(predefined_entities));
   root->root.attributes=sentinel;
-  root->attributes=(char ***) sentinel;
-  root->processing_instructions=(char ***) sentinel;
+  root->attributes=(char ***) root->root.attributes;
+  root->processing_instructions=(char ***) root->root.attributes;
   root->debug=IsEventLogging();
   root->signature=MagickSignature;
   return(&root->root);
@@ -2246,14 +2207,11 @@ MagickExport XMLTreeInfo *PruneTagFromXMLTree(XMLTreeInfo *xml_info)
 MagickExport XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
   const char *tag,const char *value)
 {
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
-  size_t
-    length;
+  ssize_t
+    j;
 
   assert(xml_info != (XMLTreeInfo *) NULL);
   assert((xml_info->signature == MagickSignature) ||
@@ -2285,7 +2243,7 @@ MagickExport XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
           "UnableToAcquireString");
       xml_info->attributes[i]=ConstantString(tag);
       xml_info->attributes[i+2]=(char *) NULL;
-      length=strlen(xml_info->attributes[i+1]);
+      (void) strlen(xml_info->attributes[i+1]);
     }
   /*
     Add new value to an existing attribute.
@@ -2418,15 +2376,15 @@ static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
   const char
     *attribute;
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
   size_t
     offset;
 
+  ssize_t
+    j;
+
   content=(char *) "";
   if (xml_info->parent != (XMLTreeInfo *) NULL)
     content=xml_info->parent->content;
@@ -2520,21 +2478,21 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
   char
     *xml;
 
-  long
-    j,
-    k;
-
   register char
     *p,
     *q;
 
-  register long
+  register ssize_t
     i;
 
   size_t
     extent,
     length;
 
+  ssize_t
+    j,
+    k;
+
   XMLTreeInfo
     *ordered,
     *parent;
