@@ -17,7 +17,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,23 +33,12 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Conjure interprets and executes scripts in the Magick Scripting Language
-%  (MSL). The Magick scripting language (MSL) will primarily benefit those
-%  that want to accomplish custom image processing tasks but do not wish
-%  to program, or those that do not have access to a Perl interpreter or a
-%  compiler. The interpreter is called conjure and here is an example script:
-%
-%      <?xml version="1.0" encoding="UTF-8"?>
-%      <image size="400x400" >
-%      <read filename="image.gif" />
-%      <get width="base-width" height="base-height" />
-%      <resize geometry="%[dimensions]" />
-%      <get width="width" height="height" />
-%      <print output="Image sized from %[base-width]x%[base-height]
-%          to %[width]x%[height].\n" />
-%      <write filename="image.png" />
-%      </image>
-%
+%  The conjure program gives you the ability to perform custom image processing
+%  tasks from a script written in the Magick Scripting Language (MSL). MSL is
+%  XML-based and consists of action statements with attributes. Actions include
+%  reading an image, processing an image, getting attributes from an image,
+%  writing an image, and more. An attribute is a key/value pair that modifies
+%  the behavior of an action.
 %
 */
 
@@ -94,11 +83,11 @@
 %
 %    o metadata: any metadata is returned here.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
-static void ConjureUsage(void)
+static MagickBooleanType ConjureUsage(void)
 {
   const char
     **p;
@@ -123,8 +112,9 @@ static void ConjureUsage(void)
       (char *) NULL
     };
 
-  (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
-  (void) printf("Copyright: %s\n\n",GetMagickCopyright());
+  (void) printf("Version: %s\n",GetMagickVersion((size_t *) NULL));
+  (void) printf("Copyright: %s\n",GetMagickCopyright());
+  (void) printf("Features: %s\n\n",GetMagickFeatures());
   (void) printf("Usage: %s [options ...] file [ [options ...] file ...]\n",
     GetClientName());
   (void) printf("\nImage Settings:\n");
@@ -133,10 +123,10 @@ static void ConjureUsage(void)
   (void) printf("\nMiscellaneous Options:\n");
   for (p=miscellaneous; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
-  (void) printf("\nIn additiion, define any key value pairs required by "
+  (void) printf("\nIn addition, define any key value pairs required by "
     "your script.  For\nexample,\n\n");
   (void) printf("    conjure -size 100x100 -color blue -foo bar script.msl\n");
-  exit(0);
+  return(MagickFalse);
 }
 
 WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
@@ -145,7 +135,7 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
 #define DestroyConjure() \
 { \
   image=DestroyImageList(image); \
-  for (i=0; i < (long) argc; i++) \
+  for (i=0; i < (ssize_t) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
 }
@@ -159,7 +149,7 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
 #define ThrowConjureInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyConjure(); \
   return(MagickFalse); \
 }
@@ -170,15 +160,15 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
   Image
     *image;
 
-  long
-    number_images;
-
   MagickStatusType
     status;
 
-  register long
+  register ssize_t
     i;
 
+  ssize_t
+    number_images;
+
   /*
     Set defaults.
   */
@@ -188,7 +178,7 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(exception != (ExceptionInfo *) NULL);
   if (argc < 2)
-    ConjureUsage();
+    return(ConjureUsage());
   image=NewImageList();
   number_images=0;
   option=(char *) NULL;
@@ -199,34 +189,47 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
     ThrowConjureException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
-  for (i=1; i < (long) argc; i++)
+      GetExceptionMessage(errno));
+  for (i=1; i < (ssize_t) argc; i++)
   {
     option=argv[i];
-    if (IsMagickOption(option) != MagickFalse)
+    if (IsCommandOption(option) != MagickFalse)
       {
+        if (LocaleCompare("concurrent",option+1) == 0)
+          break;
         if (LocaleCompare("debug",option+1) == 0)
           {
-            long
+            ssize_t
               event;
 
             if (*option == '+')
               break;
             i++;
-            if (i == (long) argc)
+            if (i == (ssize_t) argc)
               ThrowConjureException(OptionError,"MissingArgument",option);
-            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            event=ParseCommandOption(MagickLogEventOptions,MagickFalse,argv[i]);
             if (event < 0)
               ThrowConjureException(OptionError,"UnrecognizedEventType",
                 argv[i]);
             (void) SetLogEventMask(argv[i]);
             continue;
           }
+        if (LocaleCompare("duration",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) (argc-1))
+              ThrowConjureException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConjureInvalidArgumentException(option,argv[i]);
+            continue;
+          }
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
           {
             if (*option == '-')
-              ConjureUsage();
+              return(ConjureUsage());
             continue;
           }
         if (LocaleCompare("log",option+1) == 0)
@@ -234,7 +237,7 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
             if (*option == '-')
               {
                 i++;
-                if (i == (long) argc)
+                if (i == (ssize_t) argc)
                   ThrowConjureException(OptionError,"MissingLogFormat",option);
                 (void) SetLogFormat(argv[i]);
               }
@@ -251,7 +254,7 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
             if (*option == '+')
               break;
             i++;
-            if (i == (long) (argc-1))
+            if (i == (ssize_t) (argc-1))
               ThrowConjureException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowConjureInvalidArgumentException(option,argv[i]);
@@ -266,10 +269,10 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
             (LocaleCompare("-version",option+1) == 0))
           {
             (void) fprintf(stdout,"Version: %s\n",
-              GetMagickVersion((unsigned long *) NULL));
-            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-            exit(0);
-            continue;
+              GetMagickVersion((size_t *) NULL));
+            (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
+            return(MagickFalse);
           }
         /*
           Persist key/value pair.
@@ -297,13 +300,13 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
     status=image != (Image *) NULL ? MagickTrue : MagickFalse;
     number_images++;
   }
-  if (i != argc)
+  if (i != (ssize_t) argc)
     ThrowConjureException(OptionError,"MissingAnImageFilename",argv[i]);
   if (number_images == 0)
     ThrowConjureException(OptionError,"MissingAnImageFilename",argv[argc-1]);
   if (image != (Image *) NULL)
     image=DestroyImageList(image);
-  for (i=0; i < (long) argc; i++)
+  for (i=0; i < (ssize_t) argc; i++)
     argv[i]=DestroyString(argv[i]);
   argv=(char **) RelinquishMagickMemory(argv);
   return(status != 0 ? MagickTrue : MagickFalse);
