@@ -23,7 +23,7 @@
 %                                 August 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -61,12 +61,6 @@
   return(MagickFalse); \
 }
 
-/*
-  Global declarations.
-*/
-static unsigned long
-  id = 0;
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -141,8 +135,8 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
       wand->name);
   (void) ResetMagickMemory(clone_wand,0,sizeof(*clone_wand));
   clone_wand->id=AcquireWandId();
-  (void) FormatMagickString(clone_wand->name,MaxTextExtent,"%s-%lu",
-    MagickWandId,clone_wand->id);
+  (void) FormatMagickString(clone_wand->name,MaxTextExtent,"%s-%.20g",
+    MagickWandId,(double) clone_wand->id);
   clone_wand->exception=AcquireExceptionInfo();
   InheritException(clone_wand->exception,wand->exception);
   clone_wand->image_info=CloneImageInfo(wand->image_info);
@@ -365,14 +359,14 @@ WandExport ExceptionType MagickGetExceptionType(const MagickWand *wand)
 %
 %  The format of the MagickGetIteratorIndex method is:
 %
-%      long MagickGetIteratorIndex(MagickWand *wand)
+%      ssize_t MagickGetIteratorIndex(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
 %    o wand: the magick wand.
 %
 */
-WandExport long MagickGetIteratorIndex(MagickWand *wand)
+WandExport ssize_t MagickGetIteratorIndex(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
   assert(wand->signature == WandSignature);
@@ -421,7 +415,7 @@ WandExport char *MagickQueryConfigureOption(const char *option)
   ExceptionInfo
     *exception;
 
-  unsigned long
+  size_t
     number_options;
 
   exception=AcquireExceptionInfo();
@@ -453,7 +447,7 @@ WandExport char *MagickQueryConfigureOption(const char *option)
 %  The format of the MagickQueryConfigureOptions function is:
 %
 %      char **MagickQueryConfigureOptions(const char *pattern,
-%        unsigned long *number_options)
+%        size_t *number_options)
 %
 %  A description of each parameter follows:
 %
@@ -464,7 +458,7 @@ WandExport char *MagickQueryConfigureOption(const char *option)
 %
 */
 WandExport char **MagickQueryConfigureOptions(const char *pattern,
-  unsigned long *number_options)
+  size_t *number_options)
 {
   char
     **options;
@@ -707,7 +701,7 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
 %
 %  The format of the MagickQueryFonts function is:
 %
-%      char **MagickQueryFonts(const char *pattern,unsigned long *number_fonts)
+%      char **MagickQueryFonts(const char *pattern,size_t *number_fonts)
 %
 %  A description of each parameter follows:
 %
@@ -718,7 +712,7 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
 %
 */
 WandExport char **MagickQueryFonts(const char *pattern,
-  unsigned long *number_fonts)
+  size_t *number_fonts)
 {
   char
     **fonts;
@@ -749,7 +743,7 @@ WandExport char **MagickQueryFonts(const char *pattern,
 %  The format of the MagickQueryFonts function is:
 %
 %      char **MagickQueryFonts(const char *pattern,
-%        unsigned long *number_formats)
+%        size_t *number_formats)
 %
 %  A description of each parameter follows:
 %
@@ -760,7 +754,7 @@ WandExport char **MagickQueryFonts(const char *pattern,
 %
 */
 WandExport char **MagickQueryFormats(const char *pattern,
-  unsigned long *number_formats)
+  size_t *number_formats)
 {
   char
     **formats;
@@ -796,7 +790,6 @@ WandExport char **MagickQueryFormats(const char *pattern,
 %
 %    o resource: Relinquish the memory associated with this resource.
 %
-%
 */
 WandExport void *MagickRelinquishMemory(void *memory)
 {
@@ -889,7 +882,7 @@ WandExport void MagickSetFirstIterator(MagickWand *wand)
 %  The format of the MagickSetIteratorIndex method is:
 %
 %      MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
-%        const long index)
+%        const ssize_t index)
 %
 %  A description of each parameter follows:
 %
@@ -899,7 +892,7 @@ WandExport void MagickSetFirstIterator(MagickWand *wand)
 %
 */
 WandExport MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
-  const long index)
+  const ssize_t index)
 {
   Image
     *image;
@@ -976,7 +969,6 @@ WandExport void MagickWandGenesis(void)
 {
   if (IsMagickInstantiated() == MagickFalse)
     MagickCoreGenesis((char *) NULL,MagickFalse);
-  id=AcquireWandId();
 }
 
 /*
@@ -994,12 +986,12 @@ WandExport void MagickWandGenesis(void)
 %
 %  The format of the MaickWandTerminus method is:
 %
-%      void MaickWandTerminus(void)
+%      void MagickWandTerminus(void)
 %
 */
 WandExport void MagickWandTerminus(void)
 {
-  RelinquishWandId(id);
+  DestroyWandIds();
   MagickCoreTerminus();
 }
 
@@ -1015,6 +1007,9 @@ WandExport void MagickWandTerminus(void)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  NewMagickWand() returns a wand required for all other methods in the API.
+%  A fatal exception is thrown if there is not enough memory to allocate the
+%  wand.   Use DestroyMagickWand() to dispose of the wand when it is no longer
+%  needed.
 %
 %  The format of the NewMagickWand method is:
 %
@@ -1029,7 +1024,7 @@ WandExport MagickWand *NewMagickWand(void)
   MagickWand
     *wand;
 
-  unsigned long
+  size_t
     depth;
 
   depth=MAGICKCORE_QUANTUM_DEPTH;
@@ -1039,11 +1034,11 @@ WandExport MagickWand *NewMagickWand(void)
   wand=(MagickWand *) AcquireMagickMemory(sizeof(*wand));
   if (wand == (MagickWand *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   (void) ResetMagickMemory(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
-  (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%lu",MagickWandId,
-    wand->id);
+  (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%.20g",MagickWandId,
+    (double) wand->id);
   wand->exception=AcquireExceptionInfo();
   wand->image_info=AcquireImageInfo();
   wand->quantize_info=CloneQuantizeInfo((QuantizeInfo *) NULL);
@@ -1070,7 +1065,7 @@ WandExport MagickWand *NewMagickWand(void)
 %
 %  The format of the NewMagickWandFromImage method is:
 %
-%      MagickWand NewMagickWandFromImage(const Image *image)
+%      MagickWand *NewMagickWandFromImage(const Image *image)
 %
 %  A description of each parameter follows:
 %
