@@ -387,7 +387,7 @@ int main(int argc,char **argv)
   MagickWand
     *clone_wand,
     *magick_wand;
- 
+
   PixelIterator
     *iterator;
 
@@ -397,27 +397,28 @@ int main(int argc,char **argv)
     *fill,
     **pixels;
 
-  register long
+  register ssize_t
     i;
 
-  size_t
-    length;
-
-  unsigned char 
+  unsigned char
     *profile;
 
   unsigned int
     status;
 
-  unsigned long
+  size_t
     columns,
     delay,
+    length,
     number_options,
     number_profiles,
     number_properties,
     number_wands,
     rows;
 
+  (void) argc;
+  (void) argv;
+  MagickWandGenesis();
   magick_wand=NewMagickWand();
   (void) MagickSetSize(magick_wand,640,480);
   (void) MagickGetSize(magick_wand,&columns,&rows);
@@ -446,17 +447,19 @@ int main(int argc,char **argv)
   if (status == MagickFalse)
     ThrowAPIException(magick_wand);
   if (MagickGetNumberImages(magick_wand) != 5)
-    (void) fprintf(stderr,"read %lu images; expected 5\n",
-      (unsigned long) MagickGetNumberImages(magick_wand));
+    (void) fprintf(stderr,"read %.20g images; expected 5\n",
+      (double) MagickGetNumberImages(magick_wand));
   (void) fprintf(stdout,"Iterate forward...\n");
   MagickResetIterator(magick_wand);
   while (MagickNextImage(magick_wand) != MagickFalse)
-    (void) fprintf(stdout,"index %ld scene %lu\n",
-      MagickGetIteratorIndex(magick_wand),MagickGetImageScene(magick_wand));
+    (void) fprintf(stdout,"index %.20g scene %.20g\n",(double)
+      MagickGetIteratorIndex(magick_wand),(double)
+      MagickGetImageScene(magick_wand));
   (void) fprintf(stdout,"Iterate reverse...\n");
   while (MagickPreviousImage(magick_wand) != MagickFalse)
-    (void) fprintf(stdout,"index %ld scene %lu\n",
-      MagickGetIteratorIndex(magick_wand),MagickGetImageScene(magick_wand));
+    (void) fprintf(stdout,"index %.20g scene %.20g\n",(double)
+      MagickGetIteratorIndex(magick_wand),(double)
+      MagickGetImageScene(magick_wand));
   (void) fprintf(stdout,"Remove scene 1...\n");
   (void) MagickSetIteratorIndex(magick_wand,1);
   clone_wand=MagickGetImage(magick_wand);
@@ -465,8 +468,9 @@ int main(int argc,char **argv)
     ThrowAPIException(magick_wand);
   MagickResetIterator(magick_wand);
   while (MagickNextImage(magick_wand) != MagickFalse)
-    (void) fprintf(stdout,"index %ld scene %lu\n",
-      MagickGetIteratorIndex(magick_wand),MagickGetImageScene(magick_wand));
+    (void) fprintf(stdout,"index %.20g scene %.20g\n",(double)
+      MagickGetIteratorIndex(magick_wand),(double)
+      MagickGetImageScene(magick_wand));
   (void) fprintf(stdout,"Insert scene 1 back in sequence...\n");
   (void) MagickSetIteratorIndex(magick_wand,0);
   status=MagickAddImage(magick_wand,clone_wand);
@@ -474,8 +478,9 @@ int main(int argc,char **argv)
     ThrowAPIException(magick_wand);
   MagickResetIterator(magick_wand);
   while (MagickNextImage(magick_wand) != MagickFalse)
-    (void) fprintf(stdout,"index %ld scene %lu\n",
-      MagickGetIteratorIndex(magick_wand),MagickGetImageScene(magick_wand));
+    (void) fprintf(stdout,"index %.20g scene %.20g\n",(double)
+      MagickGetIteratorIndex(magick_wand),(double)
+      MagickGetImageScene(magick_wand));
   (void) fprintf(stdout,"Set scene 2 to scene 1...\n");
   (void) MagickSetIteratorIndex(magick_wand,2);
   status=MagickSetImage(magick_wand,clone_wand);
@@ -484,8 +489,9 @@ int main(int argc,char **argv)
     ThrowAPIException(magick_wand);
   MagickResetIterator(magick_wand);
   while (MagickNextImage(magick_wand) != MagickFalse)
-    (void) fprintf(stdout,"index %ld scene %lu\n",
-      MagickGetIteratorIndex(magick_wand),MagickGetImageScene(magick_wand));
+    (void) fprintf(stdout,"index %.20g scene %.20g\n",(double)
+      MagickGetIteratorIndex(magick_wand),(double)
+      MagickGetImageScene(magick_wand));
   (void) fprintf(stdout,"Apply image processing options...\n");
   status=MagickCropImage(magick_wand,60,60,10,10);
   if (status == MagickFalse)
@@ -501,7 +507,7 @@ int main(int argc,char **argv)
   border=NewPixelWand();
   (void) PixelSetColor(background,"green");
   (void) PixelSetColor(border,"black");
-  status=MagickFloodfillPaintImage(magick_wand,AllChannels,background,
+  status=MagickFloodfillPaintImage(magick_wand,CompositeChannels,background,
     0.01*QuantumRange,border,0,0,MagickFalse);
   if (status == MagickFalse)
     ThrowAPIException(magick_wand);
@@ -542,11 +548,12 @@ int main(int argc,char **argv)
       };
 
     (void) MagickSetIteratorIndex(magick_wand,2);
-    status=MagickSetImagePixels(magick_wand,10,10,3,3,"RGB",CharPixel,
+    status=MagickImportImagePixels(magick_wand,10,10,3,3,"RGB",CharPixel,
       primary_colors);
     if (status == MagickFalse)
       ThrowAPIException(magick_wand);
-    status=MagickGetImagePixels(magick_wand,10,10,3,3,"RGB",CharPixel,pixels);
+    status=MagickExportImagePixels(magick_wand,10,10,3,3,"RGB",CharPixel,
+      pixels);
     if (status == MagickFalse)
       ThrowAPIException(magick_wand);
     for (i=0; i < 9; i++)
@@ -560,6 +567,13 @@ int main(int argc,char **argv)
   status=MagickResizeImage(magick_wand,50,50,UndefinedFilter,1.0);
   if (status == MagickFalse)
     ThrowAPIException(magick_wand);
+  MagickResetIterator(magick_wand);
+  while (MagickNextImage(magick_wand) != MagickFalse)
+  {
+    (void) MagickSetImageDepth(magick_wand,8);
+    (void) MagickSetImageCompression(magick_wand,RLECompression);
+  }
+  MagickResetIterator(magick_wand);
   (void) MagickSetIteratorIndex(magick_wand,4);
   (void) fprintf(stdout,"Utilitize pixel iterator to draw diagonal...\n");
   iterator=NewPixelIterator(magick_wand);
@@ -612,7 +626,7 @@ int main(int argc,char **argv)
   options=MagickGetOptions(magick_wand,"*",&number_options);
   if (options != (char **) NULL)
     {
-      for (i=0; i < (long) number_options; i++)
+      for (i=0; i < (ssize_t) number_options; i++)
       {
         (void) fprintf(stdout,"  %s\n",options[i]);
         options[i]=(char *) MagickRelinquishMemory(options[i]);
@@ -638,7 +652,7 @@ int main(int argc,char **argv)
   properties=MagickGetImageProperties(magick_wand,"*",&number_properties);
   if (properties != (char **) NULL)
     {
-      for (i=0; i < (long) number_properties; i++)
+      for (i=0; i < (ssize_t) number_properties; i++)
       {
         (void) fprintf(stdout,"  %s\n",properties[i]);
         properties[i]=(char *) MagickRelinquishMemory(properties[i]);
@@ -664,7 +678,7 @@ int main(int argc,char **argv)
   profiles=MagickGetImageProfiles(magick_wand,"*",&number_profiles);
   if (profiles != (char **) NULL)
     {
-      for (i=0; i < (long) number_profiles; i++)
+      for (i=0; i < (ssize_t) number_profiles; i++)
       {
         (void) fprintf(stdout,"  %s\n",profiles[i]);
         profiles[i]=(char *) MagickRelinquishMemory(profiles[i]);
@@ -682,5 +696,5 @@ int main(int argc,char **argv)
   profile=(unsigned char *) MagickRelinquishMemory(profile);
   magick_wand=DestroyMagickWand(magick_wand);
   (void) fprintf(stdout,"Wand tests pass.\n");
-  return(0); 
+  return(0);
 }
